@@ -66,7 +66,6 @@ static void InitFont(BFont_Info * Font)
 			flip_image_vertically(Font->char_iso_image[i].surface);
 
 			SDL_FreeSurface(tmp_char1);
-			//--------------------
 			// Now we can go on to the next char
 			//
 			i++;
@@ -124,6 +123,7 @@ BFont_Info *LoadFont(const char *filename)
 				SetCurrentFont(Font);
 
 			} else {
+				fprintf(stderr, "Loading font \"%s\":%s\n", filename, IMG_GetError());
 				/* free memory allocated for the BFont_Info structure */
 				free(Font);
 				Font = NULL;
@@ -168,15 +168,56 @@ int FontHeight(BFont_Info * Font)
 	return (Font->h);
 };				// int FontHeight (BFont_Info * Font)
 
-/* Return the width of the "c" character */
+/**
+ * Return the width of specified character
+ */
 int CharWidth(BFont_Info * Font, unsigned char c)
 {
-
 	if (c < ' ' || c > Font->number_of_chars - 1)
 		c = '.';
 	return Font->Chars[c].w;
 }
 
+/**
+ * Get letter-spacing for specified font.
+ *
+ * Letter-spacing refers to the overall spacing of a word or block of text
+ * affecting its overall density and texture.
+ */
+int get_letter_spacing(BFont_Info *font) {
+	if (font == FPS_Display_BFont || font == Blue_BFont || font == Red_BFont)
+		return -2;
+	else if (font == Menu_BFont)
+		return -4;
+	else
+		return 0;
+}
+
+/**
+ * Handle font switching on special characters. Returns 1 if the font was
+ * changed and 0 if it was not.
+ */
+int handle_switch_font_char(unsigned char c)
+{
+	if (c == font_switchto_red[0]) {
+		SetCurrentFont(Red_BFont);
+		return TRUE;
+	} else if (c == font_switchto_blue[0]) {
+		SetCurrentFont(Blue_BFont);
+		return TRUE;
+	} else if (c == font_switchto_neon[0]) {
+		SetCurrentFont(FPS_Display_BFont);
+		return TRUE;
+	} else if (c == font_switchto_msgstat[0]) {
+		SetCurrentFont(Messagestat_BFont);
+		return TRUE;
+	} else if (c == font_switchto_msgvar[0]) {
+		SetCurrentFont(Messagevar_BFont);
+		return TRUE;
+	}
+	return FALSE;
+}
+
 /**
  * Puts a single char on the surface with the specified font 
  */
@@ -197,7 +238,6 @@ int PutCharFont(SDL_Surface * Surface, BFont_Info * Font, int x, int y, unsigned
 
 			if ((dest.x < clipping_rect.x + clipping_rect.w) && (dest.x >= clipping_rect.x)) {
 #ifdef HAVE_LIBGL
-				//--------------------
 				// We need to simulate the clipping as would be done by SDL.
 				// We simply do "all or none" clipping for left and right, but for the top/bottom of the screen,
 				// we use GL to do proper clipping in order not to see a whole line disappear at once.
@@ -262,59 +302,30 @@ int PutCharFont(SDL_Surface * Surface, BFont_Info * Font, int x, int y, unsigned
 };				// int PutCharFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, int c)
 
 /**
- *
  *
  */
-void PutString(SDL_Surface * Surface, int x, int y, char *text)
-{
-	PutStringFont(Surface, CurrentFont, x, y, text);
-};				// void PutString (SDL_Surface * Surface, int x, int y, char *text)
-
-/**
- *
- *
- */
-void PutStringFont(SDL_Surface * Surface, BFont_Info * Font, int x, int y, char *text)
+void PutString(SDL_Surface *surface, int x, int y, char *text)
 {
 	int i = 0;
-	//--------------------
-	// I added little hack to kern MenuFont..
-	// This basicly just prints them more tight on the screen.
-	// basse, 15.2.03
-	//
-	int kerning = 0;
-	if (Font == Menu_BFont)
-		kerning = -4;
-
-	if (Font == FPS_Display_BFont || Font == Blue_BFont || Font == Red_BFont)
-		kerning = -2;
 
 	while (text[i] != '\0') {
-		//--------------------
-		// Here I've added some hack to allow to give a font switching
-		// directive with a text through various menus and therefore
-		// switch the font even multiple times in one big text.
-		//                                          jp, 27.7.2002
-		//
-		switch (text[i]) {
-		case 1:
-			Font = Red_BFont;
-			kerning = -2;
-			break;
-		case 2:
-			Font = Blue_BFont;
-			kerning = -2;
-			break;
-		case 3:
-			Font = FPS_Display_BFont;
-			kerning = -2;
-			break;
-		default:
-			x += PutCharFont(Surface, Font, x, y, text[i]) + kerning;
+		int letter_spacing = get_letter_spacing(GetCurrentFont());
+		if (!handle_switch_font_char(text[i])) {
+			x += PutCharFont(surface, GetCurrentFont(), x, y, text[i]) + letter_spacing;
 		}
 		i++;
 	}
-};				// void PutStringFont (SDL_Surface * Surface, BFont_Info * Font, int x, int y, char *text)
+}
+
+/**
+ * Write a string on a surface using specified font, taking letter-spacing
+ * into account.
+ */
+void PutStringFont(SDL_Surface *surface, BFont_Info *font, int x, int y, char *text)
+{
+	SetCurrentFont(font);
+	PutString(surface, x, y, text);
+}
 
 /**
  *
@@ -322,51 +333,48 @@ void PutStringFont(SDL_Surface * Surface, BFont_Info * Font, int x, int y, char
  */
 int TextWidth(char *text)
 {
-	return (TextWidthFont(CurrentFont, text));
-};				// int TextWidth (char *text)
+	return TextWidthFont(CurrentFont, text);
+}
+
+/**
+ * Calculate the width of a string using a certain font, taking letter-spacing
+ * into account.
+ */
+int TextWidthFont(BFont_Info *font, char *text)
+{
+	int i = 0, width = 0;
+	int letter_spacing = get_letter_spacing(font);
+	while (text[i] != '\0') {
+		if (handle_switch_font_char(text[i]))
+			letter_spacing = get_letter_spacing(font);
+		else
+			width += CharWidth(font, text[i]) + letter_spacing;
+		i++;
+	}
+	return width;
+}
 
 /**
  *
  *
  */
-int TextWidthFont(BFont_Info * Font, char *text)
+static int LimitTextWidthFont(BFont_Info *font, char *text, int limit)
 {
-	int i = 0, x = 0;
-	//--------------------
-	// Based on Bastians hack: 
-	// 
-	// 'I added little hack to kern MenuFont..
-	// This basicly just prints them more tight on the screen.
-	// basse, 15.2.03'
-	//
-	// I extend this to give new text width results...
-	//
-	int kerning = 0;
-	if (Font == Menu_BFont)
-		kerning = -4;
-	if (Font == FPS_Display_BFont || Font == Blue_BFont || Font == Red_BFont)
-		kerning = -2;
-
+	int i = 0, width = 0;
+	int letter_spacing = get_letter_spacing(font);
 	while (text[i] != '\0') {
-		switch (text[i]) {
-		case 1:
-			Font = Red_BFont;
-			kerning = -2;
-			break;
-		case 2:
-			Font = Blue_BFont;
-			kerning = -2;
-			break;
-		case 3:
-			Font = FPS_Display_BFont;
-			kerning = -2;
-			break;
+		if (handle_switch_font_char(text[i])) {
+			letter_spacing = get_letter_spacing(font);
+			i++;
+			continue;
 		}
-		x += CharWidth(Font, text[i]) + kerning;
+		width += CharWidth(font, text[i]) + letter_spacing;
 		i++;
+		if (width >= limit)
+			return i;
 	}
-	return x;
-};				// int TextWidthFont (BFont_Info * Font, char *text)
+	return -1;
+}
 
 /**
  *
@@ -375,52 +383,7 @@ int TextWidthFont(BFont_Info * Font, char *text)
 int LimitTextWidth(char *text, int limit)
 {
 	return (LimitTextWidthFont(CurrentFont, text, limit));
-};				// int LimitTextWidth (char *text, int limit)
-
-/**
- *
- *
- */
-int LimitTextWidthFont(BFont_Info * Font, char *text, int limit)
-{
-	int i = 0, x = 0;
-	//--------------------
-	// Based on Bastians hack: 
-	// 
-	// 'I added little hack to kern MenuFont..
-	// This basicly just prints them more tight on the screen.
-	// basse, 15.2.03'
-	//
-	// I extend this to give new text width results...
-	//
-	int kerning = 0;
-	if (Font == Menu_BFont)
-		kerning = -4;
-	if (Font == FPS_Display_BFont || Font == Blue_BFont || Font == Red_BFont)
-		kerning = -2;
-
-	while (text[i] != '\0') {
-		switch (text[i]) {
-		case 1:
-			Font = Red_BFont;
-			kerning = -2;
-			break;
-		case 2:
-			Font = Blue_BFont;
-			kerning = -2;
-			break;
-		case 3:
-			Font = FPS_Display_BFont;
-			kerning = -2;
-			break;
-		}
-		x += CharWidth(Font, text[i]) + kerning;
-		i++;
-		if (x >= limit)
-			return i;
-	}
-	return -1;
-};				// int LimitTextWidthFont (BFont_Info * Font, char *text, int limit)
+}
 
 /* counts the spaces of the strings */
 int count(char *text)
@@ -543,133 +506,6 @@ void PrintString(SDL_Surface * Surface, int x, int y, const char *fmt, ...)
 	va_end(args);
 }
 
-void PrintStringFont(SDL_Surface * Surface, BFont_Info * Font, int x, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-		PutStringFont(Surface, Font, x, y, temp);
-		free(temp);
-	}
-	va_end(args);
-}
-
-void CenteredPrintString(SDL_Surface * Surface, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-		CenteredPutString(Surface, y, temp);
-		free(temp);
-	}
-	va_end(args);
-}
-
-void CenteredPrintStringFont(SDL_Surface * Surface, BFont_Info * Font, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-		CenteredPutStringFont(Surface, Font, y, temp);
-		free(temp);
-	}
-	va_end(args);
-
-}
-
-void RightPrintString(SDL_Surface * Surface, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-		RightPutString(Surface, y, temp);
-		free(temp);
-	}
-	va_end(args);
-}
-
-void RightPrintStringFont(SDL_Surface * Surface, BFont_Info * Font, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-		RightPutStringFont(Surface, Font, y, temp);
-		free(temp);
-	}
-	va_end(args);
-}
-
-void LeftPrintString(SDL_Surface * Surface, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-		LeftPutString(Surface, y, temp);
-		free(temp);
-	}
-	va_end(args);
-}
-
-void LeftPrintStringFont(SDL_Surface * Surface, BFont_Info * Font, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-		LeftPutStringFont(Surface, Font, y, temp);
-		free(temp);
-	}
-	va_end(args);
-}
-
-void JustifiedPrintString(SDL_Surface * Surface, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-		JustifiedPutString(Surface, y, temp);
-		free(temp);
-	}
-	va_end(args);
-}
-
-void JustifiedPrintStringFont(SDL_Surface * Surface, BFont_Info * Font, int y, const char *fmt, ...)
-{
-	va_list args;
-	char *temp;
-	va_start(args, fmt);
-
-	if ((temp = (char *)malloc(1000 + 1)) != NULL) {
-		vsprintf(temp, fmt, args);
-		JustifiedPutStringFont(Surface, Font, y, temp);
-		free(temp);
-	}
-	va_end(args);
-}
-
 /*********************************************************************************************************/
 /*********************************************************************************************************/
 /*********************************************************************************************************/
@@ -683,41 +519,6 @@ inline void PutPixel32(SDL_Surface * surface, int x, int y, Uint32 pixel)
 
 }				// void PutPixel ( ... )
 
-inline void PutPixel24(SDL_Surface * surface, int x, int y, Uint32 pixel)
-{
-	Uint8 *p;
-	p = (Uint8 *) surface->pixels + y * surface->pitch + x * 3;
-
-	if (SDL_BYTEORDER == SDL_BIG_ENDIAN) {
-		p[0] = (pixel >> 16) & 0xff;
-		p[1] = (pixel >> 8) & 0xff;
-		p[2] = pixel & 0xff;
-	} else {
-		p[0] = pixel & 0xff;
-		p[1] = (pixel >> 8) & 0xff;
-		p[2] = (pixel >> 16) & 0xff;
-	}
-
-}				// void PutPixel ( ... )
-
-inline void PutPixel16(SDL_Surface * surface, int x, int y, Uint32 pixel)
-{
-	Uint8 *p;
-	p = (Uint8 *) surface->pixels + y * surface->pitch + x * 2;
-
-	*(Uint16 *) p = pixel;
-
-}				// void PutPixel ( ... )
-
-inline void PutPixel8(SDL_Surface * surface, int x, int y, Uint32 pixel)
-{
-	Uint8 *p;
-	p = (Uint8 *) surface->pixels + y * surface->pitch + x;
-
-	*p = pixel;
-
-}				// void PutPixel ( ... )
-
 void PutPixel(SDL_Surface * surface, int x, int y, Uint32 pixel)
 {
 	int bpp = surface->format->BytesPerPixel;
@@ -730,7 +531,6 @@ void PutPixel(SDL_Surface * surface, int x, int y, Uint32 pixel)
 		}
 	}
 
-	//--------------------
 	// Here I add a security query against segfaults due to writing
 	// perhaps even far outside of the surface pixmap data.
 	//
@@ -773,31 +573,6 @@ void PutPixel(SDL_Surface * surface, int x, int y, Uint32 pixel)
  * NOTE:  I THINK THE SURFACE MUST BE LOCKED FOR THIS!
  *
  */
-inline Uint32 FdGetPixel32(SDL_Surface * Surface, Sint32 X, Sint32 Y)
-{
-	return *((Uint32 *) Surface->pixels + Y * Surface->pitch / 4 + X);
-}
-
-inline Uint32 FdGetPixel24(SDL_Surface * Surface, Sint32 X, Sint32 Y)
-{
-
-	Uint8 *bits = ((Uint8 *) Surface->pixels) + Y * Surface->pitch + X * 3;
-
-	return SDL_MapRGB(Surface->format, *((bits) + Surface->format->Rshift / 8), *((bits) + Surface->format->Gshift / 8),
-			  *((bits) + Surface->format->Bshift / 8));
-
-}
-
-inline Uint16 FdGetPixel16(SDL_Surface * Surface, Sint32 X, Sint32 Y)
-{
-	return *((Uint16 *) Surface->pixels + Y * Surface->pitch / 2 + X);
-}
-
-inline Uint8 FdGetPixel8(SDL_Surface * Surface, Sint32 X, Sint32 Y)
-{
-	return *((Uint8 *) Surface->pixels + Y * Surface->pitch + X);
-}
-
 Uint32 FdGetPixel(SDL_Surface * Surface, Sint32 X, Sint32 Y)
 {
 
