@@ -1,7 +1,7 @@
 /* 
  *
  *   Copyright (c) 2002, 2003 Johannes Prix
- *   Copyright (c) 2004-2007 Arthur Huillet
+ *   Copyright (c) 2004-2010 Arthur Huillet
  *
  *  This file is part of Freedroid
  *
@@ -56,24 +56,28 @@
 // x-offset for numbers in the "health / temperature / stamina" box
 #define HEALTH_NR_X 125
 // x-offset for numbers in the "armor / attack / damage" box
-#define ARMOR_NR_X 255
+#define ARMOR_NR_X 260
 // x-offset for values in the skills box
 #define SKILLS_VALUE_X 198
 
-// y-offsets
-#define MELEE_SKILL_Y 230
-#define RANGED_SKILL_Y 268
-#define SPELLCASTING_SKILL_Y 306
+// y-offsets - left
+// STR_X_Y - VIT_Y in defs.h
+#define POINTS_Y 245
 #define HEALTH_STAT_Y 289
 #define TEMP_STAT_Y 308
 #define STAMINA_STAT_Y 327
-#define DR_Y 140
-#define TOHIT_Y 165
-#define DAMAGE_Y 190
+
+// y-offsets - right
+#define TOHIT_Y 138
+#define DAMAGE_Y 158
+#define DAMRED_Y 178
+#define MELEE_SKILL_Y 230
+#define RANGED_SKILL_Y 268
+#define SPELLCASTING_SKILL_Y 306
 
 /**
  * This function displays all the buttons that open up the character
- * screen and the invenotry screen
+ * screen and the inventory screen
  */
 void DisplayButtons(void)
 {
@@ -94,7 +98,6 @@ void DisplayButtons(void)
 	if (GameConfig.screen_height == 480 && (GameConfig.SkillScreen_Visible || GameConfig.CharacterScreen_Visible))
 		return;
 
-	//--------------------
 	// When the Tux has some extra skill points, that can be distributed
 	// to some character stats or saved for later training with some trainer
 	// character in the city, we mark the character screen toggle button as
@@ -119,21 +122,16 @@ void DisplayButtons(void)
  */
 static void AddInfluencerItemAttributeBonus(item * BonusItem)
 {
-	//--------------------
 	// In case of no item, the thing to do is pretty easy...
 	//
 	if (BonusItem->type == (-1))
 		return;
 
-	//--------------------
-	// In case of a suffix modifier, we need to apply the suffix...
-	//
-	if (((BonusItem->suffix_code != (-1)) || (BonusItem->prefix_code != (-1))) && BonusItem->is_identified) {
-		Me.Strength += BonusItem->bonus_to_str + BonusItem->bonus_to_all_attributes;
-		Me.Dexterity += BonusItem->bonus_to_dex + BonusItem->bonus_to_all_attributes;
-		Me.Magic += BonusItem->bonus_to_mag + BonusItem->bonus_to_all_attributes;
-		Me.Vitality += BonusItem->bonus_to_vit + BonusItem->bonus_to_all_attributes;
-	}
+	// Apply the bonuses to the stats of the character.
+	Me.Strength += BonusItem->bonus_to_str + BonusItem->bonus_to_all_attributes;
+	Me.Dexterity += BonusItem->bonus_to_dex + BonusItem->bonus_to_all_attributes;
+	Me.Magic += BonusItem->bonus_to_cooling + BonusItem->bonus_to_all_attributes;
+	Me.Vitality += BonusItem->bonus_to_physique + BonusItem->bonus_to_all_attributes;
 
 };				// void AddInfluencerItemAttributeBonus( item* BonusItem )
 
@@ -143,47 +141,25 @@ static void AddInfluencerItemAttributeBonus(item * BonusItem)
  */
 static void AddInfluencerItemSecondaryBonus(item * BonusItem)
 {
-	//--------------------
 	// In case of no item, the thing to do is pretty easy...
 	//
 	if (BonusItem->type == (-1))
 		return;
 
-	//--------------------
-	// In case of present suffix or prefix modifiers, we need to apply the suffix...
-	//
-	if (((BonusItem->suffix_code != (-1)) || (BonusItem->prefix_code != (-1))) && BonusItem->is_identified) {
+	// Apply the bonuses to the stats of the character.
+	Me.to_hit += BonusItem->bonus_to_attack;
+	Me.maxenergy += BonusItem->bonus_to_health_points;
+	Me.health_recovery_rate += BonusItem->bonus_to_health_recovery;
+	Me.cooling_rate += BonusItem->bonus_to_cooling_rate;
+	Me.resist_fire += BonusItem->bonus_to_resist_fire;
+	Me.resist_electricity += BonusItem->bonus_to_resist_electricity;
 
-		//--------------------
-		// Some modifiers might not be random at all but fixed to the 
-		// item prefix or item suffix.  In that case, we must get the
-		// modifier strength from the suffix/prefix spec itself...
-		//
-		if (BonusItem->suffix_code != (-1))
-			Me.light_bonus_from_tux += SuffixList[BonusItem->suffix_code].light_bonus_value;
-		if (BonusItem->prefix_code != (-1))
-			Me.light_bonus_from_tux += PrefixList[BonusItem->prefix_code].light_bonus_value;
-
-		//--------------------
-		// Now we can apply the modifiers, that have been generated from
-		// the suffix spec and then (with some randomness) written into the
-		// item itself.  In that case we won't need the suffix- or 
-		// prefix-lists here...
-		//
-		Me.to_hit += BonusItem->bonus_to_tohit;
-		Me.max_temperature += BonusItem->bonus_to_force;
-		Me.maxenergy += BonusItem->bonus_to_life;
-		Me.health_recovery_rate += BonusItem->bonus_to_health_recovery;
-		Me.cooling_rate += BonusItem->bonus_to_cooling_rate;
-
-		Me.resist_fire += BonusItem->bonus_to_resist_fire;
-		Me.resist_electricity += BonusItem->bonus_to_resist_electricity;
-
-		// if ( ItemMap [ BonusItem->type ] . can_be_installed_in_weapon_slot )
-		// Me.freezing_enemys_property += BonusItem->freezing_time_in_seconds;
-	}
-
-};				// void AddInfluencerItemSecondaryBonus( item* BonusItem )
+	// Apply bonuses to the special abilities of the character.
+	Me.slowing_melee_targets += BonusItem->bonus_to_slow_enemy;
+	Me.paralyzing_melee_targets += BonusItem->bonus_to_paralyze_enemy;
+	Me.light_bonus_from_tux += BonusItem->bonus_to_light_radius;
+	Me.experience_factor += BonusItem->bonus_to_experience_gain / 100.0;
+}
 
 /**
  * Maybe the influencer has reached a new experience level?
@@ -198,7 +174,6 @@ void check_for_new_experience_level_reached()
 
 	Me.ExpRequired = BaseExpRequired * (exp((Me.exp_level - 1) * log(2)));
 
-	//--------------------
 	// For display reasons in the experience graph, we also state the experience 
 	// needed for the previous level inside the tux struct.  Therefore all exp/level
 	// calculations are found in this function.
@@ -217,13 +192,11 @@ void check_for_new_experience_level_reached()
 			return;
 		}
 
-		//--------------------
 		// Like in the Gothic 1 game, maximum life force will now automatically
 		// be increased upon reaching a new character level.
 		//
 		Me.base_vitality += 3;
 
-		//--------------------
 		// When a droid reaches a new experience level, all health and 
 		// force are restored to full this one time no longer.  Gothic
 		// rulez more than Diablo rulez.
@@ -231,7 +204,6 @@ void check_for_new_experience_level_reached()
 		// Me .energy = Me .maxenergy ;
 		// Me .mana   = Me .maxmana   ;
 
-		//--------------------
 		// Also when a new level is reached, we will display a big message
 		// right over the combat window.
 		//
@@ -248,7 +220,6 @@ void update_all_primary_stats()
 {
 	int i;
 
-	//--------------------
 	// Now we base PRIMARY stats
 	//
 	Me.Strength = Me.base_strength;
@@ -256,16 +227,8 @@ void update_all_primary_stats()
 	Me.Magic = Me.base_magic;
 	Me.Vitality = Me.base_vitality;
 
-	Me.freezing_melee_targets = 0;
 	Me.double_ranged_damage = FALSE;
 
-	//--------------------
-	// Now we re-initialize the SKILL LEVELS
-	//
-	for (i = 0; i < number_of_skills; i++)
-		Me.SkillLevel[i] = Me.base_skill_level[i];
-
-	//--------------------
 	// Now we add all bonuses to the influencers PRIMARY stats
 	//
 	AddInfluencerItemAttributeBonus(&Me.armour_item);
@@ -274,27 +237,6 @@ void update_all_primary_stats()
 	AddInfluencerItemAttributeBonus(&Me.shield_item);
 	AddInfluencerItemAttributeBonus(&Me.special_item);
 
-	item *itrot[5] = { &Me.armour_item, &Me.weapon_item, &Me.drive_item,
-		&Me.shield_item, &Me.special_item
-	};
-	i = 0;
-	while (i < 5) {
-		if (itrot[i]->type == -1) {
-			i++;
-			continue;
-		}
-		if (!ItemUsageRequirementsMet(itrot[i], FALSE)) {	//we have to move away the item
-			if (!AddFloorItemDirectlyToInventory(itrot[i]))
-				itrot[i]->type = -1;
-			else {	//inventory is full... ouch
-				DropItemToTheFloor(itrot[i], Me.pos.x, Me.pos.y, Me.pos.z);
-				itrot[i]->type = -1;
-			}
-		}
-		i++;
-	}
-
-	//--------------------
 	// Maybe there is some boost from a potion or magic spell going on right
 	// now...
 	//
@@ -303,7 +245,20 @@ void update_all_primary_stats()
 	if (Me.power_bonus_end_date > Me.current_game_date)
 		Me.Strength += Me.current_power_bonus;
 
-};				// void update_all_primary_stats ( )
+	// Unequip items whose requirements aren't met anymore.
+	// This needs to be done last so that temporary bonuses, for example from
+	// strength and dexterity capsules, allow the player to keep items equipped.
+	item *equipment[5] = { &Me.armour_item, &Me.weapon_item, &Me.drive_item,
+		&Me.shield_item, &Me.special_item };
+	for (i = 0; i < 5; i++) {
+		if (equipment[i]->type == -1) {
+			continue;
+		}
+		if (!ItemUsageRequirementsMet(equipment[i], FALSE)) {
+			give_item(equipment[i]);
+		}
+	}
+}
 
 /**
  * This function computes secondary stats (i.e. chances for success or
@@ -313,28 +268,23 @@ void update_all_primary_stats()
  */
 void update_secondary_stats_from_primary_stats()
 {
-	//--------------------
 	// The chance that this player character will score a hit on an enemy
 	//
 	Me.to_hit = 60 + (Me.Dexterity) * TOHIT_PERCENT_PER_DEX_POINT;
 
-	//--------------------
 	// How many life points can this character aquire currently
 	//
 	Me.maxenergy = (Me.Vitality) * Energy_Gain_Per_Vit_Point;
 
-	//--------------------
 	// The maximum mana value computed from the primary stats
 	//
 	Me.max_temperature = (Me.Magic) * Maxtemp_Gain_Per_CPU_Point;
 
-	//--------------------
 	// How long can this character run until he must take a break and
 	// walk a bit
 	//
 	Me.max_running_power = (Me.Strength) + (Me.Dexterity) + (Me.Vitality) + Me.running_power_bonus;
 
-	//--------------------
 	// base regeneration speed set to 0.2 points per second
 	Me.health_recovery_rate = 0.2;
 	Me.cooling_rate = 1.0;
@@ -350,7 +300,6 @@ void update_damage_tux_can_do()
 {
 	if (Me.weapon_item.type != (-1)) {
 		if (ItemMap[Me.weapon_item.type].item_weapon_is_melee != 0) {
-			//--------------------
 			// Damage modifier in case of MELEE WEAPON is computed:  
 			// weapon's modifier * (100+Strength)%
 			//
@@ -358,28 +307,24 @@ void update_damage_tux_can_do()
 
 			Me.damage_modifier = Me.weapon_item.damage_modifier * (Me.Strength + 100.0) / 100.0;
 
-			//--------------------
 			// Damage AND damage modifier a modified by additional melee weapon
 			// skill:  A multiplier is applied!
 			//
 			Me.damage_modifier *= MeleeDamageMultiplierTable[Me.melee_weapon_skill];
 			Me.base_damage *= MeleeDamageMultiplierTable[Me.melee_weapon_skill];
 		} else {
-			//--------------------
 			// Damage modifier in case of RANGED WEAPON is computed:  
 			// weapon's modifier * (100+Dexterity)%
 			//
 			Me.base_damage = Me.weapon_item.damage * (Me.Dexterity + 100.0) / 100.0;
 			Me.damage_modifier = Me.weapon_item.damage_modifier * (Me.Dexterity + 100.0) / 100.0;
 
-			//--------------------
 			// Damage AND damage modifier a modified by additional ranged weapon
 			// skill:  A multiplier is applied!
 			//
 			Me.damage_modifier *= RangedDamageMultiplierTable[Me.ranged_weapon_skill];
 			Me.base_damage *= RangedDamageMultiplierTable[Me.ranged_weapon_skill];
 
-			//--------------------
 			// Maybe there is a plugin for double damage with ranged
 			// weapons present?
 			//
@@ -389,7 +334,6 @@ void update_damage_tux_can_do()
 			}
 		}
 	} else {
-		//--------------------
 		// In case of no weapon equipped at all, we initialize
 		// the damage values with some simple numbers.  Currently
 		// strength and dexterity play NO ROLE in weaponless combat.
@@ -401,56 +345,23 @@ void update_damage_tux_can_do()
 	}
 };				// void update_damage_tux_can_do ( )
 
-/**
- *
- *
- */
-void update_tux_armour_damage_reduction()
-{
-	//--------------------
-	// We initialize the armour damage reduction
-
-	Me.DR = 0 ;
-
-	//--------------------
-	// Now we apply the armour bonuses from the currently equipped
-	// items to the total defence value
-	//
-	if (Me.armour_item.type != (-1)) {
-		Me.DR += Me.armour_item.dr_bonus;
-	}
-	if (Me.shield_item.type != (-1)) {
-		Me.DR += Me.shield_item.dr_bonus;
-	}
-	if (Me.special_item.type != (-1)) {
-		Me.DR += Me.special_item.dr_bonus;
-	}
-	if (Me.drive_item.type != (-1)) {
-		Me.DR += Me.drive_item.dr_bonus;
-	}
-
-};				// void update_tux_armour_damage_reduction ( )
-
 /**
  * This function should re-compute all character stats according to the
- * currently equipped items and currenly distributed stats points.
+ * currently equipped items and currently distributed stats points.
  */
 void UpdateAllCharacterStats()
 {
-	//--------------------
 	// Maybe the influencer has reached a new experience level?
 	// Let's check this...
 	// 
 	check_for_new_experience_level_reached();
 
-	//--------------------
 	// The primary status must be computed/updated first, because
 	// the secondary status (chances and the like) will depend on
 	// them...
 	//
 	update_all_primary_stats();
 
-	//--------------------
 	// At this point we know, that the primary stats of the influencer
 	// have been fully computed.  So that means, that finally we can compute
 	// all base SECONDARY stats, that are dependent upon the influencer primary
@@ -459,7 +370,6 @@ void UpdateAllCharacterStats()
 	//
 	update_secondary_stats_from_primary_stats();
 
-	//--------------------
 	// Now we compute the possible damage the player character can do.
 	// The damage value of course depends on the weapon type that the
 	// character is using.  And depending on the weapon type (melee or
@@ -467,36 +377,37 @@ void UpdateAllCharacterStats()
 	//
 	update_damage_tux_can_do();
 
-	//--------------------
 	// Update tux armour class
 	//
-	update_tux_armour_damage_reduction();
+	update_player_armor_class();
 
-	//--------------------
 	// So at this point we can finally apply all the modifiers to the influencers
 	// SECONDARY stats due to 'magical' items and spells and the like
 	//
+	Me.slowing_melee_targets = 0;
+	Me.paralyzing_melee_targets = 0;
 	Me.light_bonus_from_tux = 0;
+	Me.experience_factor = 1.0;
 	AddInfluencerItemSecondaryBonus(&Me.armour_item);
 	AddInfluencerItemSecondaryBonus(&Me.weapon_item);
 	AddInfluencerItemSecondaryBonus(&Me.drive_item);
 	AddInfluencerItemSecondaryBonus(&Me.shield_item);
 	AddInfluencerItemSecondaryBonus(&Me.special_item);
 
-	//--------------------
+	// Add light bonus if necessary
+	if (Me.light_bonus_end_date > Me.current_game_date) {
+		// Fade out the bonus in the last second
+		float ratio = min(1.0, Me.light_bonus_end_date - Me.current_game_date);
+		Me.light_bonus_from_tux += 5 * ratio;
+	}
+
 	// Check player's health and temperature
 	if (Me.energy > Me.maxenergy)
 		Me.energy = Me.maxenergy;
 	if (Me.temperature < 0)
 		Me.temperature = 0;
 
-	//--------------------
-	// Now that the defence stat is computed, we can compute the chance, that
-	// a randomly chosen lv. 1 bot will hit the Tux in any given strike...
-	//
-	Me.lv_1_bot_will_hit_percentage = 60;
-
-};				// void UpdateAllCharacterStats ( void )
+}
 
 /**
  * Now we print out the current skill levels in hacking skill, 
@@ -505,7 +416,6 @@ void UpdateAllCharacterStats()
 void show_character_screen_skills()
 {
 
-	//--------------------
 	// We add some security against skill values out of allowed
 	// bounds.
 	//
@@ -520,7 +430,6 @@ Error: melee weapon skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
 	DisplayText(_(AllSkillTexts[Me.melee_weapon_skill]),
 		    SKILLS_VALUE_X + CharacterRect.x, MELEE_SKILL_Y + 17 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 
-	//--------------------
 	// We add some security against skill values out of allowed
 	// bounds.
 	//
@@ -535,7 +444,6 @@ Error: ranged weapon skill seems out of bounds.", PLEASE_INFORM, IS_FATAL);
 	DisplayText(_(AllSkillTexts[Me.ranged_weapon_skill]),
 		    SKILLS_VALUE_X + CharacterRect.x, RANGED_SKILL_Y + 17 + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 
-	//--------------------
 	// We add some security against skill values out of allowed
 	// bounds.
 	//
@@ -562,7 +470,6 @@ void ShowCharacterScreen()
 
 	DebugPrintf(2, "\n%s(): Function call confirmed.", __FUNCTION__);
 
-	//--------------------
 	// If the log is not set to visible right now, we do not need to 
 	// do anything more, but to restore the usual user rectangle size
 	// back to normal and to return...
@@ -572,13 +479,11 @@ void ShowCharacterScreen()
 
 	SetCurrentFont(Messagestat_BFont);
 
-	// --------------------
 	// We will need the current mouse position on several spots...
 	//
 	CurPos.x = GetMousePos_x();
 	CurPos.y = GetMousePos_y();
 
-	//--------------------
 	// We define the right side of the user screen as the rectangle
 	// for our inventory screen.
 	//
@@ -635,27 +540,27 @@ void ShowCharacterScreen()
 	DisplayText(CharText, STR_X + CharacterRect.x, DEX_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 
 	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("CPU"), LEFT_TXT_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
-	SetCurrentFont(Messagevar_BFont);
-	sprintf(CharText, "%d", Me.Magic);
-	if (Me.Magic != Me.base_magic)
-		sprintf(CharText + strlen(CharText), " (%+d)", Me.Magic - Me.base_magic);
-	DisplayText(CharText, STR_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
-
-	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Life"), LEFT_TXT_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	DisplayText(_("Physique"), LEFT_TXT_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%d", Me.Vitality);
 	if (Me.Vitality != Me.base_vitality)
 		sprintf(CharText + strlen(CharText), " (%+d)", Me.Vitality - Me.base_vitality);
 	DisplayText(CharText, STR_X + CharacterRect.x, VIT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 
+	SetCurrentFont(Messagestat_BFont);
+	DisplayText(_("Cooling"), LEFT_TXT_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	SetCurrentFont(Messagevar_BFont);
+	sprintf(CharText, "%d", Me.Magic);
+	if (Me.Magic != Me.base_magic)
+		sprintf(CharText + strlen(CharText), " (%+d)", Me.Magic - Me.base_magic);
+	DisplayText(CharText, STR_X + CharacterRect.x, MAG_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+
 	SDL_Rect tmprect = CharacterRect;
 	SetCurrentFont(Messagestat_BFont);
 	DisplayText(_("Training points"), LEFT_TXT_X + CharacterRect.x, POINTS_Y + CharacterRect.y, &tmprect, TEXT_STRETCH);
 	SetCurrentFont(Messagevar_BFont);
 	sprintf(CharText, "%d", Me.points_to_distribute);
-	DisplayText(CharText, 150 + CharacterRect.x, POINTS_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	DisplayText(CharText, 155 + CharacterRect.x, POINTS_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 
 	SetCurrentFont(Messagestat_BFont);
 	DisplayText(_("Health"), LEFT_TXT_X + CharacterRect.x, HEALTH_STAT_Y + CharacterRect.y, &tmprect, TEXT_STRETCH);
@@ -675,12 +580,6 @@ void ShowCharacterScreen()
 	sprintf(CharText, "%d/%d", (int)Me.running_power, (int)Me.max_running_power);
 	DisplayText(CharText, HEALTH_NR_X + CharacterRect.x, STAMINA_STAT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 
-	SetCurrentFont(Messagestat_BFont);
-	DisplayText(_("Armor"), RIGHT_TXT_X + CharacterRect.x, DR_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
-	SetCurrentFont(Messagevar_BFont);
-	sprintf(CharText, "%d%%", (int)Me.DR);
-	DisplayText(CharText, ARMOR_NR_X + CharacterRect.x, DR_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
-
 	SetCurrentFont(Messagestat_BFont);
 	DisplayText(_("Attack"), RIGHT_TXT_X + CharacterRect.x, TOHIT_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 	SetCurrentFont(Messagevar_BFont);
@@ -690,10 +589,21 @@ void ShowCharacterScreen()
 	SetCurrentFont(Messagestat_BFont);
 	DisplayText(_("Damage"), RIGHT_TXT_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 	SetCurrentFont(Messagevar_BFont);
-	sprintf(CharText, "%d-%d", (int)Me.base_damage, (int)Me.base_damage + (int)Me.damage_modifier);
+
+	// Display range of damage, or a single value if there is no range
+	if (Me.damage_modifier)
+		sprintf(CharText, "%d-%d", (int)Me.base_damage, (int)Me.base_damage + (int)Me.damage_modifier);
+	else
+		sprintf(CharText, "%d", (int)Me.base_damage);
+
 	DisplayText(CharText, ARMOR_NR_X + CharacterRect.x, DAMAGE_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
 
-	//--------------------
+	SetCurrentFont(Messagestat_BFont);
+	DisplayText(_("Armor"), RIGHT_TXT_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+	SetCurrentFont(Messagevar_BFont);
+	sprintf(CharText, "%d", (int)Me.armor_class);
+	DisplayText(CharText, ARMOR_NR_X + CharacterRect.x, DAMRED_Y + CharacterRect.y, &CharacterRect, TEXT_STRETCH);
+
 	// Now we print out the current skill levels in hacking skill, 
 	// spellcasting, melee combat, ranged weapon combat and repairing things
 	//
