@@ -2,7 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
- *   Copyright (c) 2004-2007 Arthur Huillet
+ *   Copyright (c) 2004-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -26,7 +26,7 @@
 
 /**
  * This file contains all functions dealing with the dialog interface,
- * including blitting the chat protocol to the screen and drawing the
+ * including blitting the chat log to the screen and drawing the
  * right portrait images to the screen.
  */
 
@@ -46,9 +46,8 @@
 char *chat_initialization_code;	//first time with a character-code
 char *chat_startup_code;	//every time we start this dialog-code
 EXTERN char *PrefixToFilename[ENEMY_ROTATION_MODELS_AVAILABLE];
-char *chat_protocol = NULL;
 
-static void DoChatFromChatRosterData(enemy *ChatDroid, int ClearProtocol);
+static void run_chat(enemy *ChatDroid, int is_subdialog);
 
 /**
  * This function resets a dialog option to "empty" default values. It does NOT free the strings, this has to be done
@@ -56,15 +55,8 @@ static void DoChatFromChatRosterData(enemy *ChatDroid, int ClearProtocol);
  */
 static void clear_dialog_option(dialogue_option * d)
 {
-	int i;
 	d->option_text = "";
 	d->option_sample_file_name = "";
-
-	for (i = 0; i < MAX_REPLIES_PER_OPTION; i++) {
-		d->reply_sample_list[i] = "";
-		d->reply_subtitle_list[i] = "";
-	}
-
 	d->lua_code = NULL;
 	d->exists = 0;
 }
@@ -74,9 +66,6 @@ static void clear_dialog_option(dialogue_option * d)
  */
 static void delete_one_dialog_option(int i, int FirstInitialisation)
 {
-	int j;
-
-	//--------------------
 	// If this is not the first initialisation, we have to free the allocated
 	// strings first, or we'll be leaking memory otherwise...
 	//
@@ -91,21 +80,8 @@ static void delete_one_dialog_option(int i, int FirstInitialisation)
 		free(ChatRoster[i].lua_code);
 	ChatRoster[i].lua_code = NULL;
 
-	for (j = 0; j < MAX_REPLIES_PER_OPTION; j++) {
-		//--------------------
-		// If this is not the first initialisation, we have to free the allocated
-		// strings first, or we'll be leaking memory otherwise...
-		//
-		if (!FirstInitialisation) {
-			if (strlen(ChatRoster[i].reply_sample_list[j]))
-				free(ChatRoster[i].reply_sample_list[j]);
-			if (strlen(ChatRoster[i].reply_subtitle_list[j]))
-				free(ChatRoster[i].reply_subtitle_list[j]);
-		}
-	}
-
 	clear_dialog_option(&ChatRoster[i]);
-};				// void delete_one_dialog_option ( int i , int FirstInitialisation )
+}
 
 /**
  * This function should init the chat roster with empty values and thereby
@@ -120,7 +96,6 @@ static void InitChatRosterForNewDialogue(void)
 		delete_one_dialog_option(i, FirstInitialisation);
 	}
 
-	//--------------------
 	// Next time, we WILL have to free every used entry before cleaning it
 	// out, or we will be leaking memory...
 	//
@@ -160,7 +135,6 @@ void PlantCookie(const char *CookieString)
 {
 	int i;
 
-	//--------------------
 	// Check if the cookie has already been set. 
 	//
 	for (i = 0; i < MAX_COOKIES; i++) {
@@ -172,7 +146,6 @@ void PlantCookie(const char *CookieString)
 		}
 	}
 
-	//--------------------
 	// Now we find a good new and free position for our new cookie...
 	//
 	for (i = 0; i < MAX_COOKIES; i++) {
@@ -181,7 +154,6 @@ void PlantCookie(const char *CookieString)
 		}
 	}
 
-	//--------------------
 	// Maybe the position we have found is the last one.  That would mean too
 	// many cookies, a case that should never occur in FreedroidRPG and that is
 	// a considered a fatal error...
@@ -192,7 +164,6 @@ There were no more free positions available to store this cookie.\n\
 This should not be possible without a severe bug in FreedroidRPG.", PLEASE_INFORM, IS_FATAL);
 	}
 	
-	//--------------------
 	// Now that we know that we have found a good position for storing our
 	// new cookie, we can do it.
 	Me.cookie_list[i] = MyMalloc(strlen(CookieString)+1);
@@ -219,7 +190,7 @@ void DeleteCookie(const char *CookieString)
 	}
 
 	if (i == MAX_COOKIES) {
-		DebugPrintf(-4, "Cookie not found.");
+		ErrorMessage(__FUNCTION__, "The specified cookie \"%s\" was not found.", PLEASE_INFORM, IS_WARNING_ONLY, CookieString);
 	} else {
 		free(Me.cookie_list[i]);
 		for (i = i+1; i < MAX_COOKIES; i++) {
@@ -236,56 +207,44 @@ void DeleteCookie(const char *CookieString)
  * chat info file into the chat roster.
  *
  */
-static void LoadDialog(char *FullPathAndFullFilename)
+static void load_dialog(const char *fpath)
 {
 	char *ChatData;
 	char *SectionPointer;
 	char *EndOfSectionPointer;
-	int i, j;
-	char fpath[2048];
+	int i;
 	int OptionIndex;
 	int NumberOfOptionsInSection;
-	int NumberOfReplySubtitles;
-	int NumberOfReplySamples;
 
-	char *ReplyPointer;
-
-	sprintf(fpath, "%s", FullPathAndFullFilename);
-
-	// #define END_OF_DIALOGUE_FILE_STRING "*** End of Dialogue File Information ***"
 #define CHAT_CHARACTER_BEGIN_STRING "Beginning of new chat dialog for character=\""
 #define CHAT_CHARACTER_END_STRING "End of chat dialog for character"
 #define NEW_OPTION_BEGIN_STRING "\nNr="
+#define NO_WAIT_OPTION_STRING "\nOptionSample=\"NO_SAMPLE_HERE_AND_DONT_WAIT_EITHER\""
 
-	//--------------------
-	// At first we read the whole chat file information into memory
-	//
+	// Read the whole chat file information into memory
 	ChatData = ReadAndMallocAndTerminateFile(fpath, CHAT_CHARACTER_END_STRING);
 	SectionPointer = ChatData;
 
-	//--------------------
 	// Read the initialization code
 	//
 	if (chat_initialization_code) {
 		free(chat_initialization_code);
 		chat_initialization_code = NULL;
 	}
-	chat_initialization_code = ReadAndMallocStringFromDataOptional(SectionPointer, "FirstTime LuaCode={", "}", 0);
+	chat_initialization_code = ReadAndMallocStringFromDataOptional(SectionPointer, "FirstTime LuaCode={", "}");
 
 	if (chat_startup_code) {
 		free(chat_startup_code);
 		chat_startup_code = NULL;
 	}
-	chat_startup_code = ReadAndMallocStringFromDataOptional(SectionPointer, "EveryTime LuaCode={", "}", 0);
+	chat_startup_code = ReadAndMallocStringFromDataOptional(SectionPointer, "EveryTime LuaCode={", "}");
 
-	//--------------------
 	// At first we go take a look on how many options we have
 	// to decode from this section.
 	//
 	NumberOfOptionsInSection = CountStringOccurences(SectionPointer, NEW_OPTION_BEGIN_STRING);
 	DebugPrintf(CHAT_DEBUG_LEVEL, "\nWe have counted %d Option entries in this section.", NumberOfOptionsInSection);
 
-	//--------------------
 	// Now we see which option index is assigned to this option.
 	// It may happen, that some numbers are OMITTED here!  This
 	// should be perfectly ok and allowed as far as the code is
@@ -308,7 +267,7 @@ static void LoadDialog(char *FullPathAndFullFilename)
 		// Anything that is loaded into the chat roster doesn't need to be freed,
 		// cause this will be done by the next 'InitChatRoster' function anyway.
 		//
-		ChatRoster[OptionIndex].option_text = ReadAndMallocStringFromDataOptional(SectionPointer, "Text=\"", "\"", 0);
+		ChatRoster[OptionIndex].option_text = ReadAndMallocStringFromDataOptional(SectionPointer, "Text=\"", "\"");
 		if (!ChatRoster[OptionIndex].option_text) {
 			ChatRoster[OptionIndex].option_text = ReadAndMallocStringFromData(SectionPointer, "Text=_\"", "\"");
 		}
@@ -316,61 +275,13 @@ static void LoadDialog(char *FullPathAndFullFilename)
 		DebugPrintf(CHAT_DEBUG_LEVEL, "\nText found : \"%s\".", ChatRoster[OptionIndex].option_text);
 
 		ChatRoster[OptionIndex].option_sample_file_name =
-		    ReadAndMallocStringFromDataOptional(SectionPointer, "OptionSample=\"", "\"", 0);
+		    ReadAndMallocStringFromDataOptional(SectionPointer, "OptionSample=\"", "\"");
 
 		if (!ChatRoster[OptionIndex].option_sample_file_name)
 			ChatRoster[OptionIndex].option_sample_file_name = strdup("Sorry_No_Voice_Sample_Yet_0.wav");
 
 		DebugPrintf(CHAT_DEBUG_LEVEL, "\nOptionSample found : \"%s\".", ChatRoster[OptionIndex].option_sample_file_name);
 
-#define NEW_REPLY_SAMPLE_STRING "ReplySample=\""
-#define NEW_REPLY_SUBTITLE_STRING "NPC=_\""
-
-		//--------------------
-		// We count the number of Subtitle and Sample combinations and then
-		// we will read them out
-		//
-		NumberOfReplySamples = CountStringOccurences(SectionPointer, NEW_REPLY_SAMPLE_STRING);
-		NumberOfReplySubtitles = CountStringOccurences(SectionPointer, NEW_REPLY_SUBTITLE_STRING);
-		if (NumberOfReplySamples != NumberOfReplySubtitles && NumberOfReplySamples > 0) {
-			fprintf(stderr, "\n\nNumberOfReplySamples: %d NumberOfReplySubtitles: %d \n", NumberOfReplySamples,
-				NumberOfReplySubtitles);
-			fprintf(stderr, "The section in question looks like this: \n%s\n\n", SectionPointer);
-			ErrorMessage(__FUNCTION__, "\
-There were an unequal number of reply samples and subtitles specified\n\
-within a section of the Freedroid.dialogues file.\n\
-This is allowed in Freedroid only if there are no subtitles - ie. either specify\n\
-one subtitle per reply, or no subtitle at all.\n\
-This is currently not allowed in Freedroid and therefore indicates a\n\
-severe error.", PLEASE_INFORM, IS_FATAL);
-		}
-		//--------------------
-		// Now that we know exactly how many Sample and Subtitle sections 
-		// to read out, we can well start reading exactly that many of them.
-		// 
-		ReplyPointer = SectionPointer;
-
-		for (j = 0; j < NumberOfReplySubtitles; j++) {
-			ChatRoster[OptionIndex].reply_subtitle_list[j] =
-			    ReadAndMallocStringFromData(ReplyPointer, NEW_REPLY_SUBTITLE_STRING, "\"");
-
-			if (!NumberOfReplySamples)
-				ChatRoster[OptionIndex].reply_sample_list[j] = strdup("Sorry_No_Voice_Sample_Yet_0.wav");
-			else
-				ChatRoster[OptionIndex].reply_sample_list[j] =
-				    ReadAndMallocStringFromData(ReplyPointer, NEW_REPLY_SAMPLE_STRING, "\"");
-
-			DebugPrintf(CHAT_DEBUG_LEVEL, "\nSubtitle found : \"%s\".", ChatRoster[OptionIndex].reply_subtitle_list[j]);
-
-			DebugPrintf(CHAT_DEBUG_LEVEL, "\nReplySample found : \"%s\".", ChatRoster[OptionIndex].reply_sample_list[j]);
-
-			//--------------------
-			// Now we must move the reply pointer to after the previous combination.
-			//
-			ReplyPointer = LocateStringInData(ReplyPointer, NEW_REPLY_SUBTITLE_STRING);
-			ReplyPointer++;
-		}
-
 		if (strstr(SectionPointer, "LuaCode")) {
 			ChatRoster[OptionIndex].lua_code = ReadAndMallocStringFromData(SectionPointer, "LuaCode={", "}");
 		} else
@@ -382,16 +293,14 @@ severe error.", PLEASE_INFORM, IS_FATAL);
 			*EndOfSectionPointer = NEW_OPTION_BEGIN_STRING[0];
 	}
 
-	//--------------------
 	// Now we've got all the information we wanted from the dialogues file.
 	// We can now free the loaded file again.  Upon a new character dialogue
-	// being initiated, we'll just reload the file.  This is very conveninet,
+	// being initiated, we'll just reload the file.  This is very convenient,
 	// for it allows making and testing changes to the dialogues without even
 	// having to restart Freedroid!  Very cool!
 	//
 	free(ChatData);
-
-};				// void LoadDialog ( char* SequenceCode )
+}
 
 /**
  *
@@ -408,7 +317,6 @@ void make_sure_chat_portraits_loaded_for_this_droid(Enemy this_droid)
 	static int first_call = TRUE;
 	static int this_type_has_been_loaded[ENEMY_ROTATION_MODELS_AVAILABLE];
 
-	//--------------------
 	// We make sure we only load the portrait files once and not
 	// every time...
 	//
@@ -418,12 +326,10 @@ void make_sure_chat_portraits_loaded_for_this_droid(Enemy this_droid)
 	}
 	first_call = FALSE;
 
-	//--------------------
 	// We look up the model number for this chat partner.
 	//
 	model_number = this_droid->type;
 
-	//--------------------
 	// We should make sure, that we don't double-load images that we have loaded
 	// already, thereby wasting more resources, including OpenGL texture positions.
 	//
@@ -431,7 +337,6 @@ void make_sure_chat_portraits_loaded_for_this_droid(Enemy this_droid)
 		return;
 	this_type_has_been_loaded[model_number] = TRUE;
 
-	//--------------------
 	// At first we try to load the image, that is named after this
 	// chat section.  If that succeeds, perfect.  If not, we'll revert
 	// to a default image.
@@ -470,38 +375,17 @@ chat interface of Freedroid.  But:  Loading this file has ALSO failed.", PLEASE_
 	} else
 		chat_portrait_of_droid[model_number].surface = Large_Droid;
 
-};				// void make_sure_chat_portraits_loaded_for_this_droid ( Enemy this_droid )
+}
 
-/**
- * This function prepares the chat background window and displays the
- * image of the dialog partner and also sets the right font.
- */
-static void PrepareMultipleChoiceDialog(Enemy ChatDroid, int with_flip)
+static void show_chat_up_button(void)
 {
-	//--------------------
-	// The dialog will always take more than a few seconds to process
-	// so we need to prevent framerate distortion...
-	//
-	Activate_Conservative_Frame_Computation();
+	ShowGenericButtonFromList(CHAT_LOG_SCROLL_UP_BUTTON);
+}
 
-	//--------------------
-	// We make sure that all the chat portraits we might need are
-	// loaded....
-	//
-	make_sure_chat_portraits_loaded_for_this_droid(ChatDroid);
-
-	//--------------------
-	// We select small font for the menu interaction...
-	//
-	SetCurrentFont(FPS_Display_BFont);
-
-	blit_special_background(CHAT_DIALOG_BACKGROUND_PICTURE_CODE);
-	our_SDL_blit_surface_wrapper(chat_portrait_of_droid[ChatDroid->type].surface, NULL, Screen, &Droid_Image_Window);
-
-	if (with_flip)
-		our_SDL_flip_wrapper();
-
-};				// void PrepareMultipleChoiceDialog ( int Enum )
+static void show_chat_down_button(void)
+{
+	ShowGenericButtonFromList(CHAT_LOG_SCROLL_DOWN_BUTTON);
+}
 
 /**
  * During the Chat with a friendly droid or human, there is a window with
@@ -509,71 +393,49 @@ static void PrepareMultipleChoiceDialog(Enemy ChatDroid, int with_flip)
  * here to display said text window and it's content, scrolled to the
  * position desired by the player himself.
  */
-void display_current_chat_protocol(int background_picture_code, enemy * ChatDroid, int with_update)
+void show_chat_log(enemy *chat_enemy)
 {
-	const SDL_Rect Subtitle_Window = {.x = CHAT_SUBDIALOG_WINDOW_X,
-		.y = CHAT_SUBDIALOG_WINDOW_Y,
-		.w = CHAT_SUBDIALOG_WINDOW_W,
-		.h = CHAT_SUBDIALOG_WINDOW_H
-	};
-	int lines_needed;
-	int protocol_offset;
+	blit_special_background(CHAT_DIALOG_BACKGROUND_PICTURE_CODE);
+	our_SDL_blit_surface_wrapper(chat_portrait_of_droid[chat_enemy->type].surface, NULL, Screen, &Droid_Image_Window);
 
-#define LINES_IN_PROTOCOL_WINDOW (int)(CHAT_SUBDIALOG_WINDOW_H / (FontHeight(GetCurrentFont()) * TEXT_STRETCH))
+	chat_log.content_above_func = show_chat_up_button;
+	chat_log.content_below_func = show_chat_down_button;
+	ShowGenericButtonFromList(CHAT_LOG_SCROLL_OFF_BUTTON);
+	ShowGenericButtonFromList(CHAT_LOG_SCROLL_OFF2_BUTTON);
 
-	SetCurrentFont(FPS_Display_BFont);
+	show_text_widget(&chat_log);
+}
 
-	lines_needed = GetNumberOfTextLinesNeeded(chat_protocol, Subtitle_Window, TEXT_STRETCH) - 1;
+/**
+ * Wait for a mouse click before continuing.
+ */
+static void wait_for_click()
+{
+	SDL_Event event;
 
-	if (lines_needed <= LINES_IN_PROTOCOL_WINDOW) {
-		//--------------------
-		// When there isn't anything to scroll yet, we keep the default
-		// position and also the users clicks on up/down button will be
-		// reset immediately
-		//
-		protocol_offset = 0;
-		chat_protocol_scroll_override_from_user = 0;
-	} else {
-		// prevent the user from scrolling down too far
-		if (chat_protocol_scroll_override_from_user >= 1)
-			chat_protocol_scroll_override_from_user--;
+	while (1) {
+		SDL_WaitEvent(&event);
 
-		protocol_offset = ((int)(FontHeight(GetCurrentFont()) * TEXT_STRETCH) *
-				   (lines_needed - LINES_IN_PROTOCOL_WINDOW + chat_protocol_scroll_override_from_user));
+		if (event.type == SDL_QUIT) {
+			Terminate(0);
+		}
+
+		if (event.type == SDL_MOUSEBUTTONDOWN && event.button.button == 1)
+			return;
+
+		else if (event.type == SDL_KEYDOWN) {
+			switch (event.key.keysym.sym) {
+			case SDLK_SPACE:
+			case SDLK_RETURN:
+			case SDLK_ESCAPE:
+				return;
+				break;
+			default:
+				break;
+			}
+		}
 	}
-
-	//--------------------
-	// Prevent the player from scrolling 
-	// too high (negative protocol offset)
-	//
-	if (protocol_offset < 0) {
-		chat_protocol_scroll_override_from_user++;
-		protocol_offset = 0;
-	}
-	//--------------------
-	// Now we need to clear this window, cause there might still be some
-	// garbage from the previous subtitle in there...
-	//
-	PrepareMultipleChoiceDialog(ChatDroid, FALSE);
-
-	//--------------------
-	// Now we can display the text and update the screen...
-	//
-	SDL_SetClipRect(Screen, NULL);
-	DisplayText(chat_protocol, Subtitle_Window.x, Subtitle_Window.y - protocol_offset, &Subtitle_Window, TEXT_STRETCH);
-	if (protocol_offset > 0)
-		ShowGenericButtonFromList(CHAT_PROTOCOL_SCROLL_UP_BUTTON);
-	else
-		ShowGenericButtonFromList(CHAT_PROTOCOL_SCROLL_OFF_BUTTON);
-	if (lines_needed <= LINES_IN_PROTOCOL_WINDOW || chat_protocol_scroll_override_from_user >= 0)
-		ShowGenericButtonFromList(CHAT_PROTOCOL_SCROLL_OFF2_BUTTON);
-	else
-		ShowGenericButtonFromList(CHAT_PROTOCOL_SCROLL_DOWN_BUTTON);
-
-	if (with_update)
-		our_SDL_flip_wrapper();
-
-};				// void display_current_chat_protocol ( int background_picture_code , int with_update )
+}
 
 /**
  * This function should first display a subtitle and then also a sound
@@ -585,19 +447,22 @@ void GiveSubtitleNSample(const char *SubtitleText, const char *SampleFilename, e
 	int do_display = 1;
 	int do_wait = 1;
 
-	strcat(chat_protocol, SubtitleText);
-	strcat(chat_protocol, "\n");
+	autostr_append(chat_log.text, SubtitleText);
+	chat_log.scroll_offset = 0;
 
 	if (!strcmp(SampleFilename, "NO_WAIT")) {
 		do_wait = 0;
 	}
 
-	if (do_display)
-		display_current_chat_protocol(CHAT_DIALOG_BACKGROUND_PICTURE_CODE, ChatDroid, with_update);
+	if (do_display) {
+		show_chat_log(ChatDroid);
+		if (with_update)
+			our_SDL_flip_wrapper();
+	}
 
 	if (do_wait)
-		PlayOnceNeededSoundSample(SampleFilename, do_wait, FALSE);
-};				// void GiveSubtitleNSample( char* SubtitleText , char* SampleFilename )
+		wait_for_click();
+}
 
 void run_subdialog(const char *tmp_filename)
 {
@@ -614,7 +479,7 @@ void run_subdialog(const char *tmp_filename)
 	sprintf(finaldir, "%s", DIALOG_DIR);
 	find_file(tmp_filename, finaldir, fpath, 0);
 
-	LoadDialog(fpath);
+	load_dialog(fpath);
 
 	// we always initialize subdialogs..
 	//
@@ -626,7 +491,7 @@ void run_subdialog(const char *tmp_filename)
 	if (chat_initialization_code)
 		run_lua(chat_initialization_code);
 
-	DoChatFromChatRosterData(chat_control_chat_droid, FALSE);
+	run_chat(chat_control_chat_droid, TRUE);
 
 	push_or_pop_chat_roster(POP_ROSTER);
 
@@ -642,12 +507,10 @@ void run_subdialog(const char *tmp_filename)
  */
 static void ProcessThisChatOption(int MenuSelection, enemy *ChatDroid)
 {
-	int i;
 	//reset chat control variables for this option
 	chat_control_end_dialog = 0;
 	chat_control_next_node = -1;
 
-	//--------------------
 	// Now a menu section has been made.  We do the reaction:
 	// say the samples and the replies, later we'll set the new option values
 	//
@@ -655,27 +518,11 @@ static void ProcessThisChatOption(int MenuSelection, enemy *ChatDroid)
 	// by any reply.  This case must also be caught.
 	//
 	if (strcmp(ChatRoster[MenuSelection].option_sample_file_name, "NO_SAMPLE_HERE_AND_DONT_WAIT_EITHER")) {
-		strcat(chat_protocol, "\1TUX: ");
+		autostr_append(chat_log.text, "\1- ");
 		GiveSubtitleNSample(L_(ChatRoster[MenuSelection].option_text),
 				    ChatRoster[MenuSelection].option_sample_file_name, ChatDroid, TRUE);
-		strcat(chat_protocol, "\2");
+		autostr_append(chat_log.text, "\n");
 	}
-	//--------------------
-	// Now we can proceed to execute
-	// the rest of the reply that has been set up for this (the now maybe modified)
-	// dialog option.
-	//
-	for (i = 0; i < MAX_REPLIES_PER_OPTION; i++) {
-		//--------------------
-		// Once we encounter an empty string here, we're done with the reply...
-		//
-		if (!strlen(ChatRoster[MenuSelection].reply_subtitle_list[i]))
-			break;
-
-		GiveSubtitleNSample(L_(ChatRoster[MenuSelection].reply_subtitle_list[i]),
-				    ChatRoster[MenuSelection].reply_sample_list[i], ChatDroid, TRUE);
-	}
-
 	if (ChatRoster[MenuSelection].lua_code) {
 		run_lua(ChatRoster[MenuSelection].lua_code);
 	}
@@ -687,41 +534,30 @@ static void ProcessThisChatOption(int MenuSelection, enemy *ChatDroid)
  * disk, this function is invoked to handle the actual chat interaction
  * and the dialog flow.
  */
-static void DoChatFromChatRosterData(enemy *ChatDroid, int clear_protocol)
+static void run_chat(enemy *ChatDroid, int is_subdialog)
 {
 	int i;
-	SDL_Rect Chat_Window;
 	char *DialogMenuTexts[MAX_ANSWERS_PER_PERSON];
 	chat_control_partner_started = (ChatDroid->will_rush_tux);
-	SDL_Event event;
 
 	// Reset chat control variables.
 	chat_control_end_dialog = 0;
 	chat_control_next_node = -1;
 
-	//--------------------
-	// We always should clear the chat protocol.  Only for SUBDIALOGS it is
-	// suitable not to clear the chat protocol.
-	//
-	if (clear_protocol) {
-		if (chat_protocol != NULL)
-			free(chat_protocol);
-		chat_protocol = MyMalloc(500000);	// enough for any chat...
-		strcpy(chat_protocol, "\2--- ");
-		strcat(chat_protocol, _("Start of Dialog"));
-		strcat(chat_protocol, " ---\n");
-		chat_protocol_scroll_override_from_user = 0;
-		SetCurrentFont(FPS_Display_BFont);
-	}
+	/* Initialize the chat log widget. */
+	if (!is_subdialog)
+		init_text_widget(&chat_log, _("\3--- Start of Dialog ---\n"));
 
-	display_current_chat_protocol(CHAT_DIALOG_BACKGROUND_PICTURE_CODE, ChatDroid, TRUE);
+	chat_log.rect.x = CHAT_SUBDIALOG_WINDOW_X;
+	chat_log.rect.y = CHAT_SUBDIALOG_WINDOW_Y;
+	chat_log.rect.w = CHAT_SUBDIALOG_WINDOW_W;
+	chat_log.rect.h = CHAT_SUBDIALOG_WINDOW_H;
+	chat_log.font = FPS_Display_BFont;
+	chat_log.text_stretch = TEXT_STRETCH;
 
-	Chat_Window.x = 242;
-	Chat_Window.y = 100;
-	Chat_Window.w = 380;
-	Chat_Window.h = 314;
+	show_chat_log(ChatDroid);
+	our_SDL_flip_wrapper();
 
-	//--------------------
 	// We load the option texts into the dialog options variable..
 	//
 	for (i = 0; i < MAX_ANSWERS_PER_PERSON; i++) {
@@ -746,7 +582,6 @@ static void DoChatFromChatRosterData(enemy *ChatDroid, int clear_protocol)
 			chat_control_next_node =
 			    ChatDoMenuSelectionFlagged(_("What will you say?"), DialogMenuTexts, 1, -1,
 						       FPS_Display_BFont, ChatDroid);
-			//--------------------
 			// We do some correction of the menu selection variable:
 			// The first entry of the menu will give a 1 and so on and therefore
 			// we need to correct this to more C style.
@@ -765,19 +600,9 @@ static void DoChatFromChatRosterData(enemy *ChatDroid, int clear_protocol)
 			goto wait_click_and_out;
 	}
 
- wait_click_and_out:
-	while (1) {
-		SDL_WaitEvent(&event);
-		switch (event.type) {
-		case SDL_KEYDOWN:
-		case SDL_MOUSEBUTTONDOWN:
-			return;
-		case SDL_QUIT:
-			Terminate(0);
-		}
-	}
-
-};				// void DoChatFromChatRosterData( ... )
+wait_click_and_out:
+	while (EscapePressed() || MouseLeftPressed() || SpacePressed());
+}
 
 /**
  * When the Tux (or rather the player :) ) clicks on a friendly droid,
@@ -802,7 +627,6 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 
 #define TURN_SPEED 900.0
 
-	//--------------------
 	// We reset the mouse cursor shape and abort any other
 	// mouse global mode operation.
 	//
@@ -810,7 +634,6 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 
 	Activate_Conservative_Frame_Computation();
 
-	//--------------------
 	// We make sure the one droid in question is in the standing and not
 	// in the middle of the walking motion when turning to the chat partner...
 	//
@@ -819,7 +642,6 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 	ChatDroid->speed.x = 0;
 	ChatDroid->speed.y = 0;
 
-	//--------------------
 	// At first do some waiting before the turning around starts...
 	//
 	TurningStartTime = SDL_GetTicks();
@@ -837,14 +659,13 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 		ComputeFPSForThisFrame();
 	}
 
-	//--------------------
 	// Now we find out what the final target direction of facing should
 	// be.
 	//
 	// For this we use the atan2, which gives angles from -pi to +pi.
 	// 
 	// Attention must be paid, since 'y' in our coordinates ascends when
-	// moving down and descends when moving 'up' on the scren.  So that
+	// moving down and descends when moving 'up' on the screen.  So that
 	// one sign must be corrected, so that everything is right again.
 	//
 	RightAngle = (atan2(-(Me.pos.y - ChatDroid->pos.y), +(Me.pos.x - ChatDroid->pos.x)) * 180.0 / M_PI);
@@ -855,7 +676,6 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 	//
 	RightAngle += 90;
 
-	//--------------------
 	// Now it's time do determine which direction to move, i.e. if to 
 	// turn to the left or to turn to the right...  For this purpose
 	// we convert the current angle, which is between 270 and -90 degrees
@@ -867,7 +687,6 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 	// DebugPrintf ( 0 , "\nRightAngle: %f." , RightAngle );
 	// DebugPrintf ( 0 , "\nCurrent angle: %f." , ChatDroid -> current_angle );
 
-	//--------------------
 	// Having done these preparations, it's now easy to determine the right
 	// direction of rotation...
 	//
@@ -882,7 +701,6 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 	else
 		TurningDirection = -1;
 
-	//--------------------
 	// Now we turn and show the image until both chat partners are
 	// facing each other, mostly the chat partner is facing the Tux,
 	// since the Tux may still turn around to somewhere else all the 
@@ -900,7 +718,6 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 
 		ChatDroid->current_angle = OldAngle + TurningDirection * Frame_Time() * TURN_SPEED;
 
-		//--------------------
 		// In case of positive turning direction, we wait, till our angle is greater
 		// than the right angle.
 		// Otherwise we wait till our angle is lower than the right angle.
@@ -919,7 +736,6 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 		ComputeFPSForThisFrame();
 	}
 
-	//--------------------
 	// Now that turning around is basically done, we still wait a few frames
 	// until we start the dialog...
 	//
@@ -948,26 +764,19 @@ void DialogPartnersTurnToEachOther(Enemy ChatDroid)
 void ChatWithFriendlyDroid(enemy * ChatDroid)
 {
 	int i;
-	SDL_Rect Chat_Window;
 	char *DialogMenuTexts[MAX_ANSWERS_PER_PERSON];
 	char fpath[2048];
 	char tmp_filename[5000];
 	struct npc *npc;
 
 	chat_control_chat_droid = ChatDroid;
-	//--------------------
-	// Now that we know, that a chat with a friendly droid is planned, the 
-	// friendly droid and the Tux should first turn to each other before the
-	// real dialog is started...
-	//
+
 	DialogPartnersTurnToEachOther(ChatDroid);
 
-	Chat_Window.x = 242;
-	Chat_Window.y = 100;
-	Chat_Window.w = 380;
-	Chat_Window.h = 314;
+	Activate_Conservative_Frame_Computation();
+
+	make_sure_chat_portraits_loaded_for_this_droid(ChatDroid);
 
-	//--------------------
 	// First we empty the array of possible answers in the
 	// chat interface.
 	//
@@ -975,9 +784,6 @@ void ChatWithFriendlyDroid(enemy * ChatDroid)
 		DialogMenuTexts[i] = "";
 	}
 
-	while (MouseLeftPressed() || MouseRightPressed()) ;
-
-	//--------------------
 	// We clean out the chat roster from any previous use
 	//
 	InitChatRosterForNewDialogue();
@@ -991,7 +797,7 @@ void ChatWithFriendlyDroid(enemy * ChatDroid)
 	char finaldir[50];
 	sprintf(finaldir, "%s", DIALOG_DIR);
 	find_file(tmp_filename, finaldir, fpath, 0);
-	LoadDialog(fpath);
+	load_dialog(fpath);
 
 	chat_control_chat_flags = &npc->chat_flags[0];
 
@@ -1007,12 +813,8 @@ void ChatWithFriendlyDroid(enemy * ChatDroid)
 		npc->chat_character_initialized = 1;
 	}
 
-	//--------------------
-	// Now with the loaded chat data, we can do the real chat now...
-	//
-	DoChatFromChatRosterData(ChatDroid, TRUE);
-
-};				// void ChatWithFriendlyDroid( int Enum );
+	run_chat(ChatDroid, FALSE);
+}
 
 /**
  * Validate dialogs syntax and Lua code 
@@ -1037,7 +839,7 @@ void validate_dialogs()
 	skip_initial_menus = 1;
 
 	find_file("freedroid.levels", MAP_DIR, fpath, 0);
-	LoadShip(fpath);
+	LoadShip(fpath, 0);
 	PrepareStartOfNewCharacter("NewTuxStartGameSquare");
 	clear_player_inventory_and_stats();
 
@@ -1045,6 +847,7 @@ void validate_dialogs()
 	   text on screen and wait for clicks */
 	run_lua("function npc_says(a)\nend\n");
 	run_lua("function tux_says(a)\nend\n");
+	run_lua("function cli_says(a)\nend\n");
 
 	/* Subdialogs will be tested anyway, no need to run them from Lua */
 	run_lua("function run_subdialog(a)\nend\n");
@@ -1055,7 +858,14 @@ void validate_dialogs()
 	/* drop_dead cannot be tested because it means we would try to kill our dummy bot
 	 several times, which is not allowed by the engine */
 	run_lua("function drop_dead(a)\nend\n");
-	
+
+	run_lua("function user_input_string(a)\nreturn \"dummy\";\nend\n");
+
+	run_lua("function upgrade_items(a)\nend\n");
+	run_lua("function craft_addons(a)\nend\n");
+	/* set_bot_destination cannot be tested because this may be invoked when the bot is
+	 diffrent level then when the bot starts */
+	run_lua("function set_bot_destination(a)\nend\n");
 
 	/* This dummy will be used to test break_off_and_attack() functions and such. */
 	BROWSE_ALIVE_BOTS(dummy_partner) {
@@ -1074,7 +884,7 @@ void validate_dialogs()
 		find_file(filename, DIALOG_DIR, fpath, 0);
 
 		InitChatRosterForNewDialogue();
-		LoadDialog(fpath);
+		load_dialog(fpath);
 
 		if (chat_initialization_code) {
 			printf("\tinit code\n");
