@@ -1,6 +1,6 @@
 /* 
  *
- *   Copyright (c) 2008 Arthur Huillet
+ *   Copyright (c) 2008-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -58,7 +58,7 @@ inline int normalize_vect(float x1, float y1, float *x2, float *y2)
 	if (tmplen2 == 1)
 		return 0;
 
-	if (fabsf(tmplen2) < 0.00005)
+	if (fabsf(tmplen2) < 0.00005f)
 		return 1;	//could not normalize
 	*x2 = x1 + (*x2 - x1) * Q_rsqrt(tmplen2);
 	*y2 = y1 + (*y2 - y1) * Q_rsqrt(tmplen2);
@@ -113,7 +113,7 @@ int WalkablePassFilterCallback(colldet_filter * this, obstacle * obs, int obs_id
 	return FALSE;
 }
 colldet_filter WalkablePassFilter = { WalkablePassFilterCallback, NULL, 0, NULL };
-colldet_filter WalkableWithMarginPassFilter = { WalkablePassFilterCallback, NULL, 1, NULL };
+colldet_filter WalkableWithMarginPassFilter = { WalkablePassFilterCallback, NULL, COLLDET_WALKABLE_MARGIN, NULL };
 
 /**
  * This colldet filter is used to ignore the obstacles (such as water)
@@ -170,15 +170,16 @@ colldet_filter FlyableExceptIdPassFilter = { ObstacleByIdPassFilterCallback, NUL
  * This function checks if the connection between two points is free of
  * droids.  
  *
- * friendly_check can take three values : check all droids, only friendly droids
- * (with respect to tux), or only enemy droids
- *
  * OBSTACLES ARE NOT TAKEN INTO CONSIDERATION, ONLY DROIDS!!!
  *
  */
 int CheckIfWayIsFreeOfDroids(float x1, float y1, float x2, float y2, int OurLevel, freeway_context * ctx)
 {
 	const float Druid_Radius = 0.5;
+	float minx = min(x1, x2) - 0.5;
+	float miny = min(y1, y2) - 0.5;
+	float maxx = max(x1, x2) + 0.5;
+	float maxy = max(y1, y2) + 0.5;
 
 	float x2n = x2;
 	float y2n = y2;
@@ -197,6 +198,9 @@ int CheckIfWayIsFreeOfDroids(float x1, float y1, float x2, float y2, int OurLeve
 		    (ctx->except_bots[1] != NULL && ctx->except_bots[1] == this_enemy))
 			continue;
 
+		if (this_enemy->pos.x < minx || this_enemy->pos.y < miny || this_enemy->pos.y > maxy || this_enemy->pos.x > maxx)
+			continue;
+
 		float dist = calc_squared_distance_seg_point_normalized(x1, y1, x2, y2, x2n, y2n, this_enemy->pos.x, this_enemy->pos.y);
 
 		if (dist < Druid_Radius*Druid_Radius)
@@ -289,11 +293,11 @@ static int dlc_on_one_level(int x_tile_start, int x_tile_end, int y_tile_start,
 				// So we have our obstacle 
 
 				// Check the flags of both points against the rectangle of the object
-				moderately_finepoint rect1 = { our_obs->pos.x + obstacle_map[our_obs->type].upper_border,
-					our_obs->pos.y + obstacle_map[our_obs->type].left_border
+				moderately_finepoint rect1 = { our_obs->pos.x + obstacle_map[our_obs->type].left_border,
+					our_obs->pos.y + obstacle_map[our_obs->type].upper_border
 				};
-				moderately_finepoint rect2 = { our_obs->pos.x + obstacle_map[our_obs->type].lower_border,
-					our_obs->pos.y + obstacle_map[our_obs->type].right_border
+				moderately_finepoint rect2 = { our_obs->pos.x + obstacle_map[our_obs->type].right_border,
+					our_obs->pos.y + obstacle_map[our_obs->type].lower_border
 				};
 
 				// When DLC is called by the pathfinder, we grow a bit the obstacle's size.
@@ -301,11 +305,13 @@ static int dlc_on_one_level(int x_tile_start, int x_tile_end, int y_tile_start,
 				// from the path computed by the pathfinder. If that computed path is passing very close
 				// to an obstacle, then the actual path could go inside the obstacle, leading the character
 				// to be stuck.
-				if (filter && filter->use_margin) {
-					rect1.x -= COLLDET_MARGIN;
-					rect1.y -= COLLDET_MARGIN;
-					rect2.x += COLLDET_MARGIN;
-					rect2.y += COLLDET_MARGIN;
+				//
+				// Also, we will want to use this when dropping items.
+				if (filter && filter->extra_margin != 0.0) {
+					rect1.x -= filter->extra_margin;
+					rect1.y -= filter->extra_margin;
+					rect2.x += filter->extra_margin;
+					rect2.y += filter->extra_margin;
 				}
 
 				char p1flags = get_point_flag(rect1.x, rect1.y, rect2.x, rect2.y, p1->x, p1->y);
@@ -321,7 +327,6 @@ static int dlc_on_one_level(int x_tile_start, int x_tile_end, int y_tile_start,
 				if (ispoint)	// degenerated line, and outside the rectangle, no collision
 					continue;
 
-				//----------
 				// possible collision, check if the line is crossing the rectangle
 				// by looking if all vertices of the rectangle are in the same half-space
 				//
@@ -418,7 +423,6 @@ int DirectLineColldet(float x1, float y1, float x2, float y2, int z, colldet_fil
 	int x_start, y_start, x_end, y_end;	// intersection between the bbox and one level's limits
 	struct neighbor_data_cell *ngb_cell = NULL;
 
-	//--------------------
 	// First check on current level (small optimization)
 	//
 
@@ -434,7 +438,6 @@ int DirectLineColldet(float x1, float y1, float x2, float y2, int z, colldet_fil
 	if (!dlc_on_one_level(x_start, x_end, y_start, y_end, &p1, &p2, lvl, filter))
 		return FALSE;
 
-	//--------------------
 	// Get the neighborhood covered by the segment's bbox (see function's comment)
 	// and check each of the neighbors
 	//
@@ -500,7 +503,7 @@ struct dist_elt {
  */
 static int cmp_dist_elt(const void *elt1, const void *elt2)
 {
-	return (((struct dist_elt *)elt1)->value - ((struct dist_elt *)elt1)->value);
+	return (((struct dist_elt *)elt1)->value - ((struct dist_elt *)elt2)->value);
 }
 
 /**
@@ -517,14 +520,13 @@ int MoveOutOfObstacle(float *posX, float *posY, int posZ, obstacle * ThisObstacl
 	unsigned int i;
 	moderately_finepoint new_pos = { 0.5, 0.5 };
 
-	//--------------------
 	// Construct a sorted list of distance between character's position and the rectangle's edges
 	//
-	moderately_finepoint rect1 = { ThisObstacle->pos.x + obstacle_map[ThisObstacle->type].upper_border,
-		ThisObstacle->pos.y + obstacle_map[ThisObstacle->type].left_border
+	moderately_finepoint rect1 = { ThisObstacle->pos.x + obstacle_map[ThisObstacle->type].left_border,
+		ThisObstacle->pos.y + obstacle_map[ThisObstacle->type].upper_border
 	};
-	moderately_finepoint rect2 = { ThisObstacle->pos.x + obstacle_map[ThisObstacle->type].lower_border,
-		ThisObstacle->pos.y + obstacle_map[ThisObstacle->type].right_border
+	moderately_finepoint rect2 = { ThisObstacle->pos.x + obstacle_map[ThisObstacle->type].right_border,
+		ThisObstacle->pos.y + obstacle_map[ThisObstacle->type].lower_border
 	};
 
 	struct dist_elt dist_arr[] = { {rect2.x - (*posX), RIGHT},
@@ -535,33 +537,32 @@ int MoveOutOfObstacle(float *posX, float *posY, int posZ, obstacle * ThisObstacl
 
 	qsort(dist_arr, 4, sizeof(struct dist_elt), cmp_dist_elt);
 
-	//--------------------
 	// Check each direction of the sorted list
 	//
-	for (i = 0; i < 5; ++i) {
+	for (i = 0; i < 4; ++i) {
 		switch (dist_arr[i].direction) {
 		case (RIGHT):
 			{
-				new_pos.x = rect2.x + 0.01;	// small increment to really be outside the rectangle
+				new_pos.x = rect2.x + 0.1;	// small increment to really be outside the rectangle
 				new_pos.y = *posY;
 				break;
 			}
 		case (DOWN):
 			{
 				new_pos.x = *posX;
-				new_pos.y = rect1.y - 0.01;
+				new_pos.y = rect1.y - 0.1;
 				break;
 			}
 		case (LEFT):
 			{
-				new_pos.x = rect1.x - 0.01;
+				new_pos.x = rect1.x - 0.1;
 				new_pos.y = *posY;
 				break;
 			}
 		case (TOP):
 			{
 				new_pos.x = *posX;
-				new_pos.y = rect2.y + 0.01;
+				new_pos.y = rect2.y + 0.1;
 				break;
 			}
 		}
@@ -576,8 +577,7 @@ int MoveOutOfObstacle(float *posX, float *posY, int posZ, obstacle * ThisObstacl
 	// The caller will need to find a fallback
 	//
 	return FALSE;
-
-}				// void MoveOutOfObstacle( float* posX, float* posY, int posZ, obstacle* ThisObstacle )
+}
 
 /**
  * This function escapes a character from an obstacle, by
@@ -624,14 +624,13 @@ int EscapeFromObstacle(float *posX, float *posY, int posZ, colldet_filter * filt
 				if (obstacle_map[our_obs->type].block_area_type == COLLISION_TYPE_NONE)
 					continue;
 
-				//--------------------
 				// Now if the position lies inside the collision rectangle, then there's
 				// a collision.
 				//
-				if ((*posX > our_obs->pos.x + obstacle_map[our_obs->type].upper_border) &&
-				    (*posX < our_obs->pos.x + obstacle_map[our_obs->type].lower_border) &&
-				    (*posY > our_obs->pos.y + obstacle_map[our_obs->type].left_border) &&
-				    (*posY < our_obs->pos.y + obstacle_map[our_obs->type].right_border)) {
+				if ((*posX > our_obs->pos.x + obstacle_map[our_obs->type].left_border) &&
+				    (*posX < our_obs->pos.x + obstacle_map[our_obs->type].right_border) &&
+				    (*posY > our_obs->pos.y + obstacle_map[our_obs->type].upper_border) &&
+				    (*posY < our_obs->pos.y + obstacle_map[our_obs->type].lower_border)) {
 					// Find a new position for the character
 					return MoveOutOfObstacle(posX, posY, posZ, our_obs, filter);
 				}
