@@ -2,7 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
- *   Copyright (c) 2004-2007 Arthur Huillet 
+ *   Copyright (c) 2004-2010 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -90,7 +90,7 @@ enum {
 };
 #define NOT_LOADED_MARKER "nothing_loaded"
 
-#define ENEMY_ROTATION_MODELS_AVAILABLE 35
+#define ENEMY_ROTATION_MODELS_AVAILABLE 40
 #define ROTATION_ANGLES_PER_ROTATION_MODEL 8
 
 #define MAX_ENEMY_MOVEMENT_PHASES 50
@@ -98,6 +98,7 @@ enum {
 #define ATTACK_ANIMATION 114
 #define GETHIT_ANIMATION 115
 #define DEATH_ANIMATION 116
+#define DEAD_ANIMATION 118
 #define STAND_ANIMATION 117
 
 #define NUMBER_OF_SHADOW_IMAGES 20
@@ -106,24 +107,28 @@ enum {
 #define MAX_OBSTACLE_NAMES_PER_LEVEL 100
 #define MAX_OBSTACLE_DESCRIPTIONS_PER_LEVEL 100
 
-#define FLOOR_TILES_VISIBLE_AROUND_TUX ((GameConfig . screen_width >= 1024 ? 12 : GameConfig . screen_width >= 800 ? 9 : 7))
+#define FLOOR_TILES_VISIBLE_AROUND_TUX ((GameConfig . screen_width >= 1024 ? 13 : GameConfig . screen_width >= 800 ? 9 : 7))
 #define MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE 5
 #define MAX_ITEMS_PER_LEVEL 300
-#define MAX_CHEST_ITEMS_PER_LEVEL (MAX_ITEMS_PER_LEVEL)
-#define MAX_MAP_LABELS_PER_LEVEL 100
 #define MAX_ITEMS_IN_INVENTORY 100
+#define MAX_ITEMS_IN_NPC_INVENTORY 20
 #define INVENTORY_GRID_WIDTH 10
 #define INVENTORY_GRID_HEIGHT 6
 #define INV_SUBSQUARE_WIDTH (31)
 #define INV_SUBSQUARE_HEIGHT 32
 #define NUMBER_OF_MOUSE_CURSOR_PICTURES 2
 
+// Balancing of loot and items.
+#define MONEY_PER_BOT_CLASS 12
 #define ITEM_DROP_PERCENTAGE 10
 #define GOLD_DROP_PERCENTAGE 40
+#define SOCKET_DROP_PERCENTAGE 20
+#define ELECTRIC_SOCKET_COST 100
+#define MECHANICAL_SOCKET_COST 100
+#define UNIVERSAL_SOCKET_COST 400
 #define ARMOUR_DURABILITYLOSS_PERCENTAGE_WHEN_HIT 30
 #define WEAPON_DURABILITYLOSS_PERCENTAGE_WHEN_USED 10
 
-#define MAX_REPLIES_PER_OPTION 100
 #define CHAT_DEBUG_LEVEL 1
 
 #define MAX_BIG_SCREEN_MESSAGES 10
@@ -134,15 +139,20 @@ enum {
 #define MAX_ACTIVE_SPELLS 100
 #define MAX_NUMBER_OF_PROGRAMS 50
 
+enum upgrade_socket_types {
+	UPGRADE_SOCKET_TYPE_MECHANICAL,
+	UPGRADE_SOCKET_TYPE_ELECTRIC,
+	UPGRADE_SOCKET_TYPE_UNIVERSAL
+};
+
 enum skill_forms {
-	PROGRAM_FORM_IMMEDIATE = 10,	//instant hit whatever behind cursor
+	PROGRAM_FORM_INSTANT = 10,	//instant hit whatever behind cursor
 	PROGRAM_FORM_RADIAL,	//nova
 	PROGRAM_FORM_BULLET,	//fire a bullet
 	PROGRAM_FORM_SELF,	//instant hit player
 };
 
 enum _global_ingame_modes {
-	GLOBAL_INGAME_MODE_IDENTIFY = 7123,
 	GLOBAL_INGAME_MODE_NORMAL = 7124,
 	GLOBAL_INGAME_MODE_SCROLL_UP = 7127,
 	GLOBAL_INGAME_MODE_SCROLL_DOWN = 7128,
@@ -172,13 +182,11 @@ enum _game_config_screen_visible_parameters {
 #define UserCenter_y (User_Rect.y+User_Rect.h/2)
 
 #define MAX_CHARACTER_NAME_LENGTH (25)
-#define MAX_LENGTH_FOR_DIALOG_SECTION_NAME 100
-#define MAX_LENGTH_OF_SHORT_DESCRIPTION_STRING 200
 
 // The flags for AssembleCombatWindow are:
 enum { ONLY_SHOW_MAP = 1, DO_SCREEN_UPDATE = 2, ONLY_SHOW_MAP_AND_TEXT = 4, SHOW_GRID = 8, SHOW_TUX_AND_ENEMIES = 16, SHOW_TEXT =
-	    32, SHOW_ITEMS = 64, OMIT_OBSTACLES = 128, OMIT_TUX = 256, OMIT_ENEMIES = 512, SHOW_OBSTACLE_NAMES = 1024, ZOOM_OUT =
-	    2048, OMIT_BLASTS = 4096, USE_OWN_MOUSE_CURSOR = 8192, SKIP_LIGHT_RADIUS = 16384, NO_CURSOR = 32768 };
+	    32, SHOW_ITEMS = 64, OMIT_OBSTACLES = 128, OMIT_TUX = 256, OMIT_ENEMIES = 512, ZOOM_OUT = 1024, OMIT_BLASTS = 2048,
+		 USE_OWN_MOUSE_CURSOR = 4096, SKIP_LIGHT_RADIUS = 8192, NO_CURSOR = 16384, OMIT_ITEMS_LABEL = 32768 };
 
 // Flags for obstacles
 enum {
@@ -195,13 +203,14 @@ enum {
 	DROPS_RANDOM_TREASURE = 32,
 	NEEDS_PRE_PUT = 64,
 	GROUND_LEVEL = 128,	/* This obstacle has a small height */
-	IS_WALKABLE = 256	/* A bot can traverse this obstacle */
+	IS_WALKABLE = 256,	/* A bot can traverse this obstacle */
+    IS_CLICKABLE = 512  /* Player can click on this obstacle */
 };
 
 #define AUTOMAP_ZOOM_OUT_FACT 8.0
 
 // The color filters are:
-enum { FILTER_BLUE = 1, FILTER_GREEN = 2, FILTER_RED = 3 };
+enum { FILTER_BLUE = 1, FILTER_GREEN = 2, FILTER_RED = 3, FILTER_HIGHLIGHT = 4 };
 
 //--------------------
 // Constants for Paths and names of Data-files
@@ -243,6 +252,8 @@ enum { FILTER_BLUE = 1, FILTER_GREEN = 2, FILTER_RED = 3 };
 #define QUEST_BROWSER_BACKGROUND_CODE 29
 #define NUMBER_SELECTOR_BACKGROUND_CODE 30
 #define HUD_BACKGROUND_CODE 32
+#define ITEM_UPGRADE_BACKGROUND_CODE 33
+#define ADDON_CRAFTING_BACKGROUND_CODE 34
 
 #define ICON_FILE		"paraicon.bmp"
 
@@ -283,7 +294,7 @@ enum { FILTER_BLUE = 1, FILTER_GREEN = 2, FILTER_RED = 3 };
 //--------------------
 // dimensions of the droids as required for functions NotPassable() and CheckEnemyCollsion() )
 //
-#define DRUIDRADIUSXY	        (7*2/64.0)
+#define DROIDRADIUSXY	        (7*2/64.0)
 #define BULLET_BULLET_COLLISION_DIST (10/64.0)
 
 #define TAKEOVER_BACKGROUND_MUSIC_SOUND "Bleostrada.ogg"
@@ -354,8 +365,8 @@ enum _non_player_characters {
 	PERSON_TEST_DROID,
 	PERSON_MSCD,
 
-	PERSON_PLACEHOLDER1,
-	PERSON_PLACEHOLDER2,
+	PERSON_CNET,
+	PERSON_IRIS,
 	PERSON_PLACEHOLDER3,
 	PERSON_PLACEHOLDER4,
 	PERSON_PLACEHOLDER5,
@@ -406,64 +417,54 @@ enum {
 	BUY_BUTTON,
 	SELL_BUTTON,
 	REPAIR_BUTTON,
-	IDENTIFY_BUTTON,
 
 	OPEN_CLOSE_SKILL_EXPLANATION_BUTTON,
 
-	GO_LEVEL_NORTH_BUTTON,
-	GO_LEVEL_NORTH_BUTTON_PUSHED,
-	GO_LEVEL_SOUTH_BUTTON,
-	GO_LEVEL_SOUTH_BUTTON_PUSHED,
-	GO_LEVEL_EAST_BUTTON,
-	GO_LEVEL_EAST_BUTTON_PUSHED,
-	GO_LEVEL_WEST_BUTTON,
-	GO_LEVEL_WEST_BUTTON_PUSHED,
 	EXPORT_THIS_LEVEL_BUTTON,
 	EXPORT_THIS_LEVEL_BUTTON_PUSHED,
 	LEVEL_EDITOR_SAVE_SHIP_BUTTON,
 	LEVEL_EDITOR_SAVE_SHIP_BUTTON_PUSHED,
 	LEVEL_EDITOR_SAVE_SHIP_BUTTON_OFF,
 	LEVEL_EDITOR_SAVE_SHIP_BUTTON_OFF_PUSHED,
-	LEVEL_EDITOR_TOGGLE_WAYPOINT_BUTTON,
-	LEVEL_EDITOR_TOGGLE_WAYPOINT_BUTTON_PUSHED,
-	LEVEL_EDITOR_TOGGLE_CONNECTION_BLUE_BUTTON,
-	LEVEL_EDITOR_TOGGLE_CONNECTION_BLUE_BUTTON_PUSHED,
-	LEVEL_EDITOR_TOGGLE_CONNECTION_RED_BUTTON,
-	LEVEL_EDITOR_TOGGLE_CONNECTION_RED_BUTTON_PUSHED,
 
 	LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON,
 	LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON_PUSHED,
 	LEVEL_EDITOR_NEXT_OBSTACLE_BUTTON,
 	LEVEL_EDITOR_NEXT_OBSTACLE_BUTTON_PUSHED,
 
+	LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON,
+	LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON_OFF,
+	LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON_OFF_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON,
+	LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON_OFF,
+	LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON_OFF_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_ITEM_BUTTON,
+	LEVEL_EDITOR_TYPESELECT_ITEM_BUTTON_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_ITEM_BUTTON_OFF,
+	LEVEL_EDITOR_TYPESELECT_ITEM_BUTTON_OFF_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_WAYPOINT_BUTTON,
+	LEVEL_EDITOR_TYPESELECT_WAYPOINT_BUTTON_PUSHED,
+	LEVEL_EDITOR_TYPESELECT_WAYPOINT_BUTTON_OFF,
+	LEVEL_EDITOR_TYPESELECT_WAYPOINT_BUTTON_OFF_PUSHED,
+
 	LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON,
 	LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON_PUSHED,
 	LEVEL_EDITOR_ZOOM_IN_BUTTON,
 	LEVEL_EDITOR_ZOOM_IN_BUTTON_PUSHED,
 	LEVEL_EDITOR_ZOOM_OUT_BUTTON,
 	LEVEL_EDITOR_ZOOM_OUT_BUTTON_PUSHED,
-	LEVEL_EDITOR_RECURSIVE_FILL_BUTTON,
-	LEVEL_EDITOR_RECURSIVE_FILL_BUTTON_PUSHED,
 	LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON,
 	LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON_PUSHED,
 	LEVEL_EDITOR_NEW_MAP_LABEL_BUTTON,
 	LEVEL_EDITOR_NEW_MAP_LABEL_BUTTON_PUSHED,
-	LEVEL_EDITOR_NEW_ITEM_BUTTON,
-	LEVEL_EDITOR_NEW_ITEM_BUTTON_PUSHED,
 	LEVEL_EDITOR_EDIT_CHEST_BUTTON,
 	LEVEL_EDITOR_EDIT_CHEST_BUTTON_PUSHED,
 	LEVEL_EDITOR_ESC_BUTTON,
 	LEVEL_EDITOR_ESC_BUTTON_PUSHED,
-	LEVEL_EDITOR_LEVEL_RESIZE_BUTTON,
-	LEVEL_EDITOR_LEVEL_RESIZE_BUTTON_PUSHED,
 	LEVEL_EDITOR_QUIT_BUTTON,
 	LEVEL_EDITOR_QUIT_BUTTON_PUSHED,
-
-	LEVEL_EDITOR_UNDERGROUND_LIGHT_ON_BUTTON,
-	LEVEL_EDITOR_UNDERGROUND_LIGHT_ON_BUTTON_PUSHED,
-	LEVEL_EDITOR_UNDERGROUND_LIGHT_OFF_BUTTON,
-	LEVEL_EDITOR_UNDERGROUND_LIGHT_OFF_BUTTON_PUSHED,
-
 	LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON,
 	LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON_PUSHED,
 	LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON_OFF,
@@ -489,10 +490,6 @@ enum {
 
 	LEVEL_EDITOR_NEXT_ITEM_GROUP_BUTTON,
 	LEVEL_EDITOR_PREV_ITEM_GROUP_BUTTON,
-	LEVEL_EDITOR_NEXT_PREFIX_BUTTON,
-	LEVEL_EDITOR_PREV_PREFIX_BUTTON,
-	LEVEL_EDITOR_NEXT_SUFFIX_BUTTON,
-	LEVEL_EDITOR_PREV_SUFFIX_BUTTON,
 	LEVEL_EDITOR_CANCEL_ITEM_DROP_BUTTON,
 	LEVEL_EDITOR_UNDO_BUTTON,
 	LEVEL_EDITOR_UNDO_BUTTON_PUSHED,
@@ -519,12 +516,12 @@ enum {
 	DESCRIPTION_WINDOW_UP_BUTTON,
 	DESCRIPTION_WINDOW_DOWN_BUTTON,
 
-	DRUID_SHOW_EXIT_BUTTON,
+	DROID_SHOW_EXIT_BUTTON,
 
-	CHAT_PROTOCOL_SCROLL_UP_BUTTON,
-	CHAT_PROTOCOL_SCROLL_DOWN_BUTTON,
-	CHAT_PROTOCOL_SCROLL_OFF_BUTTON,
-	CHAT_PROTOCOL_SCROLL_OFF2_BUTTON,
+	CHAT_LOG_SCROLL_UP_BUTTON,
+	CHAT_LOG_SCROLL_DOWN_BUTTON,
+	CHAT_LOG_SCROLL_OFF_BUTTON,
+	CHAT_LOG_SCROLL_OFF2_BUTTON,
 
 	QUEST_BROWSER_EXIT_BUTTON,
 	QUEST_BROWSER_OPEN_QUESTS_BUTTON,
@@ -543,6 +540,15 @@ enum {
 	WEAPON_MODE_BUTTON,
 	SKI_ICON_BUTTON,
 
+	ITEM_UPGRADE_APPLY_BUTTON,
+	ITEM_UPGRADE_APPLY_BUTTON_DISABLED,
+	ITEM_UPGRADE_CLOSE_BUTTON,
+
+	ADDON_CRAFTING_SCROLL_UP_BUTTON,
+	ADDON_CRAFTING_SCROLL_DOWN_BUTTON,
+	ADDON_CRAFTING_SCROLL_DESC_UP_BUTTON,
+	ADDON_CRAFTING_SCROLL_DESC_DOWN_BUTTON,
+
 	//--------------------
 	// Please leave this here as the last entry, since it conveniently and
 	// automatically counts the number of buttons defined.  Other buttons
@@ -559,9 +565,8 @@ enum {
 #define STR_X 95
 #define STR_Y 140
 #define DEX_Y 165
-#define MAG_Y 190
-#define VIT_Y 215
-#define POINTS_Y 245
+#define VIT_Y 190
+#define MAG_Y 215
 
 #define BUTTON_WIDTH 35
 #define BUTTON_HEIGHT 19
@@ -570,15 +575,22 @@ enum {
 #define CHARACTERRECT_W (320)
 #define CHARACTERRECT_H (480)
 
-#define TUX_SWING_PHASES 	14
-#define TUX_BREATHE_PHASES 	1
-#define TUX_GOT_HIT_PHASES      0
-#define TUX_WALK_CYCLE_PHASES   10
-#define TUX_RUN_CYCLE_PHASES   10
-#define TUX_TOTAL_PHASES (TUX_SWING_PHASES+TUX_BREATHE_PHASES+TUX_GOT_HIT_PHASES+TUX_WALK_CYCLE_PHASES+TUX_RUN_CYCLE_PHASES)
+#define ITEM_UPGRADE_RECT_X 320
+#define ITEM_UPGRADE_RECT_Y 0
+#define ITEM_UPGRADE_RECT_W 320
+#define ITEM_UPGRADE_RECT_H 480
 
-#define TUX_RUNNING_SPEED (5.5)
-#define TUX_WALKING_SPEED (2.5)
+#define ADDON_CRAFTING_RECT_X 320
+#define ADDON_CRAFTING_RECT_Y 0
+#define ADDON_CRAFTING_RECT_W 320
+#define ADDON_CRAFTING_RECT_H 480
+
+// Number of keyframes in Tux image_archive files
+#define TUX_TOTAL_PHASES 35
+
+// The speed is measured in tiles per second.
+#define TUX_RUNNING_SPEED 5.5
+#define TUX_WALKING_SPEED 2.5
 
 #define MAX_TUX_DIRECTIONS      16
 #define RADIAL_SPELL_DIRECTIONS 16
@@ -589,6 +601,7 @@ enum {
 #define WAIT_AFTER_GAME_WON	5.0	// how long to show tux after the game is won
 #define WAIT_COLLISION		1	// after a little collision with Tux or another enemy, hold position for a while
 				    // this variable describes the amount of time in SECONDS 
+#define WAIT_BEFORE_ROTATE 0.7 // Wait a bit to avoid jittering between two rotation angles
 
 #define ALLBLASTTYPES		3	/* number of different exposions */
 
@@ -605,9 +618,9 @@ enum {
 // WARNING leave them here, they are required in struct.h
 //
 
-#define MAX_WP_CONNECTIONS 	12
+#define MIN_MAP_LINES		30
 #define MAX_MAP_LINES 		100
-#define MAX_COOKIES 100
+#define MAX_COOKIES 200
 #define MAX_INTERMEDIATE_WAYPOINTS_FOR_TUX 100
 
 #define RIGHT_WALL_BIT 1
@@ -620,32 +633,26 @@ enum {
 
 #define MAX_INFLU_POSITION_HISTORY 500
 #define MAX_MISSIONS_IN_GAME 50	// how many missions can there be at most in Freedroid
-#define MAX_MISSION_DESCRIPTION_TEXTS 10
+#define MAX_MISSION_DESCRIPTION_TEXTS 25
 
-#define MAX_LEVELS		50	// how many map levels are allowed in one ship
+#define MAX_LEVELS		100	// how many map levels are allowed in one ship
 
 #define MAX_EVENT_TRIGGERS      500	// how many event triggers at most to allow
 
-#define MAXWAYPOINTS 		500
-
 #define MAX_PHASES_IN_A_BULLET 12
 #define PHASES_OF_EACH_BLAST 20
 
-#define ITEM_INVENTORY_IMAGE_FILE_NAME "File or directory name for inventory image=\""
-#define ITEM_DROP_SOUND_FILE_NAME "Item uses drop sound with filename=\""
-
 #define UNIVERSAL_COORD_W(W) (int)((float)(W) * ((float)(GameConfig . screen_width) / 640.0))
 #define UNIVERSAL_COORD_H(H) (int)((float)(H) * ((float)(GameConfig . screen_height) / 480.0))
 
 //--------------------
-// And now we put here all the rectangle definitions and that...
-//
+// Rectangle definitions
 
 // Game message window
-#define SUBTITLEW_RECT_X 120
-#define SUBTITLEW_RECT_Y 389
-#define SUBTITLEW_RECT_W 415
-#define SUBTITLEW_RECT_H 44
+#define MESSAGE_TEXT_WIDGET_X UNIVERSAL_COORD_W(120)
+#define MESSAGE_TEXT_WIDGET_Y UNIVERSAL_COORD_H(389)
+#define MESSAGE_TEXT_WIDGET_W UNIVERSAL_COORD_W(415)
+#define MESSAGE_TEXT_WIDGET_H UNIVERSAL_COORD_H(44)
 
 // the inventory rect is the rectangle inside the inventory screen,
 // where all the items "in the backpack" including the quick inventory
@@ -775,7 +782,7 @@ enum _bullets {
 //
 enum _explosions {
 	BULLETBLAST = 0,
-	DRUIDBLAST,
+	DROIDBLAST,
 	OWNBLAST
 };
 
@@ -850,9 +857,8 @@ enum {
 //
 enum _combo_action_types {
 	NO_COMBO_ACTION_SET = -10,
-	COMBO_ACTION_OPEN_CHEST = 15,
+	COMBO_ACTION_OBSTACLE = 15,
 	COMBO_ACTION_PICK_UP_ITEM = 16,
-	COMBO_ACTION_SMASH_BARREL = 17,
 };
 
 //--------------------
@@ -864,7 +870,6 @@ enum {
 	BUY_1_ITEM = 1,
 	SELL_1_ITEM = 4,
 	REPAIR_ITEM = 12,
-	IDENTIFY_ITEM = 13
 };
 
 enum obstacle_types {
@@ -946,8 +951,8 @@ enum obstacle_types {
 	// These obstacles are generated from 'iso_pillars_and_barrels.blend' file
 	// in the cvs rep.
 	//
-	ISO_BLOCK_1 = 47,
-	ISO_BLOCK_2 = 48,
+	ISO_PILLAR_TALL = 47,
+	ISO_PILLAR_SHORT = 48,
 	ISO_TV_PILLAR_W = 49,
 
 	ISO_BARREL_1 = 50,
@@ -1377,6 +1382,19 @@ enum obstacle_types {
 	ISO_SECURITY_GATE_OPEN_S,
 	ISO_SECURITY_GATE_CLOSED_E,
 	ISO_SECURITY_GATE_CLOSED_S,
+	ISO_SOLAR_PANEL_E,
+	ISO_BOTLINE_01_N,
+	ISO_BOTLINE_01_E,
+	ISO_BOTLINE_02_E,
+	ISO_BOTLINE_02_N,
+	ISO_BOTLINE_02_W,
+	ISO_BOTLINE_02_S,
+	ISO_BOTLINE_03_N,
+	ISO_BOTLINE_03_E,
+	ISO_BOTLINE_04_N,
+	ISO_BOTLINE_04_E,
+	ISO_BOTLINE_05_N,
+	ISO_BOTLINE_05_E,
 	NUMBER_OF_OBSTACLE_TYPES
 };
 
@@ -1533,11 +1551,15 @@ enum {
 	ISO_TWOSQUARE_0001,
 	ISO_TWOSQUARE_0002,
 	ISO_TWOSQUARE_0003,
-
+	ISO_WATER_SIDEWALK_01,
+	ISO_WATER_SIDEWALK_02,
+	ISO_WATER_SIDEWALK_03,
+	ISO_WATER_SIDEWALK_04,
 	ALL_ISOMETRIC_FLOOR_TILES
 };
 
-#define ITEM_TAKE_DIST (1.2)
+// Maximum distance for both taking and dropping items
+#define ITEM_TAKE_DIST (0.7)
 // Maximum and minimum distance for a melee fight
 #define SQUARED_MELEE_MAX_DIST (2.25)
 #define MELEE_MIN_DIST (1.0)
@@ -1547,12 +1569,16 @@ enum {
 // Minimum distance to shoot with a range weapon
 #define SQUARED_RANGE_SHOOT_MIN_DIST (7.0)
 
-// Margin's size added to obstacles size when colldet is called by the pathfinder
-#define COLLDET_MARGIN (0.05)
-// Distance to next intermediate point before to follow the next subpath
-// It has to be smaller than COLLDET_MARGIN. Near half of COLLDET_MARGIN is a
-// good candidate
-#define DIST_TO_INTERM_POINT (0.03)
+// Margins added to obstacles size when an item is being dropped.
+#define COLLDET_DROP_ITEM_MARGIN (0.2)
+// Margin's size added to obstacles size when colldet is called by the pathfinder.
+// Also used to validate the walkability of a path between 2 waypoints.
+#define COLLDET_WALKABLE_MARGIN (0.04)
+// Distance to next intermediate point before to follow the next subpath.
+// Also used as the distance to next waypoint before to choose a new one.
+// It has to be smaller than COLLDET_WALKABLE_MARGIN. Near half of
+// COLLDET_WALKABLE_MARGIN is a good candidate
+#define DIST_TO_INTERM_POINT (0.02)
 
 //-----
 // Pathfinder constants
@@ -1564,4 +1590,30 @@ enum {
 	DIFFICULTY_HARD
 };
 
+enum faction_id {
+	FACTION_SELF = 0,
+	FACTION_BOTS,
+	FACTION_REDGUARD,
+	FACTION_RESISTANCE,
+	FACTION_CIVILIAN,
+	FACTION_CRAZY,
+	FACTION_SINGULARITY,
+
+	FACTION_NUMBER_OF_FACTIONS, //keep this last
+};
+
+enum faction_state {
+	HOSTILE = 0,
+	FRIENDLY,
+};
+
+#define ACCESS_OBSTACLE_EXTENSION(X,Y) ((struct obstacle_extension *)(X.arr))[Y]
+#define ACCESS_MAP_LABEL(X,Y) ((struct map_label *)(X.arr))[Y]
+
+enum obstacle_extension_type {
+	OBSTACLE_EXTENSION_LABEL = 30, // This extension indicates an obstacle label. The associated data is a const char * containing the label.
+	OBSTACLE_EXTENSION_CHEST_ITEMS, // This extension indicates an item list. The associated data is a struct dynarray containing struct item.
+	OBSTACLE_EXTENSION_DIALOGFILE, // This extension indicates the dialog filename associated to an interactive obstacle. Contains the dialog basename.
+	OBSTACLE_EXTENSION_SIGNMESSAGE = OBSTACLE_EXTENSION_DIALOGFILE, // This extension contains the message written on a signpost. It is exclusive with a dialogfile specification
+};
 #endif
