@@ -27,11 +27,17 @@ c_id = r'[\w\d]+' # Standard C identifier
 c_type = r'((?:(?:unsigned|signed|short|long)\s+)*' + c_id + r'(?:(?:\s+)|(?:\s*\*\s*)))' # C type
 
 # Regexp which search for a structure
-find_structure_rxp = re.compile(r'typedef struct .+?'
+find_structure_typedef_rxp = re.compile(r'typedef struct .+?'
                                 r'\{'
                                 r'([^\}]+)'
                                 r'\}'
                                 r'\s*(' + c_id + ').*?;', re.M | re.S)
+find_structure_notypedef_rxp = re.compile(
+                                r'^struct (' + c_id + ').'
+                                r'\{'
+                                r'([^\}]+)'
+                                r'\};', re.M | re.S)
+
 # Regexp which search for a field
 find_members_rxp = re.compile(r'\s*' + c_type + r'\s*(' + c_id + r')(?:\s*\[(.+)\])?\s*?;.*')
 
@@ -51,13 +57,13 @@ special_types = {
     'unsigned_char' : 'uchar'
 }
 
-only_dump_structs = [ "gps", "point", "moderately_finepoint", "finepoint", "tux_t", "item", "enemy", "bullet", "melee_shot", "mission", "configuration_for_freedroid", "npc" ]
+only_dump_structs = [ "gps", "point", "moderately_finepoint", "finepoint", "tux_t", "item", "enemy", "bullet", "melee_shot", "mission", "configuration_for_freedroid", "npc", "upgrade_socket" ]
 
 def main():
     if len(sys.argv) < 3:
-        print "Usage: %s <input.h> <output>" % sys.argv[0]
+        print("Usage: %s <input.h> <output>") % sys.argv[0]
         sys.exit(1)
-    
+
     # Filenames
     inpfn, outfn = sys.argv[1:]
     inpf = open(inpfn, 'r')
@@ -70,13 +76,15 @@ def main():
     data = {}
 
     header = inpf.read()
-    structures = find_structure_rxp.findall(header)
+    structures = find_structure_typedef_rxp.findall(header)
+    for s in find_structure_notypedef_rxp.findall(header):
+        structures.append((s[1], 'struct ' + s[0]))
     for s in structures:
         # s is a tuple which contains (code, name) with code = the code inside the structure
         code, name = s
 
-	if name not in only_dump_structs:
-	    continue
+        if name not in only_dump_structs:
+            continue
 
         # Avoid matching things in comments
         lines = []
@@ -91,7 +99,7 @@ def main():
                 in_comment_block = False
                 l = l.split('*/')[1]
             if not in_comment_block: lines.append(l)
-        
+
         # This list will contain all the (type, field, size) tuples
         a = []
         for l in lines:
@@ -105,28 +113,23 @@ def main():
         for f in a:
             type, field, size = f
             type = type.lower().strip()
-           
-            #print("got type " + str(type) + " field is " + str(field) + " size is " + str(size))
 
-            # Pointers - do not save them. It does not make sense.
-            if '*' in type:
-                type = type.replace('*', '').strip() + '_ptr'
-                continue
+            #print("got type " + str(type) + " field is " + str(field) + " size is " + str(size))
 
             # Spaces
             type = type.replace(' ', '_')
             if type in special_types.keys(): type = special_types[type]
-            
-            
-	    if size: 
-			type += '[%s]' % size
+
+            if size:
+                type += '[%s]' % size
             data[name].append((type, field))
-   
+
     # Writing loop
     for s_name in data.keys():
         str_save = str_read = ''
-	header = 'int save_%s(char *, %s *);\n int read_%s(char *, char *, %s *);\n' % (s_name, s_name, s_name, s_name)
-	str_save += 'int save_%s(char * tag, %s * target)\n{\nautostr_append(savestruct_autostr, "<%%s>\\n",tag);\n' % (s_name,s_name)
+        func_name = s_name.replace('struct ', '')
+        header = 'int save_%s(char *, %s *);\n int read_%s(char *, char *, %s *);\n' % (func_name, s_name, func_name, s_name)
+        str_save += 'int save_%s(char * tag, %s * target)\n{\nautostr_append(savestruct_autostr, "<%%s>\\n",tag);\n' % (func_name,s_name)
         str_read += '''int read_%s(char* buffer, char * tag, %s * target)\n{\n
 		char search[strlen(tag) + 5];
 		sprintf(search, "<%%s>", tag);
@@ -137,19 +140,28 @@ def main():
 		char * epos = strstr(buffer, search);
 		if ( ! epos ) return 2;
 		*epos = 0;
-		''' % (s_name, s_name)
+		''' % (func_name, s_name)
 
         for (type, field) in data[s_name]:
             size = None
             if "[" in type:
                 size = type.split('[')[1][:-1]
-                type = type.split('[')[0] + '_array'
-            str_save += 'save_%s("%s", %s(target->%s)%s);\n' % (type, field, '' if size else '&', field, (', %s' % size) if size else '')
-            str_read += 'read_%s(pos, "%s", %s %s(target->%s)%s);\n' % (type, field, '', '' if size else '&', field, (', %s' % size) if size else '')
+                elt_type = type.split('[')[0]
+                type = elt_type + '_array'
+            if '*' in type:
+                # Pointers are not saved (it does not make sense). They are initialized to NULL during loading
+                if size == None:
+                    str_read += 'target->%s = NULL;\n' % field
+                else:
+                    str_read += 'memset(target->%s, 0, %s * sizeof(%s));\n' % (field, size, elt_type)
+            else:
+                str_save += 'save_%s("%s", %s(target->%s)%s);\n' % (type, field, '' if size else '&', field, (', %s' % size) if size else '')
+                str_read += 'read_%s(pos, "%s", %s %s(target->%s)%s);\n' % (type, field, '', '' if size else '&', field, (', %s' % size) if size else '')
+
         str_save += 'autostr_append(savestruct_autostr, "</%s>\\n", tag);\nreturn 0;\n}\n\n'
         str_read += '''*epos = '>'; \nreturn 0;\n}\n\n'''
         outf.write(str_save)
         outf.write(str_read)
-	outh.write(header)
+        outh.write(header)
 
 if __name__ == '__main__': main()
