@@ -56,8 +56,8 @@ EXTERN float RangedDamageMultiplierTable[];
 EXTERN float RangedRechargeMultiplierTable[];
 EXTERN spell_skill_spec *SpellSkillMap;
 EXTERN tux_t Me;		/* the influence data */
-EXTERN Druidspec Druidmap;
-EXTERN Bulletspec Bulletmap;
+EXTERN druidspec *Druidmap;
+EXTERN bulletspec *Bulletmap;
 EXTERN blastspec Blastmap[ALLBLASTTYPES];
 EXTERN int skip_initial_menus;
 EXTERN int number_of_skills;
@@ -90,9 +90,10 @@ EXTERN blast AllBlasts[MAXBLASTS + 10];
 EXTERN int sound_on;		// Toggle TRUE/FALSE for turning sounds on/off 
 EXTERN int debug_level;		// 0=no debug 1=some debug messages 2=...etc 
 				// (currently only 0 or !=0 is implemented) 
-EXTERN int show_all_droids;	// display enemys regardless of IsVisible() 
+EXTERN int show_all_droids;	// display enemies regardless of IsVisible() 
 EXTERN int draw_collision_rectangles;	// to better debug collision rectangles
 EXTERN int draw_grid;		// grid to see where objects will be positioned
+EXTERN int start_editor;
 #undef EXTERN
 #ifdef _misc_c
 #define EXTERN
@@ -107,8 +108,7 @@ EXTERN char *character_descriptions[MAX_PERSONS];
 
 EXTERN char previous_part_strings[ALL_PART_GROUPS][50];
 EXTERN mouse_press_button AllMousePressButtons[MAX_MOUSE_PRESS_BUTTONS];
-EXTERN item *Item_Held_In_Hand;
-EXTERN point InventorySize;
+EXTERN item *item_held_in_hand;
 
 EXTERN iso_image MouseCursorImageList[NUMBER_OF_MOUSE_CURSOR_PICTURES];
 EXTERN iso_image SpellLevelButtonImageList[NUMBER_OF_SKILL_PAGES];
@@ -122,7 +122,6 @@ EXTERN BFont_Info *Para_BFont;
 EXTERN BFont_Info *FPS_Display_BFont;
 EXTERN BFont_Info *Blue_BFont;
 EXTERN BFont_Info *Red_BFont;
-EXTERN BFont_Info *Highscore_BFont;
 EXTERN float Overall_Average;
 EXTERN int SkipAFewFrames;
 
@@ -159,14 +158,21 @@ EXTERN list_head_t visible_level_list;
 EXTERN int gps_transform_map_dirty_flag;
 
 #define NEIGHBOR_IDX(x,len) ( ( (x) < 0) ? (0) : ( ( (x) < (len) ) ? (1) : (2) ) )
-#define NEIGHBOR_ID_NW(lvl) ( level_neighbors_map[lvl][0][0] ? level_neighbors_map[lvl][0][0]->lvl_idx : -1 )
-#define NEIGHBOR_ID_N(lvl)  ( level_neighbors_map[lvl][0][1] ? level_neighbors_map[lvl][0][1]->lvl_idx : -1 )
-#define NEIGHBOR_ID_NE(lvl) ( level_neighbors_map[lvl][0][2] ? level_neighbors_map[lvl][0][2]->lvl_idx : -1 )
-#define NEIGHBOR_ID_W(lvl)  ( level_neighbors_map[lvl][1][0] ? level_neighbors_map[lvl][1][0]->lvl_idx : -1 )
-#define NEIGHBOR_ID_E(lvl)  ( level_neighbors_map[lvl][1][2] ? level_neighbors_map[lvl][1][2]->lvl_idx : -1 )
-#define NEIGHBOR_ID_SW(lvl) ( level_neighbors_map[lvl][2][0] ? level_neighbors_map[lvl][2][0]->lvl_idx : -1 )
-#define NEIGHBOR_ID_S(lvl)  ( level_neighbors_map[lvl][2][1] ? level_neighbors_map[lvl][2][1]->lvl_idx : -1 )
-#define NEIGHBOR_ID_SE(lvl) ( level_neighbors_map[lvl][2][2] ? level_neighbors_map[lvl][2][2]->lvl_idx : -1 )
+#define NEIGHBOR_ID(lvl, x, y) (level_neighbors_map[lvl][y][x] ? level_neighbors_map[lvl][y][x]->lvl_idx : -1)
+#define NEIGHBOR_ID_NW(lvl) NEIGHBOR_ID(lvl, 0, 0)
+#define NEIGHBOR_ID_N(lvl)  NEIGHBOR_ID(lvl, 1, 0)
+#define NEIGHBOR_ID_NE(lvl) NEIGHBOR_ID(lvl, 2, 0)
+#define NEIGHBOR_ID_W(lvl)  NEIGHBOR_ID(lvl, 0, 1)
+#define NEIGHBOR_ID_E(lvl)  NEIGHBOR_ID(lvl, 2, 1)
+#define NEIGHBOR_ID_SW(lvl) NEIGHBOR_ID(lvl, 0, 2)
+#define NEIGHBOR_ID_S(lvl)  NEIGHBOR_ID(lvl, 1, 2)
+#define NEIGHBOR_ID_SE(lvl) NEIGHBOR_ID(lvl, 2, 2)
+
+typedef struct tux_part_render_set *tux_part_render_motionclass[MAX_TUX_DIRECTIONS];
+EXTERN struct tux_rendering_s {
+	struct dynarray motion_class_names;
+	tux_part_render_motionclass *render_order;
+} tux_rendering;
 
 #undef EXTERN
 #ifdef _light_c
@@ -196,6 +202,7 @@ int chat_control_partner_started;	//the dialog partner is the one who started th
 unsigned char *chat_control_chat_flags; //current chat flags
 enemy *chat_control_chat_droid;	//droid we are chatting with
 dialogue_option ChatRoster[MAX_DIALOGUE_OPTIONS_IN_ROSTER];
+EXTERN text_widget chat_log;
 
 #undef EXTERN
 #ifdef _sound_c
@@ -268,6 +275,14 @@ EXTERN float iso_floor_tile_width_over_two;
 EXTERN float iso_floor_tile_height_over_two;
 EXTERN int vid_bpp;		// bits per pixel 
 
+#undef EXTERN
+#ifdef _hud_c
+#define EXTERN
+#else
+#define EXTERN extern
+#endif
+EXTERN text_widget message_log;
+
 #undef EXTERN
 #ifdef _blocks_c
 #define EXTERN
@@ -283,8 +298,6 @@ EXTERN obstacle_spec obstacle_map[NUMBER_OF_OBSTACLE_TYPES];
 #define EXTERN extern
 #endif
 EXTERN int display_char_disabled;
-EXTERN int chat_protocol_scroll_override_from_user;
-EXTERN int game_message_protocol_scroll_override_from_user;
 EXTERN SDL_Rect Droid_Image_Window;
 
 #undef EXTERN
@@ -308,7 +321,6 @@ EXTERN SDL_Rect Droid_Image_Window;
 #define EXTERN extern
 #endif
 EXTERN point input_axis;	/* joystick (and mouse) axis values */
-EXTERN int always_show_items_text;
 
 #undef EXTERN
 #ifdef _takeover_c
@@ -336,8 +348,6 @@ EXTERN SDL_Rect SkillScreenRect;
 EXTERN int Number_Of_Item_Types;
 EXTERN itemspec *ItemMap;
 EXTERN short int item_count_per_class[10];
-EXTERN item_bonus *PrefixList;
-EXTERN item_bonus *SuffixList;
 
 EXTERN int game_root_mode;
 enum {
@@ -370,4 +380,20 @@ EXTERN char *do_benchmark;
 #endif
 
 EXTERN list_head_t npc_head;
+
+#undef EXTERN
+#ifdef _influ_c
+#define EXTERN
+#else
+#define EXTERN extern
+#endif
+
+EXTERN struct {
+	int standing_keyframe;
+	struct timebased_animation attack;
+	struct distancebased_animation walk;
+	struct distancebased_animation run;
+} tux_anim;
+
 #endif				// _global_h
+
