@@ -2,7 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
- *   Copyright (c) 2004-2007 Arthur Huillet 
+ *   Copyright (c) 2004-2010 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -226,7 +226,7 @@ void make_sure_system_mouse_cursor_is_turned_on(void)
 		SDL_ShowCursor(SDL_ENABLE);
 	}
 
-};				// void make_sure_system_mouse_cursor_is_turned_off ( void )
+};				// void make_sure_system_mouse_cursor_is_turned_on ( void )
 
 /**
  * When the system mouse cursor is now shown, we need to blit our own 
@@ -235,49 +235,37 @@ void make_sure_system_mouse_cursor_is_turned_on(void)
  */
 void blit_our_own_mouse_cursor(void)
 {
-	static int first_call = TRUE;
+	static int loaded = FALSE;
 	int i;
 	static iso_image mouse_cursors[16];
 	char constructed_filename[2000];
-	char fpath[2048];
 	int cursor_index = (-1);
 	point cursoff = { 0, 0 };
 
-	//--------------------
 	// On the first function call ever, we load the surfaces for the
 	// flags into memory.
 	//
-	if (first_call) {
+	if (!loaded) {
 		for (i = 0; i < 10; i++) {
 			sprintf(constructed_filename, "mouse_cursor_%04d.png", i);
-			find_file(constructed_filename, GRAPHICS_DIR, fpath, 0);
-			get_iso_image_from_file_and_path(fpath, &(mouse_cursors[i]), FALSE);
-			if (mouse_cursors[i].surface == NULL) {
-				fprintf(stderr, "\nFull path used: %s.", fpath);
-				ErrorMessage(__FUNCTION__, "\
-Error loading flag image.", PLEASE_INFORM, IS_FATAL);
-			}
-			if (use_open_gl) {
-				DebugPrintf(1, "\n%s(): Texture made from mouse cursor surface...", __FUNCTION__);
-				make_texture_out_of_surface(&(mouse_cursors[i]));
-			}
+			load_iso_image(&mouse_cursors[i], constructed_filename, FALSE);
 		}
-
-		first_call = FALSE;
+		loaded = TRUE;
 	}
 
 	switch (global_ingame_mode) {
 	case GLOBAL_INGAME_MODE_SCROLL_UP:
 		cursor_index = 4;
+		cursoff.x = -12;
 		break;
 	case GLOBAL_INGAME_MODE_SCROLL_DOWN:
 		cursor_index = 5;
-		break;
-	case GLOBAL_INGAME_MODE_IDENTIFY:
-		cursor_index = 1;
+		cursoff.x = -12;
 		break;
 	case GLOBAL_INGAME_MODE_NORMAL:
 		cursor_index = 0;
+		cursoff.x = -5;
+		cursoff.y = -4;
 		break;
 	case GLOBAL_INGAME_MODE_REPAIR:
 		cursor_index = 6;
@@ -291,12 +279,10 @@ Error loading flag image.", PLEASE_INFORM, IS_FATAL);
 		cursor_index = 3;
 		break;
 	default:
-		DebugPrintf(-4, "\n%s(): global_ingame_mode: %d.", __FUNCTION__, global_ingame_mode);
-		ErrorMessage(__FUNCTION__, "Illegal global ingame mode encountered!", PLEASE_INFORM, IS_FATAL);
+		ErrorMessage(__FUNCTION__, "Illegal global ingame mode encountered: %d", PLEASE_INFORM, IS_FATAL, global_ingame_mode);
 		break;
 	}
 
-	//--------------------
 	// We can now blit the mouse cursor...
 	//
 	if (use_open_gl) {
@@ -305,8 +291,7 @@ Error loading flag image.", PLEASE_INFORM, IS_FATAL);
 	} else {
 		blit_iso_image_to_screen_position(&mouse_cursors[cursor_index], GetMousePos_x() + cursoff.x, GetMousePos_y() + cursoff.y);
 	}
-
-};				// void blit_our_own_mouse_cursor ( void )
+}
 
 /**
  * Occasionally it might come in handly to have the whole image fading
@@ -452,7 +437,6 @@ int do_graphical_number_selection_in_range(int lower_range, int upper_range, int
 
 	StoreMenuBackground(1);
 
-	//--------------------
 	// Next we prepare the selection knob for all later operations
 	//
 	if (SelectionKnob == NULL) {
@@ -511,7 +495,6 @@ ERROR LOADING SELECTION KNOB IMAGE FILE!", PLEASE_INFORM, IS_FATAL);
 
 			if (event.type == SDL_MOUSEBUTTONDOWN) {
 				if (event.button.button == SDL_BUTTON_LEFT) {
-					//--------------------
 					// Maybe the user has just 'grabbed the knob?  Then we need to
 					// mark the knob as grabbed.
 					//
@@ -519,7 +502,6 @@ ERROR LOADING SELECTION KNOB IMAGE FILE!", PLEASE_INFORM, IS_FATAL);
 							(abs(event.button.y - (knob_target_rect.y + knob_target_rect.h / 2)) < knob_target_rect.h)) {
 						knob_is_grabbed = TRUE;
 					}
-					//--------------------
 					// OK pressed?  Then we can return the current scale value and
 					// that's it...
 					//
@@ -576,7 +558,6 @@ static Uint8 GetGreenComponent(SDL_Surface * surface, int x, int y)
 	Uint8 green;
 	int bpp = surface->format->BytesPerPixel;
 
-	//--------------------
 	// First we extract the pixel itself and the
 	// format information we need.
 	//
@@ -584,7 +565,6 @@ static Uint8 GetGreenComponent(SDL_Surface * surface, int x, int y)
 	SDL_LockSurface(surface);
 	// pixel = * ( ( Uint32* ) surface -> pixels ) ;
 	//
-	//--------------------
 	// Now for the longest time we had this command here (which can actually segfault!!)
 	//
 	// pixel = * ( ( ( Uint32* ) surface -> pixels ) + x + y * surface->w )  ;
@@ -593,7 +573,6 @@ static Uint8 GetGreenComponent(SDL_Surface * surface, int x, int y)
 
 	SDL_UnlockSurface(surface);
 
-	//--------------------
 	// Now we can extract the green component
 	//
 	temp = pixel & fmt->Gmask;	/* Isolate green component */
@@ -616,14 +595,12 @@ static Uint8 GetRedComponent(SDL_Surface * surface, int x, int y)
 	Uint8 red;
 	int bpp = surface->format->BytesPerPixel;
 
-	//--------------------
 	// First we extract the pixel itself and the
 	// format information we need.
 	//
 	fmt = surface->format;
 	SDL_LockSurface(surface);
 	// pixel = * ( ( Uint32* ) surface -> pixels ) ;
-	//--------------------
 	// Now for the longest time we had this command here (which can actually segfault!!)
 	//
 	// pixel = * ( ( ( Uint32* ) surface -> pixels ) + x + y * surface->w )  ;
@@ -631,7 +608,6 @@ static Uint8 GetRedComponent(SDL_Surface * surface, int x, int y)
 	pixel = *((Uint32 *) (((Uint8 *) (surface->pixels)) + (x + y * surface->w) * bpp));
 	SDL_UnlockSurface(surface);
 
-	//--------------------
 	// Now we can extract the red component
 	//
 	temp = pixel & fmt->Rmask;	/* Isolate red component */
@@ -654,14 +630,12 @@ static Uint8 GetBlueComponent(SDL_Surface * surface, int x, int y)
 	Uint8 blue;
 	int bpp = surface->format->BytesPerPixel;
 
-	//--------------------
 	// First we extract the pixel itself and the
 	// format information we need.
 	//
 	fmt = surface->format;
 	SDL_LockSurface(surface);
 	// pixel = * ( ( Uint32* ) surface -> pixels ) ;
-	//--------------------
 	// Now for the longest time we had this command here (which can actually segfault!!)
 	//
 	// pixel = * ( ( ( Uint32* ) surface -> pixels ) + x + y * surface->w )  ;
@@ -669,7 +643,6 @@ static Uint8 GetBlueComponent(SDL_Surface * surface, int x, int y)
 	pixel = *((Uint32 *) (((Uint8 *) (surface->pixels)) + (x + y * surface->w) * bpp));
 	SDL_UnlockSurface(surface);
 
-	//--------------------
 	// Now we can extract the blue component
 	//
 	temp = pixel & fmt->Bmask;	/* Isolate blue component */
@@ -692,13 +665,11 @@ Uint8 GetAlphaComponent(SDL_Surface * surface, int x, int y)
 	Uint8 alpha;
 	int bpp = surface->format->BytesPerPixel;
 
-	//--------------------
 	// First we extract the pixel itself and the
 	// format information we need.
 	//
 	fmt = surface->format;
 	SDL_LockSurface(surface);
-	//--------------------
 	// Now for the longest time we had this command here (which can actually segfault!!)
 	//
 	// pixel = * ( ( ( Uint32* ) surface -> pixels ) + x + y * surface->w )  ;
@@ -706,7 +677,6 @@ Uint8 GetAlphaComponent(SDL_Surface * surface, int x, int y)
 	pixel = *((Uint32 *) (((Uint8 *) (surface->pixels)) + (x + y * surface->w) * bpp));
 	SDL_UnlockSurface(surface);
 
-	//--------------------
 	// Now we can extract the alpha component
 	//
 	temp = pixel & fmt->Amask;	/* Isolate alpha component */
@@ -718,6 +688,18 @@ Uint8 GetAlphaComponent(SDL_Surface * surface, int x, int y)
 
 };				// Uint8 GetAlphaComponent ( SDL_Surface* SourceSurface , int x , int y )
 
+static Uint8 add_val_to_component(Uint8 component, int value)
+{
+	int tmp;
+
+	// Calculate the new value and check overflow/underflow.
+	tmp = component + value;
+	tmp = (tmp > 255) ? 255 : tmp;
+	tmp = (tmp < 0) ? 0 : tmp;
+
+    return (Uint8)tmp;
+}
+
 /**
  * This function can be used to create a new surface that has a certain
  * color filter applied to it.  The default so far will be that the blue
@@ -732,20 +714,17 @@ SDL_Surface *CreateColorFilteredSurface(SDL_Surface * FirstSurface, int FilterTy
 	Uint8 blue = 0;
 	float alpha3;
 
-	//--------------------
 	// First we check for NULL surfaces given...
 	//
 	if (FirstSurface == NULL) {
 		DebugPrintf(0, "\nERROR in SDL_Surface* CreateBlueColorFilteredSurface ( ... ) : NULL PARAMETER GIVEN.\n");
 		Terminate(ERR);
 	}
-	//--------------------
 	// Now we create a new surface, best in display format with alpha channel
 	// ready to be blitted.
 	//
 	ThirdSurface = our_SDL_display_format_wrapperAlpha(FirstSurface);
 
-	//--------------------
 	// Now we start to process through the whole surface and examine each
 	// pixel.
 	//
@@ -754,21 +733,27 @@ SDL_Surface *CreateColorFilteredSurface(SDL_Surface * FirstSurface, int FilterTy
 
 			alpha3 = GetAlphaComponent(FirstSurface, x, y);
 
+			red = GetRedComponent(FirstSurface, x, y);
+			green = GetGreenComponent(FirstSurface, x, y);
+			blue = GetBlueComponent(FirstSurface, x, y);
+
 			if (FilterType == FILTER_BLUE) {
+				blue = (red + green + blue) / 3;
 				red = 0;
 				green = 0;
-				blue = (GetBlueComponent(FirstSurface, x, y) +
-					GetRedComponent(FirstSurface, x, y) + GetGreenComponent(FirstSurface, x, y)) / 3;
 			} else if (FilterType == FILTER_GREEN) {
+				green = (red + green + blue) / 3;
 				red = 0;
 				blue = 0;
-				green = (GetBlueComponent(FirstSurface, x, y) +
-					 GetRedComponent(FirstSurface, x, y) + GetGreenComponent(FirstSurface, x, y)) / 3;
 			} else if (FilterType == FILTER_RED) {
+				red = (red + green + blue) / 3;
 				green = 0;
 				blue = 0;
-				red = (GetBlueComponent(FirstSurface, x, y) +
-				       GetRedComponent(FirstSurface, x, y) + GetGreenComponent(FirstSurface, x, y)) / 3;
+			} else if (FilterType == FILTER_HIGHLIGHT) {
+				// Set luminosity.
+				red = add_val_to_component(red, 64);
+				green = add_val_to_component(green, 64);
+				blue = add_val_to_component(blue, 64);
 			}
 
 			PutPixel(ThirdSurface, x, y, SDL_MapRGBA(ThirdSurface->format, red, green, blue, alpha3));
@@ -780,85 +765,6 @@ SDL_Surface *CreateColorFilteredSurface(SDL_Surface * FirstSurface, int FilterTy
 
 };				// SDL_Surface* CreateBlueColorFilteredSurface ( SDL_Surface* FirstSurface )
 
-/**
- * This function draws a "grid" on the screen, that means every
- * "second" pixel is blacked out, thereby generation a fading 
- * effect.  This function was created to fade the background of the 
- * Escape menu and its submenus.
- */
-void MakeGridOnScreen(SDL_Rect * GridRectangle)
-{
-	int x, y;
-	SDL_Rect TempRect;
-
-	if (GridRectangle == NULL)
-		GridRectangle = &User_Rect;
-
-	DebugPrintf(2, "\nvoid MakeGridOnScreen(...): real function call confirmed.");
-
-	//--------------------
-	// It's not very convenient to put a real grid over the screen using 
-	// OpenGL.  We do something similar but not quite as expensive in function
-	// overhead here.
-	//
-	if (use_open_gl) {
-		// our_SDL_fill_rect_wrapper ( Screen , & User_Rect , SDL_MapRGBA( Screen -> format , 0, 0, 0, 0x050 ) ) ;
-		return;
-	}
-
-	//--------------------
-	// We store the grid rectangle for later restoration.
-	//
-	Copy_Rect(*GridRectangle, TempRect);
-
-	//--------------------
-	// At first we do some sanity check to see if this rectangle does really
-	// make sense or not.
-	//
-	if (GridRectangle->x < 0) {
-		if (GridRectangle->w >= -GridRectangle->x)
-			GridRectangle->w += GridRectangle->w;
-		GridRectangle->x = 0;
-	}
-	if (GridRectangle->y < 0) {
-		if (GridRectangle->h >= -GridRectangle->y)
-			GridRectangle->h += GridRectangle->y;
-		GridRectangle->y = 0;
-	}
-	if ((GridRectangle->h <= 0) ||
-	    (GridRectangle->w <= 0) || (GridRectangle->x >= GameConfig.screen_width) || (GridRectangle->y >= GameConfig.screen_height)) {
-		Copy_Rect(TempRect, *GridRectangle);
-		return;
-	}
-	if ((GridRectangle->x + GridRectangle->w) > GameConfig.screen_width) {
-		GridRectangle->w = GameConfig.screen_width - GridRectangle->x;
-	}
-	if ((GridRectangle->y + GridRectangle->h) > GameConfig.screen_height) {
-		GridRectangle->h = GameConfig.screen_height - GridRectangle->y;
-	}
-	//--------------------
-	// Now we can start to draw the actual grid rectangle.
-	// We do so completely correctly with locked surfaces and everything.
-	//
-	SDL_LockSurface(Screen);
-	for (y = GridRectangle->y; y < (GridRectangle->h + GridRectangle->y); y++) {
-		for (x = GridRectangle->x; x < (GridRectangle->x + GridRectangle->w); x++) {
-			if ((x + y) % 2 == 0) {
-				PutPixel(Screen, x, y, 0);
-			}
-		}
-	}
-	SDL_UnlockSurface(Screen);
-
-	//--------------------
-	// We restore the original rectangle..
-	//
-	Copy_Rect(TempRect, *GridRectangle);
-
-	DebugPrintf(2, "\nvoid MakeGridOnScreen(...): end of function reached.");
-
-};				// void MakeGridOnSchreen( SDL_Rect* GridRectangle )
-
 /**
  *
  *
@@ -874,7 +780,6 @@ static void get_standard_iso_floor_tile_size(void)
 		ErrorMessage(__FUNCTION__, "\
 UNABLE TO LOAD STANDARD TILE!", PLEASE_INFORM, IS_FATAL);
 	}
-	//--------------------
 	// Warning!  The standard tile sizes should be a multiple of 2 in
 	//           both directions to prevent jittering from numerical 
 	//           rounding.
@@ -905,7 +810,6 @@ UNABLE TO LOAD STANDARD TILE!", PLEASE_INFORM, IS_FATAL);
  * ----------------------------------------------------------------- */
 void InitPictures(void)
 {
-	//--------------------
 	// First thing to do is get the size of a typical isometric
 	// floor tile, i.e. height and width of the corresponding graphics
 	// bitmap
@@ -917,54 +821,30 @@ void InitPictures(void)
 	// could box the influencer out of the ship....
 	Activate_Conservative_Frame_Computation();
 
-	ShowStartupPercentage(18);
-
 	set_mouse_cursor_to_shape(MOUSE_CURSOR_ARROW_SHAPE);
 
-	ShowStartupPercentage(22);
-
 	load_floor_tiles();
 
-	ShowStartupPercentage(25);
+	next_startup_percentage(33);
 
 	init_obstacle_data();
 	if (!GameConfig.lazyload)
 		load_all_obstacles();
 
-	ShowStartupPercentage(26);
-
-	DebugPrintf(1, "\nvoid InitPictures(void): preparing to load droids.");
-
 	Load_Enemy_Surfaces();
 
-	ShowStartupPercentage(38);
-
-	ShowStartupPercentage(43);
-
 	clear_all_loaded_tux_images(FALSE);
 
-	ShowStartupPercentage(65);
-
-	DebugPrintf(1, "\nvoid InitPictures(void): preparing to load blast image file.");
 	Load_Blast_Surfaces();
 
-	ShowStartupPercentage(70);
-
-	DebugPrintf(1, "\nvoid InitPictures(void): preparing to load items image file.");
-
-	ShowStartupPercentage(75);
+	next_startup_percentage(19);
 
 	Load_Mouse_Move_Cursor_Surfaces();
 
-	ShowStartupPercentage(80);
-
-	DebugPrintf(1, "\nvoid InitPictures(void): preparing to load bullet file.");
-	DebugPrintf(1, "\nvoid InitPictures(void): Number_Of_Bullet_Types : %d.", Number_Of_Bullet_Types);
 	iso_load_bullet_surfaces();
 
-	ShowStartupPercentage(92);
-
-};				// void InitPictures ( void )
+	next_startup_percentage(5);
+}
 
 /**
  * This function should load all the fonts we'll be using via the SDL
@@ -1054,7 +934,6 @@ void FreeOurBFonts(void)
  * -----------------------------------------------------------------*/
 void InitTimer(void)
 {
-	//--------------------
 	// Now SDL_TIMER is initialized here:
 	//
 	if (SDL_InitSubSystem(SDL_INIT_TIMER) == -1) {
@@ -1072,7 +951,6 @@ void InitTimer(void)
  */
 static void check_open_gl_libraries_present(void)
 {
-	//--------------------
 	// Here we introduce some warning output in case open_gl output is
 	// requested while the game was compiled without having the GL libs...
 	//
@@ -1109,13 +987,12 @@ static void check_open_gl_libraries_present(void)
  * This function should display the driver info obtained from the OpenGL
  * libraries.  This should be in a function of it's own (like now) to 
  * make sure that the OpenGL error checks in the video mode set functions
- * and that seem to be occuring so frequently are not coming from this 
+ * and that seem to be occurring so frequently are not coming from this 
  * chunk of code.
  */
 static void show_open_gl_driver_info(void)
 {
 #ifdef HAVE_LIBGL
-	//--------------------
 	// Since we want to use openGl, it might be good to check the OpenGL vendor string
 	// provided by the graphics driver.  Let's see...
 	//
@@ -1146,7 +1023,6 @@ static void set_double_buffering_attribute(void)
 		ErrorMessage(__FUNCTION__, "\
 Unable to set SDL_GL_DOUBLEBUFFER attribute!", PLEASE_INFORM, IS_FATAL);
 	}
-	//--------------------
 	// Since the OpenGL stuff hasn't been initialized yet, it's normal
 	// to get an GL_INVALID_OPERATION here, if we would really do the
 	// check.  So better refrain from OpenGL error checking here...
@@ -1171,7 +1047,6 @@ static void set_video_mode_for_open_gl(void)
 	Uint32 video_flags = 0;	// flags for SDL video mode 
 	int video_mode_ok_check_result;
 	int buffer_size, depth_size, red_size, green_size, blue_size, alpha_size;
-	//--------------------
 	// We need OpenGL double buffering, so we request it.  If we
 	// can't get it, something must be wrong, maybe an extremely bad 
 	// card/driver is present or some bad emulation.  Anyway, we'll
@@ -1179,13 +1054,11 @@ static void set_video_mode_for_open_gl(void)
 	//
 	set_double_buffering_attribute();
 
-	//--------------------
 	// Now we start setting up the proper OpenGL flags to pass to the
 	// SDL for creating the initial output window...
 	//
 	video_flags = SDL_OPENGL;	/* Enable OpenGL in SDL */
 
-	//--------------------
 	// Now according to the document http://sdldoc.csn.ul.ie/guidevideoopengl.php
 	// we do need the SDL_GL_SetAttribute ( SDL_GL_DOUBLEBUFFER, 1 ) and NOT
 	// this here...
@@ -1201,14 +1074,12 @@ static void set_video_mode_for_open_gl(void)
 	if (vid_info->blit_hw)
 		video_flags |= SDL_HWACCEL;
 
-	//--------------------
 	// We have 24 bit (or 32 bit) color depth in some of the graphics used,
 	// like e.g. backgrounds produced by Basse, so we try to get close to
 	// a target color depth of 24, or 32.
 	//
 	vid_bpp = 32;
 
-	//--------------------
 	// First we check to see if the mode we wish to set is really supported.  If it
 	// isn't supported, then we cancel the whole operation...
 	//
@@ -1216,7 +1087,9 @@ static void set_video_mode_for_open_gl(void)
 	switch (video_mode_ok_check_result) {
 	case 0:
 		ErrorMessage(__FUNCTION__, "\
-SDL reported, that the video mode mentioned above is not supported\nBreaking off...", PLEASE_INFORM, IS_FATAL);
+SDL reported that the video mode (%d x %d) mentioned above is not supported\n\
+To see all possible resolutions please run 'freedroidRPG -r99'\n\
+Breaking off...", PLEASE_INFORM, IS_FATAL, GameConfig.screen_width, GameConfig.screen_height);
 		break;
 	default:
 		DebugPrintf(-4, "\nTesting if color depth %d bits is available... ", vid_bpp);
@@ -1234,7 +1107,6 @@ SDL reported, that the video mode mentioned above is not supported\nBreaking off
 		}
 	}
 
-	//--------------------
 	// Now that we know which mode to go for, we can give it a try and get the
 	// output surface we want.  Of course, some extra checking will be done, so
 	// that we know that the surface we're expecting is really there...
@@ -1259,13 +1131,11 @@ SDL reported, that the video mode mentioned above is not supported\nBreaking off
 
 	safely_initialize_our_default_open_gl_parameters();
 
-	//--------------------
 	// Maybe resize the window to standard size?
 	//
 	// resizeWindow( GameConfig . screen_width, GameConfig . screen_height );
 	//
 
-	//--------------------
 	// Now under win32 we're running into problems, because there seems to be some
 	// garbage in ONE OF THE TWO BUFFERS from the double-buffering.  Maybe cleaning
 	// that out solves part of the problem.  Well, not all, since there is still the
@@ -1292,7 +1162,6 @@ void InitVideo(void)
 	char fpath[2048];
 	char window_title_string[200];
 
-	//--------------------
 	// Initialize the SDL library 
 	//
 	if (SDL_Init(SDL_INIT_VIDEO) == -1) {
@@ -1300,14 +1169,12 @@ void InitVideo(void)
 		Terminate(ERR);
 	} else {
 		DebugPrintf(1, "\nSDL Video initialisation successful.\n");
-		//--------------------
 		// So the video library could be initialized.  So it should also be
 		// cleaned up and closed once we're done and quit FreedroidRPG.
 		//
 		atexit(SDL_Quit);
 	}
 
-	//--------------------
 	// Let's get some info about the whole system here.  Is this a windows or x11 or
 	// mac or whatever graphical environment?
 	//
@@ -1320,19 +1187,16 @@ void InitVideo(void)
 		Terminate(ERR);
 	}
 
-	//--------------------
 	// We check if the program has been compiled with OpenGL libraries present
 	// and take care of the case OpenGL output requested when compiled without
 	// those libs...
 	//
 	check_open_gl_libraries_present();
 
-	//--------------------
 	// We note the screen resolution used.
 	//
 	DebugPrintf(-4, "\nUsing screen resolution %d x %d.", GameConfig.screen_width, GameConfig.screen_height);
 
-	//--------------------
 	// We query the available video configuration on this system.
 	//
 	vid_info = SDL_GetVideoInfo();
@@ -1355,7 +1219,6 @@ void InitVideo(void)
 
 	vid_bpp = 32;		/* start with the simplest */
 
-	//--------------------
 	// End of possibly open-gl dependant initialisation stuff...
 	//
 	sprintf(window_title_string, "FreedroidRPG %s", VERSION);
@@ -1373,8 +1236,6 @@ void InitVideo(void)
 	blit_special_background(FREEDROID_LOADING_PICTURE_CODE);
 	our_SDL_flip_wrapper();
 
-	ShowStartupPercentage(1);
-
 	SDL_SetGamma(GameConfig.current_gamma_correction, GameConfig.current_gamma_correction, GameConfig.current_gamma_correction);
 
 	SDL_ShowCursor(SDL_DISABLE);
@@ -1395,119 +1256,64 @@ void ClearGraphMem(void)
 };				// void ClearGraphMem( void )
 
 /**
+ * Draw a colored rectangle on screen with alpha blending in SDL.
  *
- *
+ * @param rect The rectangular area.
+ * @param r The red color value.
+ * @param g The green color value.
+ * @param b The blue color value.
+ * @param a The alpha color value.
  */
-void SDL_HighlightRectangle(SDL_Surface * Surface, SDL_Rect Area)
+static void sdl_draw_rectangle(SDL_Rect *rect, int r, int g, int b, int a)
 {
-	int x, y;
-	unsigned char red, green, blue;
-	int max;
-	float EnhancementFactor;
+	SDL_Surface *surface;
 
-	//--------------------
-	// First some sanity checks.. one can never now...
-	//
-	if (Area.x < 0)
-		Area.x = 0;
-	if (Area.x + Area.w >= Surface->w)
-		Area.w = Surface->w - Area.x - 1;
-	if (Area.y < 0)
-		Area.y = 0;
-	if (Area.y + Area.h >= Surface->h)
-		Area.h = Surface->h - Area.y - 1;
-
-	//--------------------
-	// Now we start to process through the whole surface and examine each
-	// pixel.
-	//
-	SDL_LockSurface(Surface);
-
-	for (y = Area.y; y < Area.y + Area.h; y++) {
-
-#define HIGHLIGHT_BLOCK_FRAME 2
-
-		// if ( ! ( ( y < Area . y + HIGHLIGHT_BLOCK_FRAME ) || ( y + HIGHLIGHT_BLOCK_FRAME >= Area.y+Area.h ) ) && 
-		// ! GameConfig.highlighting_mode_full ) continue ;
-
-		for (x = Area.x; x < Area.x + Area.w; x++) {
-
-			if ((!((x < Area.x + HIGHLIGHT_BLOCK_FRAME) || (x + HIGHLIGHT_BLOCK_FRAME >= Area.x + Area.w)) &&
-			     !GameConfig.highlighting_mode_full) &&
-			    (!((y < Area.y + HIGHLIGHT_BLOCK_FRAME) || (y + HIGHLIGHT_BLOCK_FRAME >= Area.y + Area.h)) &&
-			     !GameConfig.highlighting_mode_full))
-				continue;
-
-			green = GetGreenComponent(Surface, x, y);
-			red = GetRedComponent(Surface, x, y);
-			blue = GetBlueComponent(Surface, x, y);
-
-			if (green < red)
-				max = red;
-			else
-				max = green;
-			if (max < blue)
-				max = blue;
-
-			EnhancementFactor = 90;
-
-			if (red < 255 - EnhancementFactor)
-				red += EnhancementFactor;
-			else
-				red = 255;
-			if (green < 255 - EnhancementFactor)
-				green += EnhancementFactor;
-			else
-				green = 255;
-			if (blue < 255 - EnhancementFactor)
-				blue += EnhancementFactor;
-			else
-				green = 255;
-
-			PutPixel(Surface, x, y, SDL_MapRGB(Surface->format, red, green, blue));
-
-		}
+	if (a == SDL_ALPHA_OPAQUE) {
+		// Do a rectangle fill operation if the input rectangle is opaque.
+		SDL_FillRect(Screen, rect, SDL_MapRGB(Screen->format, r, g, b));
+		return;
 	}
 
-	SDL_UnlockSurface(Surface);
+	// Create an empty surface with 32 bits per pixel in video memory. This will
+	// allow SDL to take advantage of video.
+	surface = SDL_CreateRGBSurface(SDL_HWSURFACE, rect->w, rect->h, 32, 0, 0, 0, 0);
 
-};				// void Old_SDL_HighlightRectangle ( SDL_Surface* Surface , SDL_Rect Area )
+	// Perform a fast fill of the whole rectangle with color.
+	SDL_FillRect(surface, NULL, SDL_MapRGB(Screen->format, r, g, b));
+
+	// Adjust the alpha properties of a surface and active the acceleration.
+	SDL_SetAlpha(surface, SDL_SRCALPHA | SDL_RLEACCEL, a);
+
+	// Blit the surface on screen and free it.
+	SDL_BlitSurface(surface, NULL, Screen, rect);
+	SDL_FreeSurface(surface);
+}
 
 /**
- * This function draws a transparent white rectangle over a specified
+ * This function draws a transparent black rectangle over a specified
  * area on the screen.
  */
-
 void ShadowingRectangle(SDL_Surface * Surface, SDL_Rect Area)
 {
-
-	DebugPrintf(2, "\n%s(): x=%d, y=%d, w=%d, h=%d.", __FUNCTION__, Area.x, Area.y, Area.w, Area.h);
-	if (use_open_gl)
-		GL_HighlightRectangle(Surface, &Area, 0, 0, 0, 150);
-	//else
-	//SDL_HighlightRectangle ( Surface , Area );
-
-	return;
-
-};				// void ShadowingRectangle
+	if (use_open_gl) {
+		gl_draw_rectangle(&Area, 0, 0, 0, 150);
+	} else {
+		sdl_draw_rectangle(&Area, 0, 0, 0, 150);
+	}
+}
 
 /**
  * This function draws a transparent white rectangle over a specified
  * area on the screen.
  */
-
 void HighlightRectangle(SDL_Surface * Surface, SDL_Rect Area)
 {
-
-	DebugPrintf(2, "\n%s(): x=%d, y=%d, w=%d, h=%d.", __FUNCTION__, Area.x, Area.y, Area.w, Area.h);
-	if (use_open_gl)
-		GL_HighlightRectangle(Surface, &Area, 255, 255, 255, 100);
-	else
-		SDL_HighlightRectangle(Surface, Area);
-
-	return;
-
-};				// void HighlightRectangle
+	if (use_open_gl) {
+		gl_draw_rectangle(&Area, 255, 255, 255, 100);
+	} else {
+		sdl_draw_rectangle(&Area, 255, 255, 255, 100);
+	}
+}
 
 /*
  * Draw an 'expanded' pixel.
@@ -1533,7 +1339,7 @@ static void draw_expanded_pixel(SDL_Surface * Surface, int x, int y, int xincr,
  * This function draws a line in SDL mode.
  * Classical Bresenham algorithm 
  */
-void DrawLine(SDL_Surface * Surface, int x1, int y1, int x2, int y2, int r, int g, int b, int thickness)
+static void draw_line_sdl(SDL_Surface *Surface, int x1, int y1, int x2, int y2, int r, int g, int b, int thickness)
 {
 	if (use_open_gl)
 		return;
@@ -1588,6 +1394,41 @@ void DrawLine(SDL_Surface * Surface, int x1, int y1, int x2, int y2, int r, int
 	}
 }
 
+/**
+ * This function draws a line in OpenGL mode.
+ */
+static void draw_line_opengl(int x1, int y1, int x2, int y2, int r, int g, int b, int width)
+{
+#ifdef HAVE_LIBGL
+	glLineWidth(width);
+	glColor3ub(r, g, b);
+
+	glDisable(GL_TEXTURE_2D);
+
+	glBegin(GL_LINES);
+	glVertex2i(x1, y1);
+	glVertex2i(x2, y2);
+	glEnd();
+
+	glEnable(GL_TEXTURE_2D);
+#endif
+}
+
+void draw_line(float x1, float y1, float x2, float y2, Uint32 color, int width)
+{
+	float rr, gg, bb, zoom_factor;
+	zoom_factor = (GameConfig.zoom_is_on ? lvledit_zoomfact_inv() : 1.0);
+	rr = (color & 0xff0000) >> 16;
+	gg = (color & 0xff00) >> 8;
+	bb = color & 0xff;
+
+	if (!use_open_gl) {
+		draw_line_sdl(Screen, x1, y1, x2, y2, rr, gg, bb, width);
+	} else {
+		draw_line_opengl(x1, y1, x2, y2, rr, gg, bb, width);
+	}
+}
+
 /*
  * Draw an horizontally hatched quad in SDL mode
  * 
@@ -1602,15 +1443,13 @@ void DrawHatchedQuad(SDL_Surface * Surface, int x1, int y1, int x2, int y2, int
 	if (use_open_gl)
 		return;
 
-	//---------------
 	// Draw edges
 	//
-	DrawLine(Surface, x1, y1, x2, y2, r, g, b, 1);
-	DrawLine(Surface, x2, y2, x3, y3, r, g, b, 1);
-	DrawLine(Surface, x3, y3, x4, y4, r, g, b, 1);
-	DrawLine(Surface, x4, y4, x1, y1, r, g, b, 1);
+	draw_line_sdl(Surface, x1, y1, x2, y2, r, g, b, 1);
+	draw_line_sdl(Surface, x2, y2, x3, y3, r, g, b, 1);
+	draw_line_sdl(Surface, x3, y3, x4, y4, r, g, b, 1);
+	draw_line_sdl(Surface, x4, y4, x1, y1, r, g, b, 1);
 
-	//---------------
 	// Reorder vertices, so that bottom-most is the first one
 	//
 	struct vertex {
@@ -1652,7 +1491,6 @@ void DrawHatchedQuad(SDL_Surface * Surface, int x1, int y1, int x2, int y2, int
 		--min_index;
 	}
 
-	//---------------
 	// Store the 2 left edges, and the 2 right edges
 	//
 	// Note : since spans can be drawn from left to right or right to left
@@ -1691,7 +1529,6 @@ void DrawHatchedQuad(SDL_Surface * Surface, int x1, int y1, int x2, int y2, int
 	right_edges[1].deltax = (vertices[vindex[2]].x - vertices[vindex[3]].x);
 	right_edges[1].deltay = (vertices[vindex[2]].y - vertices[vindex[3]].y);
 
-	//---------------
 	// Quad filling, by creating horizontal spans between the left and right edges
 	// There is a 3 line step between two spans, to create hatches
 	//
@@ -1717,7 +1554,7 @@ void DrawHatchedQuad(SDL_Surface * Surface, int x1, int y1, int x2, int y2, int
 		float x_right = (float)REDGE.x + (float)(y_curr - REDGE.y) * (float)REDGE.deltax / (float)REDGE.deltay;
 
 		// Draw the span
-		DrawLine(Surface, x_left, y_curr, x_right, y_curr, r, g, b, 1);
+		draw_line_sdl(Surface, x_left, y_curr, x_right, y_curr, r, g, b, 1);
 	}
 
 #undef LEDGE
