@@ -40,7 +40,7 @@
 /* name of each keybinding */
 const char *keybindNames[] = {
 	/* General */
-	"keychart", "fullscreen", "quit", "wall_transparency",
+	"keychart", "fullscreen", "quit", "wall_transparency", "grab_input",
 
 	/* Ingame */
 	"inventory", "skill", "character", "quests",
@@ -70,7 +70,6 @@ const char *keybindNames[] = {
 	/* Cheat keys */
 	"cheat_xp+_1k", "cheat_xp*_2",
 	"cheat_melee", "cheat_range", "cheat_programing", "cheat_melee_down", "cheat_range_down", "cheat_programing_down",
-	"cheat_identify_all",
 	"cheat_drop_random_item", "cheat_drop_random_magical_item",
 	"cheat_respawn_level",
 	"cheat_menu", "cheat_level_editor",
@@ -166,6 +165,7 @@ void input_set_default(void)
 
 	input_set_keybind("keychart", SDLK_F1, KMOD_NONE);
 	input_set_keybind("fullscreen", SDLK_F2, KMOD_NONE);
+	input_set_keybind("grab_input", SDLK_g, KMOD_LCTRL);
 	input_set_keybind("quit", SDLK_q, KMOD_LCTRL);
 	input_set_keybind("wall_transparency", SDLK_t, KMOD_NONE);
 
@@ -242,7 +242,6 @@ void input_set_default(void)
 	input_set_keybind("cheat_melee_down", SDLK_KP4, KMOD_NONE);
 	input_set_keybind("cheat_range_down", SDLK_KP5, KMOD_NONE);
 	input_set_keybind("cheat_programing_down", SDLK_KP6, KMOD_NONE);
-	input_set_keybind("cheat_identify_all", SDLK_i, KMOD_LSHIFT);
 	input_set_keybind("cheat_drop_random_item", SDLK_r, KMOD_LCTRL);
 	input_set_keybind("cheat_drop_random_magical_item", SDLK_r, KMOD_LCTRL | KMOD_LSHIFT);
 	input_set_keybind("cheat_respawn_level", SDLK_r, KMOD_LCTRL | KMOD_LALT | KMOD_LSHIFT);
@@ -251,7 +250,7 @@ void input_set_default(void)
 
 }
 
-#define KEYCHART_RECT_X (GameConfig.screen_width / 10)
+#define KEYCHART_RECT_X (GameConfig.screen_width / 250)
 #define KEYCHART_RECT_Y (GameConfig.screen_height / 10)
 #define KEYCHART_RECT_W (0.8 * GameConfig.screen_width)
 #define KEYCHART_RECT_H (0.8 * GameConfig.screen_height)
@@ -268,7 +267,6 @@ static int display_keychart(unsigned int startidx, unsigned int cursor, int high
 	SDL_Rect keychart_rect = { KEYCHART_RECT_X, KEYCHART_RECT_Y, KEYCHART_RECT_W, KEYCHART_RECT_H };
 	int ypos = keychart_rect.y;
 	int xpos = keychart_rect.x;
-	BFont_Info *our_font = FPS_Display_BFont;
 
 	blit_special_background(SHOP_BACKGROUND_IMAGE_CODE);
 
@@ -283,19 +281,25 @@ static int display_keychart(unsigned int startidx, unsigned int cursor, int high
 
 	for (i = startidx; strcmp(keybindNames[i], "end"); i++) {
 		char modstring[10] = "";
+		const char *font_str = font_switchto_neon;
 		if (GameConfig.input_keybinds[i].mod & KMOD_LCTRL || GameConfig.input_keybinds[i].mod & KMOD_RCTRL)
 			strcat(modstring, "C-");
 		if (GameConfig.input_keybinds[i].mod & KMOD_LALT || GameConfig.input_keybinds[i].mod & KMOD_RALT)
 			strcat(modstring, "A-");
 		if (GameConfig.input_keybinds[i].mod & KMOD_LSHIFT || GameConfig.input_keybinds[i].mod & KMOD_RSHIFT)
 			strcat(modstring, "S-");
-		sprintf(txt, "%c%s%s - %s%s\n", (i == cursor && highlight) ? '\1' : '\2', (i == cursor) ? "** " : "   ",
+
+		if (i == cursor && highlight) {
+			font_str = font_switchto_red;
+		}
+
+		sprintf(txt, "%s%s%s - %s%s\n", font_str, (i == cursor) ? "** " : "   ",
 			GameConfig.input_keybinds[i].name, modstring, SDL_GetKeyName(GameConfig.input_keybinds[i].key));
-		PutStringFont(Screen, our_font, xpos, ypos, txt);
+		PutString(Screen, xpos, ypos, txt);
 
-		ypos += FontHeight(our_font);
+		ypos += FontHeight(GetCurrentFont());
 
-		if (ypos > keychart_rect.y + keychart_rect.h - FontHeight(our_font)) {
+		if (ypos > keychart_rect.y + keychart_rect.h - FontHeight(GetCurrentFont())) {
 			if (xpos > keychart_rect.x)
 				break;
 			else {
@@ -468,15 +472,6 @@ static int input_key(int keynum, int value)
 		} else if (KEYPRESS("cheat_programing_down")) {
 			if (Me.spellcasting_skill > 0)
 				Me.spellcasting_skill -= 1;
-		} else if (KEYPRESS("cheat_identify_all")) {
-			int i;
-			for (i = 0; i < MAX_ITEMS_IN_INVENTORY; i++)
-				Me.Inventory[i].is_identified = TRUE;
-			Me.weapon_item.is_identified = TRUE;
-			Me.shield_item.is_identified = TRUE;
-			Me.armour_item.is_identified = TRUE;
-			Me.special_item.is_identified = TRUE;
-			Me.drive_item.is_identified = TRUE;
 		} else if (KEYPRESS("cheat_drop_random_item")) {
 			DropRandomItem(Me.pos.z, Me.pos.x, Me.pos.y, 3, FALSE);
 		} else if (KEYPRESS("cheat_drop_random_magical_item")) {
@@ -501,8 +496,7 @@ static int input_key(int keynum, int value)
 			quest_browser_interface();
 			return 0;
 		} else if (KEYPRESS("autorun")) {
-			extern int autorun_activated;
-			autorun_activated = !autorun_activated;
+			GameConfig.autorun_activated = !GameConfig.autorun_activated;
 			return 0;
 		} else if (KEYPRESS("reload")) {
 			TuxReloadWeapon();
@@ -517,7 +511,7 @@ static int input_key(int keynum, int value)
 			Pause();
 			return 0;
 		} else if (KEYPRESS("show_item_labels")) {
-			always_show_items_text = !always_show_items_text;
+			GameConfig.show_item_labels = !GameConfig.show_item_labels;
 			return 0;
 		} else if (!strncmp(GameConfig.input_keybinds[keynum].name, "activate_program", strlen("activate_program"))
 			   && value == KEY_PRESS) {
@@ -595,7 +589,7 @@ static int input_key(int keynum, int value)
 			level_editor_cycle_marked_obstacle();
 			return 0;
 		} else if (KEYPRESS("next_tab")) {
-			GiveMouseAlertWindow("Reimplement that");
+			alert_window("Reimplement that");
 			return 0;
 		} else if (KEYPRESS("undo")) {
 			level_editor_action_undo();
@@ -604,7 +598,7 @@ static int input_key(int keynum, int value)
 			level_editor_action_redo();
 			return 0;
 		} else if (KEYPRESS("beautify_grass")) {
-			level_editor_beautify_grass_tiles(EditLevel(), 1, 1, EditLevel()->xlen - 1, EditLevel()->ylen - 1);
+			level_editor_beautify_grass_tiles(EditLevel());
 			return 0;
 		} else if (KEYPRESS("toolbar_scroll_left")) {
 			leveleditor_toolbar_scroll_left();
@@ -626,7 +620,7 @@ static int input_key(int keynum, int value)
 					     PLEASE_INFORM, IS_WARNING_ONLY, number);
 				return 0;
 			}
-			level_editor_place_aligned_obstacle(number);
+			level_editor_place_aligned_object(number);
 			return 0;
 		} else if (KEYPRESS("connect_waypoint")) {
 			level_editor_action_toggle_waypoint_connection_user(CURLEVEL(), EditX(), EditY());
@@ -651,6 +645,14 @@ static int input_key(int keynum, int value)
 		GameConfig.fullscreen_on = !GameConfig.fullscreen_on;
 #endif
 		return 0;
+	} else if (KEYPRESS("grab_input")) {
+		SDL_GrabMode mode = SDL_WM_GrabInput(SDL_GRAB_QUERY);
+		if (mode == SDL_GRAB_OFF)
+			mode = SDL_GRAB_ON;
+		else
+			mode = SDL_GRAB_OFF;
+
+		SDL_WM_GrabInput(mode);
 	} else if (KEYPRESS("wall_transparency")) {
 		GameConfig.transparency = !GameConfig.transparency;
 		return 0;
