@@ -1,6 +1,6 @@
 /* 
  *
- *   Copyright (c) 2008-2009 Arthur Huillet 
+ *   Copyright (c) 2008-2010 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -44,6 +44,22 @@
 /* Our Lua state for event execution */
 lua_State *global_lua_state;
 
+/** Helper to retrieve the enemy 
+  * a Lua function must act upon.
+  * An optional dialog name can be provided,
+  * by default the function will act upon
+  * the current chat droid.
+  */
+static enemy *get_enemy_arg(lua_State *L, int param_number) 
+{
+	const char *dialog = luaL_optstring(L, param_number, NULL);
+
+	if (!dialog)
+		return chat_control_chat_droid;
+
+	return get_enemy_with_dialog(dialog);
+}
+
 static int lua_event_teleport(lua_State * L)
 {
 	const char *label = luaL_checkstring(L, 1);
@@ -55,6 +71,16 @@ static int lua_event_teleport(lua_State * L)
 	return 0;
 }
 
+static int lua_event_teleport_npc(lua_State * L)
+{
+	const char *label = luaL_checkstring(L, 1);
+	enemy *en = get_enemy_arg(L, 2); 
+	location TempLocation;
+	ResolveMapLabelOnShip(label, &TempLocation);
+	teleport_enemy(en,TempLocation.level, TempLocation.x + 0.5, TempLocation.y + 0.5);
+	return 0;
+}
+
 static int lua_event_display_big_message(lua_State * L)
 {
 	const char *msg = luaL_checkstring(L, 1);
@@ -97,8 +123,8 @@ static int lua_event_disable_trigger(lua_State * L)
 
 static int event_change_obstacle_type(const char *obslabel, int type)
 {
-	obstacle *our_obstacle = give_pointer_to_obstacle_with_label(obslabel);
-	int obstacle_level_num = give_level_of_obstacle_with_label(obslabel);
+	int obstacle_level_num;
+	obstacle *our_obstacle = give_pointer_to_obstacle_with_label(obslabel, &obstacle_level_num);
 	level *obstacle_level = curShip.AllLevels[obstacle_level_num];
 
 	if (type != -1) {
@@ -107,7 +133,6 @@ static int event_change_obstacle_type(const char *obslabel, int type)
 		action_remove_obstacle(obstacle_level, our_obstacle);
 	}
 
-	//--------------------
 	// Now we make sure the door lists and that are all updated...
 	dirty_animated_obstacle_lists(obstacle_level->levelnum);
 
@@ -126,7 +151,7 @@ static int lua_event_get_obstacle_type(lua_State * L)
 {
 	const char *obslabel = luaL_checkstring(L, 1);
 	
-	obstacle *our_obstacle = give_pointer_to_obstacle_with_label(obslabel);
+	obstacle *our_obstacle = give_pointer_to_obstacle_with_label(obslabel, NULL);
 
 	lua_pushinteger(L, our_obstacle->type);	
 	return 1;
@@ -139,6 +164,22 @@ static int lua_event_delete_obstacle(lua_State * L)
 	return 0;
 }
 
+static int lua_change_obstacle_message(lua_State *L)
+{
+	const char *obslabel = luaL_checkstring(L, 1);
+	const char *message = luaL_checkstring(L, 2);
+	int obstacle_level_num;
+	obstacle *o = give_pointer_to_obstacle_with_label(obslabel, &obstacle_level_num);
+	level *l = curShip.AllLevels[obstacle_level_num];
+
+	message = strdup(message);
+
+	del_obstacle_extension(l, o, OBSTACLE_EXTENSION_SIGNMESSAGE);
+	add_obstacle_extension(l, o, OBSTACLE_EXTENSION_SIGNMESSAGE, (void *)message);
+
+	return 0;
+}
+
 static int lua_event_heal_tux(lua_State * L)
 {
 	Me.energy = Me.maxenergy;
@@ -154,7 +195,7 @@ static int lua_event_kill_tux(lua_State * L)
 static int lua_event_hurt_tux(lua_State * L)
 {
 	int hp = luaL_checkinteger(L, 1);
-	Me.energy -= hp;
+	hit_tux(hp, chat_control_chat_droid->id);
 	return 0;
 }
 
@@ -223,19 +264,26 @@ static int lua_event_get_skill(lua_State * L)
 static int lua_event_improve_program(lua_State * L)
 {
 	const char *pname = luaL_checkstring(L, 1);
-	Me.base_skill_level[get_program_index_with_name(pname)]++;
+	improve_program(get_program_index_with_name(pname));
 	return 0;
 }
 
 static int lua_event_downgrade_program(lua_State * L)
 {
 	const char *pname = luaL_checkstring(L, 1);
-	Me.base_skill_level[get_program_index_with_name(pname)]--;
-	if (Me.base_skill_level[get_program_index_with_name(pname)] < 0)
-		Me.base_skill_level[get_program_index_with_name(pname)] = 0;
+	Me.skill_level[get_program_index_with_name(pname)]--;
+	if (Me.skill_level[get_program_index_with_name(pname)] < 0)
+		Me.skill_level[get_program_index_with_name(pname)] = 0;
 	return 0;
 }
 
+static int lua_event_get_program_revision(lua_State * L)
+{
+	const char *pname = luaL_checkstring(L, 1);
+	lua_pushinteger(L, Me.skill_level[get_program_index_with_name(pname)]);
+	return 1;
+}
+
 static int lua_event_delete_item(lua_State * L)
 {
 	const char *itemname = luaL_checkstring(L, 1);
@@ -247,22 +295,16 @@ static int lua_event_delete_item(lua_State * L)
 static int lua_event_give_item(lua_State * L)
 {
 	const char *itemname = luaL_checkstring(L, 1);
-	int mult = luaL_checkinteger(L, 2);
+	int mult = luaL_optinteger(L, 2, 1);
 
 	item NewItem;
-	NewItem.type = GetItemIndexByName(itemname);
-	NewItem.prefix_code = (-1);
-	NewItem.suffix_code = (-1);
-	FillInItemProperties(&NewItem, TRUE, 1);
-	NewItem.multiplicity = mult;
+	NewItem = create_item_with_name(itemname, TRUE, mult);
 
-	//--------------------
 	// Either we put the new item directly into inventory or we issue a warning
 	// that there is no room and then drop the item to the floor directly under 
 	// the current Tux position.  That can't fail, right?
 	//
-	if (copy_item_into_inventory(&NewItem, NewItem.multiplicity)) {
-		DropItemToTheFloor(&NewItem, Me.pos.x, Me.pos.y, Me.pos.z);
+	if (!give_item(&NewItem)) {
 		SetNewBigScreenMessage(_("1 Item received (on floor)"));
 	} else {
 		SetNewBigScreenMessage(_("1 Item received!"));
@@ -273,13 +315,10 @@ static int lua_event_give_item(lua_State * L)
 static int lua_event_sell_item(lua_State *L)
 {
 	const char *itemname = luaL_checkstring(L, 1);
-	const char *charname = luaL_optstring(L, 2, NULL);
+	int weight = luaL_optint(L, 2, 1);
+	const char *charname = luaL_optstring(L, 3, chat_control_chat_droid->dialog_section_name);
 
-	if (!charname) {
-		charname = chat_control_chat_droid->dialog_section_name;
-	}
-
-	npc_add_shoplist(charname, itemname);
+	npc_add_shoplist(charname, itemname, weight);
 
 	return 0;
 }
@@ -293,12 +332,28 @@ static int lua_event_has_item_backpack(lua_State * L)
 	return 1;
 }
 
+static int lua_event_equip_item(lua_State * L)
+{
+	const char *item_name = luaL_checkstring(L, 1);
+
+	if (!item_name) {
+		ErrorMessage(__FUNCTION__, "Tried to add item without a name\n", PLEASE_INFORM,
+			     IS_WARNING_ONLY);
+		return 0;
+	}
+	item new_item = create_item_with_name(item_name, TRUE, 1);
+	equip_item(&new_item);
+	SetNewBigScreenMessage(_("1 Item received!"));
+	return 0;
+}
+
+
 static int lua_event_add_diary_entry(lua_State * L)
 {
 	const char *mis_name = luaL_checkstring(L, 1);
 	const char *text = luaL_checkstring(L, 2);
 
-	quest_browser_diary_add(mis_name, text);
+	mission_diary_add(mis_name, text);
 	return 0;
 }
 
@@ -342,7 +397,7 @@ static int lua_event_assign_mission(lua_State * L)
 
 	AssignMission(misname);
 	if (diarytext != NULL)
-		quest_browser_diary_add(misname, diarytext);
+		mission_diary_add(misname, diarytext);
 
 	return 0;
 }
@@ -350,8 +405,11 @@ static int lua_event_assign_mission(lua_State * L)
 static int lua_event_complete_mission(lua_State * L)
 {
 	const char *misname = luaL_checkstring(L, 1);
+	const char *diarytext = luaL_optstring(L, 2, NULL);
 
 	CompleteMission(misname);
+	if (diarytext != NULL)
+		mission_diary_add(misname, diarytext);
 
 	return 0;
 }
@@ -378,7 +436,7 @@ static int lua_event_give_xp(lua_State * L)
 {
 	int xp = luaL_checkinteger(L, 1);
 	char tmpstr[150];
-	Me.Experience += xp;
+	Me.Experience += xp * Me.experience_factor;
 	sprintf(tmpstr, _("+%d Experience Points"), xp);
 	SetNewBigScreenMessage(tmpstr);
 	return 0;
@@ -414,9 +472,9 @@ static int lua_event_add_gold(lua_State * L)
 	Me.Gold += nb;
 
 	if (nb > 0)
-		sprintf(tmpstr, _("Gained %d bucks!"), nb);
+		sprintf(tmpstr, _("Gained %d valuable circuits!"), nb);
 	else
-		sprintf(tmpstr, _("Lost %d bucks!"), nb);
+		sprintf(tmpstr, _("Lost %d valuable circuits!"), nb);
 
 	SetNewBigScreenMessage(tmpstr);
 	return 0;
@@ -472,6 +530,34 @@ static int lua_event_trade_with(lua_State * L)
 	return 0;
 }
 
+static int lua_event_upgrade_items(lua_State * L)
+{
+	item_upgrade_ui();
+
+	return 0;
+}
+
+static int lua_event_craft_addons(lua_State * L)
+{
+	addon_crafting_ui();
+
+	return 0;
+}
+
+static int lua_event_heal_npc(lua_State * L)
+{
+	enemy *en = get_enemy_arg(L, 1); 
+	en->energy = Druidmap[en->type].maxenergy;
+	return 0;
+}
+
+static int lua_display_npc_damage_amount(lua_State * L)
+{
+	enemy *en = get_enemy_arg(L, 1); 
+	lua_pushinteger(L, (int)(Druidmap[en->type].maxenergy - en->energy));
+	return 1;
+}
+
 static int lua_event_npc_dead(lua_State *L)
 {
 	const char *cname = luaL_checkstring(L, 1);
@@ -489,6 +575,15 @@ static int lua_event_npc_dead(lua_State *L)
 	return 1;
 }
 
+static int lua_event_freeze_tux_npc(lua_State * L)
+{
+	int duration = luaL_checkinteger(L, 1);
+	enemy *en = get_enemy_arg(L, 2); 
+	en->paralysation_duration_left = duration;
+	Me.paralyze_duration = duration;
+	return 0;
+}
+
 static int lua_chat_player_name(lua_State * L)
 {
 	lua_pushstring(L, Me.character_name);
@@ -499,12 +594,10 @@ static int lua_chat_tux_says(lua_State * L)
 {
 	const char *answer = luaL_checkstring(L, 1);
 	const char *sample = luaL_optstring(L, 2, "Sorry_No_Voice_Sample_Yet_0.wav");
-	extern char *chat_protocol;
 
-	strcat(chat_protocol, "\1TUX: ");
+	autostr_append(chat_log.text, "\1- ");
 	GiveSubtitleNSample(L_(answer), sample, chat_control_chat_droid, TRUE);
-	strcat(chat_protocol, "\2");
-
+	autostr_append(chat_log.text, "\n");
 	return 0;
 }
 
@@ -513,11 +606,24 @@ static int lua_chat_npc_says(lua_State * L)
 	const char *answer = luaL_checkstring(L, 1);
 	const char *sample = luaL_optstring(L, 2, "Sorry_No_Voice_Sample_Yet_0.wav");
 
+	autostr_append(chat_log.text, "\2");
+	GiveSubtitleNSample(L_(answer), sample, chat_control_chat_droid, TRUE);
+	autostr_append(chat_log.text, "\n");
+	return 0;
+}
+
+static int lua_chat_cli_says(lua_State * L)
+{
+	const char *answer = luaL_checkstring(L, 1);
+	const char *sample = luaL_optstring(L, 2, "Sorry_No_Voice_Sample_Yet_0.wav");
+
+	autostr_append(chat_log.text, "\3");
 	GiveSubtitleNSample(L_(answer), sample, chat_control_chat_droid, TRUE);
 
 	return 0;
 }
 
+
 static int lua_chat_run_subdialog(lua_State * L)
 {
 	const char *tmp_filename = luaL_checkstring(L, 1);
@@ -527,6 +633,13 @@ static int lua_chat_run_subdialog(lua_State * L)
 	return 0;
 }
 
+static int lua_start_chat(lua_State * L)
+{
+	enemy *en = get_enemy_arg(L, 1); 
+	ChatWithFriendlyDroid(en);
+	return 0;
+}
+
 static int lua_chat_set_next_node(lua_State * L)
 {
 	int nodenb = luaL_checkint(L, 1);
@@ -579,62 +692,81 @@ static int lua_chat_disable_node(lua_State * L)
 	return __lua_chat_toggle_node(L, 0);
 }
 
-static int lua_chat_break_off_and_attack(lua_State * L)
-{
-	chat_control_chat_droid->is_friendly = FALSE;
-	chat_control_end_dialog = 1;
-	return 0;
-}
-
 static int lua_chat_drop_dead(lua_State * L)
 {
-	hit_enemy(chat_control_chat_droid, chat_control_chat_droid->energy + 1, 0, -1, 0);
-	chat_control_end_dialog = 1;
-	return 0;
-}
-
-static int lua_chat_everybody_hostile(lua_State * L)
-{
-	enemy *erot;
-	BROWSE_ALIVE_BOTS(erot) {
-		erot->is_friendly = FALSE;
-	}
-	SwitchBackgroundMusicTo(BIGFIGHT_BACKGROUND_MUSIC_SOUND);
+	enemy *en = get_enemy_arg(L, 1); 
+	hit_enemy(en, en->energy + 1, 0, Druidmap[en->type].is_human - 2, 0);
+	if (en == chat_control_chat_droid)
+		chat_control_end_dialog = 1;
 	return 0;
 }
 
 static int lua_chat_set_bot_state(lua_State * L)
 {
 	const char *cmd = luaL_checkstring(L, 1);
+	enemy *en = get_enemy_arg(L, 2); 
 	if (!strcmp(cmd, "follow_tux")) {
-		chat_control_chat_droid->follow_tux = TRUE;
-		chat_control_chat_droid->CompletelyFixed = FALSE;
+		en->follow_tux = TRUE;
+		en->CompletelyFixed = FALSE;
 	} else if (!strcmp(cmd, "fixed")) {
-		chat_control_chat_droid->follow_tux = FALSE;
-		chat_control_chat_droid->CompletelyFixed = TRUE;
+		en->follow_tux = FALSE;
+		en->CompletelyFixed = TRUE;
 	} else if (!strcmp(cmd, "free")) {
-		chat_control_chat_droid->follow_tux = FALSE;
-		chat_control_chat_droid->CompletelyFixed = FALSE;
+		en->follow_tux = FALSE;
+		en->CompletelyFixed = FALSE;
+	} else if (!strcmp(cmd, "home")) {
+		en->follow_tux = FALSE;
+		en->CompletelyFixed = FALSE;
+		en->combat_state = RETURNING_HOME;
+	} else if (!strcmp(cmd, "patrol")) {
+		en->follow_tux = FALSE;
+		en->CompletelyFixed = FALSE;
+		en->combat_state = SELECT_NEW_WAYPOINT;
 	} else {
 		ErrorMessage(__FUNCTION__,
-			     "I was called with an invalid state namei %s. Accepted values are \"follow_tux\", \"fixed\" and \"free\".\n",
+			     "I was called with an invalid state named %s. Accepted values are \"follow_tux\", \"fixed\", \"free\", \"home\", and \"patrol\".\n",
 			     PLEASE_INFORM, IS_FATAL, cmd);
 	}
-
 	return 0;
 }
 
-static int lua_chat_make_tux_red_guard(lua_State * L)
+static int lua_set_bot_destination(lua_State * L)
 {
-	/*XXX get rid of this */
-	Me.is_town_guard_member = TRUE;
+	const char *label = luaL_checkstring(L, 1);
+	enemy *en = get_enemy_arg(L, 2); 
+	location TempLocation;
+	ResolveMapLabelOnShip(label, &TempLocation);
+	struct level *lvl = curShip.AllLevels[TempLocation.level];
+	int destinationwaypoint = get_waypoint(lvl, TempLocation.x + 0.5, TempLocation.y + 0.5);
+
+	if (TempLocation.level !=  en->pos.z) {
+		ErrorMessage(__FUNCTION__, "\
+				Sending bot %s to map label %s (found on level %d) cannot be done because the bot\n\
+				is not on the same level (z = %d). Doing nothing.",
+				PLEASE_INFORM, IS_WARNING_ONLY, en->dialog_section_name, label, TempLocation.level, en->pos.z);
+		return 0;
+	}
+
+	if (destinationwaypoint == -1) {
+		ErrorMessage(__FUNCTION__, "\
+				Map label %s (found on level %d) does not have a waypoint. Cannot send bot %s\n\
+				to this location. Doing nothing.\n\
+				Location coordinates x=%d, y=%d.",
+				PLEASE_INFORM, IS_WARNING_ONLY, label, TempLocation.level, en->pos.z, en->dialog_section_name, TempLocation.x, TempLocation.y);
+		return 0;
+	}
+
+	clear_out_intermediate_points(&en->pos, &en->PrivatePathway[0], 5);
+	en->lastwaypoint = destinationwaypoint;
+	en->nextwaypoint = destinationwaypoint;
+	en->combat_state = TURN_TOWARDS_NEXT_WAYPOINT;
 	return 0;
 }
 
 static int lua_chat_takeover(lua_State * L)
 {
 	int opponent_capsules = luaL_checkinteger(L, 1);
-	int player_capsules = 2 + Me.base_skill_level[get_program_index_with_name("Hacking")];
+	int player_capsules = 2 + Me.skill_level[get_program_index_with_name("Hacking")];
 	int game_length = luaL_optint(L, 2, 100);
 
 	int won = do_takeover(player_capsules, opponent_capsules, game_length);
@@ -644,28 +776,128 @@ static int lua_chat_takeover(lua_State * L)
 	return 1;
 }
 
-static int lua_chat_bot_name(lua_State * L)
+static int lua_chat_get_bot_type(lua_State * L)
 {
-	lua_pushstring(L, Druidmap[chat_control_chat_droid->type].druidname);
+	enemy *en = get_enemy_arg(L, 1); 
+	lua_pushstring(L, Druidmap[en->type].druidname);
 	return 1;
 }
 
+static int lua_chat_get_bot_name(lua_State * L)
+{
+	enemy *en = get_enemy_arg(L, 1); 
+	lua_pushstring(L, en->short_description_text);
+	return 1;
+}
+
+static int lua_chat_set_bot_name(lua_State * L)
+{
+	const char *bot_name = luaL_checkstring(L, 1);
+	enemy *en = get_enemy_arg(L, 2); 
+	free(en->short_description_text);
+	en->short_description_text = strdup(bot_name);
+	return 0;
+}
+
+static int lua_difficulty_level(lua_State * L)
+{
+	lua_pushnumber(L, GameConfig.difficulty_level);
+	return 1;
+}
+
+static int lua_set_npc_faction(lua_State *L)
+{
+	const char *fact = luaL_checkstring(L, 1);
+	enemy *en = get_enemy_arg(L, 2); 
+	en->faction = get_faction_id(fact);
+	return 0;
+}
+
+static int lua_user_input_string(lua_State *L)
+{
+	const char *title = luaL_checkstring(L, 1);
+	const char *default_str = luaL_optstring(L, 2, "");
+
+	const char *str = GetEditableStringInPopupWindow(100, title, default_str);
+
+	if (!str)
+		str = strdup("");
+
+	lua_pushstring(L, str);
+
+	free((void *)str);
+	return 1;
+}
+
+static int lua_set_faction_state(lua_State *L)
+{
+	const char *fact_name = luaL_checkstring(L, 1);
+	const char *state_str = luaL_checkstring(L, 2);
+	const char *fact2_name = luaL_optstring(L, 3, "self");
+
+	enum faction_state state;
+	enum faction_id fact_id = get_faction_id(fact_name);
+	enum faction_id fact2_id = get_faction_id(fact2_name);
+	
+	if (!strcmp(state_str, "hostile"))
+		state = HOSTILE;
+	else if (!strcmp(state_str, "friendly"))
+		state = FRIENDLY;
+	else {
+		ErrorMessage(__FUNCTION__, "Unknown faction state %s.", PLEASE_INFORM, IS_WARNING_ONLY);
+		return 0;
+	}
+
+	set_faction_state(fact_id, fact2_id, state);
+
+	return 0;
+}
+
+static int lua_create_droid(lua_State *L)
+{
+	const char *map_label = luaL_checkstring(L, 1);
+	const char *type_name = luaL_checkstring(L, 2);
+	const char *fact_name = luaL_optstring(L, 3, "ms");
+	const char *dialog    = luaL_optstring(L, 4, "AfterTakeover");
+	location loc;
+	int type;
+
+	ResolveMapLabelOnShip(map_label, &loc);
+
+	type = get_droid_type(type_name);
+
+	enemy *en = enemy_new(type);
+	enemy_reset(en);
+	en->pos.x = loc.x + 0.5;
+	en->pos.y = loc.y + 0.5;
+	en->pos.z = loc.level;
+	en->faction = get_faction_id(fact_name);
+	en->dialog_section_name = strdup(dialog);
+	enemy_insert_into_lists(en, TRUE);
+
+	return 0;
+}
+
 luaL_reg lfuncs[] = {
 	/* teleport(string map_label) 
 	 * Teleports the player to the given map label.
 	 */
 	{"teleport", lua_event_teleport}
 	,
-
+	/* teleport_npc(string map_label, [dialog name])
+	 * Teleports the current npc, or named npc to the given map label
+	 */
+	{"teleport_npc", lua_event_teleport_npc}
+	,
 	/* display_big_message(string msg)
 	 * Displays a big vanishing message on screen (seen in game, not in the dialog).
 	 */
 	{"display_big_message", lua_event_display_big_message}
 	,
-	/* display_console_message(string msg)
+	/* use display_console_message(string msg), supports [b] and [/b]
 	 * Displays a message on the game console.
 	 */
-	{"display_console_message", lua_event_display_console_message}
+	{"event_display_console_message", lua_event_display_console_message}
 	,
 
 	/* enable_trigger(string event_name)
@@ -689,6 +921,12 @@ luaL_reg lfuncs[] = {
 	{"del_obstacle", lua_event_delete_obstacle}
 	,
 
+	/* change_obstacle_message(string obstacle_label, string message)
+	 * Change the SIGNMESSAGE of the given obstacle.
+	 */
+	{"change_obstacle_message", lua_change_obstacle_message}
+	,
+
 	/* kill_tux()
 	 * heal_tux()
 	 * hurt_tux(int how_many_hp_to_remove)
@@ -722,11 +960,14 @@ luaL_reg lfuncs[] = {
 
 	/* improve_program(string program_name)
 	 * Improve the program given by one level.
+	 * get_program_revision(string program_name) returns current program revision level
 	 */
 	{"improve_program", lua_event_improve_program}
 	,
 	{"downgrade_program", lua_event_downgrade_program}
 	,
+	{"get_program_revision", lua_event_get_program_revision}
+        ,
 
 	/* del_item_backpack(string item_name[, int multiplicity = 1])
 	 * add_item(string item_name, int multiplicity)
@@ -741,7 +982,8 @@ luaL_reg lfuncs[] = {
 	,
 	{"has_item_backpack", lua_event_has_item_backpack}
 	,
-
+	{"equip_item", lua_event_equip_item}
+	,
 	{"sell_item", lua_event_sell_item}
 	,
 	{"add_diary_entry", lua_event_add_diary_entry}
@@ -752,11 +994,11 @@ luaL_reg lfuncs[] = {
 	,
 	{"del_cookie", lua_event_remove_cookie}
 	,
-	{"add_quest", lua_event_assign_mission}
+	{"assign_quest", lua_event_assign_mission}
 	,
 	{"has_quest", lua_event_is_mission_assigned}
 	,
-	{"end_quest", lua_event_complete_mission}
+	{"complete_quest", lua_event_complete_mission}
 	,
 	{"done_quest", lua_event_is_mission_complete}
 	,
@@ -776,15 +1018,24 @@ luaL_reg lfuncs[] = {
 	,
 	{"trade_with", lua_event_trade_with}
 	,
+	{"upgrade_items", lua_event_upgrade_items}
+	,
+	{"craft_addons", lua_event_craft_addons}
+	,
 
 	{"get_player_name", lua_chat_player_name}
 	,
 	{"tux_says", lua_chat_tux_says}
 	,
-	{"npc_says", lua_chat_npc_says}
+	/*use npc_says() rather than chat_npc_says() */
+	{"chat_npc_says", lua_chat_npc_says}
+	,
+	{"cli_says", lua_chat_cli_says}
 	,
 	{"run_subdialog", lua_chat_run_subdialog}
 	,
+	{"start_chat", lua_start_chat}
+	,
 	{"set_next_node", lua_chat_set_next_node}
 	,
 	{"end_dialog", lua_chat_end_dialog}
@@ -796,26 +1047,41 @@ luaL_reg lfuncs[] = {
 	{"disable_node", lua_chat_disable_node}
 	,
 
-	{"break_off_and_attack", lua_chat_break_off_and_attack}
-	,
 	{"drop_dead", lua_chat_drop_dead}
 	,
-	{"everybody_hostile", lua_chat_everybody_hostile}
-	,
 	{"set_bot_state", lua_chat_set_bot_state}
 	,
-	{"make_tux_red_guard", lua_chat_make_tux_red_guard}
+	{"set_bot_destination", lua_set_bot_destination}
 	,
-
 	{"takeover", lua_chat_takeover}
 	,
-
+	{"heal_npc", lua_event_heal_npc}
+	,
+	{"npc_damage_amount", lua_display_npc_damage_amount}
+	,
+	{"freeze_tux_npc", lua_event_freeze_tux_npc}
+	,
 	{"npc_dead", lua_event_npc_dead},
-	
-	{"bot_name", lua_chat_bot_name},
+	/* bot_type() tells you what model
+	   bot_name() tells you what name it displays
+	   set_bot_name() puts a new name in
+	*/	
+	{"bot_type", lua_chat_get_bot_type}, 
+
+	{"bot_name", lua_chat_get_bot_name},
+
+	{"set_bot_name", lua_chat_set_bot_name}, 
+
+	{"difficulty_level", lua_difficulty_level},
+
+	{"set_npc_faction", lua_set_npc_faction},
+	{"set_faction_state", lua_set_faction_state},
+
+	{"user_input_string", lua_user_input_string},
+
+	{"create_droid", lua_create_droid},
 
 	{NULL, NULL}
-	,
 };
 
 void run_lua(const char *code)
@@ -826,6 +1092,14 @@ void run_lua(const char *code)
 	}
 }
 
+void run_lua_file(const char *path)
+{
+	if (luaL_dofile(global_lua_state, path)) {
+		ErrorMessage(__FUNCTION__, "Cannot run script file %s: %s.\n",
+		         PLEASE_INFORM, IS_FATAL, path, lua_tostring(global_lua_state, -1));
+	}
+}
+
 void init_lua()
 {
 	char fpath[2048];
@@ -839,9 +1113,7 @@ void init_lua()
 		lua_setglobal(global_lua_state, lfuncs[i].name);
 	}
 
-	if (!find_file("script_helpers.lua", MAP_DIR, fpath, 1))
-		if (luaL_dofile(global_lua_state, fpath))
-			ErrorMessage(__FUNCTION__, "Cannot open script helpers file script_helpers.lua: %s.\n", PLEASE_INFORM, IS_FATAL,
-				     lua_tostring(global_lua_state, -1));
-
+	if (!find_file("script_helpers.lua", MAP_DIR, fpath, 1)) {
+		run_lua_file(fpath);
+	}
 }
