@@ -1,7 +1,7 @@
 /* 
  *
  *   Copyright (c) 2002, 2003 Johannes Prix
- *   Copyright (c) 2004-2007 Arthur Huillet
+ *   Copyright (c) 2004-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -48,8 +48,13 @@ enum ActionType {
 	ACT_CREATE_OBSTACLE,
 	ACT_REMOVE_OBSTACLE,
 	ACT_MOVE_OBSTACLE,
-	ACT_WAYPOINT_TOGGLE,
-	ACT_WAYPOINT_TOGGLE_CONNECT,
+	ACT_CREATE_ITEM,
+	ACT_REMOVE_ITEM,
+	ACT_MOVE_ITEM,
+	ACT_CREATE_WAYPOINT,
+	ACT_REMOVE_WAYPOINT,
+	ACT_TOGGLE_WAYPOINT_RSPAWN,
+	ACT_TOGGLE_WAYPOINT_CONNECTION,
 	ACT_TILE_FLOOR_SET,
 	ACT_SET_OBSTACLE_LABEL,
 	ACT_SET_MAP_LABEL,
@@ -75,10 +80,35 @@ typedef struct {
 			float newy;
 		} move_obstacle;
 
+		struct {
+			float x, y;
+			int type;		
+		} create_item;
+
+		item *delete_item;
+
+		struct {
+			item *item;
+			float newx;
+			float newy;
+		} move_item;
+
 		struct {
 			int x, y;
-			int spawn_toggle;
-		} waypoint_toggle;	/* ToogleWaypoint */
+			int suppress_random_spawn;
+		} create_waypoint;
+
+		struct {
+			int x, y;
+		} delete_waypoint;
+
+		struct {
+			int x, y;
+		} toggle_waypoint_rspawn;
+
+		struct {
+			int x, y;
+		} toggle_waypoint_connection;
 
 		struct {
 			int x, y;
@@ -95,6 +125,7 @@ typedef struct {
 		struct {
 			int id;
 			char *new_name;
+			int x, y;
 		} change_label_name;	/* EditMapLabelData */
 
 		struct {
@@ -112,19 +143,16 @@ enum {
 
 EXTERN void LevelEditor(void);
 EXTERN item *ItemDropFromLevelEditor(void);
-EXTERN void TestMap();
+EXTERN void TestMap(void);
 
 enum leveleditor_object_type {
 	OBJECT_FLOOR,
 	OBJECT_OBSTACLE,
-	OBJECT_NPC,		//just kidding, not implemented :)
 	OBJECT_WAYPOINT,
-	OBJECT_ANY,
+	OBJECT_ITEM,
+	OBJECT_NONE,
 };
 
-EXTERN iso_image *leveleditor_get_object_image(enum leveleditor_object_type type, int *array, int idx);
-EXTERN void leveleditor_print_object_info(enum leveleditor_object_type type, int *array, int idx, char *str);
-
 #ifdef _leveleditor_c
 int floor_tiles_list[] = {
 
@@ -159,6 +187,10 @@ int floor_tiles_list[] = {
         ISO_SIDEWALK_22,
         ISO_SIDEWALK_23,
         ISO_SIDEWALK_24,
+	ISO_WATER_SIDEWALK_01,
+	ISO_WATER_SIDEWALK_02,
+	ISO_WATER_SIDEWALK_03,
+	ISO_WATER_SIDEWALK_04,
 
 	ISO_FLOOR_STONE_FLOOR,
 	ISO_FLOOR_STONE_FLOOR_WITH_DOT,
@@ -374,6 +406,7 @@ int wall_tiles_list[] = {
 	ISO_V_WIRE_FENCE,
 	ISO_H_WIRE_FENCE,
 	ISO_GLASS_WALL_1,
+	ISO_BROKEN_GLASS_WALL_1,
 	ISO_GLASS_WALL_2,
 	ISO_THICK_WALL_V,
 	ISO_THICK_WALL_H,
@@ -410,8 +443,8 @@ int wall_tiles_list[] = {
 	ISO_BRICK_WALL_CABLES_CORNER_2,
 	ISO_RED_FENCE_V,
 	ISO_RED_FENCE_H,
-	ISO_BLOCK_1,
-	ISO_BLOCK_2,
+	ISO_PILLAR_TALL,
+	ISO_PILLAR_SHORT,
 	ISO_V_CURTAIN,
 	ISO_H_CURTAIN,
 
@@ -466,6 +499,20 @@ int machinery_tiles_list[] = {
 	ISO_EXIT_5,
 	ISO_EXIT_6,
 
+	ISO_SOLAR_PANEL_E,
+	ISO_BOTLINE_01_N,
+	ISO_BOTLINE_01_E,
+	ISO_BOTLINE_02_E,
+	ISO_BOTLINE_02_N,
+	ISO_BOTLINE_02_W,
+	ISO_BOTLINE_02_S,
+	ISO_BOTLINE_03_N,
+	ISO_BOTLINE_03_E,
+	ISO_BOTLINE_04_N,
+	ISO_BOTLINE_04_E,
+	ISO_BOTLINE_05_N,
+	ISO_BOTLINE_05_E,
+
 	-1,
 };
 
@@ -696,17 +743,16 @@ EXTERN int waypoint_list[];
 #endif
 
 EXTERN int all_obstacles_list[NUMBER_OF_OBSTACLE_TYPES + 1];
+EXTERN int *all_items_list;
 
-EXTERN int EditX();
-EXTERN int EditY();
-EXTERN level *EditLevel();
+EXTERN int EditX(void);
+EXTERN int EditY(void);
+EXTERN level *EditLevel(void);
 
 EXTERN char VanishingMessage[10000];
 EXTERN float VanishingMessageEndDate;
 
 EXTERN int OriginWaypoint;
 
-EXTERN int marked_obstacle_is_glued_to_here(level *, float, float);
-
 EXTERN int level_editor_done;
 #endif
