@@ -1,6 +1,6 @@
 /* 
  *
- *   Copyright (c) 2004-2009 Arthur Huillet
+ *   Copyright (c) 2004-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -39,7 +39,6 @@
 
 #include "lvledit/lvledit.h"
 #include "lvledit/lvledit_widgets.h"
-#include "lvledit/lvledit_widget_typeselect.h"
 
 /* Undo/redo action lists */
 LIST_HEAD(to_undo);
@@ -105,11 +104,35 @@ static action *action_create(int type, va_list args)
 			act->d.move_obstacle.newx = va_arg(args, double);
 			act->d.move_obstacle.newy = va_arg(args, double);
 			break;
-		case ACT_WAYPOINT_TOGGLE:
-		case ACT_WAYPOINT_TOGGLE_CONNECT:
-			act->d.waypoint_toggle.x = va_arg(args, int);
-			act->d.waypoint_toggle.y = va_arg(args, int);
-			act->d.waypoint_toggle.spawn_toggle = va_arg(args, int);
+		case ACT_CREATE_ITEM:
+			act->d.create_item.x = (float)va_arg(args, double);
+			act->d.create_item.y = (float)va_arg(args, double);
+			act->d.create_item.type = va_arg(args, int);
+			break;
+		case ACT_REMOVE_ITEM:
+			act->d.delete_item = va_arg(args, item *);
+			break;
+		case ACT_MOVE_ITEM:
+			act->d.move_item.item = va_arg(args, item *);
+			act->d.move_item.newx = (float)va_arg(args, double);
+			act->d.move_item.newy = (float)va_arg(args, double);
+			break;
+		case ACT_CREATE_WAYPOINT:
+			act->d.create_waypoint.x = va_arg(args, int);
+			act->d.create_waypoint.y = va_arg(args, int);
+			act->d.create_waypoint.suppress_random_spawn = va_arg(args, int);
+			break;
+		case ACT_REMOVE_WAYPOINT:
+			act->d.delete_waypoint.x = va_arg(args, int);
+			act->d.delete_waypoint.y = va_arg(args, int);
+			break;
+		case ACT_TOGGLE_WAYPOINT_RSPAWN:
+			act->d.toggle_waypoint_rspawn.x = va_arg(args, int);
+			act->d.toggle_waypoint_rspawn.y = va_arg(args, int);
+			break;
+		case ACT_TOGGLE_WAYPOINT_CONNECTION:
+			act->d.toggle_waypoint_connection.x = va_arg(args, int);
+			act->d.toggle_waypoint_connection.y = va_arg(args, int);
 			break;
 		case ACT_TILE_FLOOR_SET:
 			act->d.change_floor.x = va_arg(args, int);
@@ -126,6 +149,8 @@ static action *action_create(int type, va_list args)
 		case ACT_SET_MAP_LABEL:
 			act->d.change_label_name.id = va_arg(args, int);
 			act->d.change_label_name.new_name = va_arg(args, char *);
+			act->d.change_label_name.x = va_arg(args, int);
+			act->d.change_label_name.y = va_arg(args, int);
 			break;
 		case ACT_JUMP_TO_LEVEL:
 			act->d.jump_to_level.target_level = va_arg(args, int);
@@ -169,11 +194,7 @@ obstacle *action_create_obstacle(level * EditLevel, double x, double y, int new_
 			EditLevel->obstacle_list[i].pos.x = x;
 			EditLevel->obstacle_list[i].pos.y = y;
 			EditLevel->obstacle_list[i].pos.z = EditLevel->levelnum;
-			EditLevel->obstacle_list[i].name_index = (-1);
 			glue_obstacles_to_floor_tiles_for_level(EditLevel->levelnum);
-			DebugPrintf(0, "\nNew obstacle has been added!!!");
-			fflush(stdout);
-			//--------------------
 			// Now that we have disturbed the order of the obstacles on this level, we need
 			// to re-assemble the lists of pointers to obstacles, like the door list, the
 			// teleporter list and the refreshes list.
@@ -208,120 +229,54 @@ obstacle *action_create_obstacle_user(Level EditLevel, double x, double y, int n
 
 /**
  * Change an obstacle label, possibly removing it.
- * @return the number of actions that were pushed on the stack. Removing a non existing label 
- * will not create a undo action.
+ * @return the number of actions that were pushed on the stack.
  */
 static int action_change_obstacle_label(level *EditLevel, obstacle *obstacle, char *name, int undoable)
 {
-	int check_double;
-	char *old_name = NULL;
-	int index = -1;
-	int i;
-	int nbact = 0;
-
-	if (name)
-		name = strdup(name);
-
-	//--------------------
-	// If the obstacle already has a name, we can use that index for the 
-	// new name now.
-	//
-	if (obstacle->name_index >= 0)
-		index = obstacle->name_index;
-	else {
-		//--------------------
-		// Else we must find a free index in the list of obstacle names for this level
-		//
-		for (i = 0; i < MAX_OBSTACLE_NAMES_PER_LEVEL; i++) {
-			if (EditLevel->obstacle_name_list[i] == NULL) {
-				index = i;
-				break;
-			}
-		}
-		if (index < 0)
-			goto ret;
+	// If the obstacle already has a label, remove it
+	char *old_label = get_obstacle_extension(EditLevel, obstacle, OBSTACLE_EXTENSION_LABEL);
+	if (old_label) {
+		del_obstacle_extension(EditLevel, obstacle, OBSTACLE_EXTENSION_LABEL);
 	}
 
-	old_name = EditLevel->obstacle_name_list[index];
-	if (!name || strlen(name) == 0) {
-		obstacle->name_index = -1;
-		EditLevel->obstacle_name_list[index] = NULL;
+	// Create the undo action if appropriate
+	if (undoable) {
+		action_push(ACT_SET_OBSTACLE_LABEL, obstacle, old_label);
 	} else {
-		EditLevel->obstacle_name_list[index] = name;
-		obstacle->name_index = index;
+		free(old_label);
 	}
 
-	if (undoable && old_name != name) {
-		action_push(ACT_SET_OBSTACLE_LABEL, obstacle, old_name);
-		nbact ++;
-	} else if (old_name) {
-		free(old_name);
-	}
+	// If the new label is empty, we are done
+	if (!name || !strlen(name))
+		return 0;
 
-	if (obstacle->name_index == -1)
-		goto ret;
+	name = strdup(name);
 
-	//--------------------
-	// But even if we fill in something new, we should first
-	// check against double entries of the same label.  Let's
-	// do it...
-	//
-	for (check_double = 0; check_double < MAX_OBSTACLE_NAMES_PER_LEVEL; check_double++) {
-		//--------------------
-		// We must not use null pointers for string comparison...
-		//
-		if (EditLevel->obstacle_name_list[check_double] == NULL)
-			continue;
+	// Assign the new label
+	add_obstacle_extension(EditLevel, obstacle, OBSTACLE_EXTENSION_LABEL, name);
 
-		//--------------------
-		// We must not overwrite ourself with us in foolish ways :)
-		//
-		if (check_double == index)
-			continue;
-
-		//--------------------
-		// But in case of real double-entries, we'll handle them right.
-		//
-		if (!strcmp(EditLevel->obstacle_name_list[index], EditLevel->obstacle_name_list[check_double])) {
-			ErrorMessage(__FUNCTION__, "\
-		    The label %s did already exist on this map!  Deleting old entry in favour of the new one!", NO_NEED_TO_INFORM, IS_WARNING_ONLY, EditLevel->obstacle_name_list[index]);
-			EditLevel->obstacle_name_list[index] = NULL;
-			obstacle->name_index = check_double;
-			break;
-		}
-	}
-
-ret: 
-	return nbact;
+	return undoable;
 }
 
 void action_change_obstacle_label_user(level *EditLevel, obstacle *our_obstacle, char *predefined_name)
 {
-	int cur_idx;
-	char *name;
+	char *name = NULL;
 
 	if (!our_obstacle)
 		return;
 
-	cur_idx = our_obstacle->name_index;
-
-	//--------------------
-	// Maybe we must query the user for the desired new name.
-	// On the other hand, it might be that a name has been
-	// supplied as an argument.  That depends on whether the
-	// argument string is NULL or not.
-	//
 	if (predefined_name == NULL) {
-		name =
-		    GetEditableStringInPopupWindow(1000, _("\nPlease enter name for this obstacle: \n\n"),
-						   cur_idx != -1 ? EditLevel->obstacle_name_list[cur_idx] : "");
+		char *old_label = get_obstacle_extension(EditLevel, our_obstacle, OBSTACLE_EXTENSION_LABEL);
+		name = GetEditableStringInPopupWindow(1000, _("\nPlease enter obstacle label: \n\n"),
+						   old_label ? old_label : "");
 	} else {
 		name = strdup(predefined_name);
 	}
-
-	action_change_obstacle_label(EditLevel, our_obstacle, name, 1);
-
-	free(name);
+	
+	if (name) {
+		action_change_obstacle_label(EditLevel, our_obstacle, name, 1);
+		free(name);
+	}
 }
 
 /**
@@ -331,7 +286,8 @@ void action_change_obstacle_label_user(level *EditLevel, obstacle *our_obstacle,
  */
 void action_remove_obstacle(level *EditLevel, obstacle *our_obstacle)
 {
-	//--------------------
+	level *lvl = curShip.AllLevels[our_obstacle->pos.z];
+
 	// The likely case that no obstacle was currently marked.
 	//
 	if (our_obstacle == NULL)
@@ -341,20 +297,18 @@ void action_remove_obstacle(level *EditLevel, obstacle *our_obstacle)
 
 	// Remove the obstacle label if we had one
 	// with the current design this won't be undoable
-	action_change_obstacle_label(EditLevel, our_obstacle, NULL, 0);
+	action_change_obstacle_label(lvl, our_obstacle, NULL, 0);
 
-	//--------------------
 	// Now doing that must have shifted the glue!  That is a problem.  We need to
 	// reglue everything to the map...
 	//
-	glue_obstacles_to_floor_tiles_for_level(EditLevel->levelnum);
+	glue_obstacles_to_floor_tiles_for_level(lvl->levelnum);
 
-	//--------------------
 	// Now that we have disturbed the order of the obstacles on this level, we need
 	// to re-assemble the lists of pointers to obstacles, like the door list, the
 	// teleporter list and the refreshes list.
 	//
-	dirty_animated_obstacle_lists(EditLevel->levelnum);
+	dirty_animated_obstacle_lists(lvl->levelnum);
 }
 
 void action_remove_obstacle_user(Level EditLevel, obstacle * our_obstacle)
@@ -372,67 +326,199 @@ void action_remove_obstacle_user(Level EditLevel, obstacle * our_obstacle)
 	action_push(ACT_CREATE_OBSTACLE, posx, posy, type);
 }
 
-void action_toggle_waypoint(level * EditLevel, int BlockX, int BlockY, int toggle_random_spawn)
+/**
+ * Create an item on the map, add this action in the stack undo/redo
+ * \param EditLevel Pointer towards the editing level where create the item
+ * \param x The x position of the item
+ * \param y The y position of the item
+ * \param type The type of the item
+ * \return The new item created
+ */
+item *action_create_item(level *EditLevel, float x, float y, int type)
 {
-	int wpnum;
-	int isnew = 0;
+	int multiplicity = 1;
 
-	wpnum = CreateWaypoint(EditLevel, BlockX, BlockY, &isnew);
-
-	//--------------------
-	// If its waypoint already, this waypoint must either be deleted
-	// or the random spawn bit reset...
-	//
-	if (!isnew) {
-		if (toggle_random_spawn) {
-			if (EditLevel->AllWaypoints[wpnum].suppress_random_spawn)
-				EditLevel->AllWaypoints[wpnum].suppress_random_spawn = 0;
-			else
-				EditLevel->AllWaypoints[wpnum].suppress_random_spawn = 1;
-		} else
-			DeleteWaypoint(EditLevel, wpnum);
+	if (ItemMap[type].item_group_together_in_inventory) {
+		// Display a popup window with a number selector in order to choose the
+		// multiplicity of the item
+		multiplicity =
+		    do_graphical_number_selection_in_range(1, (!MatchItemWithName(type, "Valuable Circuits")) ? 100 : 1000, 1);
+		
+		// We always want create an item with a multiplicity higher than 0
+		multiplicity = (multiplicity > 0) ? multiplicity : 1;
 	}
 
-	action_push(ACT_WAYPOINT_TOGGLE, BlockX, BlockY, toggle_random_spawn);
+	// Create an item on the map
+	item *it = DropItemAt(type, EditLevel->levelnum, x, y, multiplicity);
+	if (it) {
+		action_push(ACT_REMOVE_ITEM, it);
+	}
+	return it;
 }
 
-int action_toggle_waypoint_connection(level * EditLevel, int id_origin, int id_target, int removeifpresent, int undoable)
+/**
+ * Remove an item on the map and add this action in the stack undo/redo
+ * \param EditLevel Pointer towards the editing level where delete the item
+ * \param it The item which we want deleted
+ */
+void action_remove_item(level *EditLevel, item *it)
 {
-	int i = 0;
-	waypoint *SrcWp = &(EditLevel->AllWaypoints[id_origin]);
-	for (i = 0; i < SrcWp->num_connections; i++) {
-		if (SrcWp->connections[i] == id_target) {
-			// Already a waypoint, remove it
+	float x, y;
+	int type;
+
+	// Save item information for the undo action
+	type = it->type;
+	x = it->pos.x;
+	y = it->pos.y;
+
+	// Remove the item on the map
+	DeleteItem(it);
+
+	// Make an undoable removal
+	action_push(ACT_CREATE_ITEM, x, y, type);
+}
+
+/**
+ * Move an item on the map and add this action in stack undo/redo
+ * \param EditLevel Pointer towards the editing level where move the item
+ * \param it The item which we want moved
+ * \param x The new x position of the item
+ * \param y The new y position of the item
+ */
+void action_move_item(level *EditLevel, item *it, float x, float y)
+{
+	float oldx, oldy;
+
+	// Save item position for the undo action
+	oldx = it->pos.x;
+	oldy = it->pos.y;
+
+	// Define the new position of the item
+	it->pos.x = x;
+	it->pos.y = y;
+
+	action_push(ACT_MOVE_ITEM, it, oldx, oldy);
+}
+
+/**
+ * Create a waypoint on the map, add this action in the undo/redo stack
+ * \param EditLevel Pointer towards the currently edited level where create the waypoint
+ * \param x The x position of the waypoint
+ * \param y The y position of the waypoint
+ * \param random_spawn TRUE if the waypoint can be used to place a random bot
+ * \return The new waypoint created
+ */
+waypoint *action_create_waypoint(level *EditLevel, int x, int y, int random_spawn)
+{
+	waypoint *wpts = EditLevel->waypoints.arr;
+	int wpnum;
+
+	wpnum = get_waypoint(EditLevel, x, y);
+	if (wpnum < 0) {
+		// When the waypoint doesn't exist on the map, create it
+		wpnum = add_waypoint(EditLevel, x, y, random_spawn);
+
+		// The waypoints array may have been moved by the add_waypoint call
+		wpts = EditLevel->waypoints.arr;
+
+		// Make an undoable action
+		action_push(ACT_REMOVE_WAYPOINT, wpts[wpnum].x, wpts[wpnum].y);
+	}
+
+	return &wpts[wpnum];
+}
+
+/**
+ * Remove a waypoint on the map and add this action in the undo/redo stack
+ * \param EditLevel Pointer towards the currently edited level where delete the waypoint
+ * \param x The x position of the waypoint
+ * \param y The y position of the waypoint
+ */
+void action_remove_waypoint(level *EditLevel, int x, int y)
+{
+	waypoint *wpts = EditLevel->waypoints.arr;
+
+	int wpnum = get_waypoint(EditLevel, x, y);
+	if (wpnum < 0) {
+		// When the waypoint doesn't exist on the map, we are done
+		return;
+	}
+
+	// Save waypoint information for the undo action
+	int old_random_spawn = wpts[wpnum].suppress_random_spawn;
+
+	// Remove the waypoint on the map
+	del_waypoint(EditLevel, x, y);
+
+	// Make an undoable action
+	action_push(ACT_CREATE_WAYPOINT, x, y, old_random_spawn);
+}
+
+/**
+ * Set/unset the flag for random bots and add this action in the undo/redo stack
+ * \param EditLevel Pointer towards the currently edited level where toggle the waypoint
+ * \param x The x position of the waypoint
+ * \param y The y position of the waypoint
+ */
+void action_toggle_waypoint_randomspawn(level *EditLevel, int x, int y)
+{
+	waypoint *wpts = EditLevel->waypoints.arr;
+
+	int wpnum = get_waypoint(EditLevel, x, y);
+	if (wpnum < 0) {
+		return;
+	}
+
+	// Toggle the flag for random bots
+	wpts[wpnum].suppress_random_spawn = !wpts[wpnum].suppress_random_spawn;
+
+	// Make an undoable action
+	action_push(ACT_TOGGLE_WAYPOINT_RSPAWN, x, y);
+}
+
+int action_toggle_waypoint_connection(level *EditLevel, int id_origin, int id_target, int removeifpresent, int undoable)
+{
+	waypoint *wpts = EditLevel->waypoints.arr;
+	int *connections;
+	int i;
+
+	// Get the connections of the waypoint;
+	connections = wpts[id_origin].connections.arr;
+
+	for (i = 0; i < wpts[id_origin].connections.size; i++) {
+		if (connections[i] == id_target) {
 			if (removeifpresent) {
-				memmove(SrcWp->connections + i, SrcWp->connections + i + 1,
-					(SrcWp->num_connections - (i + 1)) * sizeof(SrcWp->connections[0]));
-				SrcWp->num_connections--;
+				// Delete the connection of the waypoint
+				dynarray_del(&wpts[id_origin].connections, i, sizeof(int));
 			}
 			if (undoable)
-				action_push(ACT_WAYPOINT_TOGGLE_CONNECT, id_origin, id_target, -1);
+				action_push(ACT_TOGGLE_WAYPOINT_CONNECTION, id_origin, id_target);
 			return -1;
 		}
 	}
-	SrcWp->connections[SrcWp->num_connections] = id_target;
-	SrcWp->num_connections++;
-	SrcWp = NULL;
+
+	// Add the target connection of the waypoint
+	dynarray_add(&wpts[id_origin].connections, &id_target, sizeof(int));
 
 	if (undoable)
-		action_push(ACT_WAYPOINT_TOGGLE_CONNECT, id_origin, id_target, -1);
+		action_push(ACT_TOGGLE_WAYPOINT_CONNECTION, id_origin, id_target);
+
 	return 1;
 }
 
 void level_editor_action_toggle_waypoint_connection_user(level * EditLevel, int xpos, int ypos)
 {
+	waypoint *wpts;
 	int i;
 
 	// Determine which waypoint is currently targeted
-	for (i = 0; i < EditLevel->num_waypoints; i++) {
-		if ((EditLevel->AllWaypoints[i].x == xpos) && (EditLevel->AllWaypoints[i].y == ypos))
+	wpts = EditLevel->waypoints.arr;
+	for (i = 0; i < EditLevel->waypoints.size; i++) {
+		if ((wpts[i].x == xpos) && (wpts[i].y == ypos))
 			break;
 	}
 
-	if (i == EditLevel->num_waypoints) {
+	if (i == EditLevel->waypoints.size) {
 		sprintf(VanishingMessage, _("Sorry, don't know which waypoint you mean."));
 		VanishingMessageEndDate = SDL_GetTicks() + 7000;
 	} else {
@@ -440,15 +526,8 @@ void level_editor_action_toggle_waypoint_connection_user(level * EditLevel, int
 		VanishingMessageEndDate = SDL_GetTicks() + 7000;
 		if (OriginWaypoint == (-1)) {
 			OriginWaypoint = i;
-			if (EditLevel->AllWaypoints[OriginWaypoint].num_connections < MAX_WP_CONNECTIONS) {
-				strcat(VanishingMessage, _("\nIt has been marked as the origin of the next connection."));
-				DebugPrintf(1, "\nWaypoint nr. %d. selected as origin\n", i);
-			} else {
-				sprintf(VanishingMessage, _("\nsORRY, MAXIMAL NUMBER OF WAYPOINT-CONNECTIONS (%d) REACHED!\n"),
-					MAX_WP_CONNECTIONS);
-				DebugPrintf(0, "Operation not possible\n");
-				OriginWaypoint = (-1);
-			}
+			strcat(VanishingMessage, _("\nIt has been marked as the origin of the next connection."));
+			DebugPrintf(1, "\nWaypoint nr. %d. selected as origin\n", i);
 		} else {
 			if (OriginWaypoint == i) {
 				strcat(VanishingMessage, _("\n\nOrigin==Target --> Connection Operation cancelled."));
@@ -476,127 +555,97 @@ void action_set_floor(Level EditLevel, int x, int y, int type)
 	action_push(ACT_TILE_FLOOR_SET, x, y, old);
 }
 
-static int tile_is_free(level * EditLevel, int y_old, int x_old, int y_new, int x_new)
+static int action_change_map_label(level *EditLevel, int i, char *name, int x, int y, int undoable)
 {
+	struct map_label *map_label;
+	char *old_label = NULL;
+
+	// If the map label exist, remove it
+	if (i < EditLevel->map_labels.size) {
+		// Get the map label
+		map_label = &ACCESS_MAP_LABEL(EditLevel->map_labels, i);
+
+		// Get the old label for undoable actions
+		old_label = map_label->label_name;
+
+		// Delete the map label
+		del_map_label(EditLevel, old_label);
+	}
+
+	// Create the undo action if appropriate
+	if (undoable) {
+		action_push(ACT_SET_MAP_LABEL, i, old_label, x, y);
+	} else {
+		free(old_label);
+	}
+
+	// If the new label is empty, we are done
+	if (!name || !strlen(name))
+		return 0;
+
+	name = strdup(name);
+
+	// Create a new map label at the position of cursor
+	add_map_label(EditLevel, x, y, name);
+
+	return undoable;
+}
+
+void level_editor_action_change_map_label_user(level *EditLevel)
+{
+	struct map_label *map_label = NULL;
+	char *name;
+	char *old_name = NULL;
+	char suggested_label[200];
 	int i;
-	float x, y;
-	int wall_id = -1;
-	int obstacle_id;
-	for (i = 0; i < MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE; ++i) {
-		obstacle_id = EditLevel->map[y_new][x_new].obstacles_glued_to_here[i];
-		if (obstacle_id != -1 &&
-		    (EditLevel->obstacle_list[obstacle_id].type >= ISO_V_WALL ||
-		     EditLevel->obstacle_list[obstacle_id].type <= ISO_OUTER_WALL_E3)) {
-			wall_id = obstacle_id;
-			y = EditLevel->obstacle_list[obstacle_id].pos.y;
-			x = EditLevel->obstacle_list[obstacle_id].pos.x;
+
+	suggested_label[0] = 0;
+
+	// We check if a map label already exists for this spot
+	for (i = 0; i < EditLevel->map_labels.size; i++) {
+		map_label = &ACCESS_MAP_LABEL(EditLevel->map_labels, i);
+
+		if ((fabsf(map_label->pos.x + 0.5 - Me.pos.x) < 0.5) && 
+			 (fabsf(map_label->pos.y + 0.5 - Me.pos.y) < 0.5)) {
+			
+			// Use the old label as a suggestion
+			old_name = map_label->label_name;
+			strcpy(suggested_label, old_name);
 			break;
 		}
 	}
-	return TRUE;
-}
-
-static void action_fill_user_recursive(level * EditLevel, int x, int y, int type, int *changed)
-{
-	int source_type = EditLevel->map[y][x].floor_value;
-	/* security */
-	if (!pos_inside_level(x, y, EditLevel))
-		return;
-
-#define at(x,y) (EditLevel -> map [ y ] [ x ] . floor_value)
-	if (at(x, y) == type)
-		return;
-	action_set_floor(EditLevel, x, y, type);
-	(*changed)++;
-	if (x > 0 && at(x - 1, y) == source_type && tile_is_free(EditLevel, y, x, y, x - 1))
-		action_fill_user_recursive(EditLevel, x - 1, y, type, changed);
-	if (x < EditLevel->xlen - 1 && at(x + 1, y) == source_type && tile_is_free(EditLevel, y, x, y, x + 1))
-		action_fill_user_recursive(EditLevel, x + 1, y, type, changed);
-	if (y > 0 && at(x, y - 1) == source_type && tile_is_free(EditLevel, y, x, y - 1, x))
-		action_fill_user_recursive(EditLevel, x, y - 1, type, changed);
-	if (y < EditLevel->ylen - 1 && at(x - 1, y + 1) == source_type && tile_is_free(EditLevel, y + 1, x, y, x))
-		action_fill_user_recursive(EditLevel, x, y + 1, type, changed);
-}
-
-void action_fill_user(level * EditLevel, int BlockX, int BlockY, int SpecialMapValue)
-{
-	int number_changed = 0;
-	action_fill_user_recursive(EditLevel, BlockX, BlockY, SpecialMapValue, &number_changed);
-	action_push(ACT_MULTIPLE_ACTIONS, number_changed);
-}
-
-static void action_change_map_label(level * EditLevel, int i, char *name)
-{
-	if (EditLevel->labels[i].pos.x != -1) {
-		int check_double;
-		if (name) {
-			for (check_double = 0; check_double < MAX_MAP_LABELS_PER_LEVEL; check_double++) {
-				if (!strcmp(name, EditLevel->labels[check_double].label_name)) {
-					ErrorMessage(__FUNCTION__, "\
-				    The label just entered did already exist on this map!  Deleting old entry in favour of the new one!", PLEASE_INFORM, IS_WARNING_ONLY);
-					i = check_double;
-					break;
-				}
-			}
+		
+	// Check if the name entered already exists
+	while (1) {
+		// Show popup window to enter a new map label
+		name = GetEditableStringInPopupWindow(sizeof(suggested_label) - 1, _("\nPlease enter map label: \n\n"), suggested_label);
+		if (!name || (old_name && !strcmp(name, old_name))) {
+			// Do not change label
+			free(name);
+			return;
 		}
-		action_push(ACT_SET_MAP_LABEL, i, EditLevel->labels[i].label_name);
-	} else {
-		action_push(ACT_SET_MAP_LABEL, i, NULL);
-	}
-	if (name && strlen(name)) {
-		EditLevel->labels[i].label_name = name;
-		EditLevel->labels[i].pos.x = rintf(Me.pos.x - 0.5);
-		EditLevel->labels[i].pos.y = rintf(Me.pos.y - 0.5);
-	} else {
-		EditLevel->labels[i].label_name = ("NoLabelHere");
-		EditLevel->labels[i].pos.x = (-1);
-		EditLevel->labels[i].pos.y = (-1);
-	}
-}
 
-void level_editor_action_change_map_label_user(level * EditLevel)
-{
-	char *NewCommentOnThisSquare;
-	int i;
-
-	SetCurrentFont(FPS_Display_BFont);
-
-	//--------------------
-	// Now we see if a map label entry is existing already for this spot
-	//
-	for (i = 0; i < MAX_MAP_LABELS_PER_LEVEL; i++) {
-		if ((fabsf(EditLevel->labels[i].pos.x + 0.5 - Me.pos.x) < 0.5) &&
-		    (fabsf(EditLevel->labels[i].pos.y + 0.5 - Me.pos.y) < 0.5)) {
+		map_label = get_map_label(EditLevel, name);
+		if (!map_label) {
+			// When the new name of the map label does not exist, we are done
 			break;
 		}
-	}
-	if (i >= MAX_MAP_LABELS_PER_LEVEL) {
-		NewCommentOnThisSquare =
-		    GetEditableStringInPopupWindow(1000,
-						   _
-						   ("\nNo existing map label entry for this position found...\n Please enter new label for this map position: \n\n"),
-						   "");
 
-		i = 0;
-		for (i = 0; i < MAX_MAP_LABELS_PER_LEVEL; i++) {
-			if (EditLevel->labels[i].pos.x == (-1))
-				break;
-		}
-		if (i >= MAX_MAP_LABELS_PER_LEVEL) {
-			DisplayText(_("\nNo more free map label entry found... using first on instead ...\n"), -1, -1, &User_Rect, 1.0);
-			i = 0;
-		} else {
-			DisplayText(_("\nUsing new map label list entry...\n"), -1, -1, &User_Rect, 1.0);
-		}
-		// Terminate( ERR );
-	} else {
-		NewCommentOnThisSquare =
-		    GetEditableStringInPopupWindow(1000,
-						   _
-						   ("\nOverwriting existing map label list entry...\n Please enter new label for this map position: \n\n"),
-						   EditLevel->labels[i].label_name);
+		// When the new name already exists, we must not create an other map
+		// label with the same name, but we want to display an alert window,
+		// and then go back to input box with the name still present
+		alert_window(_("The new name of the map label already exists on this map, please choose an other name."));
+
+		// Copy the name in order to have it in the input box
+		strcpy(suggested_label, name);
+		free(name);
+
+		// Restore the menu background in order to correctly draw the next popup window
+		RestoreMenuBackground(1);
 	}
-	action_change_map_label(EditLevel, i, NewCommentOnThisSquare);
+
+	// Change a map label when the name enter by the user is valid
+	action_change_map_label(EditLevel, i, name, rintf(Me.pos.x - 0.5), rintf(Me.pos.y - 0.5), 1);
 }
 
 /**
@@ -606,11 +655,28 @@ void level_editor_action_change_map_label_user(level * EditLevel)
  */
 void action_jump_to_level(int target_level, double x, double y)
 {
+	// When the user wants to change the current edited level, reset tools
+	leveleditor_reset_tools();
+
 	action_push(ACT_JUMP_TO_LEVEL, EditLevel()->levelnum, Me.pos.x, Me.pos.y);	//< sets undo or redo stack, depending on push_mode state
 	reset_visible_levels();
 	Teleport(target_level, (float)x, (float)y, FALSE);
 }
 
+/**
+ * Jump to the center of a level.
+ *
+ * @param level_num The level id.
+ */
+void action_jump_to_level_center(int level_num)
+{
+	// Calculate the center of the level.
+	float x = curShip.AllLevels[level_num]->xlen / 2;
+	float y = curShip.AllLevels[level_num]->ylen / 2;
+
+	action_jump_to_level(level_num, x, y);
+}
+
 static void action_do(level * level, action * a)
 {
 	switch (a->type) {
@@ -623,11 +689,26 @@ static void action_do(level * level, action * a)
 	case ACT_MOVE_OBSTACLE:
 		action_move_obstacle(level, a->d.move_obstacle.obstacle, a->d.move_obstacle.newx, a->d.move_obstacle.newy);
 		break;
-	case ACT_WAYPOINT_TOGGLE:
-		action_toggle_waypoint(level, a->d.waypoint_toggle.x, a->d.waypoint_toggle.y, a->d.waypoint_toggle.spawn_toggle);
+	case ACT_CREATE_ITEM:
+		action_create_item(level, a->d.create_item.x, a->d.create_item.y, a->d.create_item.type);
 		break;
-	case ACT_WAYPOINT_TOGGLE_CONNECT:
-		action_toggle_waypoint_connection(level, a->d.waypoint_toggle.x, a->d.waypoint_toggle.y, 1, 1);
+	case ACT_REMOVE_ITEM:
+		action_remove_item(level, a->d.delete_item);
+		break;
+	case ACT_MOVE_ITEM:
+		action_move_item(level, a->d.move_item.item, a->d.move_item.newx, a->d.move_item.newy);
+		break;
+	case ACT_CREATE_WAYPOINT:
+		action_create_waypoint(level, a->d.create_waypoint.x, a->d.create_waypoint.y, a->d.create_waypoint.suppress_random_spawn);
+		break;
+	case ACT_REMOVE_WAYPOINT:
+		action_remove_waypoint(level, a->d.delete_waypoint.x, a->d.delete_waypoint.y);
+		break;
+	case ACT_TOGGLE_WAYPOINT_RSPAWN:
+		action_toggle_waypoint_randomspawn(level, a->d.toggle_waypoint_rspawn.x, a->d.toggle_waypoint_rspawn.y);
+		break;
+	case ACT_TOGGLE_WAYPOINT_CONNECTION:
+		action_toggle_waypoint_connection(level, a->d.toggle_waypoint_connection.x, a->d.toggle_waypoint_connection.y, 1, 1);
 		break;
 	case ACT_TILE_FLOOR_SET:
 		action_set_floor(level, a->d.change_floor.x, a->d.change_floor.y, a->d.change_floor.type);
@@ -639,7 +720,7 @@ static void action_do(level * level, action * a)
 		action_change_obstacle_label(level, a->d.change_obstacle_name.obstacle, a->d.change_obstacle_name.new_name, 1);
 		break;
 	case ACT_SET_MAP_LABEL:
-		action_change_map_label(level, a->d.change_label_name.id, a->d.change_label_name.new_name);
+		action_change_map_label(level, a->d.change_label_name.id, a->d.change_label_name.new_name, a->d.change_label_name.x, a->d.change_label_name.y, 1);
 		break;
 	case ACT_JUMP_TO_LEVEL:
 		action_jump_to_level(a->d.jump_to_level.target_level, a->d.jump_to_level.x, a->d.jump_to_level.y);
@@ -655,17 +736,24 @@ static void __level_editor_do_action_from_stack(struct list_head *stack)
 	if (list_empty(stack))
 		return;
 
+	// Get the top action from the undo/redo stack
 	a = list_entry(stack->next, action, node);
 
 	if (a->type == ACT_MULTIPLE_ACTIONS) {
+		// When the action is multiple, we must execute each action,
+		// and push all actions in the stack in order to undo
 		int i, max;
 
 		max = a->d.number_actions;
 		clear_action(a);
 
+		// Execute all actions
 		for (i = 0; i < max; i++) {
 			__level_editor_do_action_from_stack(stack);
 		}
+
+		// Push all actions in the stack in order to undo
+		action_push(ACT_MULTIPLE_ACTIONS, max);
 	} else {
 		action_do(EditLevel(), a);
 		clear_action(a);
@@ -687,40 +775,35 @@ void level_editor_action_redo()
 }
 
 /**
- *
- *
+ * Place an aligned object on the map with the keypad
+ * \param positionid The position of the object on the purple grid
  */
-void level_editor_place_aligned_obstacle(int positionid)
+void level_editor_place_aligned_object(int positionid)
 {
-	struct leveleditor_typeselect *ts = get_current_object_type();
-	int obstacle_id = -1;
-	int obstacle_created = FALSE;
 	float position_offset_x[9] = { 0, 0.5, 1.0, 0, 0.5, 1.0, 0, 0.5, 1.0 };
 	float position_offset_y[9] = { 1.0, 1.0, 1.0, 0.5, 0.5, 0.5, 0, 0, 0 };
+	struct leveleditor_categoryselect *cs = get_current_object_type();
+	int type = cs->indices[cs->selected_tile_nb];
+	moderately_finepoint pos;
 
 	positionid--;
 
-	if (ts->type != OBJECT_OBSTACLE && ts->type != OBJECT_FLOOR)
-		return;
-
-	switch (ts->type) {
-	case OBJECT_ANY:
-		GiveMouseAlertWindow("Quickbar support was removed.");
-		break;
+	// Calculate the position of the object
+	pos.x = (int)Me.pos.x + position_offset_x[positionid];
+	pos.y = (int)Me.pos.y + position_offset_y[positionid];
 
+	switch (cs->type) {
 	case OBJECT_OBSTACLE:
-		obstacle_id = ts->indices[ts->selected_tile_nb];
-		action_create_obstacle_user(EditLevel(), ((int)Me.pos.x) + position_offset_x[positionid],
-					    ((int)Me.pos.y) + position_offset_y[positionid], obstacle_id);
-		obstacle_created = TRUE;
-		break;
-
+			action_create_obstacle_user(EditLevel(), pos.x, pos.y, type);
+			break;
 	case OBJECT_FLOOR:
-		action_set_floor(EditLevel(), (int)Me.pos.x, (int)Me.pos.y, ts->indices[ts->selected_tile_nb]);
-		break;
-
+			action_set_floor(EditLevel(), (int)Me.pos.x, (int)Me.pos.y, type);
+			break;
+	case OBJECT_ITEM:
+			action_create_item(EditLevel(), pos.x, pos.y, type);
+			break;
 	default:
-		return;
+			break;
 	}
 }
 
@@ -734,7 +817,6 @@ void CreateNewMapLevel(int level_num)
 	level *NewLevel;
 	int i, k, l;
 
-	//--------------------
 	// Get the memory for one level 
 	//
 	NewLevel = (Level) MyMalloc(sizeof(level));
@@ -742,7 +824,6 @@ void CreateNewMapLevel(int level_num)
 	DebugPrintf(0, "\n-----------------------------------------------------------------");
 	DebugPrintf(0, "\nStarting to create and add a completely new level to the ship.");
 
-	//--------------------
 	// Now we proceed in the order of the struct 'level' in the
 	// struct.h file so that we can easily verify if we've handled
 	// all the data structure or left something out which could
@@ -753,16 +834,12 @@ void CreateNewMapLevel(int level_num)
 	NewLevel->ylen = 90;
 	NewLevel->light_bonus = 19;
 	NewLevel->minimum_light_value = 19;
+	NewLevel->infinite_running_on_this_level = FALSE;
+	NewLevel->random_dungeon = 0;
+	NewLevel->dungeon_generated = FALSE;
 	NewLevel->Levelname = strdup("New level just created");
 	NewLevel->Background_Song_Name = strdup("TheBeginning.ogg");
-	//--------------------
-	// Now we initialize the obstacle name list with 'empty' values
-	//
-	for (i = 0; i < MAX_OBSTACLE_NAMES_PER_LEVEL; i++) {
-		NewLevel->obstacle_name_list[i] = NULL;
-	}
-	
-	//--------------------
+
 	// First we initialize the floor with 'empty' values
 	//
 	for (i = 0; i < NewLevel->ylen; i++) {
@@ -774,7 +851,6 @@ void CreateNewMapLevel(int level_num)
 			}
 		}
 	}
-	//--------------------
 	// Now we initialize the level jump interface variables with 'empty' values
 	//
 	NewLevel->jump_target_north = (-1);
@@ -782,7 +858,6 @@ void CreateNewMapLevel(int level_num)
 	NewLevel->jump_target_east = (-1);
 	NewLevel->jump_target_west = (-1);
 
-	//--------------------
 	// Now we initialize the map obstacles with 'empty' information
 	//
 	for (i = 0; i < MAX_OBSTACLES_ON_MAP; i++) {
@@ -790,36 +865,8 @@ void CreateNewMapLevel(int level_num)
 		NewLevel->obstacle_list[i].pos.x = (-1);
 		NewLevel->obstacle_list[i].pos.y = (-1);
 		NewLevel->obstacle_list[i].pos.z = level_num;
-		NewLevel->obstacle_list[i].name_index = (-1);
-	}
-	for (i = 0; i < MAX_OBSTACLES_ON_MAP; i++) {
-		NewLevel->obstacle_list[i].type = (-1);
-		NewLevel->obstacle_list[i].pos.x = (-1);
-		NewLevel->obstacle_list[i].pos.y = (-1);
-		NewLevel->obstacle_list[i].name_index = (-1);
 	}
 
-	//--------------------
-	// Now we initialize the map labels array with 'empty' information
-	//
-	for (i = 0; i < MAX_MAP_LABELS_PER_LEVEL; i++) {
-		NewLevel->labels[i].pos.x = (-1);
-		NewLevel->labels[i].pos.y = (-1);
-		NewLevel->labels[i].label_name = "no_label_defined";
-	}
-	//--------------------
-	// Now we add empty waypoint information...
-	//
-	NewLevel->num_waypoints = 0;
-	for (i = 0; i < MAXWAYPOINTS; i++) {
-		NewLevel->AllWaypoints[i].x = 0;
-		NewLevel->AllWaypoints[i].y = 0;
-
-		for (k = 0; k < MAX_WP_CONNECTIONS; k++) {
-			NewLevel->AllWaypoints[i].connections[k] = -1;
-		}
-	}
-	//--------------------
 	// First we initialize the items arrays with 'empty' information
 	//
 	for (i = 0; i < MAX_ITEMS_PER_LEVEL; i++) {
@@ -827,20 +874,18 @@ void CreateNewMapLevel(int level_num)
 		NewLevel->ItemList[i].pos.y = (-1);
 		NewLevel->ItemList[i].pos.z = (-1);
 		NewLevel->ItemList[i].type = (-1);
-		NewLevel->ItemList[i].currently_held_in_hand = FALSE;
 
 	}
-	//--------------------
-	// Now we initialize the chest items arrays with 'empty' information
-	//
-	for (i = 0; i < MAX_CHEST_ITEMS_PER_LEVEL; i++) {
-		NewLevel->ChestItemList[i].pos.x = (-1);
-		NewLevel->ChestItemList[i].pos.y = (-1);
-		NewLevel->ChestItemList[i].pos.z = (-1);
-		NewLevel->ChestItemList[i].type = (-1);
-		NewLevel->ChestItemList[i].currently_held_in_hand = FALSE;
-	}
 
+	// Initialize obstacle extensions
+	dynarray_init(&NewLevel->obstacle_extensions, 10, sizeof(struct obstacle_extension));
+
+	// Initialize map labels
+	dynarray_init(&NewLevel->map_labels, 10, sizeof(struct map_label));
+
+	// Initialize waypoints
+	dynarray_init(&NewLevel->waypoints, 10, sizeof(struct waypoint));
+	
 	curShip.AllLevels[level_num] = NewLevel;
 
 	glue_obstacles_to_floor_tiles_for_level(level_num);
@@ -858,7 +903,7 @@ void delete_map_level(int lnum)
 
 	// Remove references to this level from others
 	for (i = 0; i < MAX_LEVELS; i++) {
-		if (curShip.AllLevels[i] == NULL)
+		if (!level_exists(i))
 			continue;
 
 		if (curShip.AllLevels[i]->jump_target_north == lnum)
@@ -876,27 +921,51 @@ void delete_map_level(int lnum)
 
 	curShip.AllLevels[lnum] = NULL;
 
-	if (lnum == curShip.num_levels)
+	if (lnum == curShip.num_levels - 1)
 		curShip.num_levels--;
 
 }
 
-void level_editor_edit_chest(obstacle * o)
+static int get_chest_contents(level *l, obstacle *o, item *items[MAX_ITEMS_IN_INVENTORY])
 {
-	item *chest_items[MAX_CHEST_ITEMS_PER_LEVEL];
-	item *user_items[MAX_CHEST_ITEMS_PER_LEVEL];
+	struct dynarray *itemlist = get_obstacle_extension(l, o, OBSTACLE_EXTENSION_CHEST_ITEMS);
+
+	memset(items, 0, MAX_ITEMS_IN_INVENTORY);
+
+	if (!itemlist) {
+		return 0;
+	}
+
+	int i;
+	int curitem = 0;
+	for (i = 0; i < itemlist->size; i++) {
+		items[curitem++] = &((item *)itemlist->arr)[i];
+		if (curitem == MAX_ITEMS_IN_INVENTORY - 1)
+			break;
+	}
+
+	return curitem;
+}
+
+void level_editor_edit_chest(obstacle *o)
+{
+	item *chest_items[MAX_ITEMS_IN_INVENTORY];
+	item *user_items[2];
 	int chest_nb_items;
 	int done = 0;
 	shop_decision shop_order;
 	item *tmp;
-	int idx;
 
-	item dummy_addtochest = {.type = 1,.suffix_code = -1,.prefix_code = -1,.is_identified = 1 };
+	item dummy_addtochest;
+	init_item(&dummy_addtochest);
+	dummy_addtochest.type = 1;
 	FillInItemProperties(&dummy_addtochest, 2, 1);
 
 	user_items[0] = &dummy_addtochest;
 	user_items[1] = NULL;
 
+	struct dynarray *itemlist = get_obstacle_extension(CURLEVEL(), o, OBSTACLE_EXTENSION_CHEST_ITEMS);
+
 	// Safety check
 	switch (o->type) {
 	case ISO_H_CHEST_CLOSED:
@@ -916,7 +985,7 @@ void level_editor_edit_chest(obstacle * o)
 	while (!done) {
 
 		// Build the list of items in the chest
-		chest_nb_items = AssemblePointerListForChestShow(&chest_items[0], o->pos);
+		chest_nb_items = get_chest_contents(CURLEVEL(), o, chest_items);
 
 		// Display the shop interface
 		done = GreatShopInterface(chest_nb_items, chest_items, 1, user_items, &shop_order);
@@ -926,24 +995,21 @@ void level_editor_edit_chest(obstacle * o)
 		switch (shop_order.shop_command) {
 		case BUY_1_ITEM:
 			DeleteItem(chest_items[shop_order.item_selected]);
+			dynarray_del(itemlist, shop_order.item_selected, sizeof(item));
 			break;
 		case SELL_1_ITEM:
 			tmp = ItemDropFromLevelEditor();
 			if (tmp) {
-				tmp->pos.x = o->pos.x;
-				tmp->pos.y = o->pos.y;
-				tmp->pos.z = o->pos.z;
 
-				for (idx = 0; idx < MAX_CHEST_ITEMS_PER_LEVEL; idx++) {
-					if (EditLevel()->ChestItemList[idx].type == -1)
-						break;
+				if (!itemlist) {
+					itemlist = dynarray_alloc(10, sizeof(item));
+					add_obstacle_extension(CURLEVEL(), o, OBSTACLE_EXTENSION_CHEST_ITEMS, itemlist);
 				}
 
-				if (idx == MAX_CHEST_ITEMS_PER_LEVEL) {
-					ErrorMessage(__FUNCTION__, "Chests on current level are full.", PLEASE_INFORM, IS_WARNING_ONLY);
-					idx = 0;
-				}
-				MoveItem(tmp, &EditLevel()->ChestItemList[idx]);
+				dynarray_add(itemlist, tmp, sizeof(item)); 
+
+				// delete the ground copy
+				DeleteItem(tmp);
 			}
 			break;
 		default:
@@ -952,4 +1018,4 @@ void level_editor_edit_chest(obstacle * o)
 	}
 }
 
-#undef _leveleditor_action_c
+#undef _leveledijtor_action_c
