@@ -1,6 +1,6 @@
 /* 
  *
- *   Copyright (c) 2004-2009 Arthur Huillet
+ *   Copyright (c) 2004-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -33,7 +33,6 @@ void action_push(int type, ...);
 /* Obstacle manipulation */
 obstacle *action_create_obstacle(level *, double, double, int);
 obstacle *action_create_obstacle_user(level *, double, double, int);
-void level_editor_place_aligned_obstacle(int);
 void action_remove_obstacle_user(level *, obstacle *);
 void action_remove_obstacle(level *, obstacle *);
 
@@ -42,23 +41,33 @@ void action_move_obstacle(level *, obstacle *, float, float);
 void action_change_obstacle_label_user(level *, obstacle *, char *);
 
 /* Waypoint manipulation */
-void action_toggle_waypoint(level *, int, int, int);
+waypoint *action_create_waypoint(level *, int, int, int);
+void action_remove_waypoint(level *, int, int);
+void action_toggle_waypoint_randomspawn(level *, int, int);
 int action_toggle_waypoint_connection(level *, int, int, int, int);
 void level_editor_action_toggle_waypoint_connection_user(level *, int, int);
 
 /* Floor tiles manipulation */
 void action_set_floor(level *, int, int, int);
-void action_fill_user(level *, int, int, int);
 
 /* Map manipulation */
 void level_editor_action_change_map_label_user(level *);
 void action_jump_to_level(int, double, double);
+void action_jump_to_level_center(int);
 void CreateNewMapLevel(int);
 void delete_map_level(int);
 
 /* Chest manipulation */
 void level_editor_edit_chest(obstacle *);
 
+// Item manipulation
+item *action_create_item(level *, float, float, int);
+void action_remove_item(level *, item *);
+void action_move_item(level *, item *, float, float);
+
+// Object manipulation
+void level_editor_place_aligned_object(int);
+
 #ifndef _leveleditor_actions_c
 #define EXTERN extern
 #else
