@@ -1,5 +1,5 @@
 /* 
- *   Copyright (c) 2009 Arthur Huillet
+ *   Copyright (c) 2009-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -28,12 +28,9 @@
 #define EXTERN
 #endif
 
-void leveleditor_display();
+void leveleditor_display(void);
 void draw_connection_between_tiles(float, float, float, float, int);
 
 float lvledit_zoomfact(void);
 float lvledit_zoomfact_inv(void);
 int lvledit_set_zoomfact(float);
-
-EXTERN char *tooltip_text;
-EXTERN int tooltip_date;
