@@ -22,4 +22,4 @@
  *
  */
 
-void level_editor_beautify_grass_tiles(level *, int, int, int, int);
+void level_editor_beautify_grass_tiles(level *);
