@@ -1,6 +1,6 @@
 /* 
  *
- *   Copyright (c) 2004-2009 Arthur Huillet
+ *   Copyright (c) 2004-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -37,22 +37,28 @@
 #include "lvledit/lvledit_actions.h"
 #include "lvledit/lvledit_map.h"
 #include "lvledit/lvledit_validator.h"
+#include "lvledit/lvledit_menu.h"
+
+#include "mapgen/mapgen.h"
 
 int EditLevelNrPopup(void)
 {
 	char *str, *cstr;
 	int tgt = 0;
 	str = GetEditableStringInPopupWindow(1000, _("\n Please enter new level number: \n\n"), "");
-	cstr = str;
-	while (*cstr) {
-		if (!isdigit(*cstr))
-			tgt = -1;
-		cstr++;
-	}
-
-	if (!tgt)
-		tgt = atoi(str);
+	if (str) {
+		cstr = str;
+		while (*cstr) {
+			if (!isdigit(*cstr))
+				tgt = -1;
+			cstr++;
+		}
 
+		if (!tgt)
+			tgt = atoi(str);
+	} else
+		tgt = -1;
+	
 	free(str);
 
 	return (tgt);
@@ -135,12 +141,11 @@ void EditLevelDimensions(void)
 		switch (MenuPosition) {
 		case INSERTREMOVE_COLUMN_VERY_EAST:
 			if (RightPressed()) {
-				InsertColumnVeryEast(EditLevel);
+				insert_column_east(EditLevel);
 				while (RightPressed()) ;
 			}
 			if (LeftPressed()) {
-				EditLevel->xlen--;	// making it smaller is always easy:  just modify the value for size
-				// allocation of new memory or things like that are not nescessary
+				remove_column_east(EditLevel);
 				while (LeftPressed()) ;
 			}
 			gps_transform_map_dirty_flag = TRUE;
@@ -148,11 +153,11 @@ void EditLevelDimensions(void)
 
 		case INSERTREMOVE_COLUMN_VERY_WEST:
 			if (RightPressed()) {
-				InsertColumnVeryWest(EditLevel);
+				insert_column_west(EditLevel);
 				while (RightPressed()) ;
 			}
 			if (LeftPressed()) {
-				RemoveColumnVeryWest(EditLevel);
+				remove_column_west(EditLevel);
 				while (LeftPressed()) ;
 			}
 			gps_transform_map_dirty_flag = TRUE;
@@ -160,13 +165,11 @@ void EditLevelDimensions(void)
 
 		case INSERTREMOVE_LINE_VERY_SOUTH:
 			if (RightPressed()) {
-				InsertLineVerySouth(EditLevel);
+				insert_line_south(EditLevel);
 				while (RightPressed()) ;
 			}
-
 			if (LeftPressed()) {
-				EditLevel->ylen--;	// making it smaller is always easy:  just modify the value for size
-				// allocation of new memory or things like that are not nescessary.
+				remove_line_south(EditLevel);
 				while (LeftPressed()) ;
 			}
 			gps_transform_map_dirty_flag = TRUE;
@@ -174,11 +177,11 @@ void EditLevelDimensions(void)
 
 		case INSERTREMOVE_LINE_VERY_NORTH:
 			if (RightPressed()) {
-				InsertLineVeryNorth(EditLevel);
+				insert_line_north(EditLevel);
 				while (RightPressed()) ;
 			}
 			if (LeftPressed()) {
-				RemoveLineVeryNorth(EditLevel);
+				remove_line_north(EditLevel);
 				while (LeftPressed()) ;
 			}
 			gps_transform_map_dirty_flag = TRUE;
@@ -362,7 +365,8 @@ static void SetLevelInterfaces(void)
 					while (LeftPressed()) ;
 				}
 				if (RightPressed()) {
-					EditLevel->jump_target_north++;
+					if ((EditLevel->jump_target_north + 1) < curShip.num_levels)
+						EditLevel->jump_target_north++;
 					while (RightPressed()) ;
 				}
 				gps_transform_map_dirty_flag = TRUE;
@@ -375,7 +379,8 @@ static void SetLevelInterfaces(void)
 					while (LeftPressed()) ;
 				}
 				if (RightPressed()) {
-					EditLevel->jump_target_south++;
+					if ((EditLevel->jump_target_south + 1) < curShip.num_levels)
+						EditLevel->jump_target_south++;
 					while (RightPressed()) ;
 				}
 				gps_transform_map_dirty_flag = TRUE;
@@ -388,7 +393,8 @@ static void SetLevelInterfaces(void)
 					while (LeftPressed()) ;
 				}
 				if (RightPressed()) {
-					EditLevel->jump_target_east++;
+					if ((EditLevel->jump_target_east + 1) < curShip.num_levels)
+						EditLevel->jump_target_east++;
 					while (RightPressed()) ;
 				}
 				gps_transform_map_dirty_flag = TRUE;
@@ -401,7 +407,8 @@ static void SetLevelInterfaces(void)
 					while (LeftPressed()) ;
 				}
 				if (RightPressed()) {
-					EditLevel->jump_target_west++;
+					if ((EditLevel->jump_target_west + 1) < curShip.num_levels)
+						EditLevel->jump_target_west++;
 					while (RightPressed()) ;
 				}
 				gps_transform_map_dirty_flag = TRUE;
@@ -461,7 +468,7 @@ static void AddRemLevel(void)
 				int i;
 				// search empty level, if any
 				for (i = 0; i < curShip.num_levels; ++i) {
-					if (curShip.AllLevels[i] == NULL) {
+					if (!level_exists(i)) {
 						new_level_num = i;
 						break;
 					}
@@ -475,8 +482,10 @@ static void AddRemLevel(void)
 				dirty_animated_obstacle_lists(new_level_num);
 				// Teleporting Tux will re-render the menu background
 				reset_visible_levels();
-				Teleport(new_level_num, 3, 3, FALSE);
+				action_jump_to_level_center(new_level_num);
 			}
+			else      //curShip.num_levels >= MAX_LEVELS
+				alert_window(_("Reached the maximum number of levels."));
 			break;
 		case REMOVE_CURRENT_LEVEL:
 			if (game_root_mode == ROOT_IS_GAME)
@@ -485,7 +494,7 @@ static void AddRemLevel(void)
 				SDL_Delay(1);
 
 			if (EditLevel()->levelnum == 0) {
-				GiveMouseAlertWindow("Cannot remove level number 0.\n");
+				alert_window("Cannot remove level number 0.\n");
 				break;
 			}
 
@@ -493,14 +502,13 @@ static void AddRemLevel(void)
 				int tmp = EditLevel()->levelnum; // needed due to Teleport changing EditLevel() result
 
 				dirty_animated_obstacle_lists(tmp);
+				delete_map_level(tmp);
 
 				gps_transform_map_dirty_flag = TRUE;
 				gps_transform_map_init();
 				// Teleporting Tux will re-render the menu background
 				reset_visible_levels();
 				Teleport(0, 3, 3, FALSE);
-
-				delete_map_level(tmp);
 			}
 			break;
 		case LEAVE_OPTIONS_MENU:
@@ -535,6 +543,7 @@ static void LevelOptions(void)
 		EDIT_LEVEL_DIMENSIONS,
 		SET_LEVEL_INTERFACE_POSITION,
 		SET_RANDOM_LEVEL,
+		SET_TELEPORT_PAIR,
 		CHANGE_LIGHT,
 		CHANGE_INDOOR_OUTDOOR,
 		SET_BACKGROUND_SONG_NAME,
@@ -591,6 +600,11 @@ static void LevelOptions(void)
 		MenuTexts[i] = Options[i];
 		i++;
 
+		sprintf(Options[i], "%s: %s", _("Teleport pair"),
+				mapgen_teleport_pair_str(EditLevel()->teleport_pair));
+		MenuTexts[i] = Options[i];
+		i++;
+
 		sprintf(Options[i], _("Light"));
 		strcat(Options[i], ":  ");
 		strcat(Options[i], _("Ambient"));
@@ -658,9 +672,9 @@ static void LevelOptions(void)
 			tgt = EditLevelNrPopup();
 
 			if (tgt >= 0 && tgt < curShip.num_levels) {
-				if (curShip.AllLevels[tgt] != NULL) {
+				if (level_exists(tgt)) {
 					reset_visible_levels();
-					Teleport(tgt, 3, 3, FALSE);
+					action_jump_to_level_center(tgt);
 				}
 				proceed_now = !proceed_now;
 			}
@@ -669,6 +683,9 @@ static void LevelOptions(void)
 			EditLevel()->random_dungeon++;
 			EditLevel()->random_dungeon %= 3;
 			break;
+		case SET_TELEPORT_PAIR:
+			EditLevel()->teleport_pair = mapgen_cycle_teleport_pair(EditLevel()->teleport_pair);
+			break;
 		case CHANGE_LIGHT:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
@@ -689,21 +706,22 @@ static void LevelOptions(void)
 					l = 0;
 			} else
 				l = 2;
-			reset_visible_levels();
-			Teleport(EditLevel()->levelnum, Me.pos.x, Me.pos.y, FALSE);
 			break;
 		case SET_LEVEL_NAME:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
-			EditLevel()->Levelname =
-			    GetEditableStringInPopupWindow(1000, _("\n Please enter new level name: \n\n"), EditLevel()->Levelname);
+			char *newLevelname = GetEditableStringInPopupWindow(1000, _("\n Please enter new level name: \n\n"), EditLevel()->Levelname);
+			if (newLevelname)
+				EditLevel()->Levelname = newLevelname;
 			break;
 		case SET_BACKGROUND_SONG_NAME:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
-			EditLevel()->Background_Song_Name =
-			    GetEditableStringInPopupWindow(1000, _("\n Please enter new music file name: \n\n"),
+			char* newBackgroundSongName =
+				GetEditableStringInPopupWindow(1000, _("\n Please enter new music file name: \n\n"),
 							   EditLevel()->Background_Song_Name);
+			if (newBackgroundSongName)
+				EditLevel()->Background_Song_Name = newBackgroundSongName;
 			break;
 		case SET_LEVEL_INTERFACE_POSITION:
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
@@ -750,24 +768,24 @@ static void LevelOptions(void)
 				if (LeftPressed()) {
 					// find first available level lower than the current one
 					int newlevel = EditLevel()->levelnum - 1;
-					while (curShip.AllLevels[newlevel] == NULL && newlevel >= 0)
+					while (!level_exists(newlevel) && newlevel >= 0)
 						--newlevel;
 					// teleport if new level exists
 					if (newlevel >= 0) {
 						reset_visible_levels();
-						Teleport(newlevel, 3, 3, FALSE);
+						action_jump_to_level_center(newlevel);
 					}
 					while (LeftPressed()) ;
 				}
 				if (RightPressed()) {
 					// find first available level higher than the current one
 					int newlevel = EditLevel()->levelnum + 1;
-					while (curShip.AllLevels[newlevel] == NULL && newlevel < curShip.num_levels)
+					while (!level_exists(newlevel) && newlevel < curShip.num_levels)
 						++newlevel;
 					// teleport if new level exists
 					if (newlevel < curShip.num_levels) {
 						reset_visible_levels();
-						Teleport(newlevel, 3, 3, FALSE);
+						action_jump_to_level_center(newlevel);
 					}
 					while (RightPressed()) ;
 				}
@@ -854,7 +872,7 @@ static void AdvancedOptions(void)
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
 				SDL_Delay(1);
 			LevelValidation();
-			while (!SpacePressed() && !EnterPressed() && !MouseLeftPressed())
+			while (!SpacePressed() && !EnterPressed() && !MouseLeftPressed() && !EscapePressed())
 				SDL_Delay(0);
 			//Hack: eat all pending events.
 			input_handle();
@@ -958,9 +976,9 @@ int DoLevelEditorMainMenu()
 			tgt = EditLevelNrPopup();
 
 			if (tgt >= 0 && tgt < curShip.num_levels) {
-				if (curShip.AllLevels[tgt] != NULL) {
+				if (level_exists(tgt)) {
 					reset_visible_levels();
-					Teleport(tgt, 3, 3, FALSE);
+					action_jump_to_level_center(tgt);
 				}
 				proceed_now = !proceed_now;
 			}
@@ -988,7 +1006,7 @@ int DoLevelEditorMainMenu()
 				SDL_Delay(1);
 			char fp[2048];
 			find_file("freedroid.levels", MAP_DIR, fp, 0);
-			SaveShip(fp, TRUE);
+			SaveShip(fp, TRUE, 0);
 			CenteredPutString(Screen, 11 * FontHeight(Menu_BFont), _("Your ship was saved..."));
 			our_SDL_flip_wrapper();
 			while (EnterPressed() || SpacePressed() || MouseLeftPressed())
@@ -1018,29 +1036,29 @@ int DoLevelEditorMainMenu()
 
 		if (LeftPressed() || RightPressed()) {
 			switch (MenuPosition) {
-
+			
 			case ENTER_LEVEL_POSITION:
 				if (LeftPressed()) {
 					// find first available level lower than the current one
 					int newlevel = EditLevel()->levelnum - 1;
-					while (curShip.AllLevels[newlevel] == NULL && newlevel >= 0)
+					while (!level_exists(newlevel) && newlevel >= 0)
 						--newlevel;
 					// teleport if new level exists
 					if (newlevel >= 0) {
 						reset_visible_levels();
-						Teleport(newlevel, 3, 3, FALSE);
+						action_jump_to_level_center(newlevel);
 					}
 					while (LeftPressed()) ;
 				}
 				if (RightPressed()) {
 					// find first available level higher than the current one
 					int newlevel = EditLevel()->levelnum + 1;
-					while (curShip.AllLevels[newlevel] == NULL && newlevel < curShip.num_levels)
+					while (!level_exists(newlevel) && newlevel < curShip.num_levels)
 						++newlevel;
 					// teleport if new level exists
 					if (newlevel < curShip.num_levels) {
 						reset_visible_levels();
-						Teleport(newlevel, 3, 3, FALSE);
+						action_jump_to_level_center(newlevel);
 					}
 					while (RightPressed()) ;
 				}
