@@ -1,7 +1,7 @@
 
 /* 
  *
- *   Copyright (c) 2009 Arthur Huillet
+ *   Copyright (c) 2009-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -41,7 +41,11 @@
 
 #include "lvledit/lvledit_tools.h"
 
-static struct leveleditor_move state;
+static struct leveleditor_move {
+	/* click&drag */
+	point origin;
+	moderately_finepoint c_corresponding_position;
+} state;
 
 /**
  * Input events get forwarded to us this way.
@@ -70,8 +74,8 @@ int leveleditor_move_input(SDL_Event * event)
 						    GetMousePos_y() - (GameConfig.screen_height / 2), GameConfig.zoom_is_on);
 
 		// Calculate the new position
-		Me.pos.x += (b.x - a.x) / 30;
-		Me.pos.y += (b.y - a.y) / 30;
+		Me.pos.x += exp((b.x - a.x) / 10.0) - 1.0;
+		Me.pos.y += exp((b.y - a.y) / 10.0) - 1.0;
 
 		if (Me.pos.x > curShip.AllLevels[Me.pos.z]->xlen)
 			Me.pos.x = curShip.AllLevels[Me.pos.z]->xlen - 1;
