@@ -1,7 +1,7 @@
 
 /* 
  *
- *   Copyright (c) 2009 Arthur Huillet
+ *   Copyright (c) 2009-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -24,16 +24,22 @@
  */
 
 int leveleditor_select_input(SDL_Event * event);
-int leveleditor_select_display();
+int leveleditor_select_display(void);
 
-int selection_empty();
+void leveleditor_select_reset(void);
+
+int selection_empty(void);
 void *single_tile_selection(int);
 int element_in_selection(void *);
 
-int level_editor_can_cycle_obs();
-void level_editor_cycle_marked_obstacle();
-void level_editor_cut_selection();
-void level_editor_copy_selection();
-void level_editor_paste_selection();
+point selection_start(void);
+point selection_len(void);
+int selection_type(void);
+
+int level_editor_can_cycle_obs(void);
+void level_editor_cycle_marked_obstacle(void);
+void level_editor_cut_selection(void);
+void level_editor_copy_selection(void);
+void level_editor_paste_selection(void);
 
 void clear_selection(int);
