@@ -51,6 +51,10 @@ static int lvlval_neighborhood_execute(struct level_validator *this, struct lvlv
 static void *lvlval_neighborhood_parse_excpt(char *string);
 static int lvlval_neighborhood_cmp_data(void *opaque_data1, void *opaque_data2);
 
+static int lvlval_obstacles_execute(struct level_validator *this, struct lvlval_ctx *validator_ctx);
+static void *lvlval_obstacles_parse_excpt(char *string);
+static int lvlval_obstacles_cmp_data(void *opaque_data1, void *opaque_data2);
+
 struct level_validator level_validators[] = {
 	{'C',
 	 LIST_HEAD_INIT(level_validators[0].excpt_list),
@@ -67,6 +71,11 @@ struct level_validator level_validators[] = {
 	 lvlval_neighborhood_execute,
 	 lvlval_neighborhood_parse_excpt,
 	 lvlval_neighborhood_cmp_data},
+	{'O',
+	 LIST_HEAD_INIT(level_validators[3].excpt_list),
+	 lvlval_obstacles_execute,
+	 lvlval_obstacles_parse_excpt,
+	 lvlval_obstacles_cmp_data},
 	{.initial = '\0'}
 };
 
@@ -384,7 +393,7 @@ static int lvlval_chest_cmp_data(void *opaque_data1, void *opaque_data2)
 	if (data1->obj_pos.z != data2->obj_pos.z)
 		return FALSE;
 
-	float dist = calc_euklid_distance(data1->obj_pos.x, data1->obj_pos.y, data2->obj_pos.x, data2->obj_pos.y);
+	float dist = calc_distance(data1->obj_pos.x, data1->obj_pos.y, data2->obj_pos.x, data2->obj_pos.y);
 	if (dist > DIST_EPSILON)
 		return FALSE;
 
@@ -448,7 +457,6 @@ static int lvlval_chest_execute(struct level_validator *this, struct lvlval_ctx
 
 struct waypoint_excpt_data {
 	char subtest;
-	int wp_id[2];
 	gps wp_pos[2];
 };
 
@@ -473,11 +481,9 @@ static void *lvlval_waypoint_parse_excpt(char *string)
 	case 'P':
 	case 'O':
 	case 'S':
-		ReadValueFromString(string, "Idx=", "%d", &(data->wp_id[0]), NULL);
 		ReadValueFromString(string, "X=", "%f", &(data->wp_pos[0].x), NULL);
 		ReadValueFromString(string, "Y=", "%f", &(data->wp_pos[0].y), NULL);
 		ReadValueFromString(string, "L=", "%d", &(data->wp_pos[0].z), NULL);
-		data->wp_id[1] = 0;
 		data->wp_pos[1].x = data->wp_pos[1].y = 0.0;
 		data->wp_pos[1].z = 0;
 		break;
@@ -485,11 +491,9 @@ static void *lvlval_waypoint_parse_excpt(char *string)
 	case 'D':
 	case 'W':
 	case 'Q':
-		ReadValueFromString(string, "Idx1=", "%d", &(data->wp_id[0]), NULL);
 		ReadValueFromString(string, "X1=", "%f", &(data->wp_pos[0].x), NULL);
 		ReadValueFromString(string, "Y1=", "%f", &(data->wp_pos[0].y), NULL);
 		ReadValueFromString(string, "L1=", "%d", &(data->wp_pos[0].z), NULL);
-		ReadValueFromString(string, "Idx2=", "%d", &(data->wp_id[1]), NULL);
 		ReadValueFromString(string, "X2=", "%f", &(data->wp_pos[1].x), NULL);
 		ReadValueFromString(string, "Y2=", "%f", &(data->wp_pos[1].y), NULL);
 		ReadValueFromString(string, "L2=", "%d", &(data->wp_pos[1].z), NULL);
@@ -520,12 +524,10 @@ static int lvlval_waypoint_cmp_data(void *opaque_data1, void *opaque_data2)
 
 	int i;
 	for (i = 0; i < 2; ++i) {
-		if (data1->wp_id[i] != data2->wp_id[i])
-			return FALSE;
 		if (data1->wp_pos[i].z != data2->wp_pos[i].z)
 			return FALSE;
 
-		float dist = calc_euklid_distance(data1->wp_pos[i].x, data1->wp_pos[i].y, data2->wp_pos[i].x, data2->wp_pos[i].y);
+		float dist = calc_distance(data1->wp_pos[i].x, data1->wp_pos[i].y, data2->wp_pos[i].x, data2->wp_pos[i].y);
 		if (dist > DIST_EPSILON)
 			return FALSE;
 	}
@@ -547,53 +549,47 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 	int dist_is_invalid = FALSE;
 	int path_is_invalid = FALSE;
 	int path_warning = FALSE;
-#	define TRSL_FACT 0.02
+	waypoint *wpts;
 #	define MIN_DIST 1.0
 
+	// Get all waypoints for the level
+	wpts = validator_ctx->this_level->waypoints.arr;
+
 	// Check waypoints position
-	for (i = 0; i < validator_ctx->this_level->num_waypoints; ++i) {
+	for (i = 0; i < validator_ctx->this_level->waypoints.size; ++i) {
 		struct waypoint_excpt_data to_check = { 'P',
-			{i, 0},
 			{
-			 {validator_ctx->this_level->AllWaypoints[i].x + 0.5,
-			  validator_ctx->this_level->AllWaypoints[i].y + 0.5,
-			  validator_ctx->this_level->levelnum},
-			 {0.0, 0.0, 0}
-			 }
+				{wpts[i].x + 0.5, wpts[i].y + 0.5, validator_ctx->this_level->levelnum}, {0.0, 0.0, 0}
+			}
 		};
 
 		if (lookup_exception(this, &to_check))
 			continue;
 
 		if (!SinglePointColldet
-		    (validator_ctx->this_level->AllWaypoints[i].x + 0.5, validator_ctx->this_level->AllWaypoints[i].y + 0.5,
-		     validator_ctx->this_level->levelnum, &WalkablePassFilter)) {
+		    (wpts[i].x + 0.5, wpts[i].y + 0.5, validator_ctx->this_level->levelnum, &WalkablePassFilter)) {
 			if (!pos_is_invalid) {	// First error : print header
 				validator_print_header(validator_ctx, "Unreachable waypoints list",
 						       "The following waypoints were found to be inside an obstacle.\n"
 						       "This could lead to some bots being stuck.");
 				pos_is_invalid = TRUE;
 			}
-			printf("[Type=\"WP\"] WP Idx=%d (X=%f:Y=%f:L=%d)\n", i, validator_ctx->this_level->AllWaypoints[i].x + 0.5,
-			       validator_ctx->this_level->AllWaypoints[i].y + 0.5, validator_ctx->this_level->levelnum);
+			printf("[Type=\"WP\"] WP X=%f:Y=%f:L=%d\n", wpts[i].x + 0.5, wpts[i].y + 0.5, 
+					validator_ctx->this_level->levelnum);
 		}
 	}
 	if (pos_is_invalid)
 		puts(line);
 
 	// Check waypoints connectivity
-	for (i = 0; i < validator_ctx->this_level->num_waypoints; ++i) {
+	for (i = 0; i < validator_ctx->this_level->waypoints.size; ++i) {
 		struct waypoint_excpt_data to_check = { 'O',
-			{i, 0},
 			{
-			 {validator_ctx->this_level->AllWaypoints[i].x + 0.5,
-			  validator_ctx->this_level->AllWaypoints[i].y + 0.5,
-			  validator_ctx->this_level->levelnum},
-			 {0.0, 0.0, 0}
-			 }
+				{wpts[i].x + 0.5, wpts[i].y + 0.5, validator_ctx->this_level->levelnum}, {0.0, 0.0, 0}
+			}
 		};
 
-		if (!lookup_exception(this, &to_check) && validator_ctx->this_level->AllWaypoints[i].num_connections == 0) {
+		if (!lookup_exception(this, &to_check) && wpts[i].connections.size == 0) {
 			if (!conn_is_invalid) {	// First error : print header
 				validator_print_header(validator_ctx, "Unconnected waypoints list",
 						       "The following waypoints were found to be without connection (Type=WO).\n"
@@ -601,17 +597,19 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 						       "This could lead to some bots being stuck on those waypoints.");
 				conn_is_invalid = TRUE;
 			}
-			printf("[Type=\"WO\"] WP Idx=%d (X=%f:Y=%f:L=%d)\n", i, validator_ctx->this_level->AllWaypoints[i].x + 0.5,
-			       validator_ctx->this_level->AllWaypoints[i].y + 0.5, validator_ctx->this_level->levelnum);
+			printf("[Type=\"WO\"] WP X=%f:Y=%f:L=%d\n", wpts[i].x + 0.5, wpts[i].y + 0.5, 
+					validator_ctx->this_level->levelnum);
 		}
 
 		to_check.subtest = 'S';
 		if (lookup_exception(this, &to_check))
 			continue;
 
-		for (j = 0; j < validator_ctx->this_level->AllWaypoints[i].num_connections; ++j) {
-			int wp = validator_ctx->this_level->AllWaypoints[i].connections[j];
-			if (wp == i) {
+		// Get the connections of the waypoint
+		int *connections = wpts[i].connections.arr;
+
+		for (j = 0; j < wpts[i].connections.size; ++j) {
+			if (connections[j] == i) {
 				if (!conn_is_invalid) {	// First error : print header
 					validator_print_header(validator_ctx, "Unconnected waypoints list",
 							       "The following waypoints were found to be without connection (Type=WO).\n"
@@ -619,8 +617,8 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 							       "This could lead to some bots being stuck on those waypoints.");
 					conn_is_invalid = TRUE;
 				}
-				printf("[Type=\"WS\"] WP Idx=%d (X=%f:Y=%f:L=%d)\n", i, validator_ctx->this_level->AllWaypoints[i].x + 0.5,
-				       validator_ctx->this_level->AllWaypoints[i].y + 0.5, validator_ctx->this_level->levelnum);
+				printf("[Type=\"WS\"] WP X=%f:Y=%f:L=%d\n", wpts[i].x + 0.5, wpts[i].y + 0.5, 
+						validator_ctx->this_level->levelnum);
 				continue;
 			}
 		}
@@ -629,31 +627,29 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 		puts(line);
 
 	// Check waypoints distance
-	for (i = 0; i < validator_ctx->this_level->num_waypoints - 1; ++i) {
-		gps wp_i =
-		    { validator_ctx->this_level->AllWaypoints[i].x + 0.5, validator_ctx->this_level->AllWaypoints[i].y + 0.5,
-		     validator_ctx->this_level->levelnum };
+	for (i = 0; i < validator_ctx->this_level->waypoints.size - 1; ++i) {
+		// Get the source waypoint
+		waypoint *from_wp = &wpts[i];
+
+		gps wp_i = { from_wp->x + 0.5, from_wp->y + 0.5, validator_ctx->this_level->levelnum };
+
+		for (j = i + 1; j < validator_ctx->this_level->waypoints.size; ++j) {
+			// Get the destination waypoint
+			waypoint *to_wp = &wpts[j];
 
-		for (j = i + 1; j < validator_ctx->this_level->num_waypoints; ++j) {
 			struct waypoint_excpt_data to_check = { 'D',
-				{i, j},
 				{
-				 {validator_ctx->this_level->AllWaypoints[i].x + 0.5,
-				  validator_ctx->this_level->AllWaypoints[i].y + 0.5,
-				  validator_ctx->this_level->levelnum},
-				 {validator_ctx->this_level->AllWaypoints[j].x + 0.5,
-				  validator_ctx->this_level->AllWaypoints[j].y + 0.5,
-				  validator_ctx->this_level->levelnum}
-				 }
+					{from_wp->x + 0.5, from_wp->y + 0.5, validator_ctx->this_level->levelnum},
+					{to_wp->x + 0.5, to_wp->y + 0.5, validator_ctx->this_level->levelnum}
+				}
 			};
 
 			if (lookup_exception(this, &to_check))
 				continue;
 
-			gps wp_j =
-			    { validator_ctx->this_level->AllWaypoints[j].x + 0.5, validator_ctx->this_level->AllWaypoints[j].y + 0.5,
-			     validator_ctx->this_level->levelnum };
-			float dist = calc_euklid_distance(wp_i.x, wp_i.y, wp_j.x, wp_j.y);
+			gps wp_j = { to_wp->x + 0.5, to_wp->y + 0.5, validator_ctx->this_level->levelnum };
+
+			float dist = calc_distance(wp_i.x, wp_i.y, wp_j.x, wp_j.y);
 
 			if (dist < MIN_DIST) {
 				if (!dist_is_invalid) {	// First error : print header
@@ -662,43 +658,41 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 					dist_is_invalid = TRUE;
 				}
 				printf
-				    ("[Type=\"WD\"] WP1 Idx1=%d (X1=%f:Y1=%f:L1=%d) <-> WP2 Idx2=%d (X2=%f:Y2=%f:L2=%d) : distance = %.3f\n",
-				     i, wp_i.x, wp_i.y, validator_ctx->this_level->levelnum, j, wp_j.x, wp_j.y,
+				    ("[Type=\"WD\"] WP1 X1=%f:Y1=%f:L1=%d <-> WP2 X2=%f:Y2=%f:L2=%d : distance = %.3f\n",
+				     wp_i.x, wp_i.y, validator_ctx->this_level->levelnum, wp_j.x, wp_j.y,
 				     validator_ctx->this_level->levelnum, dist);
 			}
 		}
 	}
 
 	// Check waypoint paths walkability
-	for (i = 0; i < validator_ctx->this_level->num_waypoints; ++i) {
-		if (validator_ctx->this_level->AllWaypoints[i].num_connections == 0)
+	for (i = 0; i < validator_ctx->this_level->waypoints.size; ++i) {
+		// Get the source waypoint
+		waypoint *from_wp = &wpts[i];
+
+		if (from_wp->connections.size == 0)
 			continue;
 
-		gps from_pos =
-		    { validator_ctx->this_level->AllWaypoints[i].x + 0.5, validator_ctx->this_level->AllWaypoints[i].y + 0.5,
-		 validator_ctx->this_level->levelnum };
+		gps from_pos = { from_wp->x + 0.5, from_wp->y + 0.5, validator_ctx->this_level->levelnum };
 
-		for (j = 0; j < validator_ctx->this_level->AllWaypoints[i].num_connections; ++j) {
-			int wp = validator_ctx->this_level->AllWaypoints[i].connections[j];
+		// Get the connections of the waypoint
+		int *connections = from_wp->connections.arr;
+
+		for (j = 0; j < from_wp->connections.size; ++j) {
+			// Get the destination waypoint
+			waypoint *to_wp = &wpts[connections[j]];
 
 			struct waypoint_excpt_data to_check = { 'W',
-				{i, wp},
 				{
-				 {validator_ctx->this_level->AllWaypoints[i].x + 0.5,
-				  validator_ctx->this_level->AllWaypoints[i].y + 0.5,
-				  validator_ctx->this_level->levelnum},
-				 {validator_ctx->this_level->AllWaypoints[wp].x + 0.5,
-				  validator_ctx->this_level->AllWaypoints[wp].y + 0.5,
-				  validator_ctx->this_level->levelnum}
-				 }
+					{from_wp->x + 0.5, from_wp->y + 0.5, validator_ctx->this_level->levelnum},
+					{to_wp->x + 0.5, to_wp->y + 0.5, validator_ctx->this_level->levelnum}
+				}
 			};
 
 			if (lookup_exception(this, &to_check))
 				continue;
 
-			gps to_pos =
-			    { validator_ctx->this_level->AllWaypoints[wp].x + 0.5, validator_ctx->this_level->AllWaypoints[wp].y + 0.5,
-			   validator_ctx->this_level->levelnum };
+			gps to_pos = { to_wp->x + 0.5, to_wp->y + 0.5, validator_ctx->this_level->levelnum };
 
 			enum connect_validity rtn = waypoints_connection_valid(&from_pos, &to_pos);
 			if (rtn & NO_PATH) {
@@ -708,9 +702,9 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 							       "This could lead those paths to not being usable.");
 					path_is_invalid = TRUE;
 				}
-				printf("[Type=\"WW\"] WP1 Idx1=%d (X1=%f:Y1=%f:L1=%d) -> WP2 Idx2=%d (X2=%f:Y2=%f:L2=%d) : %s\n",
-				       i, from_pos.x, from_pos.y, validator_ctx->this_level->levelnum,
-				       wp, to_pos.x, to_pos.y, validator_ctx->this_level->levelnum,
+				printf("[Type=\"WW\"] WP1 X1=%f:Y1=%f:L1=%d -> WP2 X2=%f:Y2=%f:L2=%d : %s\n",
+				       from_pos.x, from_pos.y, validator_ctx->this_level->levelnum,
+				       to_pos.x, to_pos.y, validator_ctx->this_level->levelnum,
 				       (rtn & COMPLEX_PATH) ? "too complex" : "path not found");
 			}
 		}
@@ -723,34 +717,32 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 	// So, even if the connection is in theory walkable, a bot could get stuck.
 	// By translating a bit the waypoints positions, we simulate such a behavior.
 
-	for (i = 0; i < validator_ctx->this_level->num_waypoints; ++i) {
-		if (validator_ctx->this_level->AllWaypoints[i].num_connections == 0)
+	for (i = 0; i < validator_ctx->this_level->waypoints.size; ++i) {
+		// Get the from waypoint
+		waypoint *from_wp = &wpts[i];
+
+		if (from_wp->connections.size == 0)
 			continue;
 
-		for (j = 0; j < validator_ctx->this_level->AllWaypoints[i].num_connections; ++j) {
-			int wp = validator_ctx->this_level->AllWaypoints[i].connections[j];
+		// Get the connections of the from waypoint
+		int *connections = from_wp->connections.arr;
+
+		for (j = 0; j < from_wp->connections.size; ++j) {
+			// Get the destination waypoint
+			waypoint *to_wp = &wpts[connections[j]];
 
 			struct waypoint_excpt_data to_check = { 'Q',
-				{i, wp},
 				{
-				 {validator_ctx->this_level->AllWaypoints[i].x + 0.5,
-				  validator_ctx->this_level->AllWaypoints[i].y + 0.5,
-				  validator_ctx->this_level->levelnum},
-				 {validator_ctx->this_level->AllWaypoints[wp].x + 0.5,
-				  validator_ctx->this_level->AllWaypoints[wp].y + 0.5,
-				  validator_ctx->this_level->levelnum}
-				 }
+					{from_wp->x + 0.5, from_wp->y + 0.5, validator_ctx->this_level->levelnum},
+					{to_wp->x + 0.5, to_wp->y + 0.5, validator_ctx->this_level->levelnum}
+				}
 			};
 
 			if (lookup_exception(this, &to_check))
 				continue;
 
-			gps from_pos =
-			    { validator_ctx->this_level->AllWaypoints[i].x + 0.5, validator_ctx->this_level->AllWaypoints[i].y + 0.5,
-			 validator_ctx->this_level->levelnum };
-			gps to_pos =
-			    { validator_ctx->this_level->AllWaypoints[wp].x + 0.5, validator_ctx->this_level->AllWaypoints[wp].y + 0.5,
-			   validator_ctx->this_level->levelnum };
+			gps from_pos = { from_wp->x + 0.5, from_wp->y + 0.5, validator_ctx->this_level->levelnum };
+			gps to_pos = { to_wp->x + 0.5, to_wp->y + 0.5, validator_ctx->this_level->levelnum };
 
 			// Translation vector
 			moderately_finepoint line_vector;
@@ -761,8 +753,8 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 			if (length < MIN_DIST)
 				continue;	// Too close waypoints. Already handled
 
-			line_vector.x = (line_vector.x * TRSL_FACT) / length;
-			line_vector.y = (line_vector.y * TRSL_FACT) / length;
+			line_vector.x = (line_vector.x * COLLDET_WALKABLE_MARGIN) / length;
+			line_vector.y = (line_vector.y * COLLDET_WALKABLE_MARGIN) / length;
 
 			// Translation normal
 			moderately_finepoint line_normal = { -line_vector.y, line_vector.x };
@@ -779,9 +771,9 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 							       "This could lead some bots to get stuck along those paths.");
 					path_warning = TRUE;
 				}
-				printf("[Type=\"WQ\"] WP1 Idx1=%d (X1=%f:Y1=%f:L1=%d) -> WP2 Idx2=%d (X2=%f:Y2=%f:L2=%d) (warning)\n",
-				       i, from_pos.x, from_pos.y, validator_ctx->this_level->levelnum,
-				       wp, to_pos.x, to_pos.y, validator_ctx->this_level->levelnum);
+				printf("[Type=\"WQ\"] WP1 X1=%f:Y1=%f:L1=%d -> WP2 X2=%f:Y2=%f:L2=%d (warning)\n",
+				       from_pos.x, from_pos.y, validator_ctx->this_level->levelnum,
+				       to_pos.x, to_pos.y, validator_ctx->this_level->levelnum);
 
 				continue;	// Next connection
 			}
@@ -799,9 +791,9 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 							       "This could lead some bots to get stuck along those paths.");
 					path_warning = TRUE;
 				}
-				printf("[Type=\"WQ\"] WP1 Idx1=%d (X1=%f:Y1=%f:L1=%d) -> WP2 Idx2=%d (X2=%f:Y2=%f:L2=%d) (warning)\n",
-				       i, from_pos.x, from_pos.y, validator_ctx->this_level->levelnum,
-				       wp, to_pos.x, to_pos.y, validator_ctx->this_level->levelnum);
+				printf("[Type=\"WQ\"] WP1 X1=%f:Y1=%f:L1=%d -> WP2 X2=%f:Y2=%f:L2=%d (warning)\n",
+				       from_pos.x, from_pos.y, validator_ctx->this_level->levelnum,
+				       to_pos.x, to_pos.y, validator_ctx->this_level->levelnum);
 
 				continue;	// Next connection
 			}
@@ -819,9 +811,9 @@ static int lvlval_waypoint_execute(struct level_validator *this, struct lvlval_c
 							       "This could lead some bots to get stuck along those paths.");
 					path_warning = TRUE;
 				}
-				printf("[Type=\"WQ\"] WP1 Idx1=%d (X1=%f:Y1=%f:L1=%d) -> WP2 Idx2=%d (X2=%f:Y2=%f:L2=%d) (warning)\n",
-				       i, from_pos.x, from_pos.y, validator_ctx->this_level->levelnum,
-				       wp, to_pos.x, to_pos.y, validator_ctx->this_level->levelnum);
+				printf("[Type=\"WQ\"] WP1 X1=%f:Y1=%f:L1=%d -> WP2 X2=%f:Y2=%f:L2=%d (warning)\n",
+				       from_pos.x, from_pos.y, validator_ctx->this_level->levelnum,
+				       to_pos.x, to_pos.y, validator_ctx->this_level->levelnum);
 			}
 		}
 	}
@@ -896,59 +888,67 @@ static int lvlval_neighborhood_execute(struct level_validator *this, struct lvlv
 	 * 1) check for existence of the defined neighbor
 	 */
 
-	if (validator_ctx->this_level->jump_target_north != -1 && curShip.AllLevels[validator_ctx->this_level->jump_target_north] == NULL) {
-		struct neighborhood_excpt_data to_check =
-		    { 'N', validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_north };
+	if (validator_ctx->this_level->jump_target_north != -1) {
+		if (!level_exists(validator_ctx->this_level->jump_target_north)) {
+			struct neighborhood_excpt_data to_check =
+		    		{ 'N', validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_north };
 
-		if (!lookup_exception(this, &to_check)) {
-			validator_print_header(validator_ctx, "Non existent neighbor",
-					       "The north jump target on a level points to a non existing level.");
-			is_invalid = TRUE;
+			if (!lookup_exception(this, &to_check)) {
+				validator_print_header(validator_ctx, "Non existent neighbor",
+					"The north jump target on a level points to a non existing level.");
+				is_invalid = TRUE;
 
-			printf("[Type=\"J\"] Interface:North of Level:%d points to Level:%d which does not exist.\n",
-			       validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_north);
+				printf("[Type=\"J\"] Interface:North of Level:%d points to Level:%d which does not exist.\n",
+			       		validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_north);
+			}
 		}
 	}
 
-	if (validator_ctx->this_level->jump_target_west != -1 && curShip.AllLevels[validator_ctx->this_level->jump_target_west] == NULL) {
-		struct neighborhood_excpt_data to_check =
-		    { 'W', validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_west };
+	if (validator_ctx->this_level->jump_target_west != -1) {
+		if (!level_exists(validator_ctx->this_level->jump_target_west)) {
+			struct neighborhood_excpt_data to_check =
+		    		{ 'W', validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_west };
 
-		if (!lookup_exception(this, &to_check)) {
-			validator_print_header(validator_ctx, "Non existent neighbor",
-					       "The west jump target on a level points to a non existing level.");
-			is_invalid = TRUE;
+			if (!lookup_exception(this, &to_check)) {
+				validator_print_header(validator_ctx, "Non existent neighbor",
+					"The west jump target on a level points to a non existing level.");
+				is_invalid = TRUE;
 
-			printf("[Type=\"J\"] Interface:West of Level:%d points to Level:%d which does not exist.\n",
-			       validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_west);
+				printf("[Type=\"J\"] Interface:West of Level:%d points to Level:%d which does not exist.\n",
+				       validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_west);
+			}
 		}
 	}
 
-	if (validator_ctx->this_level->jump_target_east != -1 && curShip.AllLevels[validator_ctx->this_level->jump_target_east] == NULL) {
-		struct neighborhood_excpt_data to_check =
-		    { 'E', validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_east };
+	if (validator_ctx->this_level->jump_target_east != -1) {
+		if (!level_exists(validator_ctx->this_level->jump_target_east)) {
+			struct neighborhood_excpt_data to_check =
+		    		{ 'E', validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_east };
 
-		if (!lookup_exception(this, &to_check)) {
-			validator_print_header(validator_ctx, "Non existent neighbor",
-					       "The east jump target on a level points to a non existing level.");
-			is_invalid = TRUE;
+			if (!lookup_exception(this, &to_check)) {
+				validator_print_header(validator_ctx, "Non existent neighbor",
+					"The east jump target on a level points to a non existing level.");
+				is_invalid = TRUE;
 
-			printf("[Type=\"J\"] Interface:East of Level:%d points to Level:%d which does not exist.\n",
-			       validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_east);
+				printf("[Type=\"J\"] Interface:East of Level:%d points to Level:%d which does not exist.\n",
+			       		validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_east);
+			}
 		}
 	}
 
-	if (validator_ctx->this_level->jump_target_south != -1 && curShip.AllLevels[validator_ctx->this_level->jump_target_south] == NULL) {
-		struct neighborhood_excpt_data to_check =
-		    { 'S', validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_south };
+	if (validator_ctx->this_level->jump_target_south != -1) {
+		if (!level_exists(validator_ctx->this_level->jump_target_south)) {
+			struct neighborhood_excpt_data to_check =
+		    		{ 'S', validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_south };
 
-		if (!lookup_exception(this, &to_check)) {
-			validator_print_header(validator_ctx, "Non existent neighbor",
-					       "The south jump target on a level points to a non existing level.");
-			is_invalid = TRUE;
+			if (!lookup_exception(this, &to_check)) {
+				validator_print_header(validator_ctx, "Non existent neighbor",
+					"The south jump target on a level points to a non existing level.");
+				is_invalid = TRUE;
 
-			printf("[Type=\"J\"] Interface:South of Level:%d points to Level:%d which does not exist.\n",
-			       validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_south);
+				printf("[Type=\"J\"] Interface:South of Level:%d points to Level:%d which does not exist.\n",
+			       		validator_ctx->this_level->levelnum, validator_ctx->this_level->jump_target_south);
+			}
 		}
 	}
 
@@ -1051,6 +1051,63 @@ static int lvlval_neighborhood_execute(struct level_validator *this, struct lvlv
 	return is_invalid;
 }
 
+//===========================================================
+// Obstacles Validator
+//
+// This validator checks if obstacles positions are valid
+//===========================================================
+
+/*
+ * No exceptions for obstacles: dummy functions
+ */
+
+static void *lvlval_obstacles_parse_excpt(char *string)
+{
+	return NULL;
+}
+
+static int lvlval_obstacles_cmp_data(void *opaque_data1, void *opaque_data2)
+{
+	return FALSE;
+}
+
+/*
+ * 'obstacles' validator
+ */
+
+static int lvlval_obstacles_execute(struct level_validator *this, struct lvlval_ctx *validator_ctx)
+{
+	int is_invalid = FALSE;
+	int i;
+	level *l = validator_ctx->this_level;
+
+	float max_x = (float)l->xlen;
+	float max_y = (float)l->ylen;
+
+	for (i = 0; i < MAX_OBSTACLES_ON_MAP; i++) {
+		if (l->obstacle_list[i].type == -1)
+			break;
+
+		if (l->obstacle_list[i].pos.x < 0 || l->obstacle_list[i].pos.x > max_x) {
+			validator_print_header(validator_ctx, "Invalid obstacle X position",
+					"The X position of an obstacle is invalid.");
+			is_invalid = TRUE;
+			printf("[ERROR] Obstacle n.%d on level %d, type %d, has position %f %f. Allowed X position ranges from 0 to %f.\n",
+					i, l->levelnum, l->obstacle_list[i].type, l->obstacle_list[i].pos.x, l->obstacle_list[i].pos.y, max_x);
+		}
+
+		if (l->obstacle_list[i].pos.y < 0 || l->obstacle_list[i].pos.y > max_y) {
+			validator_print_header(validator_ctx, "Invalid obstacle Y position",
+					"The Y position of an obstacle is invalid.");
+			is_invalid = TRUE;
+			printf("[ERROR] Obstacle n.%d on level %d, type %d, has position %f %f. Allowed Y position ranges from 0 to %f.\n",
+					i, l->levelnum, l->obstacle_list[i].type, l->obstacle_list[i].pos.x, l->obstacle_list[i].pos.y, max_y);
+		}
+	}
+
+	return is_invalid;
+}
+
 //===========================================================
 // ENTRY POINT
 //
@@ -1066,8 +1123,8 @@ void LevelValidation()
 	SDL_Rect report_rect = { UNIVERSAL_COORD_W(30), UNIVERSAL_COORD_H(30), UNIVERSAL_COORD_W(600), UNIVERSAL_COORD_H(430) };
 
 	BFont_Info *current_font = GetCurrentFont();
-	int raw_height = FontHeight(current_font);
-	int max_raws = (report_rect.h / raw_height) - 4;	// 4 lines are reserved for header and footer 
+	int row_height = FontHeight(current_font);
+	int max_rows = (report_rect.h / row_height) - 4;	// 4 lines are reserved for header and footer
 	int column_width = TextWidth("000: empty");
 
 	AssembleCombatPicture(ONLY_SHOW_MAP | NO_CURSOR | SKIP_LIGHT_RADIUS); //ONLY_SHOW_MAP_AND_TEXT | SHOW_GRID | SKIP_LIGHT_RADIUS);
@@ -1075,38 +1132,35 @@ void LevelValidation()
 
 	ShadowingRectangle(Screen, background_rect);
 
-	//--------------------
 	// Title
 	//
 	CenteredPutString(Screen, report_rect.y, "Level Validation tests - Summary\n");
 
-	//--------------------
 	// Load exceptions rules
 	//
 	load_excpt_lists("freedroid.lvleditor_exceptions");
 
-	//--------------------
 	// Loop on each level
 	//
 	int l;
 	int col_pos = 0;
-	int raw_pos = 0;
+	int row_pos = 0;
 
 	for (l = 0; l < curShip.num_levels; ++l) {
 		struct lvlval_ctx validator_ctx = { &report_rect, curShip.AllLevels[l] };
 
 		// Compute raw and column position, when a new column of text starts
-		if ((l % max_raws) == 0) {
-			col_pos = report_rect.x + (l / max_raws) * column_width;
-			raw_pos = report_rect.y + 2 * raw_height;	// 2 lines are reserved for the header
-			SetTextCursor(col_pos, raw_pos);
+		if ((l % max_rows) == 0) {
+			col_pos = report_rect.x + (l / max_rows) * column_width;
+			row_pos = report_rect.y + 2 * row_height;	// 2 lines are reserved for the header
 		}
 
-		if (curShip.AllLevels[l] == NULL) {
+		if (!level_exists(l)) {
 			// Empty level
 			char txt[40];
-			sprintf(txt, "%03d: \2empty\n", l);
-			DisplayText(txt, col_pos, -1, &report_rect, 1.0);
+ 			sprintf(txt, "%03d: \2empty", l);
+ 			int lines = DisplayText(txt, col_pos, row_pos, &report_rect, 1.0);
+ 			row_pos += lines * row_height;
 			SetCurrentFont(current_font);	// Reset font
 		} else {
 			// Loop on each validation function
@@ -1119,8 +1173,9 @@ void LevelValidation()
 
 			// Display report
 			char txt[40];
-			sprintf(txt, "%03d: %s\n", l, (level_is_invalid) ? "\1fail" : "pass");
-			DisplayText(txt, col_pos, -1, &report_rect, 1.0);
+			sprintf(txt, "%03d: %s", l, (level_is_invalid) ? "\1fail" : "pass");
+			int lines = DisplayText(txt, col_pos, row_pos, &report_rect, 1.0);
+			row_pos += lines * row_height;
 			SetCurrentFont(current_font);	// Reset font in case of the red "fail" was displayed
 
 			// Set global is_invalid flag
@@ -1128,35 +1183,30 @@ void LevelValidation()
 		}
 	}
 
-	//--------------------
 	// Outputs uncaught exception rules
 	//
 
 	uncaught_excpt = print_uncaught_exceptions();
 
-	//--------------------
 
 	free_exception_lists();
 
-	//--------------------
 	// This was it.  We can say so and return.
-	//
-	int posy = report_rect.y + report_rect.h - raw_height;
+	int posy = report_rect.y + report_rect.h - row_height;
 
 	CenteredPutString(Screen, posy, "--- End of List --- Press Space to return to leveleditor ---");
 
 	if (is_invalid) {
-		posy -= raw_height;
+		posy -= row_height;
 		CenteredPutString(Screen, posy, "\1Some tests were invalid. See the report in the console\3");
 	}
 
 	if (uncaught_excpt) {
-		posy -= raw_height;
+		posy -= row_height;
 		CenteredPutString(Screen, posy, "\1Some exceptions were not caught. See the report in the console\3");
 	}
 
 	our_SDL_flip_wrapper();
-
-}				// LevelValidation( )
+}
 
 #undef _leveleditor_validator_c
