@@ -1,6 +1,6 @@
 /* 
  *
- *   Copyright (c) 2009 Arthur Huillet
+ *   Copyright (c) 2009-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -43,12 +43,8 @@
 static void activate_button(struct leveleditor_button *b)
 {
 	int idx = b->btn_index;
-	int new_x, new_y;
-	struct leveleditor_typeselect *ts;
+	
 	switch (idx) {
-	case LEVEL_EDITOR_UNDERGROUND_LIGHT_ON_BUTTON:
-		EditLevel()->use_underground_lighting = !EditLevel()->use_underground_lighting;
-		break;
 	case LEVEL_EDITOR_UNDO_BUTTON:
 		level_editor_action_undo();
 		break;
@@ -59,22 +55,14 @@ static void activate_button(struct leveleditor_button *b)
 		if (game_root_mode == ROOT_IS_LVLEDIT) {	/*don't allow saving if root mode is GAME */
 			char fp[2048];
 			find_file("freedroid.levels", MAP_DIR, fp, 0);
-			SaveShip(fp, TRUE);
+			SaveShip(fp, TRUE, 0);
 
-			GiveMouseAlertWindow(_
-					     ("\nM E S S A G E\n\nYour ship was saved to file 'freedroid.levels' in the map directory.\n\nIf you have set up something cool and you wish to contribute it to FreedroidRPG, please contact the FreedroidRPG dev team."));
+			alert_window(_("M E S S A G E\n\nYour ship was saved to file 'freedroid.levels' in the map directory.\n\nIf you have set up something cool and you wish to contribute it to FreedroidRPG, please contact the FreedroidRPG dev team."));
 		} else
-			GiveMouseAlertWindow(_
-					     ("\nM E S S A G E\n\nE R R O R ! Your ship was not saved.\n\nPlaying on a map leaves the world in an unclean state not suitable for saving. Enter the editor from the main menu to be able to save."));
-		break;
-	case LEVEL_EDITOR_TOGGLE_WAYPOINT_BUTTON:
-		action_toggle_waypoint(EditLevel(), EditX(), EditY(), FALSE);
-		break;
-	case LEVEL_EDITOR_TOGGLE_CONNECTION_BLUE_BUTTON:
-		level_editor_action_toggle_waypoint_connection_user(EditLevel(), EditX(), EditY());
+			alert_window(_("M E S S A G E\n\nE R R O R ! Your ship was not saved.\n\nPlaying on a map leaves the world in an unclean state not suitable for saving. Enter the editor from the main menu to be able to save."));
 		break;
 	case LEVEL_EDITOR_BEAUTIFY_GRASS_BUTTON:
-		level_editor_beautify_grass_tiles(EditLevel(), 1, 1, EditLevel()->xlen - 1, EditLevel()->ylen - 1);
+		level_editor_beautify_grass_tiles(EditLevel());
 		break;
 	case LEVEL_EDITOR_DELETE_OBSTACLE_BUTTON:
 		level_editor_cut_selection();
@@ -82,12 +70,6 @@ static void activate_button(struct leveleditor_button *b)
 	case LEVEL_EDITOR_NEXT_OBSTACLE_BUTTON:
 		level_editor_cycle_marked_obstacle();
 		break;
-	case LEVEL_EDITOR_RECURSIVE_FILL_BUTTON:
-		ts = get_current_object_type();
-		if (ts->type == OBJECT_FLOOR) {
-			action_fill_user(EditLevel(), EditX(), EditY(), ts->indices[ts->selected_tile_nb]);
-		}
-		break;
 	case LEVEL_EDITOR_NEW_OBSTACLE_LABEL_BUTTON:
 		if (single_tile_selection(OBJECT_OBSTACLE)) {
 			action_change_obstacle_label_user(EditLevel(), single_tile_selection(OBJECT_OBSTACLE), NULL);
@@ -99,15 +81,9 @@ static void activate_button(struct leveleditor_button *b)
 	case LEVEL_EDITOR_EDIT_CHEST_BUTTON:
 		level_editor_edit_chest(single_tile_selection(OBJECT_OBSTACLE));
 		break;
-	case LEVEL_EDITOR_NEW_ITEM_BUTTON:
-		ItemDropFromLevelEditor();
-		break;
 	case LEVEL_EDITOR_ESC_BUTTON:
 		level_editor_done = DoLevelEditorMainMenu();
 		break;
-	case LEVEL_EDITOR_LEVEL_RESIZE_BUTTON:
-		EditLevelDimensions();
-		break;
 	case LEVEL_EDITOR_TOGGLE_ENEMIES_BUTTON:
 		GameConfig.omit_enemies_in_level_editor = !GameConfig.omit_enemies_in_level_editor;
 		break;
@@ -131,47 +107,6 @@ static void activate_button(struct leveleditor_button *b)
 		if (game_root_mode == ROOT_IS_GAME)
 			level_editor_done = TRUE;
 
-		break;
-	case GO_LEVEL_NORTH_BUTTON:
-		if (EditLevel()->jump_target_north < 0)
-			break;
-		if (Me.pos.x < curShip.AllLevels[EditLevel()->jump_target_north]->xlen - 1)
-			new_x = Me.pos.x;
-		else
-			new_x = 3;
-		new_y = curShip.AllLevels[EditLevel()->jump_target_north]->ylen - 4;
-		action_jump_to_level(EditLevel()->jump_target_north, new_x, new_y);
-
-		break;
-	case GO_LEVEL_SOUTH_BUTTON:
-		if (EditLevel()->jump_target_south < 0)
-			break;
-		if (Me.pos.x < curShip.AllLevels[EditLevel()->jump_target_south]->xlen - 1)
-			new_x = Me.pos.x;
-		else
-			new_x = 3;
-		new_y = 4;
-		action_jump_to_level(EditLevel()->jump_target_south, new_x, new_y);
-		break;
-	case GO_LEVEL_EAST_BUTTON:
-		if (EditLevel()->jump_target_east < 0)
-			break;
-		new_x = 3;
-		if (Me.pos.y < curShip.AllLevels[EditLevel()->jump_target_east]->ylen - 1)
-			new_y = Me.pos.y;
-		else
-			new_y = 4;
-		action_jump_to_level(EditLevel()->jump_target_east, new_x, new_y);
-		break;
-	case GO_LEVEL_WEST_BUTTON:
-		if (EditLevel()->jump_target_west < 0)
-			break;
-		new_x = curShip.AllLevels[EditLevel()->jump_target_west]->xlen - 4;
-		if (Me.pos.y < curShip.AllLevels[EditLevel()->jump_target_west]->ylen - 1)
-			new_y = Me.pos.y;
-		else
-			new_y = 4;
-		action_jump_to_level(EditLevel()->jump_target_west, new_x, new_y);
 		break;
 	case RIGHT_LEVEL_EDITOR_BUTTON:
 		leveleditor_toolbar_scroll_right();
@@ -179,6 +114,18 @@ static void activate_button(struct leveleditor_button *b)
 	case LEFT_LEVEL_EDITOR_BUTTON:
 		leveleditor_toolbar_scroll_left();
 		break;
+	case LEVEL_EDITOR_TYPESELECT_OBSTACLE_BUTTON:
+		leveleditor_select_type(OBJECT_OBSTACLE);
+		break;
+	case LEVEL_EDITOR_TYPESELECT_FLOOR_BUTTON:
+		leveleditor_select_type(OBJECT_FLOOR);
+		break;
+	case LEVEL_EDITOR_TYPESELECT_ITEM_BUTTON:
+		leveleditor_select_type(OBJECT_ITEM);
+		break;
+	case LEVEL_EDITOR_TYPESELECT_WAYPOINT_BUTTON:
+		leveleditor_select_type(OBJECT_WAYPOINT);
+		break;
 	default:
 		ErrorMessage(__FUNCTION__, "Button type %d unhandled.", PLEASE_INFORM, IS_WARNING_ONLY, idx);
 	}
@@ -281,4 +228,18 @@ void leveleditor_button_display(struct leveleditor_widget *vb)
 
 	ShowGenericButtonFromList(b->btn_index + pushoffset + activeoffset);
 
+	if (b->text) {
+		SDL_Rect btn_rect = AllMousePressButtons[b->btn_index].button_rect;
+
+		// Calculate the position of the text, we want to display the text at the
+		// center of the button
+		SDL_Rect rect;
+		rect.x = btn_rect.x + (btn_rect.w - TextWidth(b->text)) / 2;
+		rect.y = btn_rect.y + 8;
+		rect.w = btn_rect.w;
+		rect.h = btn_rect.h;
+
+		// Draw the text on the button
+		DisplayText(b->text, rect.x, rect.y, &rect, TEXT_STRETCH);
+	}
 }
