@@ -43,19 +43,6 @@ static struct leveleditor_tool *active_tool = NULL;
 moderately_finepoint mouse_mapcoord;
 int mouse_in_level; // is the mouse cursor at an existing position on the level?
 
-static struct leveleditor_widget *tool_selection_menu;
-
-void select_tool(void *t)
-{
-	if (!active_tool)
-		selected_tool = t;
-	else
-		GiveMouseAlertWindow("Cannot select another tool: busy\n");
-
-	if (tool_selection_menu)
-		tool_selection_menu->enabled = 0;
-}
-
 void leveleditor_map_init()
 {
 	leveleditor_init_tools();
@@ -91,7 +78,7 @@ void leveleditor_map_mouserelease(SDL_Event * event, struct leveleditor_widget *
 void leveleditor_map_mousepress(SDL_Event * event, struct leveleditor_widget *vm)
 {
 	(void)vm;
-	if (!active_tool) {
+	if (!active_tool && mouse_in_level) {
 		active_tool = selected_tool;
 
 		/* disable temporary-switching by CTRL because it conflicts with modifiers used in select tool
@@ -158,41 +145,6 @@ int leveleditor_map_keybevent(SDL_Event * event, struct leveleditor_widget *vm)
 		// No active tool? Cycle the currently selected one.
 		//
 		selected_tool = (selected_tool == &tool_place ? &tool_select : &tool_place);
-
-		/* Disable the annoying menu.
-		   // No active tool? Spawn a menu
-
-		   if (!tool_selection_menu) { 
-		   //We do not have a menu
-		   tool_selection_menu = create_menu();
-		   list_add(&tool_selection_menu->node, &leveleditor_widget_list);
-		   } 
-
-		   if (!tool_selection_menu->enabled) {
-
-		   struct leveleditor_menu *m = tool_selection_menu->ext;
-		   int i;
-		   SDL_Rect r = { .x = GetMousePos_x(), .y = GetMousePos_y(), .w = 100, .h = 150 };
-		   tool_selection_menu->rect = r;
-		   sprintf(m->text[0], "Cancel");
-		   sprintf(m->text[1], "Place");
-		   sprintf(m->text[2], "Move");
-		   sprintf(m->text[3], "Select");
-		   for (i = 4; i < 10; i++)
-		   m->text[i][0] = 0;
-		   m->values[0] = selected_tool;
-		   m->values[1] = &tool_place;
-		   m->values[2] = &tool_move;
-		   m->values[3] = &tool_select;
-
-		   m->done_cb = select_tool;
-
-		   tool_selection_menu->enabled = 1;
-		   } else {
-		   //Disable the existing menu
-		   tool_selection_menu->enabled = 0;
-		   }
-		 */
 	}
 	// Forward the key to the active tool
 	forward_event(event);
@@ -203,16 +155,20 @@ int leveleditor_map_keybevent(SDL_Event * event, struct leveleditor_widget *vm)
 void leveleditor_map_display_cursor()
 {
 	int oldmode = global_ingame_mode;
+	static int dragging = FALSE;
 
 	if (active_tool)
 		active_tool->display();
 
 	if (selected_tool == &tool_select) {
-		if (ShiftPressed() && !selection_empty()) {
+		if ((ShiftPressed() || dragging == TRUE) && MouseLeftPressed()
+		    && !selection_empty()) {
 			//dragdrop
 			global_ingame_mode = GLOBAL_INGAME_MODE_DRAGDROP_TOOL;
+			dragging = TRUE;
 		} else {
 			global_ingame_mode = GLOBAL_INGAME_MODE_SELECT_TOOL;
+			dragging = FALSE;
 		}
 	}
 
@@ -230,3 +186,11 @@ void leveleditor_update_tool()
 {				//time-based updating if relevant
 	forward_event(NULL);
 }
+
+void leveleditor_reset_tools()
+{
+	leveleditor_place_reset();
+	leveleditor_select_reset();
+
+	active_tool = NULL;
+}
