@@ -41,10 +41,12 @@ void leveleditor_map_mousemove(SDL_Event *, struct leveleditor_widget *);
 int leveleditor_map_keybevent(SDL_Event *, struct leveleditor_widget *);
 
 void leveleditor_map_display(struct leveleditor_widget *);
-void leveleditor_map_display_cursor();
+void leveleditor_map_display_cursor(void);
 
 void leveleditor_map_init(void);
 
 void leveleditor_update_tool(void);
+void leveleditor_reset_tools(void);
+
 EXTERN moderately_finepoint mouse_mapcoord;
 EXTERN int mouse_in_level;
