@@ -1,6 +1,6 @@
 /* 
  *
- *   Copyright (c) 2009 Arthur Huillet
+ *   Copyright (c) 2009-2010 Arthur Huillet
  *
  *
  *  This file is part of Freedroid
@@ -33,8 +33,8 @@ enum leveleditor_widget_type {
 	WIDGET_BUTTON,
 	WIDGET_TOOLBAR,
 	WIDGET_MAP,
-	WIDGET_OBJECTTYPESELECTORBUTTON,	//hahaha what a name! behold!
-	WIDGET_MENU,
+	WIDGET_CATEGORY_SELECTOR,
+	WIDGET_MINIMAP,
 };
 
 /* A widget in the level editor */
@@ -60,6 +60,8 @@ struct leveleditor_button {
 	int btn_index;		//index in AllMousePressButtons array
 	int pressed;
 	int active;
+	char *text;
+	char *tooltip;
 };
 
 struct leveleditor_toolbar {
@@ -68,7 +70,7 @@ struct leveleditor_toolbar {
 struct leveleditor_mapwidget {
 };
 
-struct leveleditor_typeselect {
+struct leveleditor_categoryselect {
 	unsigned int selected_tile_nb;
 	unsigned int toolbar_first_block;
 	char *title;
@@ -76,21 +78,18 @@ struct leveleditor_typeselect {
 	int *indices;
 };
 
-struct leveleditor_menu {
-	char text[10][100];
-	void *values[10];
-	void (*done_cb) (void *);
-	int currently_selected_idx;
-	int ispressed;
+#define WIDGET_MINIMAP_WIDTH 210
+#define WIDGET_MINIMAP_HEIGHT 140
+
+struct leveleditor_minimap {
 };
 
 void leveleditor_init_widgets(void);
 void leveleditor_display_widgets(void);
 void leveleditor_update_button_states(void);
+void leveleditor_select_type(enum leveleditor_object_type);
 
 struct leveleditor_widget *get_active_widget(int, int);
-struct leveleditor_widget *create_button(int);
-struct leveleditor_widget *create_menu();
 EXTERN struct list_head leveleditor_widget_list;
 EXTERN struct leveleditor_widget *previously_active_widget;
 
@@ -98,6 +97,6 @@ EXTERN struct leveleditor_widget *previously_active_widget;
 
 #include "lvledit/lvledit_widget_button.h"
 #include "lvledit/lvledit_widget_map.h"
-#include "lvledit/lvledit_widget_menu.h"
 #include "lvledit/lvledit_widget_toolbar.h"
-#include "lvledit/lvledit_widget_typeselect.h"
+#include "lvledit/lvledit_widget_categoryselect.h"
+#include "lvledit/lvledit_widget_minimap.h"
