@@ -103,7 +103,7 @@ void Game()
 
 		UpdateAllCharacterStats();
 
-		MoveEnemys();	// move all the enemys:
+		move_enemies();
 
 		check_tux_enemy_collision();
 
@@ -111,9 +111,7 @@ void Game()
 
 		CheckIfMissionIsComplete();
 
-		if (GameConfig.limit_framerate && Frame_Time() < 1/30.0) {
-			SDL_Delay(10);
-		}
+		limit_fps();
 
 		ComputeFPSForThisFrame();
 
@@ -133,7 +131,6 @@ int main(int argc, char *argv[])
 		fprintf(stderr, "Couldn't change working directory to %s.\n", dirname(argv[0]));
 #endif
 
-	//--------------------
 	// First we issue some message, that should appear in the debug log for
 	// windows users.
 	//
@@ -166,6 +163,7 @@ better than nothing.  Thanks anyway for your interest in FreedroidRPG.\n\
 	 */
 	sound_on = TRUE;	// default value, can be overridden by command-line 
 	use_open_gl = TRUE;	// default value, can be overridden by command-line 
+	start_editor = FALSE;	// default value, can be overridden by command-line 
 	debug_level = -1;	// -1: shut up all debug ... 0=no debug 1=first debug level (at the moment=all) 
 	GameConfig.fullscreen_on = TRUE;	// use X11-window or full screen 
 
@@ -178,8 +176,15 @@ better than nothing.  Thanks anyway for your interest in FreedroidRPG.\n\
 	}
 
 	while (!QuitProgram) {
-		StartupMenu();
-		input_handle();
+		if (start_editor) {
+			// When the user wants to start editor from the command line, skip the
+			// startup menu and load the level editor directly
+			prepare_level_editor();
+			start_editor = FALSE;
+		} else {
+			StartupMenu();
+			input_handle();
+		}
 		switch (game_root_mode) {
 		case ROOT_IS_GAME:
 			Game();
@@ -288,7 +293,6 @@ void UpdateCountersForThisFrame()
 	if (SkipAFewFrames)
 		SkipAFewFrames--;
 
-	//--------------------
 	// This is the timeout, that the tux should not start a movement
 	// some fraction of a second after an item drop.
 	//
@@ -306,7 +310,6 @@ void UpdateCountersForThisFrame()
 		}
 	}
 
-	//--------------------
 	// Maybe some items are just thrown in the air and still in the air.
 	// We need to keep track of the time the item has spent in the air so far.
 	//
@@ -324,18 +327,16 @@ void UpdateCountersForThisFrame()
 		}
 	}
 	
-	//--------------------
 	// Some bots might be frozen and some might be poisoned, some
 	// might still have a 'firewait' or a normal wait or a paralysation.
 	// In any case those counteres must be updated, but we'll only to 
 	// that for the Tux current level (at present).
 	//
 	for (level_num = 0; level_num < curShip.num_levels; level_num++) {
-		if ((curShip.AllLevels[level_num] != NULL) && level_is_visible(level_num))
+		if (level_exists(level_num) && level_is_visible(level_num))
 			update_timeouts_for_bots_on_level(level_num, latest_frame_time);
 	}
 
-	//--------------------
 	// Now we do all the things, that need to be updated for each connected
 	// player separatedly.
 	//
@@ -345,11 +346,10 @@ void UpdateCountersForThisFrame()
 	Me.MissionTimeElapsed += latest_frame_time;
 	Me.TextVisibleTime += latest_frame_time;
 
-	//--------------------
 	// We take care of the running stamina...
 	//
 	my_speed = sqrt(Me.speed.x * Me.speed.x + Me.speed.y * Me.speed.y);
-	if (my_speed >= (TUX_WALKING_SPEED + TUX_RUNNING_SPEED) * 0.5) {
+	if ((my_speed >= (TUX_WALKING_SPEED + TUX_RUNNING_SPEED) * 0.5) && (!GameConfig.cheat_running_stamina)) {
 		Me.running_power -= latest_frame_time * 3.0;
 	} else {
 		Me.running_power += latest_frame_time * 3.0;
@@ -360,7 +360,6 @@ void UpdateCountersForThisFrame()
 			Me.running_must_rest = FALSE;
 	}
 
-	//-------------------
 	// Mana and health recover over time
 	Me.energy += Me.health_recovery_rate * latest_frame_time;
 	if (Me.energy > Me.maxenergy)
@@ -379,11 +378,12 @@ void UpdateCountersForThisFrame()
 		if (Me.energy - lose_life < 1 && Me.paralyze_duration == 0) {
 			DoSkill(get_program_index_with_name("Emergency shutdown"),
 				calculate_program_heat_cost(get_program_index_with_name("Emergency shutdown")));
-			SetNewBigScreenMessage(_("Automatically shutting down due to near lethal overheat"));
+			SetNewBigScreenMessage(_("due to near lethal overheating"));
+			SetNewBigScreenMessage(_("Automatically shutting down"));
 			lose_life = (Me.temperature - Me.max_temperature) * latest_frame_time / 10;
 		}
 		if (lose_life > 0) {
-			Me.energy -= lose_life;
+			hit_tux(lose_life, -1);
 		}
 	}
 
@@ -418,19 +418,17 @@ void UpdateCountersForThisFrame()
 	if (Me.invisible_duration < 0)
 		Me.invisible_duration = 0;
 
-	//--------------------
 	// In order to know when a level can finally be respawned with
 	// enemies, we keep track to the time spent actually in the game, i.e.
 	// time actually spent passing frames...
 	//
 	for (i = 0; i < curShip.num_levels; i++) {
-		if (curShip.AllLevels[i] == NULL)
+		if (!level_exists(i))
 			continue;
 		if (Me.pos.z != i) {
 			if (Me.time_since_last_visit_or_respawn[i] > (-1)) {
 				Me.time_since_last_visit_or_respawn[i] += latest_frame_time;
 			}
-			//--------------------
 			// Now maybe it's time to respawn?  If we really have multiple
 			// players of course, this check would have to look a bit different...
 			//
@@ -446,7 +444,6 @@ void UpdateCountersForThisFrame()
 			}
 
 		} else {
-			//--------------------
 			// When the Tux is right on this level, there is absolutely no need 
 			// for respawning anything...
 			//
@@ -454,7 +451,6 @@ void UpdateCountersForThisFrame()
 		}
 	}
 
-	//--------------------
 	// On some maps, the Tux will have no enemies.  Therefore it would
 	// make sense that (for better gameplay) the running bar does not
 	// run out.
