@@ -2,7 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
- *   Copyright (c) 2004-2007 Arthur Huillet 
+ *   Copyright (c) 2004-2010 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -40,8 +40,10 @@
 #define CONNECTION_STRING	"c: "
 #define WHITE_SPACE 		" \t"
 #define MAP_BEGIN_STRING	"beginning_of_map"
-#define WP_SECTION_BEGIN_STRING		"wp"
+#define WP_BEGIN_STRING		"wp"
 #define LEVEL_END_STRING	"end_of_level"
+#define WP_END_STRING		LEVEL_END_STRING
+#define LEVEL_HEADER_LEVELNUMBER "Levelnumber:"
 
 #define LEVEL_NAME_STRING "Name of this level=_\""
 #define LEVEL_NAME_STRING_LEGACY "Name of this level="
@@ -53,31 +55,17 @@
 #define ITEM_NAME_STRING "it: n=\""
 #define ITEM_POS_X_STRING " X="
 #define ITEM_POS_Y_STRING " Y="
-#define ITEM_DR_BONUS_STRING " DR="
-#define ITEM_DAMAGE_STRING " Dmg="
-#define ITEM_DAMAGE_MODIFIER_STRING " DmgMod="
+#define ITEM_ARMOR_CLASS_BASE_STRING " AC="
 #define ITEM_MAX_DURATION_STRING " MDur="
 #define ITEM_CUR_DURATION_STRING " CDur="
 #define ITEM_AMMO_CLIP_STRING " AClip="
 #define ITEM_MULTIPLICITY_STRING " Multi="
-#define ITEM_PREFIX_CODE_STRING " Prefix="
-#define ITEM_SUFFIX_CODE_STRING " Suffix="
-#define ITEM_BONUS_TO_STR_STRING " StrBon="
-#define ITEM_BONUS_TO_DEX_STRING " StrDex="
-#define ITEM_BONUS_TO_VIT_STRING " StrVit="
-#define ITEM_BONUS_TO_MAG_STRING " StrMag="
-#define ITEM_BONUS_TO_ALLATT_STRING " StrAllAtt="
-#define ITEM_BONUS_TO_LIFE_STRING " StrLife="
-#define ITEM_BONUS_TO_FORCE_STRING " StrForce="
-#define ITEM_BONUS_TO_HEALTH_RECOVERY_STRING " StrHealthRecovery="
-#define ITEM_BONUS_TO_MANA_RECOVERY_STRING " StrManaRecovery="
-#define ITEM_BONUS_TO_TOHIT_STRING " StrToHit="
-#define ITEM_BONUS_TO_DRDAM_STRING " StrDRDam="
-#define ITEM_BONUS_TO_RESELE_STRING " ResEle="
-#define ITEM_BONUS_TO_RESFIR_STRING " ResFir="
-#define ITEM_IS_IDENTIFIED_STRING " IsIdent="
-#define CHEST_ITEMS_SECTION_BEGIN_STRING "pchestinfolvl"
-#define CHEST_ITEMS_SECTION_END_STRING "/pchestinfolvl"
+#define ITEM_SOCKETS_SIZE_STRING " Sockets="
+#define ITEM_SOCKET_TYPE_STRING "SocketType"
+#define ITEM_SOCKET_ADDON_STRING "SocketAddon"
+
+#define OBSTACLE_EXTENSIONS_BEGIN_STRING "obstacleextensions"
+#define OBSTACLE_EXTENSIONS_END_STRING "/obstacleextensions"
 
 #define STATEMENT_BEGIN_STRING "pstmtinfolvl"
 #define STATEMENT_END_STRING "/pstmtinfolvl"
@@ -110,8 +98,6 @@
 #define OBSTACLE_TYPE_STRING "t"
 #define OBSTACLE_X_POSITION_STRING "x"
 #define OBSTACLE_Y_POSITION_STRING "y"
-#define OBSTACLE_LABEL_INDEX_STRING "l"
-#define OBSTACLE_DESCRIPTION_INDEX_STRING "d"
 
 //--------------------
 // How big is the margin the walls have as far as collisions are concerned.
