@@ -2,7 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
- *   Copyright (c) 2004-2007 Arthur Huillet 
+ *   Copyright (c) 2004-2010 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -45,9 +45,6 @@ int Single_Player_Menu(void);
 void Options_Menu(void);
 
 EXTERN void LevelEditor(void);
-extern int MyCursorX;
-extern int MyCursorY;
-extern int load_game_command_came_from_inside_running_game;
 
 #define SELL_PRICE_FACTOR (0.25)
 #define REPAIR_PRICE_FACTOR (0.5)
@@ -70,7 +67,6 @@ static int MouseCursorIsOverMenuItem(int first_menu_item_pos_y, int h)
 
 	PureFraction = (GetMousePos_y() - first_menu_item_pos_y) / h;
 
-	//--------------------
 	// Now it can be that the pure difference is negative or that it is positive.
 	// However we should not always round thowards zero here, but rather always to
 	// the next LOWER integer!  This will be done here:
@@ -91,15 +87,12 @@ static int MouseCursorIsOverMenuItem(int first_menu_item_pos_y, int h)
 static void print_menu_text(char *InitialText, char *MenuTexts[], int first_menu_item_pos_y, int background_code, void *MenuFont)
 {
 	char open_gl_string[2000];
-	int h = FontHeight(GetCurrentFont());
 
-	//--------------------
 	// We need to prepare the background for the menu, so that
 	// it can be accessed with proper speed later...
 	//
 	InitiateMenu(background_code);
 
-	//--------------------
 	// Maybe if this is the very first startup menu, we should also print
 	// out some status variables like whether using OpenGL or not DIRECTLY
 	// ON THE MENU SCREEN...
@@ -122,7 +115,6 @@ static void print_menu_text(char *InitialText, char *MenuTexts[], int first_menu
 			strcat(open_gl_string, _(" NO "));
 		LeftPutString(Screen, GameConfig.screen_height - FontHeight(GetCurrentFont()), open_gl_string);
 	}
-	//--------------------
 	// Now that the possible font-changing small info printing is
 	// done, we can finally set the right font for the menu itself.
 	//
@@ -130,7 +122,6 @@ static void print_menu_text(char *InitialText, char *MenuTexts[], int first_menu
 		SetCurrentFont(Menu_BFont);
 	else
 		SetCurrentFont((BFont_Info *) MenuFont);
-	h = FontHeight(GetCurrentFont());
 
 };				// void print_menu_text ( ... )
 
@@ -162,20 +153,17 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 
 	game_status = INSIDE_MENU;
 
-	//--------------------
 	// At first we hide the system mouse cursor, because we want to use
 	// our own creation in the menus too...
 	//
 	make_sure_system_mouse_cursor_is_turned_off();
 
-	//--------------------
 	// We set the given font, if appropriate, and set the font height variable...
 	//
 	if (MenuFont != NULL)
 		SetCurrentFont(MenuFont);
 	h = FontHeight(GetCurrentFont());
 
-	//--------------------
 	// Some menus are intended to start with the default setting of the
 	// first menu option selected.  However this is not always desired.
 	// It might happen, that a submenu returns to the upper menu and then
@@ -187,7 +175,6 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 	if (FirstItem != (-1))
 		MenuPosition = FirstItem;
 
-	//--------------------
 	// First thing we do is find out how may options we have
 	// been given for the menu
 	// For each option, we compute and store its pixel-width
@@ -207,12 +194,10 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 	}
 	MenuTextWidths[NumberOfOptionsGiven] = 0;
 
-	//--------------------
 	// Clamp the longest option to the available width on the screen
 	// (50 pixels around the menu's background, and 1.5 fontheigth around the text)
 	LongestOption = min(GameConfig.screen_width - 100 - 3 * h, LongestOption);
 
-	//--------------------
 	// In those cases where we don't reset the menu position upon 
 	// initalization of the menu, we must check for menu positions
 	// outside the bounds of the current menu.
@@ -234,14 +219,12 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 	while (1) {
 		save_mouse_state();
 
-		//--------------------
 		// We write out the normal text of the menu, either by doing it once more
 		// in the open_gl case or by restoring what we have saved earlier, in the 
 		// SDL output case.
 		//
 		RestoreMenuBackground(0);
 
-		//--------------------
 		// Maybe we should display some thumbnails with the saved games entries?
 		// But this will only apply for the load_hero and the delete_hero menus...
 		//
@@ -250,18 +233,15 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 		    strcmp(MenuTexts[MenuPosition - 1], _("[down]")) &&
 		    strcmp(MenuTexts[MenuPosition - 1], _("[up]")) &&
 		    strcmp(MenuTexts[MenuPosition - 1], " ") && MenuPosition < NumberOfOptionsGiven) {
-			//--------------------
 			// We load the thumbnail, or at least we try to do it...
 			//
 			LoadAndShowThumbnail(MenuTexts[MenuPosition - 1]);
 			LoadAndShowStats(MenuTexts[MenuPosition - 1]);
 		}
-		//--------------------
 		// Draw the menu's  background
 		//
 		ShadowingRectangle(Screen, BackgroundRect);
 
-		//-------------------
 		// Display each option
 		//
 		for (i = 0; TRUE; i++) {
@@ -272,7 +252,10 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 			if (MenuTextWidths[i] == 0)
 				break;
 
-			//--------------------
+			// Don't select empty menu entries
+			if (strcmp(MenuTexts[i], " ") == 0)
+				continue;
+
 			// Define the actual text to display
 			// If the text is too long, handle autoscroll and clip it
 			if (MenuTextWidths[i] > LongestOption) {
@@ -294,7 +277,6 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 				width = MenuTextWidths[i];
 			}
 
-			//--------------------
 			// Depending on what highlight method has been used, we so some highlighting
 			// of the currently selected menu options location on the screen...
 			//
@@ -309,7 +291,6 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 					auto_scroll_start += AUTO_SCROLL_RATE;
 				}
 			}
-			//--------------------
 			// Draw the option's text
 			CenteredPutString(Screen, first_menu_item_pos_y + i * h, str);
 
@@ -319,18 +300,15 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 		if (strlen(InitialText) > 0)
 			DisplayText(InitialText, 50, 50, NULL, TEXT_STRETCH);
 
-		//--------------------
 		// Now the mouse cursor must be brought to the screen
 		//
 		make_sure_system_mouse_cursor_is_turned_off();
 		blit_our_own_mouse_cursor();
 
-		//--------------------
 		// Image should be ready now, so we can show it...
 		//
 		our_SDL_flip_wrapper();
 
-		//--------------------
 		// Now it's time to handle the possible keyboard and mouse 
 		// input from the user...
 		//
@@ -370,7 +348,6 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 				case SDLK_SPACE:
 				case SDLK_LEFT:
 				case SDLK_RIGHT:
-					//--------------------
 					// The space key or enter key or arrow keys all indicate, that
 					// the user has made a selection.
 					//
@@ -383,6 +360,11 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 				case SDLK_UP:
 					if (MenuPosition > 1)
 						MenuPosition--;
+
+					// Skip any blank positions when moving with the keyboard
+					if (strcmp(MenuTexts[MenuPosition - 1], " ") == 0)
+						MenuPosition = (MenuPosition == 1) ? MenuPosition + 1 : MenuPosition - 1;
+
 					MoveMenuPositionSound();
 					HighlightRect.x = UNIVERSAL_COORD_W(320);	// ( TextWidth ( MenuTexts [ MenuPosition - 1 ] ) ) / 2 ;
 					HighlightRect.y = first_menu_item_pos_y + (MenuPosition - 1) * h;
@@ -392,6 +374,11 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 				case SDLK_DOWN:
 					if (MenuPosition < NumberOfOptionsGiven)
 						MenuPosition++;
+
+					// Skip any blank positions when moving with the keyboard
+					if (strcmp(MenuTexts[MenuPosition - 1], " ") == 0)
+						MenuPosition = (MenuPosition == NumberOfOptionsGiven) ? MenuPosition - 1 : MenuPosition + 1;
+
 					MoveMenuPositionSound();
 					HighlightRect.x = UNIVERSAL_COORD_W(320);	// ( TextWidth ( MenuTexts [ MenuPosition - 1 ] ) ) / 2 ;
 					HighlightRect.y = first_menu_item_pos_y + (MenuPosition - 1) * h;
@@ -406,12 +393,13 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 		}
 
 		if (MouseLeftClicked()) {
-			//--------------------
 			// Only when the mouse click really occured on the menu do we
 			// interpret it as a menu choice.  Otherwise we'll just ignore
-			// it.
-			//
-			if (MouseCursorIsOverMenuItem(first_menu_item_pos_y, h) == MenuPosition) {
+			// it. Also, we completely ignore any clicks if the clicked
+			// position is blank.
+			if (MouseCursorIsOverMenuItem(first_menu_item_pos_y, h) == MenuPosition &&
+				strcmp(MenuTexts[MenuPosition - 1], " ") != 0) {
+
 				MenuItemSelectedSound();
 				ret = MenuPosition;
 				goto out;
@@ -424,13 +412,11 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 		if (MenuPosition > NumberOfOptionsGiven)
 			MenuPosition = NumberOfOptionsGiven;
 
-		//--------------------
 		// If the selected option has changed, halt eventual current autoscrolling
 		if (MenuPosition != old_menu_position) {
 			auto_scroll_run = TRUE;
 			auto_scroll_start = 0.0f;
 		}
-		//--------------------
 		// At this the while (1) overloop ends.  But for the menu, we really do not
 		// need to hog the CPU.  Therefore some waiting should be introduced here.
 		//
@@ -441,6 +427,7 @@ int DoMenuSelection(char *InitialText, char **MenuTexts, int FirstItem, int back
 	free(MenuTextWidths);
 	while (MouseLeftPressed())	//eat the click that may have happened
 		SDL_Delay(1);
+	input_handle();
 	game_status = old_game_status;
 	return ret;
 };				// int DoMenuSelection( ... )
@@ -460,7 +447,6 @@ ChatDoMenuSelectionFlagged(char *InitialText, char *MenuTexts[MAX_ANSWERS_PER_PE
 
 	DebugPrintf(2, "\nBEFORE:  First Item now: %d.", FirstItem);
 
-	//--------------------
 	// We filter out those answering options that are allowed by the flag mask
 	//
 	DebugPrintf(MENU_SELECTION_DEBUG, "\n%s(): %d \n", __FUNCTION__, FirstItem);
@@ -481,12 +467,10 @@ ChatDoMenuSelectionFlagged(char *InitialText, char *MenuTexts[MAX_ANSWERS_PER_PE
 
 	DebugPrintf(2, "\nMIDDLE:  First Item now: %d.", FirstItem);
 
-	//--------------------
 	// Now we do the usual menu selection, using only the activated chat alternatives...
 	//
 	MenuSelection = ChatDoMenuSelection(FilteredChatMenuTexts, FirstItem, MenuFont, ChatDroid);
 
-	//--------------------
 	// Now that we have an answer, we must transpose it back to the original array
 	// of all theoretically possible answering possibilities.
 	//
@@ -508,63 +492,6 @@ ChatDoMenuSelectionFlagged(char *InitialText, char *MenuTexts[MAX_ANSWERS_PER_PE
 	return (MenuSelection);
 };				// int ChatDoMenuSelectionFlagged( char* InitialText , char* MenuTexts[ MAX_ANSWERS_PER_PERSON] , ... 
 
-/**
- * Without destroying or changing anything, this function should determine
- * how many lines of text it takes to write a string with the current 
- * font into the given rectangle, provided one would start at the left
- * side as one usually does...
- */
-int GetNumberOfTextLinesNeeded(char *GivenText, SDL_Rect GivenRectangle, float text_stretch)
-{
-	int BackupOfMyCursorX, BackupOfMyCursorY;
-	int TextLinesNeeded;
-	int TestPosition;
-	int stored_height;
-
-	//--------------------
-	// If we receive an empty string, we print out a warning message and then
-	// return one line as the required amount of lines.
-	//
-	if (strlen(GivenText) <= 1) {
-		/*
-		   ErrorMessage ( __FUNCTION__  , "\
-		   Warning.  Received empty or nearly empty string!",
-		   NO_NEED_TO_INFORM, IS_WARNING_ONLY );
-		 */
-		return (1);
-	}
-	//--------------------
-	// First we make a backup of everything, so that we don't destory anything.
-	//
-	display_char_disabled = TRUE;
-	BackupOfMyCursorX = MyCursorX;
-	BackupOfMyCursorY = MyCursorY;
-
-	//--------------------
-	// Now in our simulated environment, we can blit the Text and see how many lines it takes...
-	//
-	MyCursorX = GivenRectangle.x;
-	MyCursorY = GivenRectangle.y;
-	TestPosition = MyCursorY;
-
-	stored_height = GivenRectangle.h;
-	GivenRectangle.h = 32000;
-	TextLinesNeeded = DisplayText(GivenText, GivenRectangle.x, GivenRectangle.y, &GivenRectangle, text_stretch);
-	GivenRectangle.h = stored_height;
-
-	//--------------------
-	// Now that we have found our solution, we can restore everything back to normal
-	//
-	// RestoreMenuBackground ( 1 ) ;
-	display_char_disabled = FALSE;
-
-	MyCursorX = BackupOfMyCursorX;
-	MyCursorY = BackupOfMyCursorY;
-
-	return (TextLinesNeeded);
-
-};				// int GetNumberOfTextLinesNeeded ( MenuTexts [ i ] , Choice_Window )
-
 /**
  *
  * This function performs a menu for the player to select from, using the
@@ -590,7 +517,7 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 	SDL_Rect HighlightRect;
 	int MaxLinesInMenuRectangle;
 	int OptionOffset = 0;
-	int SpaceUsedSoFar;
+	int line_count;
 	int BreakOffCauseAllDisplayed;
 	int BreakOffCauseNoRoom = FALSE;
 	int LastOptionVisible = 0;
@@ -602,7 +529,6 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 	int old_game_status = game_status;
 
 	game_status = INSIDE_MENU;
-	//--------------------
 	// First we initialize the menu positions
 	//
 	for (i = 0; i < MAX_ANSWERS_PER_PERSON; i++) {
@@ -611,18 +537,14 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 		MenuOptionLineRequirement[i] = 0;
 	}
 
-	//--------------------
 	// Now we set some viable choice window and we compute the maximum number of lines
 	// that will still fit well into the choice window.
-	//
-	Choice_Window.x = 35 * GameConfig.screen_width / 640;
-	Choice_Window.y = 335 * GameConfig.screen_height / 480;
-	Choice_Window.w = (640 - 70) * GameConfig.screen_width / 640;
-	Choice_Window.h = 118 * GameConfig.screen_height / 480;
+	Choice_Window.x = UNIVERSAL_COORD_W(37);
+	Choice_Window.y = UNIVERSAL_COORD_H(336);
+	Choice_Window.w = UNIVERSAL_COORD_W(640 - 70);
+	Choice_Window.h = UNIVERSAL_COORD_H(118);
 	MaxLinesInMenuRectangle = Choice_Window.h / (FontHeight(GetCurrentFont()) * TEXT_STRETCH);
-	DebugPrintf(1, "\nComputed number of lines in choice window at most: %d.", MaxLinesInMenuRectangle);
 
-	//--------------------
 	// We don't need the system mouse cursor, as we do have our own for
 	// the same purpose.
 	//
@@ -631,7 +553,6 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 	if (FirstItem != (-1))
 		menu_position_to_remember = FirstItem;
 
-	//--------------------
 	// First thing we do is find out how may options we have
 	// been given for the menu
 	//
@@ -648,15 +569,13 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 	}
 	NumberOfOptionsGiven = i;
 
-	//--------------------
 	// We need to prepare the background for the menu, so that
 	// it can be accessed with proper speed later...
 	//
 	SDL_SetClipRect(Screen, NULL);
-	display_current_chat_protocol(CHAT_DIALOG_BACKGROUND_PICTURE_CODE, ChatDroid, FALSE);
+	show_chat_log(ChatDroid);
 	StoreMenuBackground(0);
 
-	//--------------------
 	// Now that the possible font-changing background assembling is
 	// done, we can finally set the right font for the menu itself.
 	//
@@ -672,8 +591,7 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 		RestoreMenuBackground(0);
 		save_mouse_state();
 
-		//--------------------
-		// Now that the possible font-changing chat protocol display is
+		// Now that the possible font-changing chat log display is
 		// done, we can finally set the right font for the menu itself.
 		//
 		if (MenuFont == NULL)
@@ -682,13 +600,11 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 			SetCurrentFont((BFont_Info *) MenuFont);
 		h = FontHeight(GetCurrentFont());
 
-		//--------------------
 		// We blit to the screen all the options that are not empty and that still fit
 		// onto the screen
 		//
-		SpaceUsedSoFar = 0;
+		line_count = 0;
 		for (i = OptionOffset; i < MAX_ANSWERS_PER_PERSON; i++) {
-			//--------------------
 			// If all has been displayed already, we quit blitting...
 			//
 			if (strlen(MenuTexts[i]) == 0) {
@@ -697,46 +613,44 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 				LastOptionVisible = i;
 				break;
 			}
-			//--------------------
 			// If there is not enough room any more, we quit blitting...
-			//
-			if (SpaceUsedSoFar > MaxLinesInMenuRectangle * FontHeight(GetCurrentFont())) {
+			int lines_needed = get_lines_needed(MenuTexts[i], Choice_Window, TEXT_STRETCH);
+			if ((line_count + lines_needed) > MaxLinesInMenuRectangle) {
 				BreakOffCauseAllDisplayed = FALSE;
 				BreakOffCauseNoRoom = TRUE;
 				LastOptionVisible = i;
 				break;
 			}
-			//--------------------
 			// Now that we know, that there is enough room, we can blit the next menu option.
 			//
 			MenuPosX[i] = Choice_Window.x;
-			MenuPosY[i] = Choice_Window.y + SpaceUsedSoFar;
+			MenuPosY[i] = Choice_Window.y + (line_count * h * TEXT_STRETCH);
 			MenuOptionLineRequirement[i] =
-			    DisplayText(MenuTexts[i], Choice_Window.x, Choice_Window.y + SpaceUsedSoFar, &Choice_Window, TEXT_STRETCH);
-			SpaceUsedSoFar += MenuOptionLineRequirement[i] * (FontHeight(GetCurrentFont()) * TEXT_STRETCH);
+			    DisplayText(MenuTexts[i], MenuPosX[i], MenuPosY[i], &Choice_Window, TEXT_STRETCH);
+			line_count += MenuOptionLineRequirement[i];
 		}
 
-		//--------------------
 		// We highlight the currently selected option with a highlighting rectangle
 		//
 		// (and we add some security against 'empty' chat selection menus causing
 		// some segfaults rather easily...)
-		//
-		DebugPrintf(1, "\n%s(): menu_position_to_remember: %d.", __FUNCTION__, menu_position_to_remember);
-		DebugPrintf(1, "\n%s(): FirstItem: %d.", __FUNCTION__, FirstItem);
 		if (menu_position_to_remember <= 0)
 			menu_position_to_remember = 1;
-		HighlightRect.x = MenuPosX[menu_position_to_remember - 1] - 0 * h;
+
+#define HIGHLIGHT_H_MARGIN 4
+		HighlightRect.x = MenuPosX[menu_position_to_remember - 1] - HIGHLIGHT_H_MARGIN;
 		HighlightRect.y = MenuPosY[menu_position_to_remember - 1];
-		HighlightRect.w = TextWidth(MenuTexts[menu_position_to_remember - 1]) + 0 * h;
+		HighlightRect.w = TextWidth(MenuTexts[menu_position_to_remember - 1])
+			+ 2 * HIGHLIGHT_H_MARGIN;
 		if (HighlightRect.w > 580 * GameConfig.screen_width / 640)
 			HighlightRect.w = 580 * GameConfig.screen_width / 640;
-		HighlightRect.h = MenuOptionLineRequirement[menu_position_to_remember - 1] * (FontHeight(GetCurrentFont()) * TEXT_STRETCH);
+		HighlightRect.h = MenuOptionLineRequirement[menu_position_to_remember - 1] * h + 1;
 		if (HighlightRect.h + HighlightRect.y > UNIVERSAL_COORD_H(457))
 			HighlightRect.h = UNIVERSAL_COORD_H(457) - HighlightRect.y;
 		HighlightRectangle(Screen, HighlightRect);
 		// Display again the highlighted line
-		DisplayText(MenuTexts[menu_position_to_remember - 1], MenuPosX[menu_position_to_remember - 1],
+		DisplayText(MenuTexts[menu_position_to_remember - 1],
+			    MenuPosX[menu_position_to_remember - 1],
 			    MenuPosY[menu_position_to_remember - 1], &Choice_Window, TEXT_STRETCH);
 
 		if (BreakOffCauseNoRoom)
@@ -744,13 +658,11 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 		if (OptionOffset)
 			ShowGenericButtonFromList(SCROLL_DIALOG_MENU_UP_BUTTON);
 
-		//--------------------
 		// Now the mouse cursor must be brought to the screen
 		//
 		make_sure_system_mouse_cursor_is_turned_off();
 		blit_our_own_mouse_cursor();
 
-		//--------------------
 		// Now everything should become visible!
 		//
 		our_SDL_flip_wrapper();
@@ -810,14 +722,14 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 				break;
 
 			case SDLK_PAGEUP:
-				chat_protocol_scroll_override_from_user -= 3;
-				display_current_chat_protocol(CHAT_DIALOG_BACKGROUND_PICTURE_CODE, ChatDroid, FALSE);
+				chat_log.scroll_offset -= 3;
+				show_chat_log(ChatDroid);
 				StoreMenuBackground(0);
 				break;
 
 			case SDLK_PAGEDOWN:
-				chat_protocol_scroll_override_from_user += 3;
-				display_current_chat_protocol(CHAT_DIALOG_BACKGROUND_PICTURE_CODE, ChatDroid, FALSE);
+				chat_log.scroll_offset += 3;
+				show_chat_log(ChatDroid);
 				StoreMenuBackground(0);
 				break;
 
@@ -843,7 +755,6 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 		}
 
 		if (MouseLeftClicked()) {
-			//--------------------
 			// First we see if there was perhaps a click on one of the active scroll buttons
 			//
 			if ((MouseCursorIsOnButton(SCROLL_DIALOG_MENU_DOWN_BUTTON, GetMousePos_x(), GetMousePos_y())) &&
@@ -853,21 +764,19 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 			} else if ((MouseCursorIsOnButton(SCROLL_DIALOG_MENU_UP_BUTTON, GetMousePos_x(), GetMousePos_y())) &&
 				   (OptionOffset)) {
 				OptionOffset--;
-			} else if (MouseCursorIsOnButton(CHAT_PROTOCOL_SCROLL_UP_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-				chat_protocol_scroll_override_from_user--;
-				display_current_chat_protocol(CHAT_DIALOG_BACKGROUND_PICTURE_CODE, ChatDroid, FALSE);
+			} else if (MouseCursorIsOnButton(CHAT_LOG_SCROLL_UP_BUTTON, GetMousePos_x(), GetMousePos_y())) {
+				chat_log.scroll_offset--;
+				show_chat_log(ChatDroid);
 				StoreMenuBackground(0);
-			} else if (MouseCursorIsOnButton(CHAT_PROTOCOL_SCROLL_DOWN_BUTTON, GetMousePos_x(), GetMousePos_y())) {
-				chat_protocol_scroll_override_from_user++;
-				display_current_chat_protocol(CHAT_DIALOG_BACKGROUND_PICTURE_CODE, ChatDroid, FALSE);
+			} else if (MouseCursorIsOnButton(CHAT_LOG_SCROLL_DOWN_BUTTON, GetMousePos_x(), GetMousePos_y())) {
+				chat_log.scroll_offset++;
+				show_chat_log(ChatDroid);
 				StoreMenuBackground(0);
 			}
-			//--------------------
 			// If not, then maybe it was a click into the options window.  That alone
 			// would be enough to call it a valid user decision.
 			//
 			else {
-				//--------------------
 				// Now if the click has occured within the lines of the menu, we will count
 				// this as a valid choice of the user.
 				//
@@ -903,7 +812,6 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
 		if (cursors_menu_position > LastOptionVisible)
 			cursors_menu_position = LastOptionVisible;
 
-		//--------------------
 		// If the mouse cursor was on one of the possible lines, than we can try to translate
 		// it into a real menu position
 		//
@@ -931,7 +839,7 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
  out:
 	game_status = old_game_status;
 	return ret;
-};				// int ChatDoMenuSelection( char* InitialText , char* MenuTexts[] , asdfasd .... )
+}
 
 /**
  * This function prepares the screen for the big Escape menu and 
@@ -942,7 +850,6 @@ int ChatDoMenuSelection(char *MenuTexts[MAX_ANSWERS_PER_PERSON], int FirstItem,
  */
 void InitiateMenu(int background_code)
 {
-	//--------------------
 	// Here comes the standard initializer for all the menus and submenus
 	// of the big escape menu.  This prepares the screen, so that we can
 	// write on it further down.
@@ -951,8 +858,7 @@ void InitiateMenu(int background_code)
 
 	if (background_code == (-1)) {
 		DisplayBanner();
-		AssembleCombatPicture(USE_OWN_MOUSE_CURSOR);
-		MakeGridOnScreen(NULL);
+		AssembleCombatPicture(SHOW_ITEMS | USE_OWN_MOUSE_CURSOR);
 	} else {
 		// DisplayImage ( find_file ( BackgroundToUse , GRAPHICS_DIR, FALSE ) );
 		blit_special_background(background_code);
@@ -972,15 +878,12 @@ void Cheatmenu(void)
 	int i, l;
 	int x0, y0, line;
 	int skip_dead = 0;
-	Waypoint WpList;	// pointer on current waypoint-list  
 
-	//--------------------
 	// Prevent distortion of framerate by the delay coming from 
 	// the time spend in the menu.
 	//
 	Activate_Conservative_Frame_Computation();
 
-	//--------------------
 	// Some small font is needed, such that we can get a lot of lines on
 	// one single cheat menu page...
 	//
@@ -1001,13 +904,13 @@ void Cheatmenu(void)
 		printf_SDL(Screen, -1, -1, " L. alive robot list of current level\n");
 		printf_SDL(Screen, -1, -1, " k. dead robot list of current level\n");
 		printf_SDL(Screen, -1, -1, " d. destroy robots on current level\n");
-		printf_SDL(Screen, -1, -1, " h. Auto-aquire all skills\n");
+		printf_SDL(Screen, -1, -1, " h. Auto-acquire all skills\n");
+		printf_SDL(Screen, -1, -1, " c. Acquire 1 million circuits\n");
 		printf_SDL(Screen, -1, -1, " n. No hidden droids: %s", show_all_droids ? "ON\n" : "OFF\n");
+		printf_SDL(Screen, -1, -1, " r. Infinite running stamina: %s", GameConfig.cheat_running_stamina ? "ON\n" : "OFF\n");
 		printf_SDL(Screen, -1, -1, " x. Cheatkeys : %s", GameConfig.enable_cheatkeys ? "ON\n" : "OFF\n");
-		printf_SDL(Screen, -1, -1, " w. Print current waypoints\n");
 		printf_SDL(Screen, -1, -1, " q. RESUME game\n");
 
-		//--------------------
 		// Now we show it...
 		//
 		our_SDL_flip_wrapper();
@@ -1066,7 +969,7 @@ void Cheatmenu(void)
 						} else {
 							printf_SDL(Screen, 15, -1, "SEVERE ERROR: Type=%d. ", erot->type);
 						}
-						if (erot->is_friendly)
+						if (is_friendly(erot->faction, FACTION_SELF))
 							printf_SDL(Screen, -1, -1, " YES");
 						else
 							printf_SDL(Screen, -1, -1, "  NO");
@@ -1083,6 +986,9 @@ void Cheatmenu(void)
 						case DEATH_ANIMATION:
 							printf_SDL(Screen, -1, -1, " Deth");
 							break;
+						case DEAD_ANIMATION:
+							printf_SDL(Screen, -1, -1, " Dead");
+							break;
 						case STAND_ANIMATION:
 							printf_SDL(Screen, -1, -1, " Stnd");
 							break;
@@ -1121,46 +1027,23 @@ void Cheatmenu(void)
 
 		case 'h':	// auto-aquire all skills
 			for (i = 0; i < number_of_skills; i++)
-				Me.base_skill_level[i]++;
+				Me.skill_level[i]++;
+			break;
+
+		case 'c':	// Add 1 million circuits
+			Me.Gold += 1000000;
 			break;
 
 		case 'n':	// toggle display of all droids
 			show_all_droids = !show_all_droids;
 			break;
 
-		case 'x':
-			GameConfig.enable_cheatkeys = !GameConfig.enable_cheatkeys;
+		case 'r':
+			GameConfig.cheat_running_stamina = !GameConfig.cheat_running_stamina;
 			break;
 
-		case 'w':	/* print waypoint info of current level */
-			WpList = CURLEVEL()->AllWaypoints;
-			for (i = 0; i < MAXWAYPOINTS && WpList[i].x; i++) {
-				if (i && !(i % 20)) {
-					printf_SDL(Screen, -1, -1, " ---- MORE -----\n");
-					our_SDL_flip_wrapper();
-
-					if (getchar_raw(NULL) == 'q') {
-						i = -1;	//ugly hack to not getchar_raw twice in a row
-						break;
-					}
-				}
-				if (!(i % 20)) {
-					ClearGraphMem();
-					printf_SDL(Screen, x0, y0, "Nr.   X   Y      C1  C2  C3  C4\n");
-					printf_SDL(Screen, -1, -1, "------------------------------------\n");
-				}
-				printf_SDL(Screen, -1, -1, "%2d   %2d  %2d      %2d  %2d  %2d  %2d\n",
-					   i, WpList[i].x, WpList[i].y,
-					   WpList[i].connections[0],
-					   WpList[i].connections[1], WpList[i].connections[2], WpList[i].connections[3]);
-			}	/* for (all waypoints) */
-
-			if (i == -1)
-				break;
-
-			printf_SDL(Screen, -1, -1, " --- END ---\n");
-			our_SDL_flip_wrapper();
-			getchar_raw(NULL);
+		case 'x':
+			GameConfig.enable_cheatkeys = !GameConfig.enable_cheatkeys;
 			break;
 
 		case ' ':
@@ -1299,29 +1182,20 @@ static int Startup_handle(int n)
 		}
 		break;
 	case LVLEDIT_POSITION:	//allow starting directly in leveleditor - the hack is a little dirty but it does its work.
-		game_root_mode = ROOT_IS_LVLEDIT;
-		skip_initial_menus = 1;
-		clear_player_inventory_and_stats();
-		UpdateAllCharacterStats();
-		strcpy(Me.character_name, "MapEd");
-		char fp[2048];
-		find_file("freedroid.levels", MAP_DIR, fp, 0);
-		LoadShip(fp);
-		PrepareStartOfNewCharacter("NewTuxStartGameSquare");
-		skip_initial_menus = 0;
+		prepare_level_editor();
 		return EXIT_MENU;
 		break;
 	case TUTORIAL_POSITION:	//Similar hack to start Tutorial.
+		game_root_mode = ROOT_IS_GAME;
 		skip_initial_menus = 1;
 		clear_player_inventory_and_stats();
 		UpdateAllCharacterStats();
 		strcpy(Me.character_name, "TutorialTux");
 		char fpp[2048];
 		find_file("freedroid.levels", MAP_DIR, fpp, 0);
-		LoadShip(fpp);
+		LoadShip(fpp, 0);
 		PrepareStartOfNewCharacter("TutorialTuxStart");
 		skip_initial_menus = 0;
-		game_root_mode = ROOT_IS_GAME;
 		return EXIT_MENU;
 		break;
 	case OPTIONS_POSITION:
@@ -1368,8 +1242,7 @@ static int Game_handle(int n)
 	case DIFFICULTY:
 		GameConfig.difficulty_level++;
 		GameConfig.difficulty_level %= 3;
-		GiveMouseAlertWindow(_
-				     ("You need to restart FreedroidRPG for the difficulty change to be taken into account.\n\nSorry for the inconvenience.\n"));
+		alert_window(_("You need to restart FreedroidRPG for the difficulty change to take effect.\n\nSorry for the inconvenience."));
 		return CONTINUE_MENU;
 	case LEAVE_MENU:
 		return EXIT_MENU;
@@ -1433,9 +1306,9 @@ static int Options_handle(int n)
 
 static void Options_fill(char *MenuTexts[10])
 {
-	strncpy(MenuTexts[0], _("Game Options"), 1024);
-	strncpy(MenuTexts[1], _("Graphics Options"), 1024);
-	strncpy(MenuTexts[2], _("Sound Options"), 1024);
+	strncpy(MenuTexts[0], _("Game"), 1024);
+	strncpy(MenuTexts[1], _("Graphics"), 1024);
+	strncpy(MenuTexts[2], _("Sound"), 1024);
 	strncpy(MenuTexts[3], _("Keys"), 1024);
 	strncpy(MenuTexts[4], _("Droid Talk"), 1024);
 	strncpy(MenuTexts[5], _("On-Screen Displays"), 1024);
@@ -1541,12 +1414,10 @@ static int Resolution_handle(int n)
 			while (EnterPressed() || SpacePressed()) ;
 			GameConfig.next_time_width_of_screen = screen_resolutions[i].xres;
 			GameConfig.next_time_height_of_screen = screen_resolutions[i].yres;
-			char *fmt = _("\nYou selected %d x %d pixel.\n\n"
-				      "Change of screen resolution will\n"
-				      "take effect automatically when you next\n" "start FreedroidRPG.\n" "\n" "Thank you.\n");
+			char *fmt = _("You selected %d x %d pixels.\n\nYou need to restart FreedroidRPG for the changes to take effect.\n\nSorry for the inconvenience.");
 			char *txt = (char *)malloc(strlen(fmt) + 20);	// 20 chars should be enough to store 2 integers
 			sprintf(txt, fmt, screen_resolutions[i].xres, screen_resolutions[i].yres);
-			GiveMouseAlertWindow(txt);
+			alert_window(txt);
 			free(txt);
 			return CONTINUE_MENU;
 		}
@@ -1602,6 +1473,9 @@ static int Graphics_handle(int n)
 		if (LeftPressed()) {
 			while (LeftPressed()) ;
 			GameConfig.current_gamma_correction -= 0.05;
+			if (GameConfig.current_gamma_correction < 0.0) {
+				GameConfig.current_gamma_correction = 0.0;
+			}
 			SDL_SetGamma(GameConfig.current_gamma_correction, GameConfig.current_gamma_correction,
 				     GameConfig.current_gamma_correction);
 		}
@@ -1614,7 +1488,7 @@ static int Graphics_handle(int n)
 #ifndef __WIN32__
 		SDL_WM_ToggleFullScreen(Screen);
 #else
-		GiveMouseAlertWindow(_("\nThis option will be taken into account after you restart the game.\n\n   Thank you.\n"));
+		alert_window(_("You need to restart FreedroidRPG for the changes to take effect.\n\nSorry for the inconvenience."));
 #endif
 		break;
 
@@ -1713,15 +1587,17 @@ static int Sound_handle(int n)
 	case SET_BG_MUSIC_VOLUME:
 		if (RightPressed()) {
 			while (RightPressed()) ;
-			if (GameConfig.Current_BG_Music_Volume < 1)
-				GameConfig.Current_BG_Music_Volume += 0.05;
+			GameConfig.Current_BG_Music_Volume += 0.05;
+			if (GameConfig.Current_BG_Music_Volume > 1.0)
+				GameConfig.Current_BG_Music_Volume = 1.0;
 			SetBGMusicVolume(GameConfig.Current_BG_Music_Volume);
 		}
 
 		if (LeftPressed()) {
 			while (LeftPressed()) ;
-			if (GameConfig.Current_BG_Music_Volume > 0)
-				GameConfig.Current_BG_Music_Volume -= 0.05;
+			GameConfig.Current_BG_Music_Volume -= 0.05;
+			if (GameConfig.Current_BG_Music_Volume < 0.0)
+				GameConfig.Current_BG_Music_Volume = 0.0;
 			SetBGMusicVolume(GameConfig.Current_BG_Music_Volume);
 		}
 
@@ -1730,15 +1606,17 @@ static int Sound_handle(int n)
 	case SET_SOUND_FX_VOLUME:
 		if (RightPressed()) {
 			while (RightPressed()) ;
-			if (GameConfig.Current_Sound_FX_Volume < 1)
-				GameConfig.Current_Sound_FX_Volume += 0.05;
+			GameConfig.Current_Sound_FX_Volume += 0.05;
+			if (GameConfig.Current_Sound_FX_Volume > 1.0)
+				GameConfig.Current_Sound_FX_Volume = 1.0;
 			SetSoundFXVolume(GameConfig.Current_Sound_FX_Volume);
 		}
 
 		if (LeftPressed()) {
 			while (LeftPressed()) ;
-			if (GameConfig.Current_Sound_FX_Volume > 0)
-				GameConfig.Current_Sound_FX_Volume -= 0.05;
+			GameConfig.Current_Sound_FX_Volume -= 0.05;
+			if (GameConfig.Current_Sound_FX_Volume < 0.0)
+				GameConfig.Current_Sound_FX_Volume = 0.0;
 			SetSoundFXVolume(GameConfig.Current_Sound_FX_Volume);
 		}
 		break;
@@ -1842,8 +1720,8 @@ static void Performance_fill(char *MenuTexts[])
 	strcat(Options[i], Options[i + 1]);
 	strncpy(MenuTexts[i], Options[i], 1024);
 	i++;
-	sprintf(Options[i], _("Skip shadow blitting"));
-	sprintf(Options[i + 1], ": %s", GameConfig.skip_shadow_blitting ? _("YES") : _("NO"));
+	sprintf(Options[i], _("Show obstacle shadows"));
+	sprintf(Options[i + 1], ": %s", GameConfig.skip_shadow_blitting ? _("NO") : _("YES"));
 	strcat(Options[i], Options[i + 1]);
 	strncpy(MenuTexts[i], Options[i], 1024);
 	i++;
@@ -1993,13 +1871,11 @@ static void Get_New_Character_Name(void)
 	else
 		Temp = "MapEd";
 
-	//--------------------
 	// In case 'Escape has been pressed inside GetString, then a NULL pointer
 	// will be returned, which means no name has been given to the character
 	// yet, so we set an empty string for the character name.
 	//
 	if (Temp == NULL) {
-		//--------------------
 		// WARNING:  We should not use "" here, since that is also used inside
 		//           the menu code as the termination string for the list of menu
 		//           options.  That would cause problems in the load and display
@@ -2011,7 +1887,6 @@ static void Get_New_Character_Name(void)
 		strcpy(Me.character_name, "HaveNoName");
 		return;
 	}
-	//--------------------
 	// If a real name has been given to the character, we can copy that name into
 	// the corresponding structure and return here (not without freeing the string
 	// received.  Could be some valuable 20 bytes after all :)
@@ -2035,44 +1910,21 @@ void clear_player_inventory_and_stats(void)
 {
 	int i;
 
-	//--------------------
 	// At first we clear the inventory of the new character
 	// of any items (or numeric waste) that might be in there
 	//
 	for (i = 0; i < MAX_ITEMS_IN_INVENTORY; i++) {
-		Me.Inventory[i].type = (-1);
-		Me.Inventory[i].prefix_code = (-1);
-		Me.Inventory[i].suffix_code = (-1);
-		Me.Inventory[i].currently_held_in_hand = FALSE;
+		init_item(&Me.Inventory[i]);
 	}
 	DebugPrintf(1, "\n%s(): Inventory has been emptied...", __FUNCTION__);
 
-	//--------------------
 	// Now we add some safety, against 'none present' items
 	//
-	Me.weapon_item.type = (-1);
-	Me.drive_item.type = (-1);
-	Me.armour_item.type = (-1);
-	Me.shield_item.type = (-1);
-	Me.special_item.type = (-1);
-
-	Me.weapon_item.prefix_code = (-1);
-	Me.drive_item.prefix_code = (-1);
-	Me.armour_item.prefix_code = (-1);
-	Me.shield_item.prefix_code = (-1);
-	Me.special_item.prefix_code = (-1);
-
-	Me.weapon_item.suffix_code = (-1);
-	Me.drive_item.suffix_code = (-1);
-	Me.armour_item.suffix_code = (-1);
-	Me.shield_item.suffix_code = (-1);
-	Me.special_item.suffix_code = (-1);
-
-	Me.weapon_item.currently_held_in_hand = (-1);
-	Me.drive_item.currently_held_in_hand = (-1);
-	Me.armour_item.currently_held_in_hand = (-1);
-	Me.shield_item.currently_held_in_hand = (-1);
-	Me.special_item.currently_held_in_hand = (-1);
+	init_item(&Me.weapon_item);
+	init_item(&Me.drive_item);
+	init_item(&Me.armour_item);
+	init_item(&Me.shield_item);
+	init_item(&Me.special_item);
 
 	Me.base_vitality = 25;
 	Me.base_strength = 10;
@@ -2094,16 +1946,12 @@ static int PrepareNewHero(void)
 
 	Get_New_Character_Name();
 
-	Me.is_town_guard_member = FALSE;
-
-	//--------------------
 	// If the special string "HaveNoName" is being supplied, then
 	// we treat this as no name given and will return false.
 	//
 	if (!strcmp(Me.character_name, "HaveNoName"))
 		return (FALSE);
 
-	//--------------------
 	// If a real name has been given, then we can proceed and start the
 	// game.  If no real name has been given or 'Escape' has been pressed,
 	// then the calling function will return to the menu and do nothing
@@ -2171,7 +2019,6 @@ static int do_savegame_selection_and_act(int action)
 
 	InitiateMenu(NE_TITLE_PIC_BACKGROUND_CODE);
 
-	//--------------------
 	// We use empty strings to denote the end of any menu selection, 
 	// therefore also for the end of the list of saved characters.
 	//
@@ -2179,7 +2026,6 @@ static int do_savegame_selection_and_act(int action)
 		MenuTexts[n] = "";
 	}
 
-	//--------------------
 	// First we must find the home directory of the user.  From there on
 	// we can then construct the full directory path of the saved games directory.
 	//
@@ -2195,7 +2041,6 @@ static int do_savegame_selection_and_act(int action)
 	}
 #endif
 
-	//--------------------
 	// Now we generate the right directory for loading from the home
 	// directory.
 	//
@@ -2221,7 +2066,7 @@ static int do_savegame_selection_and_act(int action)
 				MenuTexts[cnt] = eps[cnt + saveoffset - 1]->d_name;
 			}
 
-			if (cnt >= 6) {
+			if (cnt >= 7) {
 				/* Display "down" */
 				MenuTexts[cnt++] = _("[down]");
 			} else
@@ -2275,7 +2120,7 @@ static int do_savegame_selection_and_act(int action)
 	case SAVEGAME_LOAD:
 		if (LoadGame() == OK) {
 			GetEventTriggers("freedroid.events");
-			Item_Held_In_Hand = NULL;
+			item_held_in_hand = NULL;
 			rtn = TRUE;
 		} else {
 			rtn = FALSE;
@@ -2359,10 +2204,9 @@ int Single_Player_Menu(void)
 			if (PrepareNewHero() == TRUE) {
 				char fp[2048];
 				find_file("freedroid.levels", MAP_DIR, fp, 0);
-				LoadShip(fp);
+				LoadShip(fp, 0);
 				PrepareStartOfNewCharacter("NewTuxStartGameSquare");
 				can_continue = TRUE;
-				load_game_command_came_from_inside_running_game = TRUE;
 				return (TRUE);
 			} else {
 				can_continue = FALSE;
