@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (c) 2007 Arthur Huillet
+ *   Copyright (c) 2010 Arthur Huillet
  *   Copyright (c) 2003 Johannes Prix
  *
  *
@@ -106,7 +106,7 @@ int our_SDL_blit_surface_wrapper(SDL_Surface * src, SDL_Rect * srcrect, SDL_Surf
 			bytes = src->pitch / src->w;
 
 			glDisable(GL_TEXTURE_2D);
-			glEnable(GL_ALPHA_TEST);
+			glEnable(GL_BLEND);
 
 			switch (bytes) {
 			case 4:
@@ -127,7 +127,7 @@ int our_SDL_blit_surface_wrapper(SDL_Surface * src, SDL_Rect * srcrect, SDL_Surf
 					     IS_WARNING_ONLY);
 			}
 
-			glDisable(GL_ALPHA_TEST);
+			glDisable(GL_BLEND);
 			glEnable(GL_TEXTURE_2D);
 			return (0);
 		}
@@ -498,7 +498,6 @@ void make_texture_out_of_surface(iso_image * our_image)
 
 	SDL_Surface *right_sized_image;
 
-	//--------------------
 	// This fills up the image with transparent material, so that 
 	// it will have powers of 2 as the dimensions, which is a requirement
 	// for textures on most OpenGL capable cards.
@@ -509,14 +508,12 @@ void make_texture_out_of_surface(iso_image * our_image)
 	our_image->original_image_width = our_image->surface->w;
 	our_image->original_image_height = our_image->surface->h;
 
-	//--------------------
 	// Having prepared the raw image it's now time to create the real
 	// textures.
 	//
 	do_make_texture_out_of_surface(our_image, right_sized_image->w, right_sized_image->h, right_sized_image->pixels);
 	SDL_FreeSurface(right_sized_image);
 
-	//--------------------
 	// Now that the texture has been created, we assume that the image is
 	// not needed any more and can be freed now!  
 
@@ -550,7 +547,6 @@ void open_gl_check_error_status(const char *name_of_calling_function)
 
 	switch (glGetError()) {
 	case GL_NO_ERROR:
-		//--------------------
 		// All is well.  No messages need to be generated...
 		break;
 	case GL_INVALID_ENUM:
@@ -643,7 +639,6 @@ void safely_set_some_open_gl_flags_and_shade_model(void)
 int safely_initialize_our_default_open_gl_parameters(void)
 {
 #ifdef HAVE_LIBGL
-	//--------------------
 	// Set up the screne, viewport matrix, coordinate system and all that...
 	//
 	safely_set_open_gl_viewport_and_matrix_mode();
@@ -699,7 +694,7 @@ draw_gl_textured_quad_at_map_position(iso_image * our_iso_image,
 		glEnable(GL_ALPHA_TEST);
 	}
 
-	translate_map_point_to_screen_pixel(our_col, our_line, &x, &y, zoom_factor);
+	translate_map_point_to_screen_pixel(our_col, our_line, &x, &y);
 	x += our_iso_image->offset_x * zoom_factor;
 	y += our_iso_image->offset_y * zoom_factor;
 
@@ -716,7 +711,6 @@ draw_gl_textured_quad_at_map_position(iso_image * our_iso_image,
 		glEnable(GL_BLEND);
 		glBlendFunc(GL_ONE, GL_ONE);
 
-		//--------------------
 		// Now we draw our quad AGAIN!
 		//
 		draw_gl_textured_quad_helper(x, y, x + our_iso_image->original_image_width * zoom_factor,
@@ -808,23 +802,6 @@ void draw_gl_bg_textured_quad_at_screen_position(iso_image * our_floor_iso_image
 
 }
 
-/**
- *  draws a textured quad at the given screen position, after 
- * setting blend mode to additive
- */
-void blit_semitransparent_open_gl_texture_to_screen_position(iso_image * our_floor_iso_image, int x, int y, float scale_factor)
-{
-
-#ifdef HAVE_LIBGL
-
-	glBlendFunc(GL_ONE, GL_ONE);
-	draw_gl_scaled_textured_quad_at_screen_position(our_floor_iso_image, x, y, scale_factor);
-	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-
-#endif
-
-};				// void blit_semitransparent_open_gl_texture_to_screen_position ( ... )
-
 /**
  * This function restores the menu background, that must have been stored
  * before using the function of similar name.
@@ -902,7 +879,6 @@ void StoreMenuBackground(int backup_slot)
 
 #endif
 	} else {
-		//--------------------
 		// If the memory was not yet allocated, we need to do that now...
 		//
 		// otherwise we free the old surface and create a new copy of the
@@ -930,27 +906,23 @@ void set_up_stretched_texture_for_light_radius(void)
 
 	static int texture_is_set_up_already = FALSE;
 
-	//--------------------
 	// In the non-open-gl case, this function shouldn't be called ever....
 	//
 	if (!use_open_gl)
 		return;
 
-	//--------------------
 	// Some protection against creating this texture twice...
 	//
 	if (texture_is_set_up_already)
 		return;
 	texture_is_set_up_already = TRUE;
 
-	//--------------------
 	// We create an SDL surface, so that we can make the texture for the
 	// stretched-texture method light radius from it...
 	//
 	light_radius_stretch_surface =
 	    SDL_CreateRGBSurface(SDL_SWSURFACE, LightRadiusConfig.texture_w, LightRadiusConfig.texture_h, 32, rmask, gmask, bmask, amask);
 
-	//--------------------
 	// Having prepared the raw image it's now time to create the real
 	// textures.
 	//
@@ -987,7 +959,6 @@ void light_radius_update_stretched_texture(void)
 	int alpha = 0;
 	int light_strength;
 	static float alpha_factor = 255.0 / (float)NUMBER_OF_SHADOW_IMAGES;
-	//--------------------
 	// Now it's time to edit the automap texture.
 	//
 	for (y = 0; y < LightRadiusConfig.cells_h; y++) {
@@ -1028,7 +999,6 @@ void blit_open_gl_stretched_texture_light_radius(int decay_x, int decay_y)
 #ifdef HAVE_LIBGL
 	iso_image local_iso_image;
 
-	//--------------------
 	// We make sure, that there is one single texture created before
 	// doing any of our texture-blitting or texture-modification stuff
 	// with it.
@@ -1037,7 +1007,6 @@ void blit_open_gl_stretched_texture_light_radius(int decay_x, int decay_y)
 
 	light_radius_update_stretched_texture();
 
-	//--------------------
 	// Now we blit the current automap texture to the screen.  We use standard
 	// texture blitting code for this, so we need to embed the automap texture
 	// in a surrounting 'iso_image', but that shouldn't be costly or anything...
@@ -1085,36 +1054,33 @@ void PutPixel_open_gl(int x, int y, Uint32 pixel)
 };				// void PutPixel_open_gl ( x , y , pixel ) ;
 
 /**
- * On various occasions, like inside the shop interface or in the level
- * editor tile selection bar, it's nescessary to mark an item as currently
- * selected.  We do this by drawing a part-transparent white rectangle 
- * right over it.  This function can be used to draw that rectangle.
+ * Draw a colored rectangle on screen in OpenGL.
+ *
+ * @param rect The rectangular area.
+ * @param r The red color value.
+ * @param g The green color value.
+ * @param b The blue color value.
+ * @param a The alpha color value.
  */
-void
-GL_HighlightRectangle(SDL_Surface * Surface, SDL_Rect * dstrect, unsigned char r, unsigned char g, unsigned char b, unsigned char alpha)
+void gl_draw_rectangle(SDL_Rect *rect, int r, int g, int b, int a)
 {
 #ifdef HAVE_LIBGL
-
 	glDisable(GL_TEXTURE_2D);
 	glEnable(GL_BLEND);
 
-	glColor4ub(r, g, b, alpha);
+	glColor4ub(r, g, b, a);
 
 	glBegin(GL_QUADS);
-	glVertex2i(dstrect->x, dstrect->y);
-	glVertex2i(dstrect->x, dstrect->y + dstrect->h);
-	glVertex2i(dstrect->x + dstrect->w, dstrect->y + dstrect->h);
-	glVertex2i(dstrect->x + dstrect->w, dstrect->y);
+	glVertex2i(rect->x, rect->y);
+	glVertex2i(rect->x, rect->y + rect->h);
+	glVertex2i(rect->x + rect->w, rect->y + rect->h);
+	glVertex2i(rect->x + rect->w, rect->y);
 	glEnd();
 
 	glEnable(GL_TEXTURE_2D);
 	glDisable(GL_BLEND);
-
 #endif
-
-	return;
-
-};				// void GL_HighlightRectangle
+}
 
 #define CHARACTER_SCREEN_BACKGROUND_FILE "backgrounds/character.png"
 #define SKILL_SCREEN_BACKGROUND_FILE "backgrounds/SkillScreen.png"
@@ -1149,8 +1115,10 @@ GL_HighlightRectangle(SDL_Surface * Surface, SDL_Rect * dstrect, unsigned char r
 #define NUMBER_SELECTOR_BACKGROUND_IMAGE_FILE "backgrounds/number_selector.png"
 #define GAME_MESSAGE_WINDOW_BACKGROUND_IMAGE_FILE "backgrounds/game_message_window.png"
 #define HUD_BACKGROUND_IMAGE_FILE "backgrounds/hud_background.png"
+#define ITEM_UPGRADE_BACKGROUND_IMAGE_FILE "item_upgrade/background.png"
+#define ADDON_CRAFTING_BACKGROUND_IMAGE_FILE "item_upgrade/background_crafting.png"
 
-#define ALL_KNOWN_BACKGROUNDS 33
+#define ALL_KNOWN_BACKGROUNDS 35
 static iso_image our_backgrounds[ALL_KNOWN_BACKGROUNDS];
 static int background_has_been_loaded[ALL_KNOWN_BACKGROUNDS];
 
@@ -1204,6 +1172,8 @@ void blit_special_background(int background_code)
 		NUMBER_SELECTOR_BACKGROUND_IMAGE_FILE,	// 30
 		GAME_MESSAGE_WINDOW_BACKGROUND_IMAGE_FILE,	// 31
 		HUD_BACKGROUND_IMAGE_FILE,	// 32
+		ITEM_UPGRADE_BACKGROUND_IMAGE_FILE,	// 33
+		ADDON_CRAFTING_BACKGROUND_IMAGE_FILE,	// 33
 	};
 
 	static int scaling_done[ALL_KNOWN_BACKGROUNDS];
@@ -1245,7 +1215,9 @@ void blit_special_background(int background_code)
 		{0, 0, 0, 0},	// 29
 		{0, 0, 0, 0},	// 30
 		{(65 * GameConfig.screen_width) / 640, GameConfig.screen_height - (70 * GameConfig.screen_height) / 480, 500, 70},	// 31
-		{(0 * GameConfig.screen_width) / 640, GameConfig.screen_height - (118 * GameConfig.screen_height) / 480, 500, 70}	// 32
+		{(0 * GameConfig.screen_width) / 640, GameConfig.screen_height - (118 * GameConfig.screen_height) / 480, 500, 70},	// 32
+		{ITEM_UPGRADE_RECT_X, ITEM_UPGRADE_RECT_Y, ITEM_UPGRADE_RECT_W, ITEM_UPGRADE_RECT_H},	// 33
+		{ADDON_CRAFTING_RECT_X, ADDON_CRAFTING_RECT_Y, ADDON_CRAFTING_RECT_W, ADDON_CRAFTING_RECT_H}	// 34
 	};
 
 	const int need_scaling[ALL_KNOWN_BACKGROUNDS] = {
@@ -1282,6 +1254,7 @@ void blit_special_background(int background_code)
 		TRUE,		// 30
 		TRUE,		// 31
 		TRUE,		// 32
+		FALSE,		// 33
 	};
 
 	if (!background_has_been_loaded[background_code]) {
@@ -1290,7 +1263,6 @@ void blit_special_background(int background_code)
 		find_file(background_filenames[background_code], GRAPHICS_DIR, fpath, 0);
 		get_iso_image_from_file_and_path(fpath, &(our_backgrounds[background_code]), FALSE);
 
-		//--------------------
 		// For the dialog, we need not only the dialog background, but also some smaller
 		// parts of the background image, so we can re-do the background part that is in
 		// the dialog partners chat output window.  We don't make a separate image on disk
@@ -1304,7 +1276,6 @@ void blit_special_background(int background_code)
 			src_rect.x = CHAT_SUBDIALOG_WINDOW_X;
 			src_rect.w = CHAT_SUBDIALOG_WINDOW_W;
 			src_rect.h = CHAT_SUBDIALOG_WINDOW_H;
-			//--------------------
 			// With OpenGL, the image is flipped at this point already, so we
 			// just copy the image in flipped form, cause later it should be 
 			// flipped anyway.  Cool, eh?  Of course this way only the location
