@@ -1,7 +1,7 @@
 /* 
  *
  *   Copyright (c) 2002, 2003 Johannes Prix
- *   Copyright (c) 2004-2008 Arthur Huillet 
+ *   Copyright (c) 2004-2010 Arthur Huillet 
  *
  *  This file is part of Freedroid
  *
@@ -49,13 +49,11 @@
 #define SAVEDGAME_EXT ".savegame"
 #define SAVE_GAME_THUMBNAIL_EXT ".thumbnail.bmp"
 
-int load_game_command_came_from_inside_running_game = FALSE;
-
 jmp_buf saveload_jmpbuf;
 
 #define WrapErrorMessage(a, b, c, d, ...) do { \
 	ErrorMessage(a,b,c,IS_WARNING_ONLY, ##__VA_ARGS__);\
-    	GiveMouseAlertWindow(_("An error occured when trying to load the savegame.\nA common reason for this is that the game has been updated to a newer version since the save was made, in which case the savegame is very likely not compatible.\nHowever if you see this message and you have not updated the game, make sure to report this to the developers.\nThanks!"));\
+    	alert_window(_("An error occured when trying to load the savegame.\nA common reason for this is that FreedroidRPG has been updated to a newer version since the save was made, in which case the savegame is very likely not compatible.\nIf you see this message and you have not updated the game, make sure to report this to the developers.\nThanks!"));\
         if (d==IS_FATAL)\
     	    longjmp(saveload_jmpbuf, 1);\
 } while (0)
@@ -107,7 +105,6 @@ void LoadAndShowStats(char *CoreFilename)
 
 	DebugPrintf(2, "\nTrying to get file stats for character '%s'. ", CoreFilename);
 
-	//--------------------
 	// First we save the full ship information, same as with the level editor
 	//
 
@@ -128,13 +125,11 @@ or file permissions of ~/.freedroid_rpg are somehow not right.", NO_NEED_TO_INFO
 	PutString(Screen, UNIVERSAL_COORD_W(240), GameConfig.screen_height - 3 * FontHeight(GetCurrentFont()), _("Last Modified:"));
 	PutString(Screen, UNIVERSAL_COORD_W(240), GameConfig.screen_height - 2 * FontHeight(GetCurrentFont()), InfoString);
 
-	//--------------------
 	// Now that the modification time has been set up, we can start to compute
 	// the overall disk space of all files in question.
 	//
 	FileSize = FileInfoBuffer.st_size;
 
-	//--------------------
 	// The saved ship must exist.  On not, it's a sever error!
 	//    
 	sprintf(filename, "%s/%s%s", our_config_dir, CoreFilename, ".shp");
@@ -148,7 +143,6 @@ or file permissions of ~/.freedroid_rpg are somehow not right.", NO_NEED_TO_INFO
 	}
 	FileSize += FileInfoBuffer.st_size;
 
-	//--------------------
 	// A thumbnail may not yet exist.  We won't make much fuss if it doesn't.
 	//
 	sprintf(filename, "%s/%s%s", our_config_dir, CoreFilename, SAVE_GAME_THUMBNAIL_EXT);
@@ -174,7 +168,6 @@ void SaveThumbnailOfGame(void)
 	if (!our_config_dir)
 		return;
 
-	//--------------------
 	// First we save the full ship information, same as with the level editor
 	//
 	sprintf(filename, "%s/%s%s", our_config_dir, Me.character_name, SAVE_GAME_THUMBNAIL_EXT);
@@ -184,12 +177,10 @@ void SaveThumbnailOfGame(void)
 	if (use_open_gl) {
 #ifdef HAVE_LIBGL
 		SDL_Surface *FullView;
-		//--------------------
 		// We need to make a copy in processor memory. 
 		GLvoid *imgdata = malloc((GameConfig.screen_width + 2) * (GameConfig.screen_height + 2) * 4);
 		glReadPixels(0, 1, GameConfig.screen_width, GameConfig.screen_height - 1, GL_RGB, GL_UNSIGNED_BYTE, imgdata);
 
-		//--------------------
 		// Now we need to make a real SDL surface from the raw image data we
 		// have just extracted.
 		//
@@ -229,7 +220,7 @@ int SaveGame(void)
 	FILE *SaveGameFile;
 
 	if (Me.energy <= 0) {
-		GiveMouseAlertWindow(_("You are dead. Savegame not modified."));
+		alert_window(_("You are dead. Savegame not modified."));
 		return (ERR);
 	}
 
@@ -258,7 +249,7 @@ int SaveGame(void)
 	CenteredPutStringFont(Screen, Menu_BFont, 10, _("Saving"));
 	our_SDL_flip_wrapper();
 
-	if (SaveShip(filename, FALSE) != OK) {
+	if (SaveShip(filename, FALSE, 1) != OK) {
 		ErrorMessage(__FUNCTION__, "\
 The SAVING OF THE SHIP DATA FOR THE SAVED GAME FAILED!\n\
 This is either a bug in Freedroid or an indication, that the directory\n\
@@ -322,6 +313,7 @@ or file permissions of ~/.freedroid_rpg are somehow not right.", PLEASE_INFORM,
 		save_melee_shot(str, &AllMeleeShots[i]);
 	}
 
+	save_factions(savestruct_autostr);
 	autostr_append(savestruct_autostr, "End of freedroidRPG savefile\n");
 	deflate_to_stream((unsigned char *)savestruct_autostr->value, savestruct_autostr->length+1, SaveGameFile);
 	fclose(SaveGameFile);
@@ -347,14 +339,12 @@ int DeleteGame(void)
 	if (!our_config_dir)
 		return (OK);
 
-	//--------------------
 	// First we save the full ship information, same as with the level editor
 	//
 	sprintf(filename, "%s/%s%s", our_config_dir, Me.character_name, ".shp");
 
 	remove(filename);
 
-	//--------------------
 	// First, we must determine the savedgame data file name
 	//
 	sprintf(filename, "%s/%s%s", our_config_dir, Me.character_name, SAVEDGAME_EXT);
@@ -379,32 +369,36 @@ int LoadBackupGame()
 	return ret;
 }
 
-static void load_enemies(char *LoadGameData)
+/**
+ * Load all enemies from a savegame
+ */
+static void load_enemies(char *game_data)
 {
-	ClearEnemys();
-
-	enemy *newen;
+	enemy one_enemy;
 	int done;
 	int a = 0, i;
+
+	clear_enemies();
+
 	for (i = 0; i < 2; i++) {
-		char *cpos = LoadGameData;
+		char *cpos = game_data;
 		done = 0;
 		while (!done) {
-			newen = (enemy *) calloc(1, sizeof(enemy));
 			char str[25];
 			sprintf(str, "%s enemy %d", i ? "dead" : "alive", a);
 			cpos = strstr(cpos, str);
 			if (!cpos) {
 				done = 1;
-				free(newen);
 				break;
 			}
 			cpos -= 5;
-			if (read_enemy(cpos, str, newen)) {
+			memset(&one_enemy, 0, sizeof(enemy)); // all pointers have to be zeroed before to call read_xxx() functions
+			if (read_enemy(cpos, str, &one_enemy)) {
 				done = 1;
-				free(newen);
 			} else {
-				list_add(&(newen->global_list), i ? &dead_bots_head : &alive_bots_head);
+				enemy *newen = enemy_new(one_enemy.type);
+				memcpy(newen, &one_enemy, sizeof(enemy));
+				enemy_insert_into_lists(newen, (i==0));
 			}
 			a++;
 		}
@@ -484,8 +478,7 @@ int LoadGame(void)
 	sprintf(filename, "%s/%s%s", our_config_dir, Me.character_name, ".shp");
 
 	if ((DataFile = fopen(filename, "rb")) == NULL) {
-		GiveMouseAlertWindow(_
-				     ("\nW A R N I N G !\n\nFreedroidRPG was unable to locate the saved game file you requested to load.\nThis might mean that it really isn't there cause you tried to load a game without ever having saved the game before.  \nThe other explanation of this error might be a severe error in FreedroidRPG.\nNothing will be done about it."));
+		alert_window(_("W A R N I N G !\n\nFreedroidRPG was unable to locate the saved game file you requested to load.\nThis might mean that it really isn't there cause you tried to load a game without ever having saved the game before.\nThe other explanation of this error might be a severe error in FreedroidRPG. If you think this is the case, please report this to the developers."));
 		append_new_game_message(_("Failed to load old game."));
 		return (ERR);
 	} else {
@@ -499,14 +492,14 @@ int LoadGame(void)
 		return (ERR);
 	}
 
-	LoadShip(filename);
+	LoadShip(filename, 1);
 
 	sprintf(filename, "%s/%s%s", our_config_dir, Me.character_name, SAVEDGAME_EXT);
 
 	DataFile = fopen(filename, "rb");
 	if (inflate_stream(DataFile, (unsigned char **)&LoadGameData, NULL)) {
 		fclose(DataFile);
-		GiveMouseAlertWindow("Unable to decompress saved game - this is probably an old, incompatible game. Sorry.\n");
+		alert_window("Unable to decompress saved game - this is probably an old, incompatible game. Sorry.");
 		return ERR;
 	}
 
@@ -515,21 +508,27 @@ int LoadGame(void)
 	memset(&Me, 0, sizeof(tux_t));
 	read_tux_t(LoadGameData, "player", &Me);
 
+	reset_visible_levels();
+	get_visible_levels();
+
 	load_enemies(LoadGameData);
-
 	load_npcs(LoadGameData);
-
 	load_bullets(LoadGameData);
+	load_factions(LoadGameData);
 
-	/* properly restore pointers and references */
-	DebugPrintf(SAVE_LOAD_GAME_DEBUG, "\n%s(): now correcting dangerous pointers....", __FUNCTION__);
-	Me.TextToBeDisplayed = "";
-	for (i = 0; i < 2; i++) {
-		enemy *erot, *n;
-		list_for_each_entry_safe(erot, n, i ? &alive_bots_head : &dead_bots_head, global_list) {
-			erot->TextToBeDisplayed = "";
-			erot->TextVisibleTime = 0;
-		}
+	/* post-loading: Some transient states have to be adapted */
+	enemy *erot;
+	BROWSE_DEAD_BOTS(erot) {
+		/*
+		 * It is sufficient to set ->animation_type, since ->animation_phase will be set to
+		 * last_death_animation_image[erot->type] inside animate_enemy();
+		 *
+		 * Also, we might not know the correct value of last_death_animation_image[] yet,
+		 * since it's initialized by grab_enemy_images_from_archive(erot->type), and that
+		 * function may not have been called for the current bot type.
+		 */
+		if (!level_is_visible(erot->pos.z))
+			erot->animation_type = DEAD_ANIMATION;
 	}
 
 	for (i = 0; i < MAXBULLETS; i++) {
@@ -539,30 +538,25 @@ int LoadGame(void)
 		VERSION, (int)sizeof(tux_t), (int)sizeof(enemy), (int)sizeof(bullet), (int)MAXBULLETS);
 
 	if (strcmp(Me.savegame_version_string, version_check_string) != 0) {
-		GiveMouseAlertWindow(_
-				     ("Version or structsize mismatch! The savegame is not from the same version of freedroidRPG... possible breakage.\n"));
+		alert_window(_("Version or structsize mismatch! The savegame is not from the same version of FreedroidRPG... possible breakage."));
 		our_SDL_flip_wrapper();
 		while (!MouseLeftPressed())
 			SDL_Delay(1);
 	}
-	//--------------------
 	// To prevent cheating, we remove all active spells, that might still be there
 	// from other games just played before.
 	//
 	clear_active_spells();
 
-	//--------------------
 	// Now that we have loaded the game, we must count and initialize the number
 	// of droids used in this ship.  Otherwise we might ignore some robots.
 	//
 	CountNumberOfDroidsOnShip();
-	enemy_generate_level_lists();
 
 	SwitchBackgroundMusicTo(curShip.AllLevels[Me.pos.z]->Background_Song_Name);
 
 	free((char *)LoadGameData);
 
-	//--------------------
 	// Maybe someone just lost in the game and has then pressed the load
 	// button.  Then a new game is loaded and the game-over status has
 	// to be restored as well of course.
@@ -571,7 +565,6 @@ int LoadGame(void)
 
 	DebugPrintf(SAVE_LOAD_GAME_DEBUG, "\n%s(): end of function reached.", __FUNCTION__);
 
-	//--------------------
 	// Now we know that right after loading an old saved game, the Tux might have
 	// to 'change clothes' i.e. a lot of tux images need to be updated which can
 	// take a little time.  Therefore we print some message so the user will not
@@ -580,15 +573,11 @@ int LoadGame(void)
 	PutStringFont(Screen, FPS_Display_BFont, 75, 150, _("Updating Tux images (this may take a little while...)"));
 	our_SDL_flip_wrapper();
 
-	load_game_command_came_from_inside_running_game = TRUE;
-	
-	reset_visible_levels();
-	get_visible_levels();
 	animation_timeline_reset();
-	
+
 	append_new_game_message(_("Game loaded."));
 	return OK;
-};				// int LoadGame ( void ) 
+}
 
 void read_enemy_ptr(const char *buffer, const char *tag, enemy ** val)
 {
@@ -728,17 +717,24 @@ void read_luacode(const char *buffer, const char *tag, luacode * val)
 }
 
 /* Save arrays of simple types */
-#define define_save_xxx_array(X) void save_##X##_array(const char * tag, X * val_ar, int size)\
+#define define_save_xxx_array(X) void save_##X##_array(const char *tag, X *val_ar, int size)\
 {\
-autostr_append(savestruct_autostr, "<%s array n=%d>\n", tag, size);\
-int i;\
-for ( i = 0; i < size; i ++)\
-	{\
-	char str[10];\
-	sprintf(str, "%i", i);\
-	save_##X(str, &val_ar[i]);\
+	autostr_append(savestruct_autostr, "<%s array n=%d>\n", tag, size);\
+	int i;\
+	for (i = 0; i < size; i++) {\
+		char str[strlen(tag) + 20];\
+		sprintf(str, "%s elem %d", tag, i);\
+		save_##X(str, &val_ar[i]);\
 	}\
-autostr_append(savestruct_autostr, "</%s>\n", tag);\
+	autostr_append(savestruct_autostr, "</%s>\n", tag);\
+}
+
+/* Save dynamic arrays of simple types.
+ * Requires you to first use define_save_xxx_array for the type.
+ */
+#define define_save_xxx_dynarray(X) void save_##X##_dynarray(const char *tag, X##_dynarray *array)\
+{\
+	save_##X##_array(tag, array->arr, array->size);\
 }
 
 define_save_xxx_array(int32_t);
@@ -751,44 +747,80 @@ define_save_xxx_array(uchar);
 define_save_xxx_array(char);
 define_save_xxx_array(string);
 define_save_xxx_array(mission);
+define_save_xxx_array(upgrade_socket);
+define_save_xxx_dynarray(upgrade_socket);
 define_save_xxx_array(item);
 define_save_xxx_array(gps);
 define_save_xxx_array(moderately_finepoint);
 
+/**
+ * \brief Reads the number of elements stored to a generic savestruct array.
+ * \param buffer String from which to read.
+ * \param tag Tag name of the array.
+ * \return The size of the array.
+ */
+int read_array_size(const char *buffer, const char *tag)
+{
+	// Find the beginning element.
+	char search[strlen(tag) + 20];
+	sprintf(search, "<%s array", tag);
+	char *pos = strstr(buffer, search);
+	if (!pos) WrapErrorMessage(__FUNCTION__, "Unable to find array %s\n", PLEASE_INFORM, IS_FATAL, tag);
+
+	// Find the size string.
+	char *runp = pos + 1;
+	runp += strlen(tag) + 1 + strlen("array") + 1;
+	while (!isdigit(*runp)) runp++;
+
+	// Convert it to an integer.
+	return atoi(runp);
+}
+
 /* Read arrays of simple types */
-#define define_read_xxx_array(X) void read_##X##_array(const char * buffer, const char * tag, X * val_ar, int size)\
+#define define_read_xxx_array(X) void read_##X##_array(char *buffer, const char *tag, X *val_ar, int size)\
 {\
-int i;\
-char search[strlen(tag) + 20];\
-sprintf(search, "<%s array", tag);\
-char * pos = strstr(buffer, search);\
-if ( ! pos ) WrapErrorMessage ( __FUNCTION__, "Unable to find array %s\n", PLEASE_INFORM, IS_FATAL, tag);\
-sprintf(search, "</%s>", tag);\
-char * epos = strstr(pos, search);\
-if ( ! epos ) WrapErrorMessage ( __FUNCTION__, "Unable to find array end %s\n", PLEASE_INFORM, IS_FATAL, tag);\
-epos += strlen(search);\
-char savechar = *(epos + 1);\
-*(epos+1) = '\0';\
-int nb = 0;\
-char *runp = pos + 1;\
-runp += strlen(tag) + 1 + strlen("array") + 1;\
-while ( ! isdigit(*runp) ) runp++;\
-char * erunp = runp;\
-while ( *erunp != '>' ) erunp ++;\
-*erunp = '\0';\
-nb = atoi(runp);\
-*erunp = '>';\
-if ( nb != size )  WrapErrorMessage ( __FUNCTION__, "Size mismatch for array %s, %d in file, %d in game\n", NO_NEED_TO_INFORM, IS_WARNING_ONLY, tag, nb, size);\
-for ( i = 0; i < (nb <= size ? nb : size); i ++)\
-	{\
-	char str[10];\
-	sprintf(str, "%d", i);\
-	pos = strstr(pos, str);\
-	if (!pos) break;\
-	pos -= 5;\
-	read_##X(pos, str, &val_ar[i]);\
+	int i;\
+	/* Find the beginning element. */\
+	char search[strlen(tag) + 20];\
+	sprintf(search, "<%s array", tag);\
+	char * pos = strstr(buffer, search);\
+	if (!pos) WrapErrorMessage(__FUNCTION__, "Unable to find array %s\n", PLEASE_INFORM, IS_FATAL, tag);\
+	sprintf(search, "</%s>", tag);\
+	/* Find the end element. */\
+	char * epos = strstr(pos, search);\
+	if ( ! epos ) WrapErrorMessage(__FUNCTION__, "Unable to find array end %s\n", PLEASE_INFORM, IS_FATAL, tag);\
+	epos += strlen(search);\
+	/* Check for valid size. */\
+	int nb = read_array_size(pos, tag);\
+	if ( nb != size ) WrapErrorMessage(__FUNCTION__, "Size mismatch for array %s, %d in file, %d in game\n", NO_NEED_TO_INFORM, IS_WARNING_ONLY, tag, nb, size);\
+	/* Overwrite the last character of the end element with NUL. */\
+	char savechar = *(epos + 1);\
+	*(epos+1) = '\0';\
+	/* Read array elements. */\
+	for (i = 0; i < (nb <= size ? nb : size); i++) {\
+		char str[strlen(tag) + 20];\
+		sprintf(str, "%s elem %d", tag, i);\
+		pos = strstr(pos, str);\
+		if (!pos) break;\
+		pos -= 5;\
+		read_##X(pos, str, &val_ar[i]);\
 	}\
-*(epos+1) = savechar;\
+	/* Restore the last character of the end element. */\
+	*(epos+1) = savechar;\
+}
+
+/* Read dynamic arrays of simple types
+ * Requires you to first use define_read_xxx_array for the type.
+ */
+#define define_read_xxx_dynarray(X) void read_##X##_dynarray(char *buffer, const char *tag, X##_dynarray *array)\
+{\
+	/* Read the size. */\
+	int size = read_array_size(buffer, tag);\
+	/* Allocate space. */\
+	dynarray_init((struct dynarray *) array, size, sizeof(X));\
+	array->size = size;\
+	/* Read the array elements. */\
+	read_##X##_array(buffer, tag, array->arr, array->size);\
 }
 
 define_read_xxx_array(int32_t);
@@ -800,6 +832,8 @@ define_read_xxx_array(uchar);
 define_read_xxx_array(char);
 define_read_xxx_array(string);
 define_read_xxx_array(mission);
+define_read_xxx_array(upgrade_socket);
+define_read_xxx_dynarray(upgrade_socket);
 define_read_xxx_array(item);
 define_read_xxx_array(gps);
 define_read_xxx_array(moderately_finepoint);
@@ -853,12 +887,14 @@ void read_keybind_t_array(const char *buffer, const char *tag, keybind_t * kbs,
 	int tknb;
 	char *pos = strstr(buffer, "<keybinds cmd=");
 	if (!pos) {
-		ErrorMessage(__FUNCTION__, "Unable to find keybinds.\n", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
+		WrapErrorMessage(__FUNCTION__, "Unable to find keybinds.\n", NO_NEED_TO_INFORM, IS_WARNING_ONLY);
 		return;
 	}
 	char *epos = strstr(pos, "</keybinds>\n");
-	if (!epos)
-		ErrorMessage(__FUNCTION__, "Unable to find keybinds end.\n", PLEASE_INFORM, IS_FATAL);
+	if (!epos) {
+		WrapErrorMessage(__FUNCTION__, "Unable to find keybinds end.\n", PLEASE_INFORM, IS_WARNING_ONLY);
+		return;
+	}
 	char savechar = *epos;
 	*epos = '\0';
 	int mc = 0;
