@@ -9,18 +9,28 @@ int save_enemy(char * tag, enemy * target)
 autostr_append(savestruct_autostr, "<%s>\n",tag);
 save_int16_t("id", &(target->id));
 save_int16_t("type", &(target->type));
+save_char("SpecialForce", &(target->SpecialForce));
+save_int32_t("marker", &(target->marker));
+save_int16_t("max_distance_to_home", &(target->max_distance_to_home));
+save_string("dialog_section_name", &(target->dialog_section_name));
+save_string("short_description_text", &(target->short_description_text));
+save_int16_t("on_death_drop_item_code", &(target->on_death_drop_item_code));
+save_int32_t("faction", &(target->faction));
+save_char("will_rush_tux", &(target->will_rush_tux));
+save_int32_t("combat_state", &(target->combat_state));
+save_float("state_timeout", &(target->state_timeout));
+save_int16_t("CompletelyFixed", &(target->CompletelyFixed));
+save_char("has_been_taken_over", &(target->has_been_taken_over));
+save_char("follow_tux", &(target->follow_tux));
+save_char("has_greeted_influencer", &(target->has_greeted_influencer));
+save_int16_t("nextwaypoint", &(target->nextwaypoint));
+save_int16_t("lastwaypoint", &(target->lastwaypoint));
+save_int16_t("homewaypoint", &(target->homewaypoint));
 save_gps("pos", &(target->pos));
-save_gps("virt_pos", &(target->virt_pos));
 save_finepoint("speed", &(target->speed));
 save_float("energy", &(target->energy));
 save_float("animation_phase", &(target->animation_phase));
 save_int16_t("animation_type", &(target->animation_type));
-save_int16_t("nextwaypoint", &(target->nextwaypoint));
-save_int16_t("lastwaypoint", &(target->lastwaypoint));
-save_int16_t("homewaypoint", &(target->homewaypoint));
-save_int16_t("max_distance_to_home", &(target->max_distance_to_home));
-save_int32_t("combat_state", &(target->combat_state));
-save_float("state_timeout", &(target->state_timeout));
 save_float("frozen", &(target->frozen));
 save_float("poison_duration_left", &(target->poison_duration_left));
 save_float("poison_damage_per_sec", &(target->poison_damage_per_sec));
@@ -28,28 +38,18 @@ save_float("paralysation_duration_left", &(target->paralysation_duration_left));
 save_float("pure_wait", &(target->pure_wait));
 save_float("firewait", &(target->firewait));
 save_int16_t("ammo_left", &(target->ammo_left));
-save_int16_t("CompletelyFixed", &(target->CompletelyFixed));
-save_char("follow_tux", &(target->follow_tux));
-save_char("SpecialForce", &(target->SpecialForce));
-save_int16_t("on_death_drop_item_code", &(target->on_death_drop_item_code));
-save_int32_t("marker", &(target->marker));
-save_int16_t("is_friendly", &(target->is_friendly));
-save_char("has_been_taken_over", &(target->has_been_taken_over));
 save_char("attack_target_type", &(target->attack_target_type));
 save_int16_t("bot_target_n", &(target->bot_target_n));
-save_string("dialog_section_name", &(target->dialog_section_name));
-save_string("short_description_text", &(target->short_description_text));
-save_char("will_rush_tux", &(target->will_rush_tux));
-save_char("has_greeted_influencer", &(target->has_greeted_influencer));
 save_float("previous_angle", &(target->previous_angle));
 save_float("current_angle", &(target->current_angle));
-save_float("last_phase_change", &(target->last_phase_change));
 save_float("previous_phase", &(target->previous_phase));
+save_float("last_phase_change", &(target->last_phase_change));
 save_float("last_combat_step", &(target->last_combat_step));
 save_float("TextVisibleTime", &(target->TextVisibleTime));
 save_moderately_finepoint_array("PrivatePathway", (target->PrivatePathway), 5);
 save_char("bot_stuck_in_wall_at_previous_check", &(target->bot_stuck_in_wall_at_previous_check));
 save_float("time_since_previous_stuck_in_wall_check", &(target->time_since_previous_stuck_in_wall_check));
+save_gps("virt_pos", &(target->virt_pos));
 save_list_head_t("global_list", &(target->global_list));
 save_list_head_t("level_list", &(target->level_list));
 autostr_append(savestruct_autostr, "</%s>\n", tag);
@@ -70,18 +70,28 @@ int read_enemy(char* buffer, char * tag, enemy * target)
 		*epos = 0;
 		read_int16_t(pos, "id",  &(target->id));
 read_int16_t(pos, "type",  &(target->type));
+read_char(pos, "SpecialForce",  &(target->SpecialForce));
+read_int32_t(pos, "marker",  &(target->marker));
+read_int16_t(pos, "max_distance_to_home",  &(target->max_distance_to_home));
+read_string(pos, "dialog_section_name",  &(target->dialog_section_name));
+read_string(pos, "short_description_text",  &(target->short_description_text));
+read_int16_t(pos, "on_death_drop_item_code",  &(target->on_death_drop_item_code));
+read_int32_t(pos, "faction",  &(target->faction));
+read_char(pos, "will_rush_tux",  &(target->will_rush_tux));
+read_int32_t(pos, "combat_state",  &(target->combat_state));
+read_float(pos, "state_timeout",  &(target->state_timeout));
+read_int16_t(pos, "CompletelyFixed",  &(target->CompletelyFixed));
+read_char(pos, "has_been_taken_over",  &(target->has_been_taken_over));
+read_char(pos, "follow_tux",  &(target->follow_tux));
+read_char(pos, "has_greeted_influencer",  &(target->has_greeted_influencer));
+read_int16_t(pos, "nextwaypoint",  &(target->nextwaypoint));
+read_int16_t(pos, "lastwaypoint",  &(target->lastwaypoint));
+read_int16_t(pos, "homewaypoint",  &(target->homewaypoint));
 read_gps(pos, "pos",  &(target->pos));
-read_gps(pos, "virt_pos",  &(target->virt_pos));
 read_finepoint(pos, "speed",  &(target->speed));
 read_float(pos, "energy",  &(target->energy));
 read_float(pos, "animation_phase",  &(target->animation_phase));
 read_int16_t(pos, "animation_type",  &(target->animation_type));
-read_int16_t(pos, "nextwaypoint",  &(target->nextwaypoint));
-read_int16_t(pos, "lastwaypoint",  &(target->lastwaypoint));
-read_int16_t(pos, "homewaypoint",  &(target->homewaypoint));
-read_int16_t(pos, "max_distance_to_home",  &(target->max_distance_to_home));
-read_int32_t(pos, "combat_state",  &(target->combat_state));
-read_float(pos, "state_timeout",  &(target->state_timeout));
 read_float(pos, "frozen",  &(target->frozen));
 read_float(pos, "poison_duration_left",  &(target->poison_duration_left));
 read_float(pos, "poison_damage_per_sec",  &(target->poison_damage_per_sec));
@@ -89,28 +99,20 @@ read_float(pos, "paralysation_duration_left",  &(target->paralysation_duration_l
 read_float(pos, "pure_wait",  &(target->pure_wait));
 read_float(pos, "firewait",  &(target->firewait));
 read_int16_t(pos, "ammo_left",  &(target->ammo_left));
-read_int16_t(pos, "CompletelyFixed",  &(target->CompletelyFixed));
-read_char(pos, "follow_tux",  &(target->follow_tux));
-read_char(pos, "SpecialForce",  &(target->SpecialForce));
-read_int16_t(pos, "on_death_drop_item_code",  &(target->on_death_drop_item_code));
-read_int32_t(pos, "marker",  &(target->marker));
-read_int16_t(pos, "is_friendly",  &(target->is_friendly));
-read_char(pos, "has_been_taken_over",  &(target->has_been_taken_over));
 read_char(pos, "attack_target_type",  &(target->attack_target_type));
 read_int16_t(pos, "bot_target_n",  &(target->bot_target_n));
-read_string(pos, "dialog_section_name",  &(target->dialog_section_name));
-read_string(pos, "short_description_text",  &(target->short_description_text));
-read_char(pos, "will_rush_tux",  &(target->will_rush_tux));
-read_char(pos, "has_greeted_influencer",  &(target->has_greeted_influencer));
+target->bot_target_addr = NULL;
 read_float(pos, "previous_angle",  &(target->previous_angle));
 read_float(pos, "current_angle",  &(target->current_angle));
-read_float(pos, "last_phase_change",  &(target->last_phase_change));
 read_float(pos, "previous_phase",  &(target->previous_phase));
+read_float(pos, "last_phase_change",  &(target->last_phase_change));
 read_float(pos, "last_combat_step",  &(target->last_combat_step));
 read_float(pos, "TextVisibleTime",  &(target->TextVisibleTime));
+target->TextToBeDisplayed = NULL;
 read_moderately_finepoint_array(pos, "PrivatePathway",  (target->PrivatePathway), 5);
 read_char(pos, "bot_stuck_in_wall_at_previous_check",  &(target->bot_stuck_in_wall_at_previous_check));
 read_float(pos, "time_since_previous_stuck_in_wall_check",  &(target->time_since_previous_stuck_in_wall_check));
+read_gps(pos, "virt_pos",  &(target->virt_pos));
 read_list_head_t(pos, "global_list",  &(target->global_list));
 read_list_head_t(pos, "level_list",  &(target->level_list));
 *epos = '>'; 
@@ -135,14 +137,13 @@ save_int16_t("owner", &(target->owner));
 save_float("angle", &(target->angle));
 save_char("was_reflected", &(target->was_reflected));
 save_char("ignore_wall_collisions", &(target->ignore_wall_collisions));
-save_int16_t("to_hit", &(target->to_hit));
 save_char("pass_through_hit_bodies", &(target->pass_through_hit_bodies));
 save_char("pass_through_explosions", &(target->pass_through_explosions));
 save_int16_t("freezing_level", &(target->freezing_level));
 save_float("poison_duration", &(target->poison_duration));
 save_float("poison_damage_per_sec", &(target->poison_damage_per_sec));
 save_float("paralysation_duration", &(target->paralysation_duration));
-save_char("is_friendly", &(target->is_friendly));
+save_int32_t("faction", &(target->faction));
 save_char("hit_type", &(target->hit_type));
 autostr_append(savestruct_autostr, "</%s>\n", tag);
 return 0;
@@ -175,19 +176,45 @@ read_int16_t(pos, "owner",  &(target->owner));
 read_float(pos, "angle",  &(target->angle));
 read_char(pos, "was_reflected",  &(target->was_reflected));
 read_char(pos, "ignore_wall_collisions",  &(target->ignore_wall_collisions));
-read_int16_t(pos, "to_hit",  &(target->to_hit));
 read_char(pos, "pass_through_hit_bodies",  &(target->pass_through_hit_bodies));
 read_char(pos, "pass_through_explosions",  &(target->pass_through_explosions));
 read_int16_t(pos, "freezing_level",  &(target->freezing_level));
 read_float(pos, "poison_duration",  &(target->poison_duration));
 read_float(pos, "poison_damage_per_sec",  &(target->poison_damage_per_sec));
 read_float(pos, "paralysation_duration",  &(target->paralysation_duration));
-read_char(pos, "is_friendly",  &(target->is_friendly));
+read_int32_t(pos, "faction",  &(target->faction));
 read_char(pos, "hit_type",  &(target->hit_type));
 *epos = '>'; 
 return 0;
 }
 
+int save_upgrade_socket(char * tag, upgrade_socket * target)
+{
+autostr_append(savestruct_autostr, "<%s>\n",tag);
+save_int32_t("type", &(target->type));
+save_string("addon", &(target->addon));
+autostr_append(savestruct_autostr, "</%s>\n", tag);
+return 0;
+}
+
+int read_upgrade_socket(char* buffer, char * tag, upgrade_socket * target)
+{
+
+		char search[strlen(tag) + 5];
+		sprintf(search, "<%s>", tag);
+	        char * pos = strstr(buffer, search);
+		if ( ! pos ) return 1;
+		pos += 1 + strlen(tag);
+		sprintf(search, "</%s>", tag);
+		char * epos = strstr(buffer, search);
+		if ( ! epos ) return 2;
+		*epos = 0;
+		read_int32_t(pos, "type",  &(target->type));
+read_string(pos, "addon",  &(target->addon));
+*epos = '>'; 
+return 0;
+}
+
 int save_point(char * tag, point * target)
 {
 autostr_append(savestruct_autostr, "<%s>\n",tag);
@@ -308,7 +335,8 @@ save_string("dialog_basename", &(target->dialog_basename));
 save_uchar("chat_character_initialized", &(target->chat_character_initialized));
 save_uchar_array("chat_flags", (target->chat_flags), MAX_ANSWERS_PER_PERSON);
 save_string_array("shoplist", (target->shoplist), MAX_ITEMS_IN_INVENTORY);
-save_item_array("npc_inventory", (target->npc_inventory), MAX_ITEMS_IN_INVENTORY);
+save_int32_t_array("shoplistweight", (target->shoplistweight), MAX_ITEMS_IN_INVENTORY);
+save_item_array("npc_inventory", (target->npc_inventory), MAX_ITEMS_IN_NPC_INVENTORY);
 save_float("last_trading_date", &(target->last_trading_date));
 save_list_head_t("node", &(target->node));
 autostr_append(savestruct_autostr, "</%s>\n", tag);
@@ -331,7 +359,8 @@ int read_npc(char* buffer, char * tag, npc * target)
 read_uchar(pos, "chat_character_initialized",  &(target->chat_character_initialized));
 read_uchar_array(pos, "chat_flags",  (target->chat_flags), MAX_ANSWERS_PER_PERSON);
 read_string_array(pos, "shoplist",  (target->shoplist), MAX_ITEMS_IN_INVENTORY);
-read_item_array(pos, "npc_inventory",  (target->npc_inventory), MAX_ITEMS_IN_INVENTORY);
+read_int32_t_array(pos, "shoplistweight",  (target->shoplistweight), MAX_ITEMS_IN_INVENTORY);
+read_item_array(pos, "npc_inventory",  (target->npc_inventory), MAX_ITEMS_IN_NPC_INVENTORY);
 read_float(pos, "last_trading_date",  &(target->last_trading_date));
 read_list_head_t(pos, "node",  &(target->node));
 *epos = '>'; 
@@ -346,6 +375,7 @@ save_int32_t("current_power_bonus", &(target->current_power_bonus));
 save_float("power_bonus_end_date", &(target->power_bonus_end_date));
 save_int32_t("current_dexterity_bonus", &(target->current_dexterity_bonus));
 save_float("dexterity_bonus_end_date", &(target->dexterity_bonus_end_date));
+save_float("light_bonus_end_date", &(target->light_bonus_end_date));
 save_finepoint("speed", &(target->speed));
 save_gps("pos", &(target->pos));
 save_gps("teleport_anchor", &(target->teleport_anchor));
@@ -367,8 +397,6 @@ save_int32_t("running_must_rest", &(target->running_must_rest));
 save_int32_t("running_power_bonus", &(target->running_power_bonus));
 save_float("health_recovery_rate", &(target->health_recovery_rate));
 save_float("cooling_rate", &(target->cooling_rate));
-save_int16_t("LastMouse_X", &(target->LastMouse_X));
-save_int16_t("LastMouse_Y", &(target->LastMouse_Y));
 save_double("busy_time", &(target->busy_time));
 save_int32_t("busy_type", &(target->busy_type));
 save_double("phase", &(target->phase));
@@ -389,13 +417,14 @@ save_int32_t("Vitality", &(target->Vitality));
 save_int32_t("points_to_distribute", &(target->points_to_distribute));
 save_float("base_damage", &(target->base_damage));
 save_float("damage_modifier", &(target->damage_modifier));
-save_float("DR", &(target->DR));
+save_float("armor_class", &(target->armor_class));
 save_float("to_hit", &(target->to_hit));
-save_int32_t("lv_1_bot_will_hit_percentage", &(target->lv_1_bot_will_hit_percentage));
 save_int32_t("resist_fire", &(target->resist_fire));
 save_int32_t("resist_electricity", &(target->resist_electricity));
-save_int32_t("freezing_melee_targets", &(target->freezing_melee_targets));
+save_int32_t("slowing_melee_targets", &(target->slowing_melee_targets));
+save_int32_t("paralyzing_melee_targets", &(target->paralyzing_melee_targets));
 save_int32_t("double_ranged_damage", &(target->double_ranged_damage));
+save_float("experience_factor", &(target->experience_factor));
 save_uint32_t("Experience", &(target->Experience));
 save_int32_t("exp_level", &(target->exp_level));
 save_uint32_t("ExpRequired", &(target->ExpRequired));
@@ -407,8 +436,7 @@ save_int32_t("marker", &(target->marker));
 save_float("LastCrysoundTime", &(target->LastCrysoundTime));
 save_float("TextVisibleTime", &(target->TextVisibleTime));
 save_int32_t("readied_skill", &(target->readied_skill));
-save_int32_t_array("SkillLevel", (target->SkillLevel), MAX_NUMBER_OF_PROGRAMS);
-save_int32_t_array("base_skill_level", (target->base_skill_level), MAX_NUMBER_OF_PROGRAMS);
+save_int32_t_array("skill_level", (target->skill_level), MAX_NUMBER_OF_PROGRAMS);
 save_int32_t("melee_weapon_skill", &(target->melee_weapon_skill));
 save_int32_t("ranged_weapon_skill", &(target->ranged_weapon_skill));
 save_int32_t("spellcasting_skill", &(target->spellcasting_skill));
@@ -421,11 +449,7 @@ save_item("special_item", &(target->special_item));
 save_uchar_array("HaveBeenToLevel", (target->HaveBeenToLevel), MAX_LEVELS);
 save_float_array("time_since_last_visit_or_respawn", (target->time_since_last_visit_or_respawn), MAX_LEVELS);
 save_string_array("cookie_list", (target->cookie_list), MAX_COOKIES);
-save_int32_t("is_town_guard_member", &(target->is_town_guard_member));
 save_moderately_finepoint_array("next_intermediate_point", (target->next_intermediate_point), MAX_INTERMEDIATE_WAYPOINTS_FOR_TUX);
-save_uint16_t_array("KillRecord", (target->KillRecord), 200);
-save_uint16_t_array("TakeoverSuccesses", (target->TakeoverSuccesses), 200);
-save_uint16_t_array("TakeoverFailures", (target->TakeoverFailures), 200);
 save_automap_data_t_array("Automap", (target->Automap), MAX_LEVELS);
 save_int32_t("current_zero_ring_index", &(target->current_zero_ring_index));
 save_gps_array("Position_History_Ring_Buffer", (target->Position_History_Ring_Buffer), MAX_INFLU_POSITION_HISTORY);
@@ -437,6 +461,11 @@ save_float("paralyze_duration", &(target->paralyze_duration));
 save_float("invisible_duration", &(target->invisible_duration));
 save_int32_t("quest_browser_changed", &(target->quest_browser_changed));
 save_int32_t_array("program_shortcuts", (target->program_shortcuts), 10);
+save_int32_t_array("TakeoverSuccesses", (target->TakeoverSuccesses), NB_DROID_TYPES);
+save_int32_t_array("TakeoverFailures", (target->TakeoverFailures), NB_DROID_TYPES);
+save_int32_t_array("destroyed_bots", (target->destroyed_bots), NB_DROID_TYPES);
+save_int32_t_array("damage_dealt", (target->damage_dealt), NB_DROID_TYPES);
+save_float("meters_traveled", &(target->meters_traveled));
 autostr_append(savestruct_autostr, "</%s>\n", tag);
 return 0;
 }
@@ -458,12 +487,14 @@ read_int32_t(pos, "current_power_bonus",  &(target->current_power_bonus));
 read_float(pos, "power_bonus_end_date",  &(target->power_bonus_end_date));
 read_int32_t(pos, "current_dexterity_bonus",  &(target->current_dexterity_bonus));
 read_float(pos, "dexterity_bonus_end_date",  &(target->dexterity_bonus_end_date));
+read_float(pos, "light_bonus_end_date",  &(target->light_bonus_end_date));
 read_finepoint(pos, "speed",  &(target->speed));
 read_gps(pos, "pos",  &(target->pos));
 read_gps(pos, "teleport_anchor",  &(target->teleport_anchor));
 read_gps(pos, "mouse_move_target",  &(target->mouse_move_target));
 read_int16_t(pos, "current_enemy_target_n",  &(target->current_enemy_target_n));
 read_char(pos, "god_mode",  &(target->god_mode));
+target->current_enemy_target_addr = NULL;
 read_int32_t(pos, "mouse_move_target_combo_action_type",  &(target->mouse_move_target_combo_action_type));
 read_int32_t(pos, "mouse_move_target_combo_action_parameter",  &(target->mouse_move_target_combo_action_parameter));
 read_int32_t(pos, "light_bonus_from_tux",  &(target->light_bonus_from_tux));
@@ -479,8 +510,6 @@ read_int32_t(pos, "running_must_rest",  &(target->running_must_rest));
 read_int32_t(pos, "running_power_bonus",  &(target->running_power_bonus));
 read_float(pos, "health_recovery_rate",  &(target->health_recovery_rate));
 read_float(pos, "cooling_rate",  &(target->cooling_rate));
-read_int16_t(pos, "LastMouse_X",  &(target->LastMouse_X));
-read_int16_t(pos, "LastMouse_Y",  &(target->LastMouse_Y));
 read_double(pos, "busy_time",  &(target->busy_time));
 read_int32_t(pos, "busy_type",  &(target->busy_type));
 read_double(pos, "phase",  &(target->phase));
@@ -501,13 +530,14 @@ read_int32_t(pos, "Vitality",  &(target->Vitality));
 read_int32_t(pos, "points_to_distribute",  &(target->points_to_distribute));
 read_float(pos, "base_damage",  &(target->base_damage));
 read_float(pos, "damage_modifier",  &(target->damage_modifier));
-read_float(pos, "DR",  &(target->DR));
+read_float(pos, "armor_class",  &(target->armor_class));
 read_float(pos, "to_hit",  &(target->to_hit));
-read_int32_t(pos, "lv_1_bot_will_hit_percentage",  &(target->lv_1_bot_will_hit_percentage));
 read_int32_t(pos, "resist_fire",  &(target->resist_fire));
 read_int32_t(pos, "resist_electricity",  &(target->resist_electricity));
-read_int32_t(pos, "freezing_melee_targets",  &(target->freezing_melee_targets));
+read_int32_t(pos, "slowing_melee_targets",  &(target->slowing_melee_targets));
+read_int32_t(pos, "paralyzing_melee_targets",  &(target->paralyzing_melee_targets));
 read_int32_t(pos, "double_ranged_damage",  &(target->double_ranged_damage));
+read_float(pos, "experience_factor",  &(target->experience_factor));
 read_uint32_t(pos, "Experience",  &(target->Experience));
 read_int32_t(pos, "exp_level",  &(target->exp_level));
 read_uint32_t(pos, "ExpRequired",  &(target->ExpRequired));
@@ -518,9 +548,9 @@ read_mission_array(pos, "AllMissions",  (target->AllMissions), MAX_MISSIONS_IN_G
 read_int32_t(pos, "marker",  &(target->marker));
 read_float(pos, "LastCrysoundTime",  &(target->LastCrysoundTime));
 read_float(pos, "TextVisibleTime",  &(target->TextVisibleTime));
+target->TextToBeDisplayed = NULL;
 read_int32_t(pos, "readied_skill",  &(target->readied_skill));
-read_int32_t_array(pos, "SkillLevel",  (target->SkillLevel), MAX_NUMBER_OF_PROGRAMS);
-read_int32_t_array(pos, "base_skill_level",  (target->base_skill_level), MAX_NUMBER_OF_PROGRAMS);
+read_int32_t_array(pos, "skill_level",  (target->skill_level), MAX_NUMBER_OF_PROGRAMS);
 read_int32_t(pos, "melee_weapon_skill",  &(target->melee_weapon_skill));
 read_int32_t(pos, "ranged_weapon_skill",  &(target->ranged_weapon_skill));
 read_int32_t(pos, "spellcasting_skill",  &(target->spellcasting_skill));
@@ -533,11 +563,7 @@ read_item(pos, "special_item",  &(target->special_item));
 read_uchar_array(pos, "HaveBeenToLevel",  (target->HaveBeenToLevel), MAX_LEVELS);
 read_float_array(pos, "time_since_last_visit_or_respawn",  (target->time_since_last_visit_or_respawn), MAX_LEVELS);
 read_string_array(pos, "cookie_list",  (target->cookie_list), MAX_COOKIES);
-read_int32_t(pos, "is_town_guard_member",  &(target->is_town_guard_member));
 read_moderately_finepoint_array(pos, "next_intermediate_point",  (target->next_intermediate_point), MAX_INTERMEDIATE_WAYPOINTS_FOR_TUX);
-read_uint16_t_array(pos, "KillRecord",  (target->KillRecord), 200);
-read_uint16_t_array(pos, "TakeoverSuccesses",  (target->TakeoverSuccesses), 200);
-read_uint16_t_array(pos, "TakeoverFailures",  (target->TakeoverFailures), 200);
 read_automap_data_t_array(pos, "Automap",  (target->Automap), MAX_LEVELS);
 read_int32_t(pos, "current_zero_ring_index",  &(target->current_zero_ring_index));
 read_gps_array(pos, "Position_History_Ring_Buffer",  (target->Position_History_Ring_Buffer), MAX_INFLU_POSITION_HISTORY);
@@ -549,6 +575,11 @@ read_float(pos, "paralyze_duration",  &(target->paralyze_duration));
 read_float(pos, "invisible_duration",  &(target->invisible_duration));
 read_int32_t(pos, "quest_browser_changed",  &(target->quest_browser_changed));
 read_int32_t_array(pos, "program_shortcuts",  (target->program_shortcuts), 10);
+read_int32_t_array(pos, "TakeoverSuccesses",  (target->TakeoverSuccesses), NB_DROID_TYPES);
+read_int32_t_array(pos, "TakeoverFailures",  (target->TakeoverFailures), NB_DROID_TYPES);
+read_int32_t_array(pos, "destroyed_bots",  (target->destroyed_bots), NB_DROID_TYPES);
+read_int32_t_array(pos, "damage_dealt",  (target->damage_dealt), NB_DROID_TYPES);
+read_float(pos, "meters_traveled",  &(target->meters_traveled));
 *epos = '>'; 
 return 0;
 }
@@ -560,32 +591,33 @@ save_gps("pos", &(target->pos));
 save_gps("virt_pos", &(target->virt_pos));
 save_sdl_rect("text_slot_rectangle", &(target->text_slot_rectangle));
 save_int32_t("type", &(target->type));
-save_int32_t("currently_held_in_hand", &(target->currently_held_in_hand));
-save_int32_t("is_identified", &(target->is_identified));
 save_int32_t("max_duration", &(target->max_duration));
 save_float("current_duration", &(target->current_duration));
 save_float("throw_time", &(target->throw_time));
-save_int32_t("prefix_code", &(target->prefix_code));
-save_int32_t("suffix_code", &(target->suffix_code));
 save_int32_t("bonus_to_dex", &(target->bonus_to_dex));
 save_int32_t("bonus_to_str", &(target->bonus_to_str));
-save_int32_t("bonus_to_vit", &(target->bonus_to_vit));
-save_int32_t("bonus_to_mag", &(target->bonus_to_mag));
-save_int32_t("bonus_to_life", &(target->bonus_to_life));
-save_int32_t("bonus_to_force", &(target->bonus_to_force));
+save_int32_t("bonus_to_physique", &(target->bonus_to_physique));
+save_int32_t("bonus_to_cooling", &(target->bonus_to_cooling));
+save_int32_t("bonus_to_health_points", &(target->bonus_to_health_points));
 save_float("bonus_to_health_recovery", &(target->bonus_to_health_recovery));
 save_float("bonus_to_cooling_rate", &(target->bonus_to_cooling_rate));
-save_int32_t("bonus_to_tohit", &(target->bonus_to_tohit));
+save_int32_t("bonus_to_attack", &(target->bonus_to_attack));
 save_int32_t("bonus_to_all_attributes", &(target->bonus_to_all_attributes));
-save_int32_t("bonus_to_dr_or_damage", &(target->bonus_to_dr_or_damage));
+save_int32_t("bonus_to_armor_class", &(target->bonus_to_armor_class));
+save_int32_t("bonus_to_damage", &(target->bonus_to_damage));
 save_int32_t("bonus_to_resist_fire", &(target->bonus_to_resist_fire));
 save_int32_t("bonus_to_resist_electricity", &(target->bonus_to_resist_electricity));
-save_int32_t("dr_bonus", &(target->dr_bonus));
+save_int32_t("bonus_to_paralyze_enemy", &(target->bonus_to_paralyze_enemy));
+save_int32_t("bonus_to_slow_enemy", &(target->bonus_to_slow_enemy));
+save_int32_t("bonus_to_light_radius", &(target->bonus_to_light_radius));
+save_int32_t("bonus_to_experience_gain", &(target->bonus_to_experience_gain));
+save_int32_t("armor_class", &(target->armor_class));
 save_int32_t("damage", &(target->damage));
 save_int32_t("damage_modifier", &(target->damage_modifier));
 save_int32_t("multiplicity", &(target->multiplicity));
 save_int32_t("ammo_clip", &(target->ammo_clip));
 save_point("inventory_position", &(target->inventory_position));
+save_upgrade_socket_dynarray("upgrade_sockets", &(target->upgrade_sockets));
 autostr_append(savestruct_autostr, "</%s>\n", tag);
 return 0;
 }
@@ -606,32 +638,33 @@ int read_item(char* buffer, char * tag, item * target)
 read_gps(pos, "virt_pos",  &(target->virt_pos));
 read_sdl_rect(pos, "text_slot_rectangle",  &(target->text_slot_rectangle));
 read_int32_t(pos, "type",  &(target->type));
-read_int32_t(pos, "currently_held_in_hand",  &(target->currently_held_in_hand));
-read_int32_t(pos, "is_identified",  &(target->is_identified));
 read_int32_t(pos, "max_duration",  &(target->max_duration));
 read_float(pos, "current_duration",  &(target->current_duration));
 read_float(pos, "throw_time",  &(target->throw_time));
-read_int32_t(pos, "prefix_code",  &(target->prefix_code));
-read_int32_t(pos, "suffix_code",  &(target->suffix_code));
 read_int32_t(pos, "bonus_to_dex",  &(target->bonus_to_dex));
 read_int32_t(pos, "bonus_to_str",  &(target->bonus_to_str));
-read_int32_t(pos, "bonus_to_vit",  &(target->bonus_to_vit));
-read_int32_t(pos, "bonus_to_mag",  &(target->bonus_to_mag));
-read_int32_t(pos, "bonus_to_life",  &(target->bonus_to_life));
-read_int32_t(pos, "bonus_to_force",  &(target->bonus_to_force));
+read_int32_t(pos, "bonus_to_physique",  &(target->bonus_to_physique));
+read_int32_t(pos, "bonus_to_cooling",  &(target->bonus_to_cooling));
+read_int32_t(pos, "bonus_to_health_points",  &(target->bonus_to_health_points));
 read_float(pos, "bonus_to_health_recovery",  &(target->bonus_to_health_recovery));
 read_float(pos, "bonus_to_cooling_rate",  &(target->bonus_to_cooling_rate));
-read_int32_t(pos, "bonus_to_tohit",  &(target->bonus_to_tohit));
+read_int32_t(pos, "bonus_to_attack",  &(target->bonus_to_attack));
 read_int32_t(pos, "bonus_to_all_attributes",  &(target->bonus_to_all_attributes));
-read_int32_t(pos, "bonus_to_dr_or_damage",  &(target->bonus_to_dr_or_damage));
+read_int32_t(pos, "bonus_to_armor_class",  &(target->bonus_to_armor_class));
+read_int32_t(pos, "bonus_to_damage",  &(target->bonus_to_damage));
 read_int32_t(pos, "bonus_to_resist_fire",  &(target->bonus_to_resist_fire));
 read_int32_t(pos, "bonus_to_resist_electricity",  &(target->bonus_to_resist_electricity));
-read_int32_t(pos, "dr_bonus",  &(target->dr_bonus));
+read_int32_t(pos, "bonus_to_paralyze_enemy",  &(target->bonus_to_paralyze_enemy));
+read_int32_t(pos, "bonus_to_slow_enemy",  &(target->bonus_to_slow_enemy));
+read_int32_t(pos, "bonus_to_light_radius",  &(target->bonus_to_light_radius));
+read_int32_t(pos, "bonus_to_experience_gain",  &(target->bonus_to_experience_gain));
+read_int32_t(pos, "armor_class",  &(target->armor_class));
 read_int32_t(pos, "damage",  &(target->damage));
 read_int32_t(pos, "damage_modifier",  &(target->damage_modifier));
 read_int32_t(pos, "multiplicity",  &(target->multiplicity));
 read_int32_t(pos, "ammo_clip",  &(target->ammo_clip));
 read_point(pos, "inventory_position",  &(target->inventory_position));
+read_upgrade_socket_dynarray(pos, "upgrade_sockets",  &(target->upgrade_sockets));
 *epos = '>'; 
 return 0;
 }
@@ -672,7 +705,6 @@ save_int16_t("bot_target_n", &(target->bot_target_n));
 save_int16_t("to_hit", &(target->to_hit));
 save_int16_t("damage", &(target->damage));
 save_int16_t("owner", &(target->owner));
-save_char("level", &(target->level));
 autostr_append(savestruct_autostr, "</%s>\n", tag);
 return 0;
 }
@@ -692,10 +724,10 @@ int read_melee_shot(char* buffer, char * tag, melee_shot * target)
 		read_char(pos, "attack_target_type",  &(target->attack_target_type));
 read_char(pos, "mine",  &(target->mine));
 read_int16_t(pos, "bot_target_n",  &(target->bot_target_n));
+target->bot_target_addr = NULL;
 read_int16_t(pos, "to_hit",  &(target->to_hit));
 read_int16_t(pos, "damage",  &(target->damage));
 read_int16_t(pos, "owner",  &(target->owner));
-read_char(pos, "level",  &(target->level));
 *epos = '>'; 
 return 0;
 }
@@ -716,6 +748,7 @@ save_int32_t("CharacterScreen_Visible", &(target->CharacterScreen_Visible));
 save_int32_t("SkillScreen_Visible", &(target->SkillScreen_Visible));
 save_int32_t("Automap_Visible", &(target->Automap_Visible));
 save_int32_t("spell_level_visible", &(target->spell_level_visible));
+save_int32_t("autorun_activated", &(target->autorun_activated));
 save_string("freedroid_version_string", &(target->freedroid_version_string));
 save_int32_t("skip_light_radius", &(target->skip_light_radius));
 save_int32_t("skill_explanation_screen_visible", &(target->skill_explanation_screen_visible));
@@ -741,7 +774,10 @@ save_int32_t("do_fadings", &(target->do_fadings));
 save_int32_t("fullscreen_on", &(target->fullscreen_on));
 save_int32_t("talk_to_bots_after_takeover", &(target->talk_to_bots_after_takeover));
 save_int32_t("xray_vision_for_tux", &(target->xray_vision_for_tux));
+save_int32_t("cheat_running_stamina", &(target->cheat_running_stamina));
 save_int32_t("lazyload", &(target->lazyload));
+save_int32_t("show_item_labels", &(target->show_item_labels));
+save_int32_t("last_edited_level", &(target->last_edited_level));
 save_int32_t("difficulty_level", &(target->difficulty_level));
 save_keybind_t_array("input_keybinds", (target->input_keybinds), 100);
 autostr_append(savestruct_autostr, "</%s>\n", tag);
@@ -773,6 +809,7 @@ read_int32_t(pos, "CharacterScreen_Visible",  &(target->CharacterScreen_Visible)
 read_int32_t(pos, "SkillScreen_Visible",  &(target->SkillScreen_Visible));
 read_int32_t(pos, "Automap_Visible",  &(target->Automap_Visible));
 read_int32_t(pos, "spell_level_visible",  &(target->spell_level_visible));
+read_int32_t(pos, "autorun_activated",  &(target->autorun_activated));
 read_string(pos, "freedroid_version_string",  &(target->freedroid_version_string));
 read_int32_t(pos, "skip_light_radius",  &(target->skip_light_radius));
 read_int32_t(pos, "skill_explanation_screen_visible",  &(target->skill_explanation_screen_visible));
@@ -798,7 +835,10 @@ read_int32_t(pos, "do_fadings",  &(target->do_fadings));
 read_int32_t(pos, "fullscreen_on",  &(target->fullscreen_on));
 read_int32_t(pos, "talk_to_bots_after_takeover",  &(target->talk_to_bots_after_takeover));
 read_int32_t(pos, "xray_vision_for_tux",  &(target->xray_vision_for_tux));
+read_int32_t(pos, "cheat_running_stamina",  &(target->cheat_running_stamina));
 read_int32_t(pos, "lazyload",  &(target->lazyload));
+read_int32_t(pos, "show_item_labels",  &(target->show_item_labels));
+read_int32_t(pos, "last_edited_level",  &(target->last_edited_level));
 read_int32_t(pos, "difficulty_level",  &(target->difficulty_level));
 read_keybind_t_array(pos, "input_keybinds",  (target->input_keybinds), 100);
 *epos = '>'; 
