@@ -1,7 +1,7 @@
 /* 
  *
  *   Copyright (c) 2002, 2003 Johannes Prix
- *   Copyright (c) 2004-2007 Arthur Huillet 
+ *   Copyright (c) 2004-2010 Arthur Huillet 
  *
  *  This file is part of Freedroid
  *
@@ -55,8 +55,8 @@
 #define RECHARGE_X 260
 #define RECHARGE_Y 200
 
-#define DR_X 260
-#define DR_Y 171
+#define DAMRED_X 260
+#define DAMRED_Y 171
 
 #define INV_BUTTON_X 600
 #define INV_BUTTON_Y 400
@@ -82,15 +82,38 @@ void ImproveSkill(int *skill)
 	(*skill)++;
 };				// void ImproveSkill ( int * skill )
 
+/**
+ * This function improves a program by one
+ * returns 1 if it can't be improved any further, returns 0 otherwise
+ *
+ */
+int improve_program(int prog_id)
+{
+	if(prog_id < 0)
+		return 0;
+
+	if(Me.skill_level[prog_id] >= NUMBER_OF_SKILL_LEVELS - 1)
+		return 1;
+
+	Me.skill_level[prog_id]++;
+	return 0;
+}
+
 /* ------------------
  * This function calculates the heat cost of running a given program (source or blob), based on current program level and casting ability
  * -----------------*/
 int calculate_program_heat_cost(int program_id)
 {
-//                                               0.9^0, 0.9^1, 0.9^2 ... ...0.9^9
+//	                                           0.9^0, 0.9^1, 0.9^2 ... ...0.9^9
 	float cost_ratio[NUMBER_OF_SKILL_LEVELS] = { 1.0, 0.9, 0.81, 0.73, 0.66, 0.59, 0.53, 0.48, 0.43, 0.39 };
-	return cost_ratio[Me.spellcasting_skill] * (SpellSkillMap[program_id].heat_cost +
-						    SpellSkillMap[program_id].heat_cost_per_level * (Me.SkillLevel[program_id] - 1));
+
+	if (program_id == get_program_index_with_name("Emergency shutdown") ) { //then use cost_ratio^-1
+		return (1/cost_ratio[Me.spellcasting_skill]) * (SpellSkillMap[program_id].heat_cost +
+							    	SpellSkillMap[program_id].heat_cost_per_level * (Me.skill_level[program_id] - 1));
+	} else {
+		return cost_ratio[Me.spellcasting_skill] * (SpellSkillMap[program_id].heat_cost +
+							    SpellSkillMap[program_id].heat_cost_per_level * (Me.skill_level[program_id] - 1));
+	}
 };
 
 /* ------------------
@@ -98,7 +121,7 @@ int calculate_program_heat_cost(int program_id)
  * -----------------*/
 static int calculate_program_hit_damage(int program_id)
 {
-	return (SpellSkillMap[program_id].damage_base + SpellSkillMap[program_id].damage_per_level * (Me.SkillLevel[program_id] - 1) +
+	return (SpellSkillMap[program_id].damage_base + SpellSkillMap[program_id].damage_per_level * (Me.skill_level[program_id] - 1) +
 		MyRandom(SpellSkillMap[program_id].damage_mod));
 }
 
@@ -107,13 +130,13 @@ static int calculate_program_hit_damage(int program_id)
  *-----------------*/
 static int calculate_program_hit_damage_low(int program_id)
 {
-	return (SpellSkillMap[program_id].damage_base + SpellSkillMap[program_id].damage_per_level * (Me.SkillLevel[program_id] - 1));
+	return (SpellSkillMap[program_id].damage_base + SpellSkillMap[program_id].damage_per_level * (Me.skill_level[program_id] - 1));
 
 }
 
 static int calculate_program_hit_damage_high(int program_id)
 {
-	return (SpellSkillMap[program_id].damage_base + SpellSkillMap[program_id].damage_per_level * (Me.SkillLevel[program_id] - 1) +
+	return (SpellSkillMap[program_id].damage_base + SpellSkillMap[program_id].damage_per_level * (Me.skill_level[program_id] - 1) +
 		SpellSkillMap[program_id].damage_mod);
 }
 
@@ -124,7 +147,7 @@ static int calculate_program_hit_damage_high(int program_id)
 static float calculate_program_effect_duration(int program_id)
 {
 	return (SpellSkillMap[program_id].effect_duration +
-		SpellSkillMap[program_id].effect_duration_per_level * (Me.SkillLevel[program_id] - 1));
+		SpellSkillMap[program_id].effect_duration_per_level * (Me.skill_level[program_id] - 1));
 }
 
 /**
@@ -162,28 +185,47 @@ static void TeleportHome(void)
 {
 	location HomeSpot;
 
-	if ((!Me.teleport_anchor.x) && (!Me.teleport_anchor.y))	//if there is no anchor, teleport home
-	{
-		Me.teleport_anchor.x = Me.pos.x;
-		Me.teleport_anchor.y = Me.pos.y;
-		Me.teleport_anchor.z = Me.pos.z;
-		teleport_arrival_sound();
-		ResolveMapLabelOnShip("TeleportHomeTarget", &(HomeSpot));
-		reset_visible_levels();
-		Teleport(HomeSpot.level, HomeSpot.x, HomeSpot.y, TRUE);
-		clear_active_bullets();
-	} else			//we must teleport back to the anchor
-	{
+	// Find homespot position.
+
+	ResolveMapLabelOnShip("TeleportHomeTarget", &(HomeSpot));
+
+	// If homespot was not found, a fatal error has been generated, but we
+	// however check it once again, to prevent any bug.
+
+	if (HomeSpot.x == -1)
+		return;
+
+	// Case 1 : Tux is in homespot's level, and there is a teleport anchor
+	//          -> teleport back to previous position
+	//
+	if (Me.pos.z == HomeSpot.level && ((Me.teleport_anchor.x != 0) || (Me.teleport_anchor.y != 0))) {
+
+		// Teleport
 		teleport_arrival_sound();
 		reset_visible_levels();
 		Teleport(Me.teleport_anchor.z, Me.teleport_anchor.x, Me.teleport_anchor.y, TRUE);
 		clear_active_bullets();
+
+		// Reset teleport anchor
 		Me.teleport_anchor.x = 0;
 		Me.teleport_anchor.y = 0;
 		Me.teleport_anchor.z = 0;
+
+		return;
 	}
 
-};				// void TeleportHome ( void )
+	// Any other cases : Store current position and teleport Tux to homespot
+	//
+	Me.teleport_anchor.x = Me.pos.x;
+	Me.teleport_anchor.y = Me.pos.y;
+	Me.teleport_anchor.z = Me.pos.z;
+
+	teleport_arrival_sound();
+	reset_visible_levels();
+	Teleport(HomeSpot.level, HomeSpot.x, HomeSpot.y, TRUE);
+	clear_active_bullets();
+
+}				// void TeleportHome ( void )
 
 /**
  * This function handles the program the player has just activated.
@@ -200,7 +242,7 @@ void HandleCurrentlyActivatedSkill()
 		if (ItemMap[Me.Inventory[Grabbed_InvPos].type].item_can_be_applied_in_combat)
 			return;	//if the cursor is over an item that can be applied, then the player wants to apply it not trigger a spell
 
-	if (Me.SkillLevel[Me.readied_skill] <= 0)
+	if (Me.skill_level[Me.readied_skill] <= 0)
 		return;
 
 	/* We calculate the spellcost and check the power limit override - the temperature is raised further down, when the actual effect
@@ -208,14 +250,13 @@ void HandleCurrentlyActivatedSkill()
 	int SpellCost = calculate_program_heat_cost(Me.readied_skill);
 
 	if (Me.temperature > Me.max_temperature - SpellCost && !Override_Power_Limit) {
-		//Not_Enough_Mana_Sound(  );
 		Override_Power_Limit = 1;
 		return;
 	}
 	Override_Power_Limit = 0;
 
 	switch (SpellSkillMap[Me.readied_skill].form) {
-	case PROGRAM_FORM_IMMEDIATE:
+	case PROGRAM_FORM_INSTANT:
 	case PROGRAM_FORM_BULLET:
 	case PROGRAM_FORM_RADIAL:
 		if (!MouseCursorIsInUserRect(GetMousePos_x(), GetMousePos_y()))
@@ -247,7 +288,7 @@ int DoSkill(int skill_index, int SpellCost)
 
 	/*we handle the form of the program now */
 	switch (SpellSkillMap[skill_index].form) {
-	case PROGRAM_FORM_IMMEDIATE:
+	case PROGRAM_FORM_INSTANT:
 		droid_below_mouse_cursor = GetLivingDroidBelowMouseCursor();
 		if (droid_below_mouse_cursor == NULL)
 			goto done_handling_instant_hits;
@@ -285,6 +326,7 @@ int DoSkill(int skill_index, int SpellCost)
 		Me.slowdown_duration += strcmp(SpellSkillMap[skill_index].effect, "slowdown") ? 0 : effdur;
 		Me.paralyze_duration += strcmp(SpellSkillMap[skill_index].effect, "paralyze") ? 0 : effdur;
 		Me.invisible_duration += strcmp(SpellSkillMap[skill_index].effect, "invisibility") ? 0 : effdur;
+		Me.light_bonus_end_date = Me.current_game_date + (strcmp(SpellSkillMap[skill_index].effect, "light") ? 0 : effdur);
 		break;
 
 	case PROGRAM_FORM_BULLET:
@@ -306,7 +348,6 @@ int DoSkill(int skill_index, int SpellCost)
 		bul_parms.poison_damage_per_sec = strcmp(SpellSkillMap[skill_index].effect, "poison") ? 0 : hitdmg;
 		bul_parms.paralysation_duration = strcmp(SpellSkillMap[skill_index].effect, "paralyze") ? 0 : effdur;
 		bul_parms.damage = hitdmg;
-		bul_parms.to_hit = SpellHitPercentageTable[Me.spellcasting_skill];
 		if (SpellSkillMap[skill_index].hurt_humans && SpellSkillMap[skill_index].hurt_bots)
 			bul_parms.hit_type = ATTACK_HIT_ALL;
 		else if (SpellSkillMap[skill_index].hurt_humans)
@@ -378,15 +419,11 @@ int DoSkill(int skill_index, int SpellCost)
 								       (float)input_axis.y, FALSE), Me.pos.z, &FlyablePassFilter))
 			goto out;
 
-		if (!droid_below_mouse_cursor->is_friendly) {
-			//--------------------
-			// Only droids can be hacked.  Humans can't be 
-			// hacked.
+		if (!is_friendly(droid_below_mouse_cursor->faction, FACTION_SELF)) {
+			// Only hostile droids can be hacked. 
 			//
 			if (droid_takeover(droid_below_mouse_cursor)) {
 				// upon successful takeover
-				// restore original heat
-				// Me . temperature -= SpellCost;
 				// go directly to chat to choose droid program
 				if (GameConfig.talk_to_bots_after_takeover)
 					ChatWithFriendlyDroid(droid_below_mouse_cursor);
@@ -408,7 +445,6 @@ int DoSkill(int skill_index, int SpellCost)
 	if (!strcmp(SpellSkillMap[skill_index].effect, "repair")) {
 		if (!MouseCursorIsInInvRect(GetMousePos_x(), GetMousePos_y())
 		    || (!GameConfig.Inventory_Visible)) {
-			//--------------------
 			// Do nothing here.  The right mouse click while in inventory screen
 			// will be handled in the inventory screen management function.
 			//
@@ -417,29 +453,6 @@ int DoSkill(int skill_index, int SpellCost)
 		goto out;
 	}
 
-	if (!strcmp(SpellSkillMap[skill_index].effect, "identify")) {
-		//--------------------
-		// Maybe the identify mode has already been triggered and
-		// is activated right now.  Then of course this (second) mouse
-		// click must be ignored completely.
-		//
-		if (global_ingame_mode == GLOBAL_INGAME_MODE_IDENTIFY)
-			Me.temperature -= SpellCost;
-
-		silently_unhold_all_items();
-		global_ingame_mode = GLOBAL_INGAME_MODE_IDENTIFY;
-
-		goto out;
-	}
-
-	if (!strcmp(SpellSkillMap[skill_index].effect, "invisibility")) {
-		goto out;
-	}
-
-	if (!strcmp(SpellSkillMap[skill_index].effect, "burnup")) {
-		goto out;
-	}
-
 	if (!strcmp(SpellSkillMap[skill_index].effect, "teleport_home")) {
 		TeleportHome();
 		goto out;
@@ -456,7 +469,6 @@ int DoSkill(int skill_index, int SpellCost)
  */
 int CursorIsOnWhichSkillButton(int x, int y)
 {
-	//--------------------
 	// First we check if the cursor is in at least horizontally
 	// in the row of the skill items
 	//
@@ -465,7 +477,6 @@ int CursorIsOnWhichSkillButton(int x, int y)
 	if (x < SkillScreenRect.x + 16)
 		return (-1);
 
-	//--------------------
 	// Now we can check on which skill rectangle exactly the cursor
 	// is hovering, since we know that it is hitting, horizontally
 	// at least, the row of skill icons.
@@ -507,7 +518,6 @@ static int CursorIsOnWhichSpellPageButton(int x, int y)
 {
 	int i;
 
-	//--------------------
 	// First we check if the cursor is in at least horizontally
 	// and vertically in the line with the spell level buttons.
 	//
@@ -520,7 +530,6 @@ static int CursorIsOnWhichSpellPageButton(int x, int y)
 	if (y < SkillScreenRect.y + SPELL_LEVEL_BUTTONS_Y)
 		return (-1);
 
-	//--------------------
 	// Now we can check on which skill rectangle exactly the cursor
 	// is hovering, since we know that it is hitting, horizontally
 	// at least, the row of skill icons.
@@ -544,12 +553,10 @@ static void ShowSkillsExplanationScreen(void)
 	int TEXT_OFFSET_X = 15;
 	SDL_Rect TargetSkillRect;
 
-	//--------------------
 	// This should draw the background...
 	//
 	blit_special_background(SKILL_SCREEN_EXPLANATION_BACKGROUND_CODE);
 
-	//--------------------
 	// Draws the skill icon at the correct position
 	//
 	TargetSkillRect.x = ICON_OFFSET_X;
@@ -562,7 +569,6 @@ static void ShowSkillsExplanationScreen(void)
 		our_SDL_blit_surface_wrapper(SpellSkillMap[Me.readied_skill].icon_surface.surface, NULL, Screen, &TargetSkillRect);
 	}
 
-	//--------------------
 	// Draws the explanation text
 	// (We will use the FPS display font, cause the small one isn't 
 	// very well readable on the silver background.)
@@ -595,7 +601,7 @@ static void establish_skill_subset_map(int *SkillSubsetMap)
 		SkillSubsetMap[i] = (-1);
 	}
 	for (i = 0; i < number_of_skills; i++) {
-		if (Me.SkillLevel[i] > 0) {
+		if (Me.skill_level[i] > 0) {
 			SkillSubsetMap[NextPosition] = i;
 			NextPosition++;
 		}
@@ -610,11 +616,10 @@ static void establish_skill_subset_map(int *SkillSubsetMap)
 void activate_nth_skill(int skill_num)
 {
 
-	//--------------------
 	// If the n-th skill does exist, we activate the n-th skill,
 	// otherwise we leave the last readied skill.
 	//
-	if (Me.SkillLevel[skill_num] > 0) {
+	if (Me.skill_level[skill_num] > 0) {
 		Me.readied_skill = skill_num;
 	} else {
 		ErrorMessage(__FUNCTION__,
@@ -634,14 +639,13 @@ void set_nth_quick_skill(int quick_skill)
 					       GetMousePos_y()) +
 		    NUMBER_OF_SKILLS_PER_SKILL_PAGE * GameConfig.spell_level_visible;
 		
-		//--------------------
 		// Variable number is an index into already aquired skills.
 		// We change it into index into array of all skills.
 		int SkillSubsetMap[number_of_skills];
 		establish_skill_subset_map(&(SkillSubsetMap[0]));
 		ski = SkillSubsetMap[ski];
 			
-		if (Me.SkillLevel[ski] <=  0) {
+		if (Me.skill_level[ski] <=  0) {
 			// Invalid skill was selected
 			ErrorMessage(__FUNCTION__,
 				"Tried to set skill number %d in quick skills. Skill was not aquired yet.\n",
@@ -690,7 +694,6 @@ void ShowSkillsScreen(void)
 		SkillRectLocations[i].y = SkillScreenRect.y + FIRST_SKILLRECT_Y + i * (64 + INTER_SKILLRECT_DIST) + 3;
 	}
 
-	//--------------------
 	// If the log is not set to visible right now, we do not need to 
 	// do anything more, but to restore the usual user rectangle size
 	// back to normal and to return...
@@ -698,26 +701,22 @@ void ShowSkillsScreen(void)
 	if (GameConfig.SkillScreen_Visible == FALSE)
 		return;
 
-	//--------------------
 	// We will use the FPS display font, cause the small one isn't 
 	// very well readable on the silver background
 	//
 	SetCurrentFont(FPS_Display_BFont);
 
-	//--------------------
 	// Maybe the skill circle images for clicking between different spell circles
 	// have not been loaded yet.  Then it is time to do so.  If this was already
 	// done before, then the function will know it and don't do anything anyway.
 	//
 	Load_Skill_Level_Button_Surfaces();
 
-	// --------------------
 	// We will need the current mouse position on several spots...
 	//
 	CurPos.x = GetMousePos_x();
 	CurPos.y = GetMousePos_y();
 
-	//--------------------
 	// We will draw only those skills to the skills inventory, that are
 	// already present in the Tux.  That way the game remains open for new
 	// skills to the player and he doesn't now in advance which skills there
@@ -725,7 +724,6 @@ void ShowSkillsScreen(void)
 	//
 	establish_skill_subset_map(SkillSubsetMap);
 
-	//--------------------
 	// At this point we know, that the skill screen is desired and must be
 	// displayed in-game:
 	//
@@ -737,7 +735,6 @@ void ShowSkillsScreen(void)
 	if (GameConfig.skill_explanation_screen_visible)
 		ShowSkillsExplanationScreen();
 
-	//--------------------
 	// According to the page in the spell book currently opened,
 	// we draw a 'button' or activation mark over the appropriate spot
 	//
@@ -745,7 +742,6 @@ void ShowSkillsScreen(void)
 	SpellLevelRect.y = SkillScreenRect.y + SPELL_LEVEL_BUTTONS_Y;
 	our_SDL_blit_surface_wrapper(SpellLevelButtonImageList[GameConfig.spell_level_visible].surface, NULL, Screen, &SpellLevelRect);
 
-	//--------------------
 	// Now we fill in the skills available to this bot.  ( For now, these skills 
 	// are not class-specific, like in diablo or something, but this is our first
 	// approach to the topic after all.... :)
@@ -797,7 +793,7 @@ void ShowSkillsScreen(void)
 		    FIRST_SKILLRECT_Y - 8 + i * (64 + INTER_SKILLRECT_DIST) + SkillScreenRect.y + 2 * FontHeight(GetCurrentFont());
 
 		// Program revision
-		sprintf(CharText, _("Program revision: %c%d%c "), font_switchto_msgvar[0], Me.SkillLevel[SkillOfThisSlot],
+		sprintf(CharText, _("Program revision: %c%d%c "), font_switchto_msgvar[0], Me.skill_level[SkillOfThisSlot],
 			font_switchto_msgstat[0]);
 		DisplayText(CharText, 16 + 64 + 16 + SkillScreenRect.x, nextypos, &SkillScreenRect, TEXT_STRETCH);
 		nextypos += FontHeight(GetCurrentFont());
@@ -843,7 +839,7 @@ void ShowSkillsScreen(void)
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "poison"))
 				sprintf(CharText, _("Poison"));
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "takeover")) {
-				tmp = Me.SkillLevel[SkillOfThisSlot] + 2;
+				tmp = Me.skill_level[SkillOfThisSlot] + 2;
 				sprintf(CharText, _("Takeover charges: %c%d%c "), font_switchto_msgvar[0], tmp, font_switchto_msgstat[0]);
 			} else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "teleport_home"))
 				sprintf(CharText, _("Escape"));
@@ -858,7 +854,7 @@ void ShowSkillsScreen(void)
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "satellite"))
 				sprintf(CharText, " ");
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "light"))
-				sprintf(CharText, " ");
+				sprintf(CharText, _("Lighten area"));
 			else if (!strcmp(SpellSkillMap[SkillOfThisSlot].effect, "burnup"))
 				sprintf(CharText, " ");
 
@@ -873,7 +869,6 @@ void ShowSkillsScreen(void)
 
 	}
 
-	//--------------------
 	// Now we see if perhaps the player has just clicked on one of the skills
 	// available to this class.  In this case of course we must set a different
 	// skill/spell as the currently activated skill/spell.
@@ -892,7 +887,6 @@ void ShowSkillsScreen(void)
 		while (MouseLeftPressed())
 			SDL_Delay(1);
 	}
-	//--------------------
 	// Now we see if perhaps the player has just clicked on another skill level
 	// button.  In this case of course we must set a different skill/spell level
 	// as the currently visible spell level.
@@ -900,7 +894,6 @@ void ShowSkillsScreen(void)
 	if ((CursorIsOnWhichSpellPageButton(CurPos.x, CurPos.y) != (-1)) && MouseLeftClicked()) {
 		GameConfig.spell_level_visible = CursorIsOnWhichSpellPageButton(CurPos.x, CurPos.y);
 	}
-	//--------------------
 	// We want to know, if the button was pressed the previous frame when we
 	// are in the next frame and back in this function.  Therefore we store
 	// the current button situation, so that we can conclude on button just
