@@ -27,13 +27,6 @@
 
 #include "sound.h"
 
-/** ============================================ EFFECTS ============================================ */
-
-void CrySound(void)
-{
-	play_sample_using_WAV_cache("effects/Cry_Sound_0.ogg", FALSE, FALSE);
-}
-
 void play_open_chest_sound(void)
 {
 	PlayOnceNeededSoundSample("effects/open_chest_sound.ogg", FALSE, TRUE);
@@ -44,11 +37,6 @@ void Play_Spell_ForceToEnergy_Sound()
 	play_sample_using_WAV_cache("effects/Spell_ForceToEnergy_Sound_0.ogg", FALSE, FALSE);
 }
 
-void Play_Spell_DetectItems_Sound()
-{
-	play_sample_using_WAV_cache("effects/Spell_DetectItems_Sound_0.ogg", FALSE, FALSE);
-}
-
 /**
  * This function plays a voice sample, stating that not enough power
  * (strength) is available to use a certain item.
@@ -109,34 +97,12 @@ void Not_Enough_Dist_Sound(void)
 	}
 }
 
-/**
- * This function plays a voice sample, stating that not enough magical
- * energy (force) is available to cast a certain spell.
- * The sample must of course only be played, if it hasn't been played just
- * milliseconds before, so a check is made to see that the file is played
- * with at least a certain interval in between to the last occasion of the
- * file being played.
- */
-void Not_Enough_Mana_Sound(void)
-{
-
-	static Uint32 PreviousNotEnoughForceSound = (-1);
-	Uint32 now;
-
-	now = SDL_GetTicks();
-	if (SDL_GetTicks() - PreviousNotEnoughForceSound >= 1.15 * 1000) {
-		play_sample_using_WAV_cache("effects/tux_ingame_comments/Not_Enough_Mana_0.ogg", FALSE, FALSE);
-		PreviousNotEnoughForceSound = now;
-	}
-
-}
-
 /**
  * Whenever the Tux meets someone in the game for the very first time, 
  * this enemy or friend will issue the first-time greeting.
  * Depending on whether that is typically a hostile unit or not, we play
  * the corresponding sound from preloaded memory or, if there is no combat
- * to expect, by loading it freshly from disk and then forgeting about it
+ * to expect, by loading it freshly from disk and then forgetting about it
  * again.
  */
 void PlayGreetingSound(int SoundCode)
@@ -218,14 +184,12 @@ void play_death_sound_for_bot(enemy * ThisRobot)
 {
 	char filename[5000];
 
-	//--------------------
 	// If the keyword 'none' for the death sound file name is encountered,
 	// nothing will be done...
 	//.
 	if (!strcmp(Druidmap[ThisRobot->type].droid_death_sound_file_name, "none"))
 		return;
 
-	//--------------------
 	// Now we play the given death sound, looking for the file in the
 	// appropriate sound folder.
 	//
@@ -236,32 +200,6 @@ void play_death_sound_for_bot(enemy * ThisRobot)
 
 }
 
-/**
- * Whenever a bot dies, that should create a dying sound.  But so far,
- * this will be done only for fully animated bots, since the other bots
- * just explode and that has a sound of it's own.
- */
-void play_attack_animation_sound_for_bot(enemy * ThisRobot)
-{
-	char filename[5000];
-
-	//--------------------
-	// If the keyword 'none' for the death sound file name is encountered,
-	// nothing will be done...
-	//
-	if (!strcmp(Druidmap[ThisRobot->type].droid_attack_animation_sound_file_name, "none"))
-		return;
-
-	//--------------------
-	// Now we play the given death sound, looking for the file in the
-	// appropriate sound folder.
-	//
-	strcpy(filename, "effects/bot_sounds/");
-	strcat(filename, Druidmap[ThisRobot->type].droid_attack_animation_sound_file_name);
-	PlayOnceNeededSoundSample(filename, FALSE, FALSE);
-
-}
-
 /**
  * Whenever a bot starts to attack the Tux, he'll issue the attack cry.
  * This is done here, and no respect to loading time issues for now...
@@ -333,7 +271,6 @@ void play_item_sound(int item_type)
 {
 	char final_file_name[5000];
 
-	//--------------------
 	// First some sanity check...
 	//
 	if (item_type < 0) {
@@ -341,7 +278,6 @@ void play_item_sound(int item_type)
 		ErrorMessage(__FUNCTION__, "\
 		                           negative item type received!\n", PLEASE_INFORM, IS_FATAL);
 	}
-	//--------------------
 	// Now we can proceed and just play the sound associated with that item...
 	//
 	strcpy(final_file_name, "effects/item_sounds/");
@@ -378,19 +314,6 @@ void CantCarrySound(void)
 	}
 }
 
-/**
- *
- */
-void TransferSound(void)
-{
-	play_sample_using_WAV_cache("effects/Takeover_Sound_0.ogg", FALSE, FALSE);
-}
-
-;				// void TransferSound (void)
-
-/**
- *
- */
 void Mission_Status_Change_Sound(void)
 {
 	play_sample_using_WAV_cache("effects/Mission_Status_Change_Sound_0.ogg", FALSE, FALSE);
@@ -405,29 +328,6 @@ void teleport_arrival_sound(void)
 	play_sample_using_WAV_cache("effects/new_teleporter_sound.ogg", FALSE, FALSE);
 }
 
-/**
- * When the Tux casts a healing spell, this sound will be played.
- */
-void healing_spell_sound(void)
-{
-	play_sample_using_WAV_cache("effects/new_healing_sound.ogg", FALSE, FALSE);
-}
-
-/**
- * Some items, that can be applied inside the running game, like e.g. 
- * spellbooks, do have a certain character stat requirement.  If that stat
- * requirement isn't met, a sound will be played, which uses this function.
- */
-void application_requirements_not_met_sound(void)
-{
-	play_sample_using_WAV_cache("effects/tux_ingame_comments/i_cannot_use_this_item_yet.ogg", FALSE, FALSE);
-}
-
-void GotHitSound(void)
-{
-	play_sample_using_WAV_cache("effects/Got_Hit_Sound_0.ogg", FALSE, FALSE);
-}
-
 /**
  * Whenever the Tux gets hit, we *might* play a sound sample for the hit,
  * depending on random results.  Since we don't want to hear the same 
@@ -476,10 +376,6 @@ void MoveMenuPositionSound(void)
 	play_sample_using_WAV_cache("effects/Move_Menu_Position_Sound_0.ogg", FALSE, FALSE);
 }
 
-/**
- *
- *
- */
 void ThouArtDefeatedSound(void)
 {
 	if (!sound_on)
@@ -609,48 +505,16 @@ void Takeover_Game_Lost_Sound(void)
 	play_sample_using_WAV_cache("effects/Takeover_Game_Lost_Sound_0.ogg", FALSE, FALSE);
 }
 
-/**
- * 
- */
-void BounceSound(void)
-{
-	play_sample_using_WAV_cache("effects/Collision_Sound_0.ogg", FALSE, FALSE);
-}
-
-/**
- *
- */
 void DruidBlastSound(void)
 {
 	play_sample_using_WAV_cache("effects/Blast_Sound_0.ogg", FALSE, FALSE);
 }
 
-/**
- *
- */
 void ExterminatorBlastSound(void)
 {
 	play_sample_using_WAV_cache("effects/Blast_Sound_0.ogg", FALSE, FALSE);
 }
 
-/**
- * 
- *
- */
-void PlayLevelCommentSound(int levelnum)
-{
-	switch (levelnum) {
-	case 0:
-		// I've been away for a far too long time it seems...
-		break;
-	case 3:
-		play_sample_using_WAV_cache("effects/MSMachinesClose_0.ogg", FALSE, FALSE);
-		break;
-	default:
-		break;
-	};
-}
-
 /**
  * Whenever an enemy is hit by the tux with a melee weapon, then the
  * following sound is played...
@@ -672,14 +536,9 @@ void PlayEnemyGotHitSound(int enemytype)
 
 /**
  * This function plays a sound of a bullet being reflected. It is only
- * used, when a bullets is compensated by the tux armour.
+ * used, when a bullets is compensated by the tux armor.
  */
 void BulletReflectedSound(void)
 {
 	play_sample_using_WAV_cache("effects/Bullet_Reflected_Sound_0.ogg", FALSE, FALSE);
 }
-
-/**
- *
- *
- */
