@@ -2,7 +2,7 @@
  *
  *   Copyright (c) 1994, 2002, 2003 Johannes Prix
  *   Copyright (c) 1994, 2002 Reinhard Prix
- *   Copyright (c) 2004-2007 Arthur Huillet 
+ *   Copyright (c) 2004-2010 Arthur Huillet 
  *
  *
  *  This file is part of Freedroid
@@ -49,6 +49,24 @@ typedef struct list_head {
 	struct list_head *prev;
 } list_head_t;
 
+/**
+ * Dynamic arrays
+ */
+struct dynarray {
+	void *arr;
+	int size;
+	int capacity;
+};
+
+typedef struct upgrade_socket_dynarray {
+	struct upgrade_socket *arr;
+	int size;
+	int capacity;
+} upgrade_socket_dynarray;
+
+/* prototype for BFont_Info from BFont.h */
+struct BFont_Info;
+
 /* ----------------------------------------------------------------------
  * Here comes the struct for an iso image.  It also contains some 
  * placeholder for a possible 'zoomed-out' version of the iso image and
@@ -61,7 +79,7 @@ typedef struct list_head {
  * so that later changes to the struct can be made with minimal effort 
  * and mistakes.
  * ---------------------------------------------------------------------- */
-typedef struct iso_image_s {
+typedef struct iso_image {
 	SDL_Surface *surface;
 	short offset_x;
 	short offset_y;
@@ -83,15 +101,14 @@ typedef struct iso_image_s {
 } iso_image, *Iso_image;
 #define UNLOADED_ISO_IMAGE { NULL , 0 , 0 , NULL , 0 , 0 , 0, 0, 0, 0, 0.0, 0.0, 0.0, 0.0 }
 
-typedef struct mouse_press_button_s {
+typedef struct mouse_press_button {
 	iso_image button_image;
 	char *button_image_file_name;
 	SDL_Rect button_rect;
 	char scale_this_button;
-	char use_true_alpha_blending;
 } mouse_press_button, *Mouse_press_button;
 
-typedef struct keybind_s {
+typedef struct keybind {
 	char *name;
 		/**< keybinding name, taken from keybindNames */
 	int key;
@@ -102,7 +119,7 @@ typedef struct keybind_s {
 
 typedef char *string;
 
-typedef struct configuration_for_freedroid_s {
+typedef struct configuration_for_freedroid {
 	float WantedTextVisibleTime;
 	int Draw_Framerate;
 	int Draw_Position;
@@ -118,6 +135,8 @@ typedef struct configuration_for_freedroid_s {
 	int Automap_Visible;
 	int spell_level_visible;
 
+	int autorun_activated;
+
 	string freedroid_version_string;
 	int skip_light_radius;
 	int skill_explanation_screen_visible;
@@ -143,7 +162,10 @@ typedef struct configuration_for_freedroid_s {
 	int fullscreen_on;
 	int talk_to_bots_after_takeover;
 	int xray_vision_for_tux;
+	int cheat_running_stamina;
 	int lazyload;
+	int show_item_labels;
+	int last_edited_level;
 
 	int difficulty_level;
 
@@ -153,40 +175,40 @@ typedef struct configuration_for_freedroid_s {
 	keybind_t input_keybinds[100];
 } configuration_for_freedroid;
 
-typedef struct point_s {
+typedef struct point {
 	int x;
 	int y;
-} point, *Point;
+} point;
 
-typedef struct moderately_finepoint_s {
+typedef struct moderately_finepoint {
 	float x;
 	float y;
-} moderately_finepoint, *Moderately_finepoint;
+} moderately_finepoint;
 
-typedef struct finepoint_s {
+typedef struct finepoint {
 	double x;
 	double y;
-} finepoint, *Finepoint;
+} finepoint;
 
-typedef struct gps_s {
+typedef struct gps {
 	float x;
 	float y;
 	int z;
-} gps, *GPS;
+} gps;
 
-typedef struct location_s {
+typedef struct location {
 	unsigned char level;
 	int x;
 	int y;
-} location, *Location;
+} location;
 
-typedef struct map_label_s {
+typedef struct map_label {
 	char *label_name;
 	point pos;		// how many blocks does this big map insert cover?
-} map_label, *Map_Label;
+} map_label;
 
 typedef char *luacode;
-typedef struct mission_s {
+typedef struct mission {
 	string mission_name;
 	int MissionWasAssigned;	// has be influencer been assigned to this mission? , currently uninitialized
 	int MissionIsComplete;	// has the mission been completed?
@@ -208,13 +230,13 @@ typedef struct mission_s {
 	string mission_diary_texts[MAX_MISSION_DESCRIPTION_TEXTS];
 	float mission_description_time[MAX_MISSION_DESCRIPTION_TEXTS];
 	int expanded_display_for_this_mission;
-} mission, *Mission;
+} mission;
 
 //--------------------
 // This structure can contain conditions that must be fulfilled, so that a special
 // event is triggered.  Such conditions may be specified in the mission file as well
 //
-typedef struct event_trigger_s {
+typedef struct event_trigger {
 	// Maybe the event is triggerd by the influencer stepping somewhere
 	int Influ_Must_Be_At_Level;
 	point Influ_Must_Be_At_Point;
@@ -226,54 +248,35 @@ typedef struct event_trigger_s {
 	char *name;
 	int enabled;		//is the trigger enabled?
 	int silent;		//do we have to advertise this trigger to the user? (teleporters..)
-} event_trigger, *Event_trigger;
+} event_trigger;
 
-typedef struct item_image_spec_s {
+typedef struct item_image_spec {
 	point inv_size;
 	SDL_Surface *Surface;
 	iso_image ingame_iso_image;
-	SDL_Surface *scaled_surface_for_shop;
-} item_image_spec, *Item_image_spec;
+	iso_image shop_iso_image;
+} item_image_spec;
 
-typedef struct item_bonus_s {
-	char *bonus_name;
-	short base_bonus_to_dex;
-	short modifier_to_bonus_to_dex;
-	short base_bonus_to_str;
-	short modifier_to_bonus_to_str;
-	short base_bonus_to_vit;
-	short modifier_to_bonus_to_vit;
-	short base_bonus_to_mag;
-	short modifier_to_bonus_to_mag;
-	short base_bonus_to_all_attributes;
-	short modifier_to_bonus_to_all_attributes;
+struct addon_bonus {
+	char *name;
+	int value;
+};
 
-	short base_bonus_to_life;
-	short modifier_to_bonus_to_life;
-	short base_bonus_to_force;
-	short modifier_to_bonus_to_force;
+struct addon_material {
+	char *name;
+	int value;
+};
 
-	float base_bonus_to_health_recovery;
-	float base_bonus_to_cooling_rate;
+struct addon_spec {
+	int type;
+	int upgrade_cost;
+	char *requires_socket;
+	char *requires_item;
+	struct dynarray bonuses;
+	struct dynarray materials;
+};
 
-	short base_bonus_to_tohit;
-	short modifier_to_bonus_to_tohit;
-	short base_bonus_to_dr_or_damage;	// this is a percentage
-	short modifier_to_bonus_to_dr_or_damage;	// this is a percentage
-
-	char base_bonus_to_resist_fire;	// this is a percentage /*XXX bonus to resist whatever ARE NOT READ YET!*/
-	char modifier_to_bonus_to_resist_fire;	// this is a percentage
-	char base_bonus_to_resist_electricity;	// this is a percentage
-	char modifier_to_bonus_to_resist_electricity;	// this is a percentage
-
-	short int light_bonus_value;
-
-	short int level;	//"level" of the prefix (is it good or not)
-	float price_factor;
-
-} item_bonus, *Item_bonus;
-
-typedef struct itemspec_s {
+typedef struct itemspec {
 	char *item_name;
 	char *item_rotation_series_prefix;
 	char *item_description;
@@ -306,12 +309,13 @@ typedef struct itemspec_s {
 	// the following values have only relevance in case of a melee weapon
 	short int item_weapon_is_melee;
 	float item_gun_start_angle_modifier;	// where to start with a melee weapon swing
-	char item_gun_use_ammunition;	// which ammunition does this gun use? - 1 laser 2 plasma 3 exterminator etc.
+	int item_gun_use_ammunition;        // which ammunition does this gun use? see ammo_desc_for_weapon()
 	char item_gun_requires_both_hands;	// is this a (strictly) 2-handed weapon?
+	short int motion_class;				// Tux's motion class to use
 
 	// how good is the item as armour or shield or other protection???
-	short base_dr_bonus;
-	short dr_bonus_modifier;
+	short base_armor_class;
+	short armor_class_modifier;
 
 	// which requirement for strength, dexterity and magic (force) does the item have?
 	short int item_require_strength;
@@ -330,64 +334,66 @@ typedef struct itemspec_s {
 	short int min_drop_class;
 	short int max_drop_class;
 
-} itemspec, *Itemspec;
+} itemspec;
 
-typedef struct item_s {
+typedef struct upgrade_socket {
+	int type;
+	string addon;
+} upgrade_socket;
+
+typedef struct item {
 	// Here are the rather short-lived properties of the item
 	gps pos;
 	gps virt_pos;
 	SDL_Rect text_slot_rectangle;
 	int type;
-	int currently_held_in_hand;	// is the item currently held 'in hand' with the mouse cursor?
-	int is_identified;	// is the item identified already?
 	int max_duration;	// the maximum item durability reachable for this item
 	float current_duration;	// the currently remaining durability for this item
 	float throw_time;	// has this item just jumped out from a chest maybe or is it jumping right now?
 
-	// Here are the rather long-lived properties of the item
-	int prefix_code;
-	int suffix_code;
-
 	int bonus_to_dex;
 	int bonus_to_str;
-	int bonus_to_vit;
-	int bonus_to_mag;
-	int bonus_to_life;
-	int bonus_to_force;
+	int bonus_to_physique;
+	int bonus_to_cooling;
+	int bonus_to_health_points;
 	float bonus_to_health_recovery;
 	float bonus_to_cooling_rate;
-	int bonus_to_tohit;
+	int bonus_to_attack;
 	int bonus_to_all_attributes;
-	int bonus_to_dr_or_damage;	// this is a percentage
+	int bonus_to_armor_class;
+	int bonus_to_damage;
 	int bonus_to_resist_fire;	// this is a percentage
 	int bonus_to_resist_electricity;	// this is a percentage
+	int bonus_to_paralyze_enemy;
+	int bonus_to_slow_enemy;
+	int bonus_to_light_radius;
+	int bonus_to_experience_gain;	// this is a percentage
 
-	int dr_bonus;		// how much is dr increased by this item worn
+	int armor_class;
 	int damage;		// how much damage does this item
 	int damage_modifier;	// how much additional damage can add to the base damage
 	int multiplicity;
 	int ammo_clip;		// how much bullets in the clip, IN CASE OF WEAPON
 	point inventory_position;
-} item, *Item;
 
-typedef struct druidspec_s {
+	struct upgrade_socket_dynarray upgrade_sockets;
+} item;
+
+typedef struct druidspec {
 	char *druidname;
+	char *default_short_description;
 	char *droid_portrait_rotation_series_prefix;
 	char *droid_death_sound_file_name;
 	char *droid_attack_animation_sound_file_name;
 	int class;
 	float maxenergy;	// the maximum energy the batteries can carry 
-	float max_temperature;	// the maximum force this droids mind can carry 
 
-	float lose_health;	// the energy/time the droid gains (heals)
+	float healing_friendly; // the energy/second the droid heals as a friendly towards Tux
+	float healing_hostile;  // the energy/second the droid heals as a hostile towards Tux
 
 	float maxspeed;
 
 	short experience_reward;	// experience_reward for the elimination of one droid of this type 
-	short int brain;
-	short int sensor1;
-	short int sensor2;
-	short int sensor3;
 
 	float aggression_distance;
 	float time_spent_eyeing_tux;
@@ -410,66 +416,92 @@ typedef struct druidspec_s {
 	char *notes;		// notes on the druid of this type 
 	short int is_human;
 	short individual_shape_nr;
-} druidspec, *Druidspec;
+} druidspec;
 
-typedef struct enemy_s {
-	short int id;
-	short type;		// the number of the droid specifications in Druidmap 
-	gps pos;		// coordinates of the current position in the level
-	gps virt_pos;		// the virtual position (position of the bot if he was on this level, differs from above when it is on a neighboring level)
-	finepoint speed;	// current speed  
-	float energy;		// current energy of this droid
+typedef struct enemy {
+	// There are three sets of attributes, which are initialized and
+	// possibly re-initialized by 3 different codes:
+	//
+	// 1) 'identity' attributes.
+	//    The identity attributes define the basic information about a droid.
+	//    They should not change during the game, apart in some very
+	//    specific cases.
+	// 2) global state' attributes.
+	//    Those attributes define the global behavior of the enemy.
+	//    The global state of a droid can possibly change during the game.
+	// 3) 'transient state' attributes
+	//    Their values are used by the animation and AI code.
+	//    They change very frequently during the game.
+	//
+	// 1st and 2nd sets are initialized to default values in enemy_new().
+	// The 3rd set is unconditionally reseted in enemy_reset().
+	// The 2nd set contains attributes whose values depend on the
+	// type of the droid. They are first positioned in GetThisLevelsDroids()
+	// and GetThisLevelsSpecialForces(), and are conditionally reseted in
+	// respawn_level().
 
-	float animation_phase;	// the current animation frame for this enemy (starting at 0 of course...)
-	short animation_type;	// walk-animation, attack-animation, gethit animation, death animation
-
-	short nextwaypoint;	// the next waypoint target
-	short lastwaypoint;	// the waypoint from whence this robot just came from
-	short homewaypoint;	// the waypoint this robot started at
-	short max_distance_to_home;	// how far this robot will go before returning to it's home waypoint
-
-	int combat_state;	// current state of the bot
-	float state_timeout;	// time spent in this state (used by "timeout" states such as STOP_AND_EYE_TARGET only)
-
-	float frozen;		// is this droid currently frozen and for how long will it stay this way?
-	float poison_duration_left;	// is this droid currently poisoned and for how long will it stay this way?
-	float poison_damage_per_sec;	// is this droid currently poisoned and how much poison is at work?
-	float paralysation_duration_left;	// is this droid currently paralyzed and for how long will it stay this way?
-	float pure_wait;	// time till the droid will start to move again
-	float firewait;		// time this robot still takes until its weapon will be fully reloaded
-	short ammo_left;	// ammunition left in the charger
-
-	short int CompletelyFixed;	// set this flag to make the robot entirely immobile
-	char follow_tux;	// does this robot try to follow tux via it's random movements?
-	char SpecialForce;	// This flag will exclude the droid from initial shuffling of droids
-	short on_death_drop_item_code;	// drop a pre-determined item when dying?
-
-	int marker;		// This provides a marker for special mission targets
-
-	short int is_friendly;	// is this a friendly droid or is it a MS controlled one?
-	char has_been_taken_over;	// has the Tux made this a friendly bot via takeover subgame?
-	char attack_target_type;	// attack NOTHING, PLAYER, or BOT
-	short int bot_target_n;
-	struct enemy_s *bot_target_addr;
+	//--------------------
+	// 1st set (identity)
+	//
+	short int id;                       // unique id of the droid. start at 1.
+	short int type;                     // the number of the droid specifications in Druidmap
+	char SpecialForce;                  // this flag will exclude the droid from initial shuffling of droids
+	int marker;                         // this provides a marker for special mission targets
+	short int max_distance_to_home;     // how far this robot will go before returning to it's home waypoint
 	string dialog_section_name;
 	string short_description_text;
-	char will_rush_tux;	// will this robot approach the Tux on sight and open communication?
-	char has_greeted_influencer;	// has this robot issued his first-time-see-the-Tux message?
-	float previous_angle;	// which angle has this robot been facing the frame before?
-	float current_angle;	// which angle will the robot be facing now?
-	float last_phase_change;	// when did the robot last change his (8-way-)direction of facing
-	float previous_phase;	// which (8-way) direction did the robot face before?
-	float last_combat_step;	// when did this robot last make a step to move in closer or farther away from Tux in combat?
+	short int on_death_drop_item_code;  // drop a pre-determined item when dying
 
+	//--------------------
+	// 2nd set ('global state')
+	//
+	int faction;
+	char will_rush_tux;           // will this robot approach the Tux on sight and open communication?
+	int combat_state;             // current state of the bot
+	float state_timeout;          // time spent in this state (used by "timeout" states such as STOP_AND_EYE_TARGET only)
+	short int CompletelyFixed;    // set this flag to make the robot entirely immobile
+	char has_been_taken_over;     // has the Tux made this a friendly bot via takeover subgame?
+	char follow_tux;              // does this robot try to follow tux via it's random movements?
+	char has_greeted_influencer;  // has this robot issued his first-time-see-the-Tux message?
+	short int nextwaypoint;       // the next waypoint target
+	short int lastwaypoint;       // the waypoint from whence this robot just came from
+	short int homewaypoint;       // the waypoint this robot started at
+	gps pos;                      // coordinates of the current position in the level
+
+	//--------------------
+	// 3rd set ('transient state')
+	//
+	finepoint speed;                   // current speed
+	float energy;                      // current energy of this droid
+	float animation_phase;             // the current animation frame for this enemy (starting at 0 of course...)
+	short int animation_type;          // walk-animation, attack-animation, gethit animation, death animation
+	float frozen;                      // is this droid currently frozen and for how long will it stay this way?
+	float poison_duration_left;        // is this droid currently poisoned and for how long will it stay this way?
+	float poison_damage_per_sec;       // is this droid currently poisoned and how much poison is at work?
+	float paralysation_duration_left;  // is this droid currently paralyzed and for how long will it stay this way?
+	float pure_wait;                   // time till the droid will start to move again
+	float firewait;                    // time this robot still takes until its weapon will be fully reloaded
+	short int ammo_left;               // ammunition left in the charger
+	char attack_target_type;           // attack NOTHING, PLAYER, or BOT
+	short int bot_target_n;
+	struct enemy *bot_target_addr;
+	float previous_angle;              // which angle has this robot been facing the frame before?
+	float current_angle;               // which angle will the robot be facing now?
+	float previous_phase;              // which (8-way) direction did the robot face before?
+	float last_phase_change;           // when did the robot last change his (8-way-)direction of facing
+	float last_combat_step;            // when did this robot last make a step to move in closer or farther away from Tux in combat?
 	float TextVisibleTime;
-	char *TextToBeDisplayed;
+	char *TextToBeDisplayed;           // WARNING!!! Only use static texts
 	moderately_finepoint PrivatePathway[5];
-
 	char bot_stuck_in_wall_at_previous_check;
 	float time_since_previous_stuck_in_wall_check;
 
-	list_head_t global_list;	// entry of this bot in the global bot lists (alive or dead)
-	list_head_t level_list;	// entry of this bot in the level bot list (alive only)
+	//--------------------
+	// Misc attributes
+	//
+	gps virt_pos;             // the virtual position (position of the bot if he was on this level, differs from above when it is on a neighboring level)
+	list_head_t global_list;  // entry of this bot in the global bot lists (alive or dead)
+	list_head_t level_list;   // entry of this bot in the level bot list (alive only)
 } enemy, *Enemy;
 
 typedef struct npc {
@@ -478,7 +510,8 @@ typedef struct npc {
 	unsigned char chat_flags[MAX_ANSWERS_PER_PERSON];
 
 	string shoplist[MAX_ITEMS_IN_INVENTORY]; //list of items that can be put in the inventory of the NPC
-	item npc_inventory[MAX_ITEMS_IN_INVENTORY]; // current NPC stock (what can be readily bought)
+	int shoplistweight[MAX_ITEMS_IN_INVENTORY]; //weight of each item: relative probability of appearance in inventory
+	item npc_inventory[MAX_ITEMS_IN_NPC_INVENTORY]; // current NPC stock (what can be readily bought)
 
 	float last_trading_date; // when did we trade with this NPC the last time?
 
@@ -486,13 +519,14 @@ typedef struct npc {
 } npc;
 
 typedef char automap_data_t[100][100];
-typedef struct tux_s {
+typedef struct tux {
 	float current_game_date;	// seconds since game start, will be printed as a different 'date'
 	// inside the game, like 14:23 is afternoon
 	int current_power_bonus;
 	float power_bonus_end_date;
 	int current_dexterity_bonus;
 	float dexterity_bonus_end_date;
+	float light_bonus_end_date;
 
 	finepoint speed;	// the current speed of the druid 
 	gps pos;		// current position in the whole ship 
@@ -522,9 +556,6 @@ typedef struct tux_s {
 	float health_recovery_rate;	//points of health recovered each second
 	float cooling_rate;	//temperature points recovered each second
 
-	int16_t LastMouse_X;	// mostly for other players:  Where was the last mouseclick...
-	int16_t LastMouse_Y;	// mostly for other players:  Where was the last mouseclick...
-
 	double busy_time;	// time remaining, until the weapon is ready to fire again...
 	int busy_type;		// reason why tux is busy (enum)
 	double phase;		// the current phase of animation 
@@ -547,14 +578,15 @@ typedef struct tux_s {
 	int points_to_distribute;	// these are the points that are available to distribute upon the character stats
 	float base_damage;	// the current damage the influencer does
 	float damage_modifier;	// the modifier to the damage the influencer currently does
-	float DR;		// the current Armour Damage Reduction of the influencer
+	float armor_class;
 	float to_hit;		// percentage chance, that Tux will hit a random lv 1 bot
-	int lv_1_bot_will_hit_percentage;	// percentage chance that a random lv 1 bot will hit
 	int resist_fire;	// percentage to reduce from fire damage
 	int resist_electricity;	// percentage to reduce from electricity damage
 
-	int freezing_melee_targets;	// does this Tux freeze melee targets upon hit?
+	int slowing_melee_targets;	// duration for how long hit enemies are slowed down
+	int paralyzing_melee_targets;	// duration for how long hit enemies are paralyzed
 	int double_ranged_damage;	// does this Tux do double ranged weapon damage?
+	float experience_factor; // multiplier for the experience gained from bots
 
 	unsigned int Experience;	// character Experience = 'spare droid elements found'
 	int exp_level;		// which 'experience level' is the influencer currenly at?
@@ -567,15 +599,14 @@ typedef struct tux_s {
 	int marker;		// In case you've taken over a marked droid, this will contain the marker
 	float LastCrysoundTime;
 	float TextVisibleTime;
-	char *TextToBeDisplayed;
+	char *TextToBeDisplayed;                    // WARNING!!! Only use static texts
 
 	//--------------------
 	// Here we note all the 'skill levels' of the Tux and also which skill is
 	// currently readied and that...
 	//
 	int readied_skill;
-	int SkillLevel[MAX_NUMBER_OF_PROGRAMS];
-	int base_skill_level[MAX_NUMBER_OF_PROGRAMS];
+	int skill_level[MAX_NUMBER_OF_PROGRAMS];
 	int melee_weapon_skill;
 	int ranged_weapon_skill;
 	int spellcasting_skill;
@@ -599,16 +630,12 @@ typedef struct tux_s {
 	float time_since_last_visit_or_respawn[MAX_LEVELS];	// record of the levels the player has visited yet.
 
 	string cookie_list[MAX_COOKIES];
-	int is_town_guard_member;
 
 	//--------------------
 	// THE FOLLOWING ARE INFORMATION, THAT ARE HUGE AND THAT ALSO DO NOT NEED
 	// TO BE COMMUNICATED FROM THE CLIENT TO THE SERVER OR VICE VERSA
 	//
 	moderately_finepoint next_intermediate_point[MAX_INTERMEDIATE_WAYPOINTS_FOR_TUX];	// waypoints for the tux, when target not directly reachable
-	unsigned short int KillRecord[200];	// how many ( of the first 1000 monster types) have been killed yet?
-	unsigned short int TakeoverSuccesses[200]; // how many (of each type) did Tux takeover and make friendly?
-	unsigned short int TakeoverFailures[200];  // how many times for (each type) did Tux fail at a takeover attempt?
 	automap_data_t Automap[MAX_LEVELS];
 	int current_zero_ring_index;
 	gps Position_History_Ring_Buffer[MAX_INFLU_POSITION_HISTORY];
@@ -624,15 +651,24 @@ typedef struct tux_s {
 	int quest_browser_changed;
 
 	int program_shortcuts[10];
-} tux_t, *Tux_t;
 
-typedef struct bulletspec_s {
+	// STATISTICS ABOUT TUX
+	// Should warn if less than Number_Of_Droid_Types + 2
+#define NB_DROID_TYPES 50
+	int TakeoverSuccesses[NB_DROID_TYPES]; // how many did Tux takeover and make friendly?
+	int TakeoverFailures[NB_DROID_TYPES];  // how many did Tux fail at a takeover attempt?
+	int destroyed_bots[NB_DROID_TYPES];    // how many bots have been destroyed?
+	int damage_dealt[NB_DROID_TYPES];      // how much damage dealt?
+	float meters_traveled;     // how many meters has Tux traveled?
+} tux_t;
+
+typedef struct bulletspec {
 	int phases;		// how many phases in motion to show 
 	double phase_changes_per_second;	// how many different phases to display every second
 	iso_image image[BULLET_DIRECTIONS][MAX_PHASES_IN_A_BULLET];
-} bulletspec, *Bulletspec;
+} bulletspec;
 
-typedef struct bullet_s {
+typedef struct bullet {
 	short int type;
 	unsigned char phase;
 	signed char mine;
@@ -649,7 +685,6 @@ typedef struct bullet_s {
 
 	char was_reflected;
 	char ignore_wall_collisions;
-	short int to_hit;
 	char pass_through_hit_bodies;	// does this bullet go through hit bodies (e.g. like a laser sword stike)
 	char pass_through_explosions;	// does this bullet go through explosions (e.g. laser sword stike though dead droid)
 	short int freezing_level;	// does this bullet freeze the target?
@@ -657,11 +692,11 @@ typedef struct bullet_s {
 	float poison_damage_per_sec;
 	float paralysation_duration;
 
-	char is_friendly;	//ugly hack, great idea - if the bot has the same value, the bullet just goes through
+	int faction;
 	char hit_type;		//hit bots, humans, both?
 } bullet, *Bullet;
 
-typedef struct melee_shot_s	// this is a melee shot
+typedef struct melee_shot	// this is a melee shot
 {
 	char attack_target_type;	//type of attack
 	char mine;		//is it mine?
@@ -670,16 +705,15 @@ typedef struct melee_shot_s	// this is a melee shot
 	short int to_hit;	//chance to hit, percent
 	short int damage;
 	short int owner;
-	char level;
 } melee_shot;
 
-typedef struct blastspec_s {
+typedef struct blastspec {
 	int phases;
 	float total_animation_time;
 	iso_image image[PHASES_OF_EACH_BLAST];
-} blastspec, *Blastspec;
+} blastspec;
 
-typedef struct blast_s {
+typedef struct blast {
 	gps pos;
 	int type;
 	float phase;
@@ -687,7 +721,7 @@ typedef struct blast_s {
 	float damage_per_second;
 } blast, *Blast;
 
-typedef struct spell_active_s {
+typedef struct spell_active {
 	int img_type;		// what type of spell is active?
 	int damage;
 	int poison_duration;
@@ -700,9 +734,9 @@ typedef struct spell_active_s {
 	char active_directions[RADIAL_SPELL_DIRECTIONS];
 	int mine;
 	char hit_type;
-} spell_active, *Spell_Active;
+} spell_active;
 
-typedef struct spell_skill_spec_s {
+typedef struct spell_skill_spec {
 	char *name;
 	char *icon_name;
 	iso_image icon_surface;
@@ -720,29 +754,32 @@ typedef struct spell_skill_spec_s {
 	float effect_duration;
 	float effect_duration_per_level;
 	int graphics_code;
-} spell_skill_spec, *Spell_Skill_Spec;
+} spell_skill_spec;
 
-typedef struct waypoint_s {
+typedef struct waypoint {
 	int x;
 	int y;
-	int num_connections;
 	int suppress_random_spawn;
-	int connections[MAX_WP_CONNECTIONS];
-} waypoint, *Waypoint;
+	struct dynarray connections;
+} waypoint;
 
-typedef struct obstacle_s {
+typedef struct obstacle {
 	int type;
 	gps pos;
-	gps vpos;
-	int name_index;
-} obstacle, *Obstacle;
+} obstacle;
 
-typedef struct map_tile_s {
+typedef struct map_tile {
 	Uint16 floor_value;
 	int obstacles_glued_to_here[MAX_OBSTACLES_GLUED_TO_ONE_MAP_TILE];
-} map_tile, *Map_tile;
+} map_tile;
 
-typedef struct level_s {
+struct obstacle_extension {
+	obstacle *obs;
+	enum obstacle_extension_type type;
+	void *data;
+}; /** This contains "extension data" for obstacles - labels, item lists, ... */
+
+typedef struct level {
 	int levelnum;
 	int xlen;
 	int ylen;
@@ -750,9 +787,9 @@ typedef struct level_s {
 	int minimum_light_value;
 	int infinite_running_on_this_level;
 	int random_dungeon;
+	int dungeon_generated;
 	char *Levelname;
 	char *Background_Song_Name;
-	char *obstacle_name_list[MAX_OBSTACLE_NAMES_PER_LEVEL];
 
 	map_tile *map[MAX_MAP_LINES];	// this is a vector of pointers
 	int jump_target_north;
@@ -762,20 +799,21 @@ typedef struct level_s {
 	int use_underground_lighting;
 
 	obstacle obstacle_list[MAX_OBSTACLES_ON_MAP];
-
-	map_label labels[MAX_MAP_LABELS_PER_LEVEL];
-	int num_waypoints;
-	waypoint AllWaypoints[MAXWAYPOINTS];
 	item ItemList[MAX_ITEMS_PER_LEVEL];
-	item ChestItemList[MAX_ITEMS_PER_LEVEL];
+
+	struct dynarray obstacle_extensions;
+	struct dynarray map_labels;
+	struct dynarray waypoints;
+
+	int teleport_pair;
 } level, *Level;
 
-typedef struct obstacle_spec_s {
+typedef struct obstacle_spec {
 	iso_image image;
 	iso_image shadow_image;
-	SDL_Surface *automap_version;
 
 	char image_loaded;
+    char *label; 
 
 	//--------------------
 	// Some obstacles will block the Tux from walking through them.
@@ -785,10 +823,10 @@ typedef struct obstacle_spec_s {
 	//
 	float block_area_parm_1;
 	float block_area_parm_2;
-	float upper_border;
-	float lower_border;
 	float left_border;
 	float right_border;
+	float upper_border;
+	float lower_border;
 
 	float diaglength;
 
@@ -805,12 +843,13 @@ typedef struct obstacle_spec_s {
 	char transparent;
 	
 	char *filename;
+    void (*action) (level *obst_lvl, int obstacle_id);
 	
 	//--------------------
 	// Some obstacles have an associated animation.
 	// This property defines the function to call to animate them
 	int (*animate_fn) (level *obstacle_lvl, int obstacle_idx);
-} obstacle_spec, *Obstacle_spec;
+} obstacle_spec;
 
 struct visible_level {
 	int valid;
@@ -821,38 +860,27 @@ struct visible_level {
 	struct list_head node;
 };
 
-typedef struct ship_s {
+typedef struct ship {
 	int num_levels;
 	level *AllLevels[MAX_LEVELS];
-} ship, *Ship;
+} ship;
 
-typedef struct dialogue_option_s {
+typedef struct dialogue_option {
 	char *option_text;
 	char *option_sample_file_name;
-
-	char *reply_sample_list[MAX_REPLIES_PER_OPTION];
-	char *reply_subtitle_list[MAX_REPLIES_PER_OPTION];
-
 	luacode lua_code;
 
-	char exists;
-} dialogue_option, *Dialogue_option;
+	int exists;
+} dialogue_option;
 
-typedef struct supported_languages_s {
-	char *code;
-	char *name;
-	char *font_class;
-	char *encoding;
-} supported_languages_t;
-
-typedef struct colldet_filter_s {
-	int (*callback) (struct colldet_filter_s * filter, obstacle * obs, int obs_idx);
+typedef struct colldet_filter {
+	int (*callback) (struct colldet_filter *filter, obstacle *obs, int obs_idx);
 	void *data;
-	int use_margin;
-	struct colldet_filter_s *next;
+	float extra_margin;
+	struct colldet_filter *next;
 } colldet_filter;
 
-typedef struct light_radius_config_s {
+typedef struct light_radius_config {
 	uint32_t cells_w;
 	uint32_t cells_h;
 	uint32_t texture_w;
@@ -861,7 +889,7 @@ typedef struct light_radius_config_s {
 	float scale_factor;
 } light_radius_config;
 
-typedef struct screen_resolution_s {
+typedef struct screen_resolution {
 	int xres;
 	int yres;
 	char *comment;
@@ -874,7 +902,7 @@ typedef struct screen_resolution_s {
  * Note : '2' excepted bots are needed when the pathfinder is called
  *         to let a bot reach an other one (used during attack, for example).
  */
-typedef struct freeway_context_s {
+typedef struct freeway_context {
 	int check_tux;		// Check if Tux is along the way
 	Enemy except_bots[2];	// Do not check if those bots are along the way (see note below)
 } freeway_context;
@@ -882,7 +910,7 @@ typedef struct freeway_context_s {
 /*
  * Pathfinder's execution context
  */
-typedef struct pathfinder_context_s {
+typedef struct pathfinder_context {
 	colldet_filter *dlc_filter;	// DLC filter to use
 	freeway_context *frw_ctx;	// CheckIfWayIsFreeOfDroids's execution context to use
 } pathfinder_context;
@@ -899,4 +927,50 @@ struct auto_string {
 	unsigned long capacity;
 };
 
+/* text_widget.c */
+typedef struct text_widget {
+	SDL_Rect rect;             /* The area in which the text should be displayed */
+	struct BFont_Info *font;
+	struct auto_string *text;
+	float text_stretch;
+
+	int scroll_offset;         /* 0 means bottom, negative means above bottem. */
+	int mouse_already_handled;
+
+	void (*content_below_func)(void);
+	void (*content_above_func)(void);
+} text_widget;
+
+/*
+ * Specification of Animations
+ */
+
+struct timebased_animation {
+	float duration;			// Duration of the animation
+	int first_keyframe;
+	int last_keyframe;
+	int nb_keyframes;		// Set to (last - first). Avoid to compute it each time this information is needed
+};
+
+struct distancebased_animation {
+	float distance;			// Distance covered during the whole animation
+	int first_keyframe;
+	int last_keyframe;
+	int nb_keyframes;		// Set to (last - first). Avoid to compute it each time this information is needed
+};
+
+/**
+ * Specification of Tux's parts rendering order
+ * Defines how to order Tux's parts for a 'set' of animation phases.
+ * A linked list of sets is associated to a (motion_class, rotation) pair
+ * (see the definition of struct tux_rendering_s in global.h).
+ */
+
+struct tux_part_render_set {
+	int phase_start;	// First animation phase of the set
+	int phase_end;		// Last animation phase of the set
+	void (*render_funcs[ALL_PART_GROUPS])(int, int, int, int, int);	// Ordered array of rendering functions to call
+	struct tux_part_render_set *next;
+};
+
 #endif
