@@ -42,20 +42,20 @@ void to_show_banner(const char *left, const char *right);
 					/* inclusive "inactive" phase */
 
 /* Dimensions of the game-blocks */
-#define TO_BLOCKS		11	/* anzahl versch. Game- blocks */
+#define TO_BLOCKS		11	/* quantity of different game- blocks */
 
 #define NUM_TO_BLOCKS		2*NUM_PHASES*TO_BLOCKS	// total number of takover blocks
 #define TO_ELEMENTS		6
-#define TO_BLOCKLEN		2*32	/* dimension of a Game- block */
+#define TO_BLOCKLEN		2*32	/* dimension of a game- block */
 #define TO_BLOCKHEIGHT	 	2*8
 
 /* Dimensions of the fill-blocks (in led-column */
-#define NUM_FILL_BLOCKS		3	// yellow, violett and black
+#define NUM_FILL_BLOCKS		3	// yellow, purple and black
 #define FILL_BLOCK_LEN		2*16
 #define FILL_BLOCK_HEIGHT  	2*7
 
 /* Dimensions of a capsule */
-#define NUM_CAPS_BLOCKS		3	// yellow, violett and red (?what for)
+#define NUM_CAPS_BLOCKS		3	// yellow, purple and red (?what for)
 #define CAPSULE_LEN		2*7
 #define CAPSULE_HEIGHT		2*7
 
@@ -71,12 +71,12 @@ void to_show_banner(const char *left, const char *right);
 #define LEADERBLOCKHEIGHT	2*19
 
 /* --------------- Timing parameters --------------- */
-#define COLOR_COUNTDOWN		100	/* Zeit zum Farbe auswaehlen */
-#define GAME_COUNTDOWN		100	/* Zeit fuer das Spiel */
-#define CAPSULE_COUNTDOWN	 40	/* 1/10 sec. Lebensdauer einer Kapsel */
+#define COLOR_COUNTDOWN		100	/* time to choose color */
+#define GAME_COUNTDOWN		100	/* Time to end the game */
+#define CAPSULE_COUNTDOWN	 40	/* 1/10 sec. lifetime of a capsule */
 
-#define WAIT_MOVEMENT		0	/* 1/18 sekunden Bewegungsgeschw. */
-#define WAIT_COLOR_ROTATION	2	/* 1/18 sekunden aktive-Kabel */
+#define WAIT_MOVEMENT		0	/* 1/18 sec. ovementspeed. */
+#define WAIT_COLOR_ROTATION	2	/* 1/18 seC active-capsule */
 #define WAIT_AFTER_GAME		2*18	/* Wait after a deadlock */
 
 #define TO_TICK_LENGTH		40	/* Time in ms between ticks */
@@ -91,51 +91,51 @@ void to_show_banner(const char *left, const char *right);
 #define LEDCOLUMN_X		2*136
 #define LEDCOLUMN_Y		2*27
 
-/* Positions of Druid-pictures */
-#define GELB_DRUID_X		2*40
-#define GELB_DRUID_Y		0
-#define VIOLETT_DRUID_X		2*220
-#define VIOLETT_DRUID_Y		GELB_DRUID_Y
+/* Positions of Droid-pictures */
+#define YELLOW_DROID_X		2*40
+#define YELLOW_DROID_Y		0
+#define PURPLE_DROID_X		2*220
+#define PURPLE_DROID_Y		YELLOW_DROID_Y
 
 /* Left-Capsules positions */
-#define GELB_LEFT_CAPSULES_X	4
-#define GELB_LEFT_CAPSULES_Y	2*27
-#define VIOLETT_LEFT_CAPSULES_X	RIGHT_OFFS_X + COLUMNBLOCKLEN -10
-#define VIOLETT_LEFT_CAPSULES_Y	2*27
+#define YELLOW_LEFT_CAPSULES_X	4
+#define YELLOW_LEFT_CAPSULES_Y	2*27
+#define PURPLE_LEFT_CAPSULES_X	RIGHT_OFFS_X + COLUMNBLOCKLEN -10
+#define PURPLE_LEFT_CAPSULES_Y	2*27
 
 /* start-pos of Current Capsule */
-#define GELB_CUR_CAPSULE_X	2*26
-#define GELB_CUR_CAPSULE_Y	2*19
-#define VIOLETT_CUR_CAPSULE_X	2*255
-#define VIOLETT_CUR_CAPSULE_Y	2*19
+#define YELLOW_CUR_CAPSULE_X	2*26
+#define YELLOW_CUR_CAPSULE_Y	2*19
+#define PURPLE_CUR_CAPSULE_X	2*255
+#define PURPLE_CUR_CAPSULE_Y	2*19
 
 #define MAX_CAPSULES		13	/* a 999 has 13 !!! */
 
 /* Start-Pos of playground */
-#define GELB_PLAYGROUND_X	2*33
-#define GELB_PLAYGROUND_Y	2*26
-#define VIOLETT_PLAYGROUND_X	2*159
-#define VIOLETT_PLAYGROUND_Y	2*26
+#define YELLOW_PLAYGROUND_X	2*33
+#define YELLOW_PLAYGROUND_Y	2*26
+#define PURPLE_PLAYGROUND_X	2*159
+#define PURPLE_PLAYGROUND_Y	2*26
 
-#define LEFT_OFFS_X		2*10	/* Offset der linken "Saeule" */
+#define LEFT_OFFS_X		2*10	/* Offset of the left column */
 #define LEFT_OFFS_Y		2*15
 
-#define MID_OFFS_X		2*129	/* -- mittleren -- */
+#define MID_OFFS_X		2*129	/* -- of the middle one -- */
 #define MID_OFFS_Y		2*8
 
-#define RIGHT_OFFS_X		2*255	/* -- rechten --- */
+#define RIGHT_OFFS_X		2*255	/* -- and the right one --- */
 #define RIGHT_OFFS_Y		2*15
 
 enum _groundblocks {
-	GELB_OBEN,
-	GELB_MITTE,
-	GELB_UNTEN,
-	VIOLETT_OBEN,
-	VIOLETT_MITTE,
-	VIOLETT_UNTEN
+	YELLOW_HIGH,
+	YELLOW_MIDDLE,
+	YELLOW_LOW,
+	PURPLE_HIGH,
+	PURPLE_MIDDLE,
+	PURPLE_LOW
 };
 
-/* Konditions in Connection-layer */
+/* Conditions in Connection-layer */
 enum condition {
 	INACTIVE = 0,
 	ACTIVE1,
@@ -153,34 +153,34 @@ enum to_opponents {
 /* Color-names */
 #define TO_COLORS		2
 enum to_colors {
-	GELB = 0,
-	VIOLETT,
-	REMIS
+	YELLOW = 0,
+	PURPLE,
+	DRAW
 };
 
 /* Element - Names */
 enum to_elements {
-	EL_KABEL,
-	EL_KABELENDE,
-	EL_VERSTAERKER,
-	EL_FARBTAUSCHER,
-	EL_VERZWEIGUNG,
-	EL_GATTER
+	EL_CABLE,
+	EL_CABLE_END,
+	EL_AMPLIFIER,
+	EL_COLOR_EXCHANGER,
+	EL_SEPARATOR,
+	EL_GATE
 };
 
 /* Block-Names */
 enum to_block_tyes {
-	KABEL,
-	KABELENDE,
-	VERSTAERKER,
-	FARBTAUSCHER,
-	VERZWEIGUNG_O,
-	VERZWEIGUNG_M,
-	VERZWEIGUNG_U,
-	GATTER_O,
-	GATTER_M,
-	GATTER_U,
-	LEER
+	CABLE,
+	CABLE_END,
+	AMPLIFIER,
+	COLOR_EXCHANGER,
+	SEPARATOR_H,
+	SEPARATOR_M,
+	SEPARATOR_L,
+	GATE_H,
+	GATE_M,
+	GATE_L,
+	EMPTY
 };
 
 /* there are two classes of blocks: connectors and non-connectors */
