@@ -44,11 +44,42 @@
 #include "lvledit/lvledit_display.h"
 
 // curent text insertion position
-int MyCursorX;
-int MyCursorY;
+static int MyCursorX;
+static int MyCursorY;
 
 int display_char_disabled_local;
 
+/**
+ * This function determines how many lines of text it takes to write a string
+ * with the current font into the given rectangle, provided one would start at
+ * the left side.
+ */
+int get_lines_needed(const char *text, SDL_Rect t_rect, float text_stretch)
+{
+	t_rect.h = 32767; // arbitrary large number
+	display_char_disabled = TRUE;
+	int lines = DisplayText(text, t_rect.x, t_rect.y, &t_rect, text_stretch);
+	display_char_disabled = FALSE;
+	return lines;
+}
+
+/**
+ * Display one character, updating MyCursorX to new position.
+ */
+static void display_char(unsigned char c)
+{
+	if (handle_switch_font_char(c))
+		return;
+
+	if (c < ' ' || c > GetCurrentFont()->number_of_chars - 1)
+		c = '.';
+
+	if ((!display_char_disabled) && (!display_char_disabled_local))
+		PutCharFont(Screen, GetCurrentFont(), MyCursorX, MyCursorY, c);
+
+	MyCursorX += CharWidth(GetCurrentFont(), c) + get_letter_spacing(GetCurrentFont());
+}
+
 /**
  *
  *
@@ -75,67 +106,100 @@ void show_backgrounded_label_at_pixel_position(char *LabelText, int pos_x, int p
 void show_backgrounded_label_at_map_position(char *LabelText, float fill_status, float pos_x, float pos_y, int zoom_is_on)
 {
 	int pixel_x, pixel_y;
-	float zoom_factor = (zoom_is_on ? lvledit_zoomfact_inv() : 1.0);
 
-	translate_map_point_to_screen_pixel(pos_x, pos_y, &pixel_x, &pixel_y, zoom_factor);
+	translate_map_point_to_screen_pixel(pos_x, pos_y, &pixel_x, &pixel_y);
 	show_backgrounded_label_at_pixel_position(LabelText, pixel_x, pixel_y);
 
 };				// void show_backgrounded_label_at_map_position ( char* LabelText , float fill_status , float pos_x , float pos_y )
 
-/**
- *
- *
- */
-void show_backgrounded_text_rectangle(const char *text, int x, int y, int w, int h)
-{
-	SDL_Rect TargetRect;
-
-	TargetRect.w = w;
-	TargetRect.h = h;
-	TargetRect.x = x;
-	TargetRect.y = y;
-	our_SDL_fill_rect_wrapper(Screen, &TargetRect, SDL_MapRGB(Screen->format, 0, 0, 0));
-
 #define IN_WINDOW_TEXT_OFFSET 15
-	TargetRect.w -= IN_WINDOW_TEXT_OFFSET;
-	TargetRect.h -= IN_WINDOW_TEXT_OFFSET;
-	TargetRect.x += IN_WINDOW_TEXT_OFFSET;
-	TargetRect.y += IN_WINDOW_TEXT_OFFSET;
-
-	SetCurrentFont(FPS_Display_BFont);
-	DisplayText(text, TargetRect.x, TargetRect.y, &TargetRect, 1.0);
-
-};				// void show_backgrounded_text_rectangle ( char* text , int x , int y , int w , int h )
 
 /**
- * In some cases it will be nescessary to inform the user of something in
- * a big important style.  Then a popup window is suitable, with a mouse
- * button to confirm and make it go away again.
+ * Show text inside a filled rectangle. The specified height is taken as a
+ * minimum value, and it will be expanded vertically to fit the given text.
+ * @Ret: The height of the rectangle
  */
-void GiveMouseAlertWindow(const char *WindowText)
+int show_backgrounded_text_rectangle(const char *text, struct BFont_Info *font, int x, int y, int w, int h)
 {
-	SDL_Event e;
+	BFont_Info *old_font = GetCurrentFont();
+	SetCurrentFont(font);
+
+	SDL_Rect t_rect;
+	t_rect.x = x;
+	t_rect.y = y;
+	
+	// Find out the number of lines the text will occupy. This is done by
+	// using the drawing function with drawing temporarily disabled.
+	t_rect.w = w - (IN_WINDOW_TEXT_OFFSET * 2);
+	t_rect.h = GameConfig.screen_height;
+	int lines = get_lines_needed(text, t_rect, 1.0);
+
+	// Calculate the rectangle height
+	int f_height = FontHeight(GetCurrentFont());
+	int r_height = (lines * f_height) + (IN_WINDOW_TEXT_OFFSET * 2);
+
+	// Set up and fill the rectangle.
+	t_rect.w = w;
+	t_rect.h = r_height;
+	our_SDL_fill_rect_wrapper(Screen, &t_rect, SDL_MapRGB(Screen->format, 0, 0, 0));
+	
+	// Show the text inside our newly drawn rectangle.
+	t_rect.w -= IN_WINDOW_TEXT_OFFSET * 2;
+	t_rect.h -= IN_WINDOW_TEXT_OFFSET;
+	t_rect.x += IN_WINDOW_TEXT_OFFSET;
+	t_rect.y += IN_WINDOW_TEXT_OFFSET;
+	DisplayText(text, t_rect.x, t_rect.y, &t_rect, 1.0);
+
+	SetCurrentFont(old_font);
+	return r_height;
+}
+
+/**
+ * In some cases it will be necessary to inform the user of something in
+ * a big important style.  Then a popup window is suitable, with a mouse
+ * click to confirm and make it go away again.
+ */
+void alert_window(const char *text)
+{
+	int w = 440;   // arbitrary
+	int h = 60;    // arbitrary
+	int x = (GameConfig.screen_width  - w) / 2;	// center of screen
+	int y = (GameConfig.screen_height - h) / 5 * 2; // 2/5 of screen from top
 
 	Activate_Conservative_Frame_Computation();
+	make_sure_system_mouse_cursor_is_turned_off();
+	StoreMenuBackground(1);
 
-	show_backgrounded_text_rectangle(WindowText, (640 - 440) / 2, (480 - 340) / 2, 440, 340);
-
-	our_SDL_flip_wrapper();
-
+	SDL_Event e;
 	while (1) {
+		SDL_Delay(1);
+		RestoreMenuBackground(1);
+
+		int r_height = show_backgrounded_text_rectangle(text, FPS_Display_BFont, x, y, w, h);
+		show_backgrounded_text_rectangle(_("Click to continue..."), Red_BFont, x, y + r_height, w, 10);
+
+		blit_our_own_mouse_cursor();
+		our_SDL_flip_wrapper();
+		save_mouse_state();
+		
 		SDL_WaitEvent(&e);
+		
 		switch (e.type) {
 		case SDL_KEYDOWN:
 			if (e.key.keysym.sym == SDLK_SPACE || e.key.keysym.sym == SDLK_RETURN || e.key.keysym.sym == SDLK_ESCAPE)
-				return;
+				goto wait_click_and_out;
 			break;
 		case SDL_MOUSEBUTTONDOWN:
 			if (e.button.button == 1)
-				return;
+				goto wait_click_and_out;
 			break;
 		}
 	}
-};				// void GiveMouseAlertWindow( char* WindowText )
+
+wait_click_and_out:
+	while (SpacePressed() || EnterPressed() || EscapePressed() || MouseLeftPressed());
+
+};				// void alert_window( char* text )
 
 /**
  * 
@@ -173,7 +237,7 @@ void EnemyHitByBulletText(enemy * ThisRobot)
 		return;
 
 	ThisRobot->TextVisibleTime = 0;
-	if (!ThisRobot->is_friendly)
+	if (ThisRobot->faction == FACTION_BOTS)
 		switch (MyRandom(4)) {
 		case 0:
 			ThisRobot->TextToBeDisplayed = _("Unhandled exception fault.  Press ok to reboot.");
@@ -194,17 +258,6 @@ void EnemyHitByBulletText(enemy * ThisRobot)
 		ThisRobot->TextToBeDisplayed = _("Aargh, I got hit.  Ugh, I got a bad feeling...");
 };				// void EnemyHitByBullet( int Enum );
 
-/**
- * This function sets the text cursor used in DisplayText.
- */
-void SetTextCursor(int x, int y)
-{
-	MyCursorX = x;
-	MyCursorY = y;
-
-	return;
-};				// void SetTextCursor ( int x , int y )
-
 /* -----------------------------------------------------------------
  *
  * This function scrolls a given text down inside the User-window, 
@@ -236,7 +289,6 @@ int ScrollText(char *Text, int background_code)
 		if (background_code != (-1))
 			blit_special_background(background_code);
 		Number_Of_Line_Feeds = DisplayText(Text, ScrollRect.x, InsertLine, &ScrollRect, TEXT_STRETCH);
-		//--------------------
 		// We might add some buttons to be displayed here, so that, if you don't have
 		// a mouse wheel and don't know about cursor keys, you can still click on these
 		// buttons to control the scrolling speed of the text.
@@ -295,7 +347,6 @@ int ScrollText(char *Text, int background_code)
 
 		InsertLine -= speed;
 
-		//--------------------
 		// impose some limit on the amount to scroll away downwards and topwards
 		//
 		if (InsertLine > StartInsertLine && (speed < 0)) {
@@ -388,13 +439,16 @@ int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip,
 	SDL_Rect Temp_Clipping_Rect;	// adding this to prevent segfault in case of NULL as parameter
 	SDL_Rect store_clip;
 	short int nblines = 1;
+	int empty_lines_started = 0;
 
-	int kerning = 0;
-	if (GetCurrentFont() == FPS_Display_BFont || GetCurrentFont() == Blue_BFont || GetCurrentFont() == Red_BFont)
-		kerning = -2;
-	int tab_width = TABWIDTH * (CharWidth(GetCurrentFont(), TABCHAR) + kerning);
+	int letter_spacing = get_letter_spacing(GetCurrentFont());
+	int tab_width = TABWIDTH * (CharWidth(GetCurrentFont(), TABCHAR) + letter_spacing);
+
+	if (!*Text) {
+		nblines = 0;
+		return nblines;
+	}
 
-	//--------------------
 	// We position the internal text cursor on the right spot for
 	// the first character to be printed.
 	//
@@ -403,13 +457,11 @@ int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip,
 	if (starty != -1)
 		MyCursorY = starty;
 
-	//--------------------
 	// We make a backup of the current clipping rect, so we can respore
 	// it later.
 	//
 	SDL_GetClipRect(Screen, &store_clip);
 
-	//--------------------
 	// If we did receive some clipping rect in the parameter list (like e.g. it's
 	// always the case with dialog output) we enforce this new clipping rect, otherwise
 	// we just set the clipping rect to contain the whole screen.
@@ -424,7 +476,6 @@ int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip,
 		Temp_Clipping_Rect.h = GameConfig.screen_height;
 	}
 
-	//--------------------
 	// Now we can start to print the actual text to the screen.
 	//
 	// The running text pointer must be initialized.
@@ -434,7 +485,7 @@ int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip,
 		if (((*tmp == ' ') || (*tmp == '\t'))
 		    && (ImprovedCheckLineBreak(tmp, clip, text_stretch) == 1))	// dont write over right border 
 		{		/*THE CALL ABOVE HAS DONE THE CARRIAGE RETURN FOR US !!! */
-			nblines++;
+			empty_lines_started++;
 			++tmp;
 			continue;
 		}
@@ -442,14 +493,22 @@ int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip,
 		if (*tmp == '\n') {
 			MyCursorX = clip->x;
 			MyCursorY += (int)(FontHeight(GetCurrentFont()) * text_stretch);
-			nblines++;
+			empty_lines_started++;
 		} else if (*tmp == '\t') {
 			MyCursorX = (int)ceilf((float)MyCursorX / (float)(tab_width)) * (tab_width);
 		} else {
 			if (MyCursorY <= clip->y - (int)(FontHeight(GetCurrentFont()) * text_stretch))
 				display_char_disabled_local = TRUE;
-			DisplayChar(*tmp);
+			display_char(*tmp);
 			display_char_disabled_local = FALSE;
+
+			// At least one visible character must follow a line break or else
+			// the line is a trailing empty line not visible to the user. Such
+			// empty lines don't contribute to the visual height of the string.
+			if (isgraph(*tmp)) {
+				nblines += empty_lines_started;
+				empty_lines_started = 0;
+			}
 		}
 
 		tmp++;
@@ -459,47 +518,7 @@ int DisplayText(const char *Text, int startx, int starty, const SDL_Rect * clip,
 	SDL_SetClipRect(Screen, &store_clip);	// restore previous clip-rect 
 
 	return nblines;
-
-};				// int DisplayText(...)
-
-/* -----------------------------------------------------------------
- * This function displays a char.  It uses Menu_BFont now
- * to do this.  MyCursorX is  updated to new position.
- * ----------------------------------------------------------------- */
-void DisplayChar(unsigned char c)
-{
-
-	if (c == font_switchto_red[0]) {
-		SetCurrentFont(Red_BFont);
-		return;
-	} else if (c == font_switchto_blue[0]) {
-		SetCurrentFont(Blue_BFont);
-		return;
-	} else if (c == font_switchto_neon[0]) {
-		SetCurrentFont(FPS_Display_BFont);
-		return;
-	} else if (c == font_switchto_msgstat[0]) {
-		SetCurrentFont(Messagestat_BFont);
-		return;
-	} else if (c == font_switchto_msgvar[0]) {
-		SetCurrentFont(Messagevar_BFont);
-		return;
-	}
-	// stupid kerning hack, to get smooth font, but tighter look
-	int kerning = 0;
-	if (GetCurrentFont() == FPS_Display_BFont || GetCurrentFont() == Blue_BFont || GetCurrentFont() == Red_BFont)
-		kerning = -2;
-
-	if (c < ' ' || c > GetCurrentFont()->number_of_chars - 1) {
-		//printf("l: %u of %u \n",c,GetCurrentFont()->number_of_chars);
-		c = '.';
-	}
-	if (!display_char_disabled && !display_char_disabled_local)
-		PutCharFont(Screen, GetCurrentFont(), MyCursorX, MyCursorY, c);
-
-	MyCursorX += CharWidth(GetCurrentFont(), c) + kerning;
-
-};				// void DisplayChar(...)
+}
 
 /**
  * This function checks if the next word still fits in this line
@@ -516,18 +535,16 @@ int ImprovedCheckLineBreak(char *Resttext, const SDL_Rect * clip, float text_str
 {
 	int NeededSpace = 0;
 
-	int kerning = 0;
-	if (GetCurrentFont() == FPS_Display_BFont || GetCurrentFont() == Blue_BFont || GetCurrentFont() == Red_BFont)
-		kerning = -2;
+	int letter_spacing = get_letter_spacing(GetCurrentFont());
 
 	if (*Resttext == ' ')
-		NeededSpace = CharWidth(GetCurrentFont(), ' ') + kerning;
+		NeededSpace = CharWidth(GetCurrentFont(), ' ') + letter_spacing;
 	else if (*Resttext == '\t')
-		NeededSpace = TABWIDTH * (CharWidth(GetCurrentFont(), TABCHAR) + kerning);
+		NeededSpace = TABWIDTH * (CharWidth(GetCurrentFont(), TABCHAR) + letter_spacing);
 
 	Resttext++;
 	while ((*Resttext != ' ') && (*Resttext != '\t') && (*Resttext != '\n') && (*Resttext != 0)) {
-		NeededSpace += CharWidth(GetCurrentFont(), *Resttext) + kerning;
+		NeededSpace += CharWidth(GetCurrentFont(), *Resttext) + letter_spacing;
 		Resttext++;
 	}
 
@@ -657,7 +674,7 @@ char *GetString(int MaxLen, int background_code, const char *text_for_overhead_p
  * NOTE: MaxLen is the maximal _strlen_ of the string (excl. \0 !)
  * 
  * ----------------------------------------------------------------- */
-char *GetEditableStringInPopupWindow(int MaxLen, char *PopupWindowTitle, char *DefaultString)
+char *GetEditableStringInPopupWindow(int MaxLen, const char *PopupWindowTitle, const char *DefaultString)
 {
 	char *input;		// pointer to the string entered by the user
 	int key;		// last 'character' entered 
@@ -671,22 +688,11 @@ char *GetEditableStringInPopupWindow(int MaxLen, char *PopupWindowTitle, char *D
 
 #define EDIT_WINDOW_TEXT_OFFSET 15
 
-	// if ( MaxLen > 60 ) MaxLen = 60;
-
-	if (((int)strlen(DefaultString)) >= MaxLen - 1)
-		DefaultString[MaxLen - 1] = 0;
-
-	//--------------------
-	// We prepare a new string, mostly empty...
-	//
 	input = MyMalloc(MaxLen + 5);
-	// memset (input, '.', MaxLen);
-	// input[MaxLen] = 0;
-	// input [ 0 ] = 0 ;
-	strcpy(input, DefaultString);
+	strncpy(input, DefaultString, MaxLen - 1);
+	input[MaxLen] = 0;
 	curpos = strlen(input);
 
-	//--------------------
 	// Now we prepare a rectangle for our popup window...
 	//
 	TargetRect.w = 440;
@@ -698,7 +704,6 @@ char *GetEditableStringInPopupWindow(int MaxLen, char *PopupWindowTitle, char *D
 	TargetRect.x += EDIT_WINDOW_TEXT_OFFSET;
 	TargetRect.y += EDIT_WINDOW_TEXT_OFFSET;
 
-	//--------------------
 	// Now we find the right position for the new string to start by writing
 	// out the title text once, just to get the cursor positioned right...
 	//
@@ -707,7 +712,6 @@ char *GetEditableStringInPopupWindow(int MaxLen, char *PopupWindowTitle, char *D
 	y0 = MyCursorY;
 
 	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
-	//--------------------
 	// Now we can start the enter-string cycle...
 	//
 	finished = FALSE;
@@ -728,19 +732,20 @@ char *GetEditableStringInPopupWindow(int MaxLen, char *PopupWindowTitle, char *D
 
 		DisplayText(PopupWindowTitle, TargetRect.x, TargetRect.y, &TargetRect, TEXT_STRETCH);
 
-		// PutString (Screen, x0, y0, input);
-		TargetRect.y = x0;
-		TargetRect.y = y0;
+		if (PopupWindowTitle[strlen(PopupWindowTitle) - 1] != '\n')
+			DisplayText("\n\n", x0, y0, &TargetRect, TEXT_STRETCH);
+
+		TargetRect.y = MyCursorX;
+		TargetRect.y = MyCursorY;
 		DisplayText(input, TargetRect.x, TargetRect.y, &TargetRect, TEXT_STRETCH);
 
-		//--------------------
 		// We position the cursor right on its real location
 		//
 		tmp_char = input[curpos];
 		input[curpos] = 0;
-		CursorRect.x = x0 + TextWidth(input);
+		CursorRect.x = MyCursorX;
 		input[curpos] = tmp_char;
-		CursorRect.y = y0;
+		CursorRect.y = TargetRect.y;
 		CursorRect.h = FontHeight(GetCurrentFont());
 		CursorRect.w = 8;
 		HighlightRectangle(Screen, CursorRect);
@@ -752,8 +757,11 @@ char *GetEditableStringInPopupWindow(int MaxLen, char *PopupWindowTitle, char *D
 		if (key == SDLK_RETURN || key == SDLK_KP_ENTER) {
 			// input[curpos] = 0;
 			finished = TRUE;
+		} else if (key == SDLK_ESCAPE) {
+			while (EscapePressed())
+				SDL_Delay(1);
+			return NULL;
 		} else if (isprint(key) && (curpos < MaxLen)) {
-			//--------------------
 			// If a printable character has been entered, it is either appended to
 			// the end of the current input string or the rest of the string is being
 			// moved and the new character inserted at the end.
@@ -864,4 +872,39 @@ void printf_SDL(SDL_Surface * screen, int x, int y, const char *fmt, ...)
 
 };				// void printf_SDL (SDL_Surface *screen, int x, int y, char *fmt, ...)
 
+/**
+ * Find the longest line in text, taking letter-spacing into account. Return the
+ * minimum line width needed to print that line, using the current font.
+ */
+int longest_line_width(char *text)
+{
+	int width = 0;
+	char *line_start = text;
+
+	for (; TRUE; text++) {
+		// Find the end of the current line.
+		if (*text != '\n' && *text != '\0') {
+			continue;
+		}
+
+		// Get the width of the current line.
+		char tmp = *text;
+		*text = '\0';
+		int line_width = TextWidth(line_start);
+		*text = tmp;
+
+		// Update the width of the longest line.
+		if (line_width > width) {
+			width = line_width;
+		}
+
+		// Handle the next line, if any remain.
+		if (*text != '\0') {
+			line_start = text + 1;
+		} else {
+			return width;
+		}
+	}
+}
+
 #undef _text_c
