@@ -2,7 +2,7 @@
  *
  *
  *   Copyright (c) 2003 Johannes Prix
- *   Copyright (c) 2004-2007 Arthur Huillet 
+ *   Copyright (c) 2004-2010 Arthur Huillet 
  *
  *  This file is part of Freedroid
  *
@@ -43,7 +43,6 @@ void load_item_surfaces_for_item_type(int item_type);
 
 extern int Number_Of_Item_Types;
 extern int debug_level;
-extern dialogue_option ChatRoster[MAX_DIALOGUE_OPTIONS_IN_ROSTER];
 
 /** 
  * This function works a malloc, except that it also checks for
@@ -96,7 +95,7 @@ void ErrorMessage(const char *FunctionName, const char *fmt, int InformDeveloper
 
 	fprintf(stderr, "\n----------------------------------------------------------------------\n\
 Freedroid has encountered a problem:\n");
-	fprintf(stderr, "In Function: %s.\n", FunctionName);
+	fprintf(stderr, "In function: %s.\n", FunctionName);
 	fprintf(stderr, "FreedroidRPG package and version number: %s %s.\n", PACKAGE, VERSION);
 	vfprintf(stderr, fmt, args);
 
@@ -108,11 +107,13 @@ If you encounter this message, please inform the Freedroid developers\n\
 about the problem, by either\n\
 \n\
   sending e-mail to:\n\
-    freedroid-discussion@lists.sourceforge.net\n\
+    freedroid-discussion AT lists.sourceforge.net\n\
   mention it to someone of the developers on our IRC channel:\n\
     channel: #freedroid on irc.freenode.net\n\
   post on our forum at:\n\
     https://sourceforge.net/apps/phpbb/freedroid\n\
+  report a bug on our tracker at:\n\
+    https://sourceforge.net/tracker/?group_id=54521&atid=474016\n\
 \n\
 Thanks a lot!\n\n");
 	}
@@ -121,7 +122,7 @@ Thanks a lot!\n\n");
 		fprintf(stderr, "Freedroid will terminate now to draw attention to the problems it could\n\
 not resolve.  Sorry if that interrupts a major game of yours...\n");
 	} else {
-		fprintf(stderr, "The problem mentionned above is not fatal, we continue the execution.\n");
+		fprintf(stderr, "The problem mentioned above is not fatal, we continue the execution.\n");
 		fprintf(stderr, "----------------------------------------------------------------------\n");
 	}
 
@@ -153,18 +154,15 @@ void *MyMemmem(char *haystack, size_t haystacklen, char *needle, size_t needlele
 	DebugPrintf(3, "%s(): haystack = %d, len = %d, needle=%s\n", __FUNCTION__, haystack, haystacklen, needle);
 
 	while (haystacklen - SearchPos > 0) {
-		//--------------------
 		// We search for the first match OF THE FIRST CHARACTER of needle
 		//
 		NextFoundPointer = memchr(haystack + SearchPos, needle[0], haystacklen - SearchPos);
 
-		//--------------------
 		// if not even that was found, we can immediately return and report our failure to find it
 		//
 		if (NextFoundPointer == NULL)
 			return (NULL);
 
-		//--------------------
 		// Otherwise we see, if also the rest of the strings match this time ASSUMING THEY ARE STRINGS!
 		// In case of a match, we can return immediately
 		//
@@ -177,7 +175,6 @@ void *MyMemmem(char *haystack, size_t haystacklen, char *needle, size_t needlele
 		if (MatchPointer != NULL)
 			return (MatchPointer);
 
-		//--------------------
 		// At this point, we know that we had no luck with this one occasion of a first-character-match
 		// and must continue after this one occasion with our search
 		SearchPos = NextFoundPointer - haystack + 1;
@@ -192,23 +189,17 @@ void *MyMemmem(char *haystack, size_t haystacklen, char *needle, size_t needlele
  * it, copies it there and returns it.
  * The original source string specified should in no way be modified.
  * Returns 0 if the prefix string is not present
- * Optionally, a char can be given as "end of record".  Usually '\n'.
- * Set to 0 when not used.
  */
-char *ReadAndMallocStringFromDataOptional(char *SearchString, const char *StartIndicationString, const char *EndIndicationString,
-					  char Terminator)
+char *ReadAndMallocStringFromDataOptional(char *SearchString, const char *StartIndicationString, const char *EndIndicationString)
 {
 	char *SearchPointer;
 	char *EndOfStringPointer;
 	char *ReturnString = "";
-	char *Limit;
 	int StringLength;
 
-	for (Limit = SearchString; *Limit && *Limit != Terminator; Limit++) ;
-
 	SearchPointer = strstr(SearchString, StartIndicationString);
 
-	if ((!SearchPointer) || (SearchPointer >= Limit))
+	if (!SearchPointer)
 		return 0;
 	else {
 		// Now we move to the beginning
@@ -216,7 +207,7 @@ char *ReadAndMallocStringFromDataOptional(char *SearchString, const char *StartI
 
 		// Now we move to the end with the end pointer
 		EndOfStringPointer = strstr(SearchPointer, EndIndicationString);
-		if ((!EndOfStringPointer) || (EndOfStringPointer >= Limit))
+		if (!EndOfStringPointer)
 			return 0;
 
 		// Now we allocate memory and copy the string...
@@ -229,11 +220,9 @@ char *ReadAndMallocStringFromDataOptional(char *SearchString, const char *StartI
 		} else {
 			ReturnString = "";
 		}
-
-		DebugPrintf(2, "\nchar* ReadAndMalocStringFromData (...): Successfully identified string : %s.", ReturnString);
 	}
-	return (ReturnString);
-};				// char* ReadAndMallocStringFromDataOptional ( ... )
+	return ReturnString;
+}
 
 /**
  * This function looks for a string begin indicator and takes the string
@@ -244,7 +233,7 @@ char *ReadAndMallocStringFromDataOptional(char *SearchString, const char *StartI
  */
 char *ReadAndMallocStringFromData(char *SearchString, const char *StartIndicationString, const char *EndIndicationString)
 {
-	char *result = ReadAndMallocStringFromDataOptional(SearchString, StartIndicationString, EndIndicationString, 0);
+	char *result = ReadAndMallocStringFromDataOptional(SearchString, StartIndicationString, EndIndicationString);
 	if (!result) {
 		fprintf(stderr, "\n\nStartIndicationString: '%s'\nEndIndicationString: '%s'\n", StartIndicationString, EndIndicationString);
 		ErrorMessage(__FUNCTION__, "\
@@ -279,16 +268,13 @@ int CountStringOccurences(char *SearchString, const char *TargetString)
  * terminates the whole read in file with a 0 character, so that it
  * can easily be treated like a common string.
  */
-char *ReadAndMallocAndTerminateFile(char *filename, const char *File_End_String)
+char *ReadAndMallocAndTerminateFile(const char *filename, const char *File_End_String)
 {
 	FILE *DataFile;
 	char *Data;
 	char *ReadPointer;
 	long MemoryAmount;
 
-	DebugPrintf(1, "\n%s() : The filename is: %s", __FUNCTION__, filename);
-
-	//--------------------
 	// Read the whole theme data to memory.  We use binary mode, as we
 	// don't want to have to deal with any carriage return/line feed 
 	// convention mess on win32 or something...
@@ -342,7 +328,7 @@ This indicates a serious bug in this installation of Freedroid.", PLEASE_INFORM,
 		Data[MemoryAmount - 100] = 0;
 
 	return (Data);
-};				// char* ReadAndMallocAndTerminateFile( char* filename) 
+}
 
 /**
  * This function tries to locate a string in some given data string.
@@ -389,7 +375,6 @@ ReadValueFromStringWithDefault(char *SearchBeginPointer, const char *ValuePrecee
 	else
 		SourceLocation = DefaultValueString;
 
-	//--------------------
 	// Attention!!! 
 	// Now we try to read in the value!!!
 	//
@@ -425,7 +410,6 @@ ReadValueFromString(char *SearchBeginPointer, const char *ValuePreceedText, cons
 	SourceLocation = LocateStringInData(SearchBeginPointer, ValuePreceedText);
 	SourceLocation += strlen(ValuePreceedText);
 
-	//--------------------
 	// Attention!!! 
 	// Now we try to read in the value!!!
 	//
